package com.persagy.fm.sop.aop;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.text.CharSequenceUtil;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.exception.BusinessException;

import java.util.Collection;

/**
 * @author yuecaipu
 * @version 1.0
 * @description 参数校验
 * @date 2020/12/1 11:53
 */
public class BusinessAssert extends Assert {

	private static final String EMPTY_ERROR_MSG_TEMPLATE = "argument [{}] must not empty";

	private static final String CONTENT_NOT_EXIST_MSG_TEMPLATE = "content [{}] not exist";

	private static final String CONTENT_NOT_MATCH_MSG_TEMPLATE = "content [{}] not match";

	private static final String UPLOAD_FAIL = "upload fail";

	public static void isNotEmpty(String text, Object params) throws IllegalArgumentException {
		if (CharSequenceUtil.isBlank(text)) {
			throw new BusinessException(ResponseCode.A10000.getCode(), CharSequenceUtil.format(EMPTY_ERROR_MSG_TEMPLATE, params));
		}
	}

	public static void isNotEmpty(Object obj, Object params) throws IllegalArgumentException {
		if (null == obj) {
			throw new BusinessException(ResponseCode.A10000.getCode(), CharSequenceUtil.format(EMPTY_ERROR_MSG_TEMPLATE, params));
		}
	}

	public static void isNotEmpty(String text, String code, Object params) throws IllegalArgumentException {
		if (CharSequenceUtil.isBlank(text)) {
			throw new BusinessException(code, CharSequenceUtil.format(EMPTY_ERROR_MSG_TEMPLATE, params));
		}
	}

	public static void isNotEmpty(Collection collection, Object params) throws IllegalArgumentException {
		if (CollUtil.isEmpty(collection)) {
			throw new BusinessException(ResponseCode.A10000.getCode(), CharSequenceUtil.format(EMPTY_ERROR_MSG_TEMPLATE, params));
		}
	}

	public static void notExist(Object params) throws IllegalArgumentException {
		throw new BusinessException(ResponseCode.A10000.getCode(), CharSequenceUtil.format(CONTENT_NOT_EXIST_MSG_TEMPLATE, params));
	}

	public static void notMatch(Object params) throws IllegalArgumentException {
		throw new BusinessException(ResponseCode.A10000.getCode(), CharSequenceUtil.format(CONTENT_NOT_MATCH_MSG_TEMPLATE, params));
	}

}
