package com.persagy.fm.sop.controller;

import com.persagy.fm.common.constant.CommonConstant;
import com.persagy.fm.common.model.vo.BaseVo;
import com.persagy.fm.common.utils.FileUtil;
import com.persagy.fm.sop.aop.BusinessAssert;
import com.persagy.fm.sop.constant.SopQueryConst;
import com.persagy.fm.sop.service.IBatchHandleSopService;
import com.persagy.log.core.annotation.IgnoreLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/***
 * Description: 批量导入下载SOP的controler类
 * @author : lijie
 * @date :2021/4/28 10:45
 * Update By lijie 2021/4/28 10:45
 */
@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping("/restInputSopService/")
@Api("批量导入SOP业务")
public class BatchHandleSopController {

    private final IBatchHandleSopService batchHandleSopService;

    /**
     * Description: 集团sop-新增页-添加发布状态sop
     * @param baseVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/3/31 20:46
     * Update By lijie 2021/3/31 20:46
     */
    @ApiOperation("下载导入模板")
    @PostMapping("downloadSopTemplate")
    @CrossOrigin
    public void addPublishedSop(@RequestBody @Validated BaseVo baseVo, HttpServletResponse httpServletResponse) throws Exception {
        batchHandleSopService.downloadSopTemplate(baseVo,httpServletResponse);
    }
    /***
     * Description: 批量导入SOP
     * @param projectId : 项目id
     * @param file : 上传的文件流
     * @param response : 响应体对象
     * @param request : 请求体对象
     * @return : void
     * @author : lijie
     * @date :2021/4/30 12:08
     * Update By lijie 2021/4/30 12:08
     */
    @ApiOperation("SOP批量导入")
    @PostMapping("uploadSop")
    @CrossOrigin
    @IgnoreLog
    public void uploadSop(@RequestParam(value = SopQueryConst.PROJECT_ID_HUMP,required = false) String projectId,
                          @RequestParam(SopQueryConst.DTO_FILE) MultipartFile file,
                          HttpServletRequest request,
                          HttpServletResponse response){
        try {
            if (file.isEmpty()){
                BusinessAssert.isNotEmpty(file.getName(),SopQueryConst.DTO_FILE);
            }
            String originalFilename = file.getOriginalFilename();
            if (StringUtils.isNotBlank(originalFilename)
                    && !originalFilename.endsWith(CommonConstant.EXCEL_SUFFIX)){
                BusinessAssert.notMatch(CommonConstant.EXCEL_SUFFIX);
            }
            batchHandleSopService.uploadSop(projectId,file,request,response);
        }catch (Exception exr){
            log.error("上传文件失败",exr);
            List<String> errorList = new ArrayList<>();
            errorList.add(exr.getMessage());
            FileUtil.writeErrorContentToResponse(errorList,response,request,SopQueryConst.UPLOAD_SOP_ERR_FILE_NAME);
        }
    }
    /***
     * Description: 导出SOP
     * @param projectId : 项目id
     * @param response : 响应体对象
     * @return : void
     * @author : lijie
     * @date :2021/4/30 12:08
     * Update By lijie 2021/4/30 12:08
     */
    @ApiOperation("导出SOP")
    @PostMapping("exportSop")
    @CrossOrigin
    @IgnoreLog
    public void exportSop(@RequestParam(value = SopQueryConst.PROJECT_ID_HUMP,required = false) String projectId,
                          HttpServletResponse response) throws Exception {
        batchHandleSopService.exportSop(projectId,response);
    }


}
