package com.persagy.fm.sop.deserializers;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.JsonToken;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.deser.std.StringCollectionDeserializer;
import com.persagy.fm.common.helper.SpringHelper;
import com.persagy.fm.sop.constant.SopQueryConst;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.jackson.JsonComponent;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@JsonComponent
public class SpecialParamDeserializer extends JsonDeserializer<Object> {

    @Override
    public Object deserialize(JsonParser jsonParser, DeserializationContext deserializationContext) throws IOException, JsonProcessingException {
        JsonToken currentToken = jsonParser.getCurrentToken();
        if (jsonParser.getCurrentName().equals(SopQueryConst.VERSION_EXPLAIN)){
            if (currentToken==JsonToken.START_ARRAY){
                return deserializationContext.readValue(jsonParser,
                        deserializationContext.getTypeFactory().constructCollectionType(ArrayList.class, String.class));
            }
            String value = jsonParser.getValueAsString();
            // 解决老的版本中前端的bug,每次传version_explain都会变成多一层双引号
            if (StrUtil.isNotBlank(value) &&
                    (SopQueryConst.SPECIAL_ROD.equals(value)
                            || SopQueryConst.SPECIAL_QUO.equals(value)
                            || value.matches("^(\\u005C*\\u0022*\\u005C*)+?$"))){
                return new ArrayList<>();
            }
        }
        String value = jsonParser.getValueAsString();
        if (SopQueryConst.SPECIAL_ROD.equals(value)){
            return null;
        }
        if (jsonParser.getCurrentName().equals(SopQueryConst.VERSION_COUNT)){
            if (StrUtil.isNotBlank(value) && NumberUtil.isNumber(value)){
                return Integer.parseInt(value);
            }
        }
        return value;
    }
}
