package com.persagy.fm.sop.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;

import java.io.Serializable;
import java.util.List;

import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.persagy.fm.sop.handler.JsonTypeHandlerFactory;
import com.persagy.fm.sop.model.dto.*;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * <p>
 * 
 * </p>
 *
 * @author lijie
 * @since 2021-04-02
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName(value = "sop_edit",autoResultMap = true)
public class SopEdit extends Model<SopEdit> {

    private static final long serialVersionUID = 1L;
    /**有效状态*/
    public static final String VALID = "valid";
    /**SOP的名称*/
    public static final String SOP_NAME = "sop_name";
    /**SOP的状态*/
    public static final String SOP_STATUS = "sop_status";
    /**SOP的更新时间*/
    public static final String UPDATE_TIME = "update_time";

    /**
     * 主键id
     */
    @TableId(value = "sop_edit_id", type = IdType.ASSIGN_ID)
    private String sopEditId;

    /**
     * SOP的主键id
     */
    @TableField("sop_id")
    private String sopId;

    /**
     * 项目id,集团SOP的值为group_sop_id
     */
    @TableField("project_id")
    private String projectId;

    /**
     * 建筑功能类型
     */
    @TableField("extend_field")
    private String extendField;

    /**
     * 操作人信息
     */
    @TableField(value = "operator",typeHandler = JacksonTypeHandler.class)
    private OperatorBean operator;

    /**
     * SOP的名称,目前长度定为100.交互支持50个字符
     */
    @TableField("sop_name")
    private String sopName;

    /**
     * SOP类型,1-通用,2-项目专用
     */
    @TableField("sop_type")
    private String sopType;

    /**
     * 是否有无对象的步骤,0-否 1-是.当为引用的SOP时,就算被引用的没有对象也为1
     */
    @TableField("no_obj")
    private String noObj;

    /**
     * 专业编码,[code,code]
     */
    @TableField(value = "domains",typeHandler = JacksonTypeHandler.class)
    private List<String> domains;

    /**
     * 工具数组,[{"tool"：“……”,"from_step":true}]
     */
    @TableField(value = "tools",typeHandler = JsonTypeHandlerFactory.ToolsBeanHandler.class)
    private List<ToolsBean> tools;

    /**
     * 关联的工单类型,数组[code,code]
     */
    @TableField(value = "order_type",typeHandler = JacksonTypeHandler.class)
    private List<String> orderType;

    /**
     * 适用对象
     */
    @TableField(value = "fit_objs",typeHandler = JsonTypeHandlerFactory.FitObjsBeanHandler.class)
    private List<FitObjsBean> fitObjs;

    /**
     * 设备型号
     */
    @TableField(value = "equip_model_ids",typeHandler = JacksonTypeHandler.class)
    private List<String> equipModelIds;

    /**
     * 自定义标签
     */
    @TableField(value = "labels",typeHandler = JacksonTypeHandler.class)
    private List<String> labels;

    /**
     * 步骤
     */
    @TableField(value = "steps",typeHandler = JsonTypeHandlerFactory.SopStepsBeanHandler.class)
    private List<SopStepsBean> steps;

    /**
     * 相关资料,[{"name":"***",“url”：“***”},{}]
     */
    @TableField(value = "related_data",typeHandler = JsonTypeHandlerFactory.RelatedDataBeanHandler.class)
    private List<RelatedDataBean> relatedData;

    /**
     * 1-已发布、2-已作废
     */
    @TableField("sop_status")
    private String sopStatus;

    /**
     * 1-正常、2-待修订、3-修订中
     */
    @TableField("publish_status")
    private String publishStatus;

    /**
     * 引用的sop
     */
    @TableField(value = "ref_sop_ids",typeHandler = JacksonTypeHandler.class)
    private List<String> refSopIds;

    /**
     * 版本号
     */
    @TableField("version")
    private String version;

    /**
     * 版本数量
     */
    @TableField("version_count")
    private Integer versionCount;

    /**
     * 版本说明,[“描述”,“描述”]
     */
    @TableField(value = "version_explain",typeHandler = JacksonTypeHandler.class)
    private List<String> versionExplain;

    /**
     * 创建时间,格式为yyyyMMddHHmmss
     */
    @TableField(value = "create_time",fill = FieldFill.INSERT)
    private String createTime;

    /**
     * 更新时间,格式为yyyyMMddHHmmss
     */
    @TableField(value = "update_time",fill = FieldFill.INSERT_UPDATE)
    private String updateTime;

    /**
     * 是否有效
     */
    @TableField("valid")
    private Integer valid;


    @Override
    protected Serializable pkVal() {
        return this.sopEditId;
    }

}
