package com.persagy.fm.sop.filters;

import com.persagy.fm.sop.service.BaseService;
import org.apache.log4j.Logger;
import org.springframework.core.annotation.Order;

import javax.servlet.*;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;


@WebFilter
@Order(-1)
public class LoginFilter implements Filter {
	private static final Logger log = Logger.getLogger(LoginFilter.class);
	
	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		log.info("init loginFilter....");
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		if (request instanceof HttpServletRequest
                && response instanceof HttpServletResponse) {
            doFilter((HttpServletRequest) request, (HttpServletResponse) response, chain);
        } else {
            chain.doFilter(request, response);
        }
	}
	

	@Override
	public void destroy() {
		log.info("destroy loginFilter....");
	}

	private void doFilter(HttpServletRequest request, HttpServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		
		try {
			String url = request.getRequestURL().toString();
			// 获取请求路径与参数，并输出到日志中
			RequestWrapper requestWrapper = new RequestWrapper(request);
			ServletInputStream inputStream = requestWrapper.getInputStream();
			byte[] bytes = input2byte(inputStream);
			String params = this.inputStreamToString(new ByteArrayInputStream(bytes != null ? bytes : new byte[0]));
			log.info("前端请求：" + url + "--" + params);
			//处理验证接口
			chain.doFilter(requestWrapper, response);
		} catch (Exception e) {
			String exMsg = e.toString() + "--" + Arrays.toString(e.getStackTrace());
			log.error(exMsg);
			response.setStatus(401);
			response.getWriter().write(exMsg);
		}
	}

	public String inputStreamToString(InputStream in) throws IOException {
		StringBuffer out = new StringBuffer();
		byte[] b = new byte[4096];
		for (int n; (n = in.read(b)) != -1;) {
			out.append(new String(b, 0, n, "UTF-8"));
		}
		return out.toString();
	}

	public static final byte[] input2byte(InputStream inStream) throws IOException {
		ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
		byte[] buff = new byte[100];
		int rc = 0;
		while ((rc = inStream.read(buff, 0, 100)) > 0) {
			swapStream.write(buff, 0, rc);
		}
		byte[] in2b = swapStream.toByteArray();
		return in2b;
	}
	


}
