package com.persagy.fm.sop.model.dto;

import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.persagy.fm.sop.entity.SopCell;
import com.persagy.fm.sop.entity.SopObjRel;
import com.persagy.fm.sop.entity.SopToolRel;
import com.persagy.fm.sop.model.vo.SignVo;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;
/***
 * Description: SOP的DTO类
 * @author : lijie
 * @date :2021/4/10 16:27
 * Update By lijie 2021/4/10 16:27
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
public class SopBean {
    /**SOP的主键id*/
    @ApiModelProperty("SOP的主键id")
    @JsonProperty("sop_id")
    @JSONField(name = "sop_id")
    private String sopId;
    /**项目id,集团SOP的值为group_sop_id*/
    @ApiModelProperty("项目id,集团SOP的值为group_sop_id")
    @JsonProperty("project_id")
    @JSONField(name = "project_id")
    private String projectId;
    /**建筑功能类型*/
    @ApiModelProperty("建筑功能类型")
    @JsonProperty("extend_field")
    @JSONField(name = "extend_field")
    private String extendField;
    /**建筑功能类型名称*/
    @ApiModelProperty("建筑功能类型名称")
    @JsonProperty("extend_field_name")
    @JSONField(name = "extend_field_name")
    private String extendFieldName;
    /**操作人id,为人员id或账号id*/
    @ApiModelProperty(value = "操作人id,为人员id或账号id",hidden = true)
    @JsonProperty("operator_id")
    @JSONField(name = "operator_id")
    @JsonIgnore
    private String operatorId;
    /**操作人姓名,为人员名称或账号名称*/
    @ApiModelProperty(value = "操作人姓名,为人员名称或账号名称",hidden = true)
    @JsonProperty("operator_name")
    @JSONField(name = "operator_name")
    @JsonIgnore
    private String operatorName;
    /**SOP的名称,目前长度定为100.交互支持50个字符*/
    @ApiModelProperty(value = "SOP的名称,目前长度定为100.交互支持50个字符")
    @JsonProperty("sop_name")
    @JSONField(name = "sop_name")
    private String sopName;
    /**SOP类型,1-通用,2-项目专用*/
    @ApiModelProperty(value = "SOP类型,1-通用,2-项目专用")
    @JsonProperty("sop_type")
    @JSONField(name = "sop_type")
    private String sopType;
    /**是否有无对象的步骤,0-否 1-是.当为引用的SOP时,就算被引用的没有对象也为1*/
    @ApiModelProperty(value = "是否有无对象的步骤,0-否 1-是.当为引用的SOP时,就算被引用的没有对象也为1")
    @JsonProperty("no_obj")
    @JSONField(name = "no_obj")
    private String noObj;
    /**相关资料,[{"name":"***","url"："***"}]*/
    @ApiModelProperty(value = "相关资料,[{\"name\":\"***\",\"url\"：\"***\"}]")
    @JsonProperty("related_data")
    @JSONField(name = "related_data")
    private Object relatedData;
    /**1-已发布、2-已作废*/
    @ApiModelProperty(value = "1-已发布、2-已作废")
    @JsonProperty("sop_status")
    @JSONField(name = "sop_status")
    private String sopStatus;
    /**1-正常、2-待修订、3-修订中*/
    @ApiModelProperty(value = "1-正常、2-待修订、3-修订中")
    @JsonProperty("publish_status")
    @JSONField(name = "publish_status")
    private String publishStatus;
    /**版本号*/
    @ApiModelProperty(value = "1-正常、2-待修订、3-修订中")
    private String version;
    /**版本数*/
    @ApiModelProperty(value = "当前版本号")
    @JsonProperty("version_count")
    @JSONField(name = "version_count")
    private Integer versionCount;
    /**版本说明,[“描述”,“描述”]*/
    @ApiModelProperty(value = "版本说明,[“描述”,“描述”]")
    @JsonProperty("version_explain")
    @JSONField(name = "version_explain")
    private List<String> versionExplain;
    /**创建时间,格式为yyyyMMddHHmmss*/
    @ApiModelProperty(value = "创建时间")
    @JsonProperty("create_time")
    @JSONField(name = "create_time")
    private String createTime;
    /**更新时间,格式为yyyyMMddHHmmss*/
    @ApiModelProperty(value = "更新时间")
    @JsonProperty("update_time")
    @JSONField(name = "update_time")
    private String updateTime;
    /**
     * 是否有效
     */
    @ApiModelProperty(value = "是否有效",hidden = true)
    @JsonIgnore
    private Boolean valid;
    /**操作人对象*/
    @ApiModelProperty(value = "操作人对象")
    @JsonProperty("operator")
    @JSONField(name = "operator")
    private OperatorBean operator;
    /**关联的专业*/
    @ApiModelProperty(value = "专业列表")
    @JsonProperty("domains")
    @JSONField(name = "domains")
    private List<String> domains;
    /**关联的工作类型*/
    @ApiModelProperty(value = "关联的工作类型")
    @JsonProperty("order_type")
    @JSONField(name = "order_type")
    private List<String> orderType;
    /**关联的工具对象*/
    @ApiModelProperty(value = "关联的工具对象")
    @JsonProperty("tools")
    @JSONField(name = "tools")
    private List<ToolsBean> tools;
    /**关联的适用范围对象*/
    @ApiModelProperty(value = "关联的适用范围对象")
    @JsonProperty("fit_objs")
    @JSONField(name = "fit_objs")
    private List<SopObjRel> fitObjs;
    /**关联的设备模型id*/
    @ApiModelProperty(value = "关联的设备模型id列表")
    @JsonProperty("equip_model_ids")
    @JSONField(name = "equip_model_ids")
    private List<String> equipModelIds;
    /**关联的标签*/
    @ApiModelProperty(value = "关联的标签列表")
    @JsonProperty("labels")
    @JSONField(name = "labels")
    private List<String> labels;
    /**关联的steps*/
    @ApiModelProperty(value = "关联的steps")
    @JsonProperty("steps")
    @JSONField(name = "steps")
    private List<SopCell> steps;
    /**步骤长度*/
    @ApiModelProperty(value = "步骤长度")
    @JsonProperty("step_count")
    @JSONField(name = "step_count")
    private Integer stepCount;
    /**标志*/
    @ApiModelProperty(value = "标志")
    @JsonProperty("sign")
    @JSONField(name = "sign")
    private SopSignBean sign;
    /**状态*/
    @ApiModelProperty(value = "状态")
    @JsonProperty("status")
    @JSONField(name = "status")
    private String status;
    /**scope数组*/
    @ApiModelProperty(value = "scope数组")
    @JsonProperty("scopes")
    @JSONField(name = "scopes")
    private List<String> scopes;
    /**状态表达式*/
    @ApiModelProperty(value = "状态表达式")
    @JsonProperty("stauts_explain")
    @JSONField(name = "stauts_explain")
    private String stautsExplain;
    /**专业名称 ,非空（querySopDetailById接口返回）*/
    @ApiModelProperty(value = "专业名称 ,非空（querySopDetailById接口返回）")
    @JsonProperty("domains_names")
    @JSONField(name = "domains_names")
    private List<String> domainNames;
    /**专业名称 ,非空（querySopDetailById接口返回）*/
    @ApiModelProperty(value = "工单类型名（querySopDetailById接口返回）")
    @JsonProperty("order_type_names")
    @JSONField(name = "order_type_names")
    private List<String> orderTypeNames;
    /**专业名称 ,非空（querySopDetailById接口返回）*/
    @ApiModelProperty(value = "设备型号对象列表")
    @JsonProperty("equip_models")
    @JSONField(name = "equip_models")
    private List<EquipModelBean> equipModels;
    /**
     * 状态表达式
     */
    @JsonProperty("ref_sop_ids")
    @JSONField(name = "refSopIds")
    private List<String> refSopIds;
}
