package com.persagy.fm.sop.model.dto;

import com.fasterxml.jackson.annotation.JsonAlias;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.persagy.fm.common.validator.Insert;
import com.persagy.fm.sop.validator.ProjectInsert;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class StepContentBean {
    /**
     * from_sop : false
     * pre_execute : 1
     * pre_conform : *******
     * notice : *****
     * content_objs : [{"obj_id":"***","obj_name":"对象名称1","obj_type":"equip_class","type_code":"","parents":[{"parent_ids":["***","***","***"],"parent_names":["建筑1","楼层1","空间"]},{"parent_ids":["***","***"],"parent_names":["专业1","系统1"]},{"parent_ids":["***","***","***"],"parent_names":["专业1","系统大类","设备大类"]}]}]
     * space_scope : [{"obj_id":"***","obj_name":"对象名称1","obj_type":"space_class"}]
     * obj_required_control : ["code","code"]
     * obj_show_item : ["code","code"]
     * time_limit : 5
     * confirm_result : [{"description":"","pic":["",""],"att":[{"key":"","name":""}],"info_points":[{"id":"***","code":"****","name":"****","unit":"***","cmpt":"Numberentry02","wrong_ranges":[{"type":"gt","values":"12"},{"type":"lt","values":"6"}],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"id":"***","code":"****","name":"****","unit":"***","cmpt":"Multiselect01","cmpt_data":[{"code":"***","name":"***"}],"wrongs":["选项1","选项2"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}}],"customs":[{"is_select":true,"name":"异常情况","type":"2","items":["正常","异常"],"wrongs":["异常"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息2","type":"2","items":["选项1","选项2","选项3"],"wrongs":["选项1","选项2"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息3","type":"3","items":["选项1","选项2","选项3"],"wrongs":["选项1","选项2"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息4","type":"4","wrong_ranges":[{"type":"gt","values":"12"},{"type":"lt","values":"6"}],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息5","type":"5","unit":"***","wrong_ranges":[{"type":"range","values":["12","20"]}],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}}]}]
     * domain : *****
     * sop_id : ****
     */
    @ApiModelProperty("是否引用的SOP")
    @JsonProperty("from_sop")
    @JsonAlias({"from_sop","fromSop"})
    @NotNull(message = "缺少非空项:step_content.from_sop",groups = {Insert.class, ProjectInsert.class})
    private Boolean fromSop;
    @ApiModelProperty("执行前验证,默认为空,1-设备运行继续执行,2-设备停用继续执行,3-设备停机继续执行")
    @JsonProperty("pre_execute")
    @JsonAlias({"pre_execute","preExecute"})
    private String preExecute;
    @ApiModelProperty("强制确认")
    @JsonProperty("pre_conform")
    @JsonAlias({"pre_conform","preConform"})
    private String preConform;
    @ApiModelProperty("注意事项")
    private String notice;
    @ApiModelProperty("工作时长(分钟数)")
    @JsonProperty("time_limit")
    @JsonAlias({"time_limit","timeLimit"})
    private Integer timeLimit;
    @ApiModelProperty("专业code")
    private String domain;
    @ApiModelProperty("SOP数据唯一标识id")
    @JsonProperty("sop_id")
    @JsonAlias({"sop_id","sopId"})
    private String sopId;
    @ApiModelProperty("工作内容中涉及的对象")
    @JsonProperty("content_objs")
    @JsonAlias({"contentObjs","content_objs"})
    private List<@Valid ContentObjsBean> contentObjs;
    @ApiModelProperty("空间范围：只有选择设备类时才有此设置")
    @JsonProperty("space_scope")
    @JsonAlias({"spaceScope","space_scope"})
    private List<SpaceScopeBean> spaceScope;
    @ApiModelProperty("对象管控需求")
    @JsonProperty("obj_required_control")
    @JsonAlias({"obj_required_control","objRequiredControl"})
    private List<String> objRequiredControl;
    @ApiModelProperty("对象展示项 working_time-对象运行时间")
    @JsonProperty("obj_show_item")
    @JsonAlias({"obj_show_item","objShowItem"})
    private List<String> objShowItem;
    @ApiModelProperty("操作内容")
    @JsonProperty("confirm_result")
    @JsonAlias({"confirm_result","confirmResult"})
    private List<@Valid ConfirmResultBean> confirmResult;
    @ApiModelProperty(value = "专业名称",hidden = true)
    @JsonProperty("domain_name")
    @JsonAlias({"domain_name","domainName"})
    private String domainName;
    @ApiModelProperty(value = "步骤内容中的content,兼容老数据产生",hidden = true)
    private String content;
    @ApiModelProperty(value = "前端传递的参数",hidden = true)
    @JsonProperty("edit_content")
    @JsonAlias({"edit_content","editContent"})
    private Boolean editContent;
    @ApiModelProperty(value = "引用的SOP类",hidden = true)
    private RefedSopBean profile;
    @ApiModelProperty(value = "在到达每个对象位置时拍照",hidden = true)
    @JsonAlias({"isPhoto","is_photo"})
    @JsonProperty("isPhoto")
    private Boolean isPhoto;
    @ApiModelProperty(value = "在到达每个对象位置时扫码",hidden = true)
    @JsonAlias({"isScan","is_scan"})
    @JsonProperty("isScan")
    private Boolean isScan;
    @ApiModelProperty(value = "在到达每个指定对象时进行NFC识别",hidden = true)
    @JsonAlias({"isNfc","is_nfc"})
    @JsonProperty("isNfc")
    private Boolean isNfc;
    @ApiModelProperty(value = "对象运行时间",hidden = true)
    @JsonAlias({"workingTime","working_time"})
    @JsonProperty("working_time")
    private Boolean workingTime;
    @ApiModelProperty(value = "前端传参,兼容老数据产生",hidden = true)
    @JsonAlias({"preConformFocus","pre_conform_focus"})
    @JsonProperty("pre_conform_focus")
    private Boolean preConformFocus;
    @ApiModelProperty(value = "前端传参,兼容老数据产生",hidden = true)
    @JsonAlias({"noticeFocus","notice_focus"})
    @JsonProperty("notice_focus")
    private Boolean noticeFocus;
    @ApiModelProperty(value = "前端传参,兼容老数据产生",hidden = true)
    @JsonAlias({"contentFocus","content_focus"})
    @JsonProperty("content_focus")
    private Boolean contentFocus;
    @ApiModelProperty(value = "前端传参,兼容老数据产生",hidden = true)
    private String verified;
}