package com.persagy.fm.sop.model.vo;

import com.fasterxml.jackson.annotation.JsonAlias;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.persagy.fm.common.model.vo.BaseVo;
import com.persagy.fm.common.validator.Insert;
import com.persagy.fm.common.validator.Update;
import com.persagy.fm.sop.deserializers.SpecialParamDeserializer;
import com.persagy.fm.sop.model.dto.*;
import com.persagy.fm.sop.validator.*;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/***
 * Description: 新增SOP的VO类
 * @author : lijie
 * @date :2021/4/1 10:02
 * Update By lijie 2021/4/1 10:02
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class SaveSopVo extends BaseVo {

    /**项目id*/
    public static final String PROJECT_ID = "project_id";
    /**账号id*/
    public static final String USER_ID = "user_id";
    /**SOP名称*/
    public static final String SOP_NAME = "sop_name";
    /**steps*/
    public static final String STEPS = "steps";
    /**obj_id*/
    public static final String OBJ_ID = "obj_id";
    /**fit_objs*/
    public static final String FIT_OBJS = "fit_objs";
    /**content_objs*/
    public static final String CONTENT_OBJS = "content_objs";
    /**steps*/
    public static final String OBJ_NAME = "obj_name";
    /**obj_type*/
    public static final String OBJ_TYPE = "obj_type";
    /**confirm_result*/
    public static final String CONFIRM_RESULT = "confirm_result";
    /**description*/
    public static final String DESCRIPTION = "description";
    /**info_points*/
    public static final String INFO_POINTS = "info_points";
    /**customs*/
    public static final String CUSTOMS = "customs";
    /**id*/
    public static final String ID = "id";
    /**name*/
    public static final String NAME = "name";
    /**code*/
    public static final String CODE = "code";
    /**type*/
    public static final String TYPE = "type";
    /**from_sop*/
    public static final String FROM_SOP = "from_sop";
    /**step_content*/
    public static final String STEP_CONTENT = "step_content";

    /**
     * user_id : ***
     * project_id : ***
     * extend_field : ***
     * operator : {"id":"***","name":"燕小六"}
     * sop_id : ***
     * sop_name : ***
     * domains : ["code1","code2"]
     * order_type : ["code1","code2"]
     * no_obj : 1
     * tools : [{"tool":"name","from_step":true},{"tool":"name","from_step":true}]
     * fit_objs : [{"obj_id":"***","obj_name":"对象名称1","obj_code":"XXX","obj_type":"system","obj_source":"content_obj"}]
     * equip_model_ids : ["***","***"]
     * labels : ["name","name"]
     * related_data : [{"name":"***","url":"***"}]
     * steps : [{"from_sop":true,"sop_id":"****"},{"from_sop":false,"step_content":[{"from_sop":false,"pre_execute":"1","pre_conform":"*******","notice":"*****","content_objs":[{"obj_id":"***","obj_name":"对象名称1","obj_type":"equip_class","type_code":"","parents":[{"parent_ids":["***","***","***"],"parent_names":["建筑1","楼层1","空间"]},{"parent_ids":["***","***"],"parent_names":["专业1","系统1"]},{"parent_ids":["***","***","***"],"parent_names":["专业1","系统大类","设备大类"]}]}],"space_scope":[{"obj_id":"***","obj_name":"对象名称1","obj_type":"space_class"}],"obj_required_control":["code","code"],"obj_show_item":["code","code"],"time_limit":5,"confirm_result":[{"description":"","pic":["",""],"att":[{"key":"","name":""}],"info_points":[{"id":"***","code":"****","name":"****","unit":"***","cmpt":"Numberentry02","wrong_ranges":[{"type":"gt","values":"12"},{"type":"lt","values":"6"}],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"id":"***","code":"****","name":"****","unit":"***","cmpt":"Multiselect01","cmpt_data":[{"code":"***","name":"***"}],"wrongs":["选项1","选项2"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}}],"customs":[{"is_select":true,"name":"异常情况","type":"2","items":["正常","异常"],"wrongs":["异常"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息2","type":"2","items":["选项1","选项2","选项3"],"wrongs":["选项1","选项2"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息3","type":"3","items":["选项1","选项2","选项3"],"wrongs":["选项1","选项2"],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息4","type":"4","wrong_ranges":[{"type":"gt","values":"12"},{"type":"lt","values":"6"}],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}},{"name":"确认信息5","type":"5","unit":"***","wrong_ranges":[{"type":"range","values":["12","20"]}],"abnormal_set":{"execute_type":"","execute_dept_id":"","execute_dept_name":"","problem_type":"","problem_type_name":""}}]}],"domain":"*****"},{"from_sop":true,"sop_id":"****"}]},{"from_sop":false,"step_content":[]}]
     */
    @ApiModelProperty(value = "项目id")
    @NotBlank(message = "缺少非空项:project_id",groups = {ProjectInsert.class, ProjectEdit.class, ProjectUpdate.class})
    @JsonAlias({"projectId","project_id"})
    @JsonProperty("project_id")
    private String projectId;
    @ApiModelProperty(value = "建筑功能类型")
    @JsonAlias({"extendField","extend_field"})
    @JsonProperty("extend_field")
    private String extendField;
    @ApiModelProperty(value = "操作人信息,若账号有关联的人员 id和name保存人员，否则保存账号")
    @JsonProperty("operator")
    @JsonAlias({"operator"})
    private OperatorBean operator;
    @ApiModelProperty(value = "SOP类型,1-通用,2-项目专用",hidden = true)
    @JsonProperty("sop_type")
    @JsonAlias({"sopType","sop_type"})
    private String sopType;
    @ApiModelProperty(value = "Sop的唯一标识id")
    @JsonProperty("sop_id")
    @JsonAlias({"sopId","sop_id"})
    @NotBlank(message = "缺少非空项:sop_id", groups = {Update.class, ProjectUpdate.class})
    private String sopId;
    @NotBlank(message = "缺少非空项:sop_name",groups = {Insert.class, ProjectInsert.class,ProjectEdit.class,ProjectUpdate.class})
    @ApiModelProperty(value = "SOP名称")
    @JsonProperty("sop_name")
    @JsonAlias({"sopName","sop_name"})
    private String sopName;
    @ApiModelProperty(value = "是否包含无对象的步骤  1-是 0或空-否 ps.若被引用的sop中有无对象的步骤 则 引用的sop该值亦为1")
    @JsonProperty("no_obj")
    @JsonAlias({"noObj","no_obj"})
    private String noObj;
    @ApiModelProperty(value = "专业编码")
    private List<String> domains;
    @ApiModelProperty(value = "工单类型")
    @JsonProperty("order_type")
    @JsonAlias({"orderType","order_type"})
    private List<String> orderType;
    @ApiModelProperty(value = "工具,编辑时来源步骤中的工具，只能在步骤中删除")
    private List<ToolsBean> tools;
    @ApiModelProperty(value = "适用对象，适用范围中的建筑/楼层/空间/系统/设备")
    @JsonProperty("fit_objs")
    @JsonAlias({"fitObjs","fit_objs"})
    private List<@Valid FitObjsBean> fitObjs;
    @ApiModelProperty(value = "设备型号id集合")
    @JsonProperty("equip_model_ids")
    @JsonAlias({"equipModelIds","equip_model_ids"})
    private List<String> equipModelIds;
    @ApiModelProperty(value = "标签")
    private List<String> labels;
    @ApiModelProperty(value = "相关资料")
    @JsonProperty("related_data")
    @JsonAlias({"relatedData","related_data"})
    private List<RelatedDataBean> relatedData;
    @ApiModelProperty(value = "步骤信息")
    @NotEmpty(message = "缺少非空项:steps",groups = {Insert.class, ProjectInsert.class,ProjectUpdate.class})
    private List<@Valid SopStepsBean> steps;
    @ApiModelProperty(value = "状态,0-草稿、1-已发布，修订中")
    @JsonAlias({"sop_status","sopStatus"})
    @JsonProperty("sop_status")
    private String sopStatus;
    @ApiModelProperty(value = "版本号",hidden = true)
    @JsonAlias({"version_count","versionCount"})
    @JsonProperty("version_count")
    @JsonDeserialize(using = SpecialParamDeserializer.class)
    private Integer versionCount;
    @ApiModelProperty(value = "版本表达式",hidden = true)
    @JsonAlias({"version_explain","versionExplain"})
    @JsonProperty("version_explain")
    @JsonDeserialize(using = SpecialParamDeserializer.class)
    private List<String> versionExplain;
    @ApiModelProperty(value = "发布状态",hidden = true)
    @JsonAlias({"publish_status","publishStatus"})
    @JsonProperty("publish_status")
    private String publishStatus;
    @ApiModelProperty(value = "版本号",hidden = true)
    private String version;
    @ApiModelProperty(value = "配置服务名称",hidden = true)
    @JsonAlias({"_configServiceName"})
    private String configServiceName;






}
