package com.persagy.fm.sop.serializers;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
@Slf4j
public class JsonMapper extends ObjectMapper {
    // 静态内部类方式实现的单例模式
    private JsonMapper(){}

    public static JsonMapper getInstance(){
        return SingletonHolder.JSON_MAPPER;
    }

    private static class SingletonHolder{
        private static final JsonMapper JSON_MAPPER = new JsonMapper();
    }

    /**
     * Object可以是POJO，也可以是Collection或数组。
     * 如果对象为Null, 返回"null".
     * 如果集合为空集合, 返回"[]".
     *
     * @param object the object to json
     * @return toJson result
     */
    public String toJson(Object object) {
        try {
            return super.writeValueAsString(object);
        } catch (IOException e) {
            log.error("write to json string error:" + object, e);
            return null;
        }
    }
}
