package com.persagy.fm.sop.serializers;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.google.common.collect.ObjectArrays;
import com.persagy.fm.sop.annotations.JsonFieldFilter;
import com.persagy.fm.sop.annotations.JsonFieldFilters;
import com.persagy.fm.sop.filters.SuberJacksonFilterProvider;

/**
 * JSON 序列化，用于API返回字段过滤
 *
 * @author zhufeihong
 * @since 2020/12/28 15:11
 */
public class ResponseJsonFilterSerializer {

    // JsonMapper 继承自com.fasterxml.jackson.databind.ObjectMapper
    JsonMapper mapper = JsonMapper.getInstance();
    SuberJacksonFilterProvider filterProvider = new SuberJacksonFilterProvider();

    /**
     * json数据返回时过滤字段
     *
     * @param clazz   需要设置规则的Class
     * @param include 转换时包含哪些字段
     * @param exclude 转换时过滤哪些字段
     */
    public void filter(Class<?> clazz, String[] include, String[] exclude) {
        if (clazz == null) {
            return;
        }
        if (include != null && include.length > 0) {
            filterProvider.include(clazz, include);
        } else if (exclude != null && exclude.length > 0) {
            filterProvider.exclude(clazz, exclude);
        }
        mapper.addMixIn(clazz, filterProvider.getClass());
    }

    /**
     * json数据返回时过滤字段
     *
     * @param fieldFilters 注解数组进行过滤
     * @since 2021/1/4 17:26
     */
    public void filter(JsonFieldFilters fieldFilters) {
        for (JsonFieldFilter json : fieldFilters.value()) {
            this.filter(json.type(), json.include(),
                    ObjectArrays.concat(json.exclude(), json.addExclude(), String.class));
        }
    }

    public String toJson(Object object) throws JsonProcessingException {
        mapper.setFilterProvider(filterProvider);
        return JSON.toJSONString(object);
    }
}