package com.persagy.fm.sop.service;

import com.persagy.fm.sop.model.dto.RefedSopBean;
import com.persagy.fm.sop.model.dto.SopBean;
import com.persagy.fm.sop.model.vo.*;

import java.text.ParseException;
import java.util.List;

/***
 * Description: 集团SOP业务逻辑接口
 * @author : lijie
 * @date :2021/3/31 20:54
 * Update By lijie 2021/3/31 20:54
 */
public interface IGroupSopService {
    /***
     * Description: 集团sop-新增页-添加发布状态sop
     * @param saveSopVo : 请求参数
     * @return : void  
     * @author : lijie
     * @date :2021/4/1 14:46
     * Update By lijie 2021/4/1 14:46
     */
    void addPublishedSop(SaveSopVo saveSopVo);
    /***
     * Description: 集团sop-编辑页-编辑sop时验证sop名称
     * @param queryVerifySopNameBaseVo : 请求参数
     * @return : com.persagy.fm.sop.model.vo.VerifySopNameResult
     * @author : lijie
     * @date :2021/4/2 12:19
     * Update By lijie 2021/4/2 12:19
     */
    VerifySopNameVo verifySopNameForUpdate(QueryVerifySopNameBaseVo queryVerifySopNameBaseVo);
    /***
     * Description: 新增页:查询可供选择的sop，复制、引用时用
     * @param queryVerifySopNameBaseVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/2 14:13
     * Update By lijie 2021/4/2 14:13
     */
    VerifySopNameVo verifySopNameForAdd(QueryVerifySopNameBaseVo queryVerifySopNameBaseVo);
    /***
     * Description: 集团sop-新增页-添加草稿状态sop
     * @param saveSopVo : 保存SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/2 14:30
     * Update By lijie 2021/4/2 14:30
     */
    QuerySopVo addDraftSop(SaveSopVo saveSopVo);
    /***
     * Description: 集团sop-新增页-验证sop
     * @param saveSopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/2 15:28
     * Update By lijie 2021/4/2 15:28
     */
    List<String> verifySopBeforePublish(SaveSopVo saveSopVo) throws Exception;
    /***
     * Description: 集团sop-编辑页-发布sop
     * @param saveSopVo : 更新的SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/8 14:05
     * Update By lijie 2021/4/8 14:05
     */
    QuerySopVo publishSop(SaveSopVo saveSopVo) throws Exception;
    /***
     * Description: 集团sop-编辑页-保存草稿状态sop
     * @param saveSopVo : 保存SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/8 17:24
     * Update By lijie 2021/4/8 17:24
     */
    QuerySopVo saveDraftSop(SaveSopVo saveSopVo);
    /***
     * Description: 集团sop-列表页:查询已发布的sop列表
     * @param queryPublishedListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 15:56
     * Update By lijie 2021/4/10 15:56
     */
    PublishedListVo queryPublishedSopList(QueryPublishedListVo queryPublishedListVo) throws Exception;
    /***
     * Description: 集团sop-列表页 查询编辑中的sop列表
     * @param queryPublishedListBaseVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 18:22
     * Update By lijie 2021/4/10 18:22
     */
    List<SopBean> queryEditingSopList(QueryPublishedListBaseVo queryPublishedListBaseVo) throws Exception;
    /***
     * Description: 集团sop-列表页  查询已作废中的sop列表
     * @param queryPublishedListBaseVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 18:37
     * Update By lijie 2021/4/10 18:37
     */
    List<SopBean> queryDestroyedSopList(QueryPublishedListBaseVo queryPublishedListBaseVo) throws Exception;
    /***
     * Description: 新增页:查询可供选择的sop，复制、引用时用
     * @param queryPublishedSelVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/14 12:23
     * Update By lijie 2021/4/14 12:23
     */
    PublishedListVo querySopListForSel(QueryPublishedSelVo queryPublishedSelVo) throws Exception;
    /***
     * Description: 查询当前sop被引用的sop列表
     * @param queryReferencedSopVo : 请求参数
     * @return : java.util.List<com.persagy.fm.sop.model.vo.ReferencedSopListVo>  
     * @author : lijie
     * @date :2021/4/19 10:53
     * Update By lijie 2021/4/19 10:53
     */
    List<RefedSopBean> queryReferencedSopListById(QueryReferencedSopVo queryReferencedSopVo) throws ParseException;
}
