package com.persagy.fm.sop.service;


import com.alibaba.fastjson.JSONObject;
import com.persagy.fm.sop.entity.*;
import com.persagy.fm.sop.model.dto.RefedSopBean;
import com.persagy.fm.sop.model.dto.SopBean;
import com.persagy.fm.sop.model.vo.*;

import java.text.ParseException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 服务接口
 *
 * @author lijie
 * @since 2021-03-31 18:08:21
 * @description 由 Mybatisplus Code Generator 创建
 */
public interface IProjectSopService {

    /***
     * Description: 添加草稿状态sop
     * @param saveSopVo : 保存SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/2 14:30
     * Update By lijie 2021/4/2 14:30
     */
    QuerySopVo addDraftSop(SaveSopVo saveSopVo);

    /***
     * Description: 添加发布状态sop
     * @param saveSopVo : 请求参数
     * @return : void  
     * @author : lijie
     * @date :2021/4/1 16:00
     * Update By lijie 2021/4/1 16:00
     */
    void addPublishedSop(SaveSopVo saveSopVo);

    /***
     * Description: 填充SOP详情
     * @param sops : sop的集合
     * @return : void
     * @author : lijie
     * @date :2021/4/7 9:26
     * Update By lijie 2021/4/7 9:26
     */
    List<SopBean> fillSopDetail(List<Sop> sops);

    /***
     * Description: 编辑页-发布sop
     * @param saveSopVo : 更新的SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/8 14:05
     * Update By lijie 2021/4/8 14:05
     */
    QuerySopVo publishSop(SaveSopVo saveSopVo) throws Exception;

    /***
     * Description: 根据SOP的id集合查询关联的设备模型
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopLabelRel>>
     * @author : lijie
     * @date :2021/4/6 18:05
     * Update By lijie 2021/4/6 18:05
     */
    Map<String, List<SopEquipModelRel>> queryEquipModelMapBySopIds(Set<String> sopIds);

    /***
     * Description: 根据设备模型id查询SOP设备模型关系列表
     * @param equipModelIds : 设备模型id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopEquipModelRel>
     * @author : lijie
     * @date :2021/4/6 20:16
     * Update By lijie 2021/4/6 20:16
     */
    List<SopEquipModelRel> queryEquipModelRelsByEquipModelIds(List<String> equipModelIds);

    /***
     * Description: 根据适用范围对象id集合查询关联对象
     * @param fitObjIds : 适用范围对象id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopObjRel>
     * @author : lijie
     * @date :2021/4/6 20:38
     * Update By lijie 2021/4/6 20:38
     */
    List<SopObjRel> queryFitObjsByIds(List<String> fitObjIds);

    /***
     * Description: 根据SOP的id集合查询关联的标签
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopLabelRel>>
     * @author : lijie
     * @date :2021/4/6 18:05
     * Update By lijie 2021/4/6 18:05
     */
    Map<String, List<SopLabelRel>> queryLabelMapBySopIds(Set<String> sopIds);

    /***
     * Description: 根据标签名称查询SOP与标签关系数据
     * @param labels : 标签名称列表
     * @return : java.util.List<com.persagy.fm.sop.entity.SopLabelRel>
     * @author : lijie
     * @date :2021/4/6 20:24
     * Update By lijie 2021/4/6 20:24
     */
    List<SopLabelRel> queryLabelsByLabelName(List<String> labels);

    /***
     * Description: 根据sop的id集合和所属类型查询对象映射
     * @param sopIds : SOP的id集合
     * @param objOwner : 所属类型
     * @return : java.util.List<com.persagy.fm.sop.entity.SopObjRel>
     * @author : lijie
     * @date :2021/4/7 9:32
     * Update By lijie 2021/4/7 9:32
     */
    // Map<String, List<SopObjRel>> queryObjMapBySopIds(Set<String> sopIds, String objOwner);

    /***
     * Description: 根据工单类型id查询所有的SOP
     * @param orderTypeIds : 工单类型id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopOrderTypeRel>
     * @author : lijie
     * @date :2021/4/6 20:28
     * Update By lijie 2021/4/6 20:28
     */
    List<SopOrderTypeRel> queryOrderTypeByOrderTypeIds(List<String> orderTypeIds);

    /***
     * Description: 根据SOP的id集合查询关联的工单类型id
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<java.lang.String>>
     * @author : lijie
     * @date :2021/4/6 17:44
     * Update By lijie 2021/4/6 17:44
     */
    Map<String,List<String>> queryOrderTypeMapBySopIds(Set<String> sopIds);

    /***
     * Description: 根据sop的id集合批量查询SOP
     * @param sopIds : sop的id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.Sop>
     * @author : lijie
     * @date :2021/4/7 20:17
     * Update By lijie 2021/4/7 20:17
     */
    List<Sop> querySopByIds(Set<String> sopIds,Boolean fillDetailFlag);

    /***
     * Description: 根据父级id查询关联的专业
     * @param parentIds : 父级id的集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopDomainRel>>
     * @author : lijie
     * @date :2021/4/7 14:11
     * Update By lijie 2021/4/7 14:11
     */
    Map<String, List<SopDomainRel>> querySopDomainRelsByParentIds(Set<String> parentIds);

    /***
     * Description: 根据SOP的id查询SOP的步骤
     * @param sopIds : sop的Id
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopCell>>
     * @author : lijie
     * @date :2021/4/7 11:09
     * Update By lijie 2021/4/7 11:09
     */
    Map<String, List<SopCell>> queryStepsBySopIds(Set<String> sopIds);

    /***
     * Description: 根据SOP的id查询关联的工具对象
     * @param sopIds : sop的id集合
     * @return : java.util.Map<java.lang.String,java.util.List<com.persagy.fm.sop.entity.SopToolRel>>
     * @author : lijie
     * @date :2021/4/7 14:27
     * Update By lijie 2021/4/7 14:27
     */
    // Map<String, List<SopToolRel>> queryToolMapBySopIds(Set<String> sopIds);

    /***
     * Description: 编辑页-保存草稿状态sop
     * @param saveSopVo : 保存SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/8 17:24
     * Update By lijie 2021/4/8 17:24
     */
    QuerySopVo saveDraftSop(SaveSopVo saveSopVo);

    /***
     * Description: 验证sop
     * @param saveSopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/2 15:28
     * Update By lijie 2021/4/2 15:28
     */
    List<String> verifySopForPublish(SaveSopVo saveSopVo) throws Exception;

    /***
     * Description: 新增sop时验证sop名称
     * @param queryVerifySopNameBaseVo : 请求参数
     * @return : com.persagy.fm.sop.model.vo.VerifySopNameResult
     * @author : lijie
     * @date :2021/4/2 12:19
     * Update By lijie 2021/4/2 12:19
     */
    VerifySopNameVo verifySopNameForAdd(QueryVerifySopNameBaseVo queryVerifySopNameBaseVo);

    /***
     * Description: 编辑sop时验证sop名称
     * @param queryVerifySopNameBaseVo : 请求参数
     * @return : com.persagy.fm.sop.model.vo.VerifySopNameResult
     * @author : lijie
     * @date :2021/4/2 12:19
     * Update By lijie 2021/4/2 12:19
     */
    VerifySopNameVo verifySopNameForUpdate(QueryVerifySopNameBaseVo queryVerifySopNameBaseVo);

    /***
     * Description: 根据SOP的id集合查询SOP列表
     * @param sopIds : sop的id数组
     * @return : java.util.List<com.persagy.fm.sop.entity.Sop>
     * @author : lijie
     * @date :2021/4/10 14:07
     * Update By lijie 2021/4/10 14:07
     */
    List<SopBean> queryPublishedSopListByIds(List<String> sopIds);
    /***
     * Description: 当对象名称变化或对象删除时更新sop
     * @param objectChangeVo : 请求参数
     * @author : lijie
     * @date :2021/4/10 14:34
     * Update By lijie 2021/4/10 14:34
     */
    void updateSopWhenObjectChange(ObjectChangeVo objectChangeVo, boolean equals);
    /***
     * Description: 查询已发布的sop列表
     * @param queryPublishedListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 15:56
     * Update By lijie 2021/4/10 15:56
     */
    PublishedListVo queryPublishedSopList(QueryPublishedListVo queryPublishedListVo) throws Exception;
    /***
     * Description: 查询编辑中的SOP列表
     * @param queryPublishedListBaseVo : 请求参数
     * @return : java.util.List<com.persagy.fm.sop.model.dto.SopBean>  
     * @author : lijie
     * @date :2021/4/10 18:12
     * Update By lijie 2021/4/10 18:12
     */
    List<SopBean> queryEditingSopList(QueryPublishedListBaseVo queryPublishedListBaseVo) throws Exception;
    /***
     * Description: 查询已作废的sop列表
     * @param queryPublishedListBaseVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 18:27
     * Update By lijie 2021/4/10 18:27
     */
    List<SopBean> queryDestroyedSopList(QueryPublishedListBaseVo queryPublishedListBaseVo) throws Exception;
    /***
     * Description: 根据Id查询已发布的sop信息
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 18:47
     * Update By lijie 2021/4/10 18:47
     */
    SopBean queryPublishedSopById(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 根据Id查询可编辑的sop信息
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 19:27
     * Update By lijie 2021/4/10 19:27
     */
    SopBean querySopForEditById(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 作废sop信息
     * @param queryDestroySopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/13 9:10
     * Update By lijie 2021/4/13 9:10
     */
    void destroySopById(QueryDestroySopVo queryDestroySopVo) throws Exception;
    /***
     * Description: 恢复已作废的sop信息
     * @param queryDestroySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/13 16:12
     * Update By lijie 2021/4/13 16:12
     */
    List<String> recoverSopById(QueryDestroySopVo queryDestroySopVo) throws Exception;
    /***
     * Description: 查询可供选择的sop，复制、引用时用
     * @param queryPublishedSelVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/14 12:23
     * Update By lijie 2021/4/14 12:23
     */
    PublishedListVo querySopListForSel(QueryPublishedSelVo queryPublishedSelVo) throws Exception;
    /***
     * Description: 查询当前sop被引用的sop列表
     * @param queryReferencedSopVo : 请求参数
     * @return : java.util.List<com.persagy.fm.sop.model.dto.RefedSopBean>
     * @author : lijie
     * @date :2021/4/19 10:53
     * Update By lijie 2021/4/19 10:53
     */
    List<RefedSopBean> queryReferencedSopListById(QueryReferencedSopVo queryReferencedSopVo) throws ParseException;
    /***
     * Description:取消修订的sop信息
     * @param querySopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/19 12:30
     * Update By lijie 2021/4/19 12:30
     */
    void cancelReviseSopById(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 删除草稿的sop信息
     * @param querySopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/19 19:59
     * Update By lijie 2021/4/19 19:59
     */
    void deleteDraftSopById(QuerySopVo querySopVo);
    /***
     * Description: 编辑列表发布sop前验证
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 20:06
     * Update By lijie 2021/4/19 20:06
     */
    List<String> verifySopOnList(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 发布sop信息
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 20:53
     * Update By lijie 2021/4/19 20:53
     */
    QuerySopVo publishSopOnList(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 查询引用的sop作废的消息列表
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 21:03
     * Update By lijie 2021/4/19 21:03
     */
    List<DestroyMsgVo> querySopDestroyMsgList(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 标记sop作废消息已读
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 21:55
     * Update By lijie 2021/4/19 21:55
     */
    void markReadSopDestroyMsg(QuerySopVo querySopVo) throws Exception;
    /***
     * Description: 查询待修订中的消息列表
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 11:22
     * Update By lijie 2021/4/20 11:22
     */
    List<ReviseMsgVo> querySopReviseMsgList(QuerySopVo querySopVo) throws ParseException;
    /***
     * Description: 忽略修订消息
     * @param queryReviseMsgVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 12:17
     * Update By lijie 2021/4/20 12:17
     */
    void ignoreSopReviseMsg(QueryReviseMsgVo queryReviseMsgVo);
    /***
     * Description: 查询sop的历史版本列表
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 14:06
     * Update By lijie 2021/4/20 14:06
     */
    List<SopHisListVo> querySopHisList(QuerySopVo querySopVo) throws ParseException;
    /***
     * Description: 根据ID查询历史sop或者引用的sop详细信息
     * @param querySopDetailVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 14:46
     * Update By lijie 2021/4/20 14:46
     */
    SopBean querySopDetailById(QuerySopDetailVo querySopDetailVo) throws Exception;
    /***
     * Description: 查询自动修订方案
     * @param queryAutoReviseMsgVo :请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 18:52
     * Update By lijie 2021/4/20 18:52
     */
    List<AutoReviseMsgVo> queryAutoRevisePlan(QueryAutoReviseMsgVo queryAutoReviseMsgVo);
    /***
     * Description: 自动修订
     * @param autoReviseSopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 21:10
     * Update By lijie 2021/4/20 21:10
     */
    void autoReviseSop(AutoReviseSopVo autoReviseSopVo);
    /***
     * Description: 根据工单类型id作废sop与工单类型的关系
     * @param jsonObject :
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 15:19
     * Update By lijie 2021/4/21 15:19
     */
    String deleteSopOrderTypeRelByOrderType(JSONObject jsonObject) throws Exception;
    /***
     * Description: 组装工单令牌
     * @param jsonObject : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:18
     * Update By lijie 2021/4/21 16:18
     */
    String buildWorkOrder(JSONObject jsonObject) throws Exception;
    /***
     * Description: 获得工单事项预览
     * @param jsonObject : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:47
     * Update By lijie 2021/4/21 16:47
     */
    String getWoMattersPreview(JSONObject jsonObject) throws Exception;
    /***
     * Description: 我的工单-新增页:查询推荐的sop
     * @param jsonObject : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:59
     * Update By lijie 2021/4/21 16:59
     */
    String queryRecommendSop(JSONObject jsonObject) throws Exception;
    /***
     * Description: 批量添加SOP方法
     * @param saveSopVos : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/1 16:30
     * Update By lijie 2021/4/1 16:30
     */
    void batchAddPublishedSop(List<SaveSopVo> saveSopVos);
}
