package com.persagy.fm.sop.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.EnumUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.*;
import cn.hutool.poi.excel.cell.CellHandler;
import cn.hutool.poi.excel.cell.CellUtil;
import cn.hutool.poi.excel.cell.FormulaCellValue;
import cn.hutool.poi.excel.editors.TrimEditor;
import com.alibaba.excel.util.SheetUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.fm.common.constant.CommonConstant;
import com.persagy.fm.common.constant.enums.ObjTagEnum;
import com.persagy.fm.common.constant.enums.ResultEnum;
import com.persagy.fm.common.model.vo.BaseVo;
import com.persagy.fm.common.model.vo.GeneralDictVo;
import com.persagy.fm.common.service.DownloadService;
import com.persagy.fm.common.utils.FileUtil;
import com.persagy.fm.sop.client.SaasClient;
import com.persagy.fm.sop.constant.SopCommonConst;
import com.persagy.fm.sop.constant.SopQueryConst;
import com.persagy.fm.sop.entity.CustomInfoPoint;
import com.persagy.fm.sop.entity.Sop;
import com.persagy.fm.sop.entity.SopCell;
import com.persagy.fm.sop.entity.SopObjRel;
import com.persagy.fm.sop.enums.*;
import com.persagy.fm.sop.model.dto.*;
import com.persagy.fm.sop.model.vo.PublishedListVo;
import com.persagy.fm.sop.model.vo.QueryPublishedListVo;
import com.persagy.fm.sop.model.vo.SaveSopVo;
import com.persagy.fm.sop.service.IBatchHandleSopService;
import com.persagy.fm.sop.service.IProjectSopService;
import com.persagy.fm.sop.service.ISopService;
import com.persagy.fm.sop.service.ObjService;
import com.persagy.fm.sop.validator.ProjectInsert;
import com.persagy.fm.sop.validator.ProjectUpdate;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellAddress;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.ss.util.SheetUtil;
import org.apache.poi.xssf.usermodel.*;
import org.apache.poi.xssf.usermodel.helpers.XSSFFormulaUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
@Slf4j
public class BatchHandleSopServiceImpl implements IBatchHandleSopService {
    private final ISopService sopService;
    private final SaasClient saasClient;
    private final IProjectSopService projectSopService;

    /**SOP批量导入模板的位置*/
    private static final String TEMPLATE_FILE_PATH="/template/SOP导入模板.xlsx";
    /**SOP批量导入模板的位置*/
    private static final String EXPORT_FILE_PATH="/template/SOP数据.xlsx";
    /**存储codeData的Sheet页的行数*/
    private static final ThreadLocal<Integer> LOOK_INDEX = new ThreadLocal<>();
    /**存储infoData的Sheet页的行数*/
    private static final ThreadLocal<Integer> INFO_INDEX = new ThreadLocal<>();

    /***
     * Description: 下载批量导入的SOP模板
     * @param baseVo : 请求参数
     * @param response : 响应对象
     * @return : void
     * @author : lijie
     * @date :2021/4/28 10:53
     * Update By lijie 2021/4/28 10:53
     */
    @Override
    public void downloadSopTemplate(BaseVo baseVo, HttpServletResponse response) throws IOException {
        // 下载Excel模板类
        new DownloadService() {
            @Override
            protected void handleWorkbook(Workbook workbook) {
                try{
                    // 1.填充对象类数据
                    LOOK_INDEX.set(0);
                    INFO_INDEX.set(0);
                    fillObjClassDataToWorkbook(workbook);
                }finally {
                    LOOK_INDEX.remove();
                    INFO_INDEX.remove();
                }
            }

            @Override
            protected String getFilePath() {
                return TEMPLATE_FILE_PATH;
            }

            @Override
            protected HttpServletResponse getHttpServletResponse() {
                return response;
            }
        }.downloadFile();
    }

    /***
     * Description: 批量上传SOP
     * @param projectId : 项目id
     * @param file : 文件流
     * @param request : 请求体对象
     * @param response : 响应体对象
     * @return : void
     * @author : lijie
     * @date :2021/4/30 12:15
     * Update By lijie 2021/4/30 12:15
     */
    @Override
    public void uploadSop(String projectId, MultipartFile file, HttpServletRequest request, HttpServletResponse response) throws IOException {
        if (StrUtil.isBlank(projectId)){
            projectId = SopCommonConst.GROUP_SOP_ID;
        }
        ExcelReader firstSheet = ExcelUtil.getReader(file.getInputStream(),0);
        ExcelReader classDataSheet = ExcelUtil.getReader(file.getInputStream(),1);
        ExcelReader codeDataSheet = ExcelUtil.getReader(file.getInputStream(),2);
        ExcelReader infoDataSheet = ExcelUtil.getReader(file.getInputStream(),3);
        List<List<Object>> uploadData = firstSheet.read();
        List<List<Object>> classListList = classDataSheet.read();
        List<List<Object>> codeListList = codeDataSheet.read();
        List<List<Object>> infoListList = infoDataSheet.read();
        List<String> checkFileErrList = checkFileIsValid(uploadData);
        if (CollUtil.isNotEmpty(checkFileErrList)){
            FileUtil.writeErrorContentToResponse(checkFileErrList,response,request,SopQueryConst.UPLOAD_SOP_ERR_FILE_NAME);
            return;
        }
        // 1.大类名称集合
        Set<String> classNameSet = new HashSet<>();
        for (int i = 0; i < classListList.size(); i++) {
            List<Object> objects = classListList.get(i);
            classNameSet.add(CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(0))));
        }
        // 2.分类映射
        Map<String,String> codeMap = new HashMap<>();
        Map<String,String> objTypeMap = new HashMap<>();
        for (List<Object> objects : codeListList) {
            String objName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(0)));
            String objCode = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(1)));
            String objType = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(2)));
            codeMap.putIfAbsent(objName,objCode);
            objTypeMap.putIfAbsent(objName,objType);
        }
        // 3.信息点映射
        Map<String,InfoPointsBean> infoMap = new HashMap<>();
        for (List<Object> objects : infoListList) {
            String objName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(0)));
            String objCode = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(1)));
            if (StrUtil.isBlank(objCode) || !objCode.startsWith("{")){
                objCode = "{}";
            }
            infoMap.putIfAbsent(objName,JSONObject.parseObject(objCode,InfoPointsBean.class));
        }
        // 4.检验数据合法性
        List<String> checkDataErrList = checkDataIsValid(uploadData, classNameSet, codeMap, infoMap,firstSheet);
        if (CollUtil.isNotEmpty(checkDataErrList)){
            FileUtil.writeErrorContentToResponse(checkDataErrList,response,request,SopQueryConst.UPLOAD_SOP_ERR_FILE_NAME);
            return;
        }
        List<SaveSopVo> saveSopVos = generateSaveSopVo(uploadData,codeMap,infoMap,firstSheet,projectId,objTypeMap);
        projectSopService.batchAddPublishedSop(saveSopVos);
    }

    /***
     * Description: 导出SOP
     * @param projectId : 项目id
     * @param response : 响应体对象
     * @return : void
     * @author : lijie
     * @date :2021/5/6 12:09
     * Update By lijie 2021/5/6 12:09
     */
    @Override
    public void exportSop(String projectId, HttpServletResponse response) throws Exception {
        // 下载Excel模板类
        new DownloadService() {
            @SneakyThrows
            @Override
            protected void handleWorkbook(Workbook workbook) {
                try{
                    // 1.填充对象类数据
                    LOOK_INDEX.set(0);
                    INFO_INDEX.set(0);
                    fillObjClassDataToWorkbook(workbook);
                    // 2.填充SOP的数据
                    fillSopDataToWorkbook(projectId, workbook);
                }finally {
                    LOOK_INDEX.remove();
                    INFO_INDEX.remove();
                }
            }

            @Override
            protected String getFilePath() {
                return EXPORT_FILE_PATH;
            }

            @Override
            protected HttpServletResponse getHttpServletResponse() {
                return response;
            }
        }.downloadFile();
    }

    /***
     * Description: 填充SOP数据
     * @param workbook : 工作簿
     * @return : void
     * @author : lijie
     * @date :2021/5/6 12:14
     * Update By lijie 2021/5/6 12:14
     */
    private void fillSopDataToWorkbook(String projectId, Workbook workbook) throws Exception {
        // 1.查询所有SOP数据
        if (StrUtil.isBlank(projectId)){
            projectId = SopCommonConst.GROUP_SOP_ID;
        }
        QueryPublishedListVo queryPublishedListVo = QueryPublishedListVo.builder().build();
        queryPublishedListVo.setProjectId(projectId);
        queryPublishedListVo.setNeedReturnCriteria(false);
        PublishedListVo listVo = projectSopService.queryPublishedSopList(queryPublishedListVo);
        // 2.写入数据到Excel里
        List<SopBean> beans = listVo.getContent();
        if (CollUtil.isEmpty(beans)){
            return;
        }
        Sheet firstSheet = workbook.getSheetAt(0);
        Sheet classDataSheet = workbook.getSheetAt(1);
        Sheet codeDataSheet = workbook.getSheetAt(2);
        Map<String,String> classMap = new HashMap<>();
        for (int i=0;i<classDataSheet.getLastRowNum();i++){
            List<Object> objectList = RowUtil.readRow(RowUtil.getOrCreateRow(classDataSheet, i), null);
            if (CollUtil.isEmpty(objectList) || objectList.size()<2){
                continue;
            }
            String firstName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objectList.get(0)));
            for (int j = 1; j < objectList.size(); j++) {
                String trimToEmpty = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objectList.get(j)));
                if (StrUtil.isBlank(trimToEmpty)){
                    continue;
                }
                classMap.putIfAbsent(trimToEmpty,firstName);
            }
        }
        List<List<Object>> codeListList = new ArrayList<>();
        for (int i=0;i<codeDataSheet.getLastRowNum();i++){
            codeListList.add(RowUtil.readRow(RowUtil.getOrCreateRow(codeDataSheet,i), null));
        }
        // 2.分类映射
        Map<String,String> codeMap = new HashMap<>();
        for (List<Object> objects : codeListList) {
            String objName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(0)));
            String objCode = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(objects.get(1)));
            codeMap.putIfAbsent(objCode,objName);
        }
        // 从第三行开始写入
        int sheetRowIndex = 2;
        for (int i = 0; i < beans.size(); i++) {
            SopBean sopBean = beans.get(i);
            List<SopCell> steps = sopBean.getSteps();
            if (CollUtil.isEmpty(steps)){
                continue;
            }
            for (int j = 0; j < steps.size(); j++) {
                SopCell stepCell = steps.get(j);
                if (stepCell.getFromSop()){
                    // 引用SOP不写入excel
                    continue;
                }
                // 工作内容那一层
                List<SopCell> stepContents = stepCell.getStepContent();
                if (CollUtil.isEmpty(stepContents)){
                    continue;
                }
                for (int m = 0; m < stepContents.size(); m++) {
                    SopCell stepContent = stepContents.get(m);
                    Row row = RowUtil.getOrCreateRow(firstSheet, sheetRowIndex);
                    List<SopObjRel> contentObjs = stepContent.getContentObjs();
                    if (CollUtil.isNotEmpty(contentObjs)){
                        SopObjRel contentObj = contentObjs.get(0);
                        String objName = codeMap.get(SopCommonConst.UNDER_LINE+contentObj.getObjId());
                        if (StrUtil.isNotBlank(objName)){
                            CellUtil.setCellValue(CellUtil.getOrCreateCell(row,1),classMap.getOrDefault(objName,""),null);
                            CellUtil.setCellValue(CellUtil.getOrCreateCell(row,2),objName,null);
                            CellUtil.setCellValue(CellUtil.getOrCreateCell(row,3),SopCommonConst.UNDER_LINE+contentObj.getObjId(),null);
                        }
                        List<SopObjRel> spaceScopes = stepContent.getSpaceScope();
                        if (CollUtil.isNotEmpty(spaceScopes)){
                            SopObjRel spaceScope = spaceScopes.get(0);
                            CellUtil.setCellValue(CellUtil.getOrCreateCell(row,4),spaceScope.getObjId(),null);
                            CellUtil.setCellValue(CellUtil.getOrCreateCell(row,5),spaceScope.getObjName(),null);
                        }
                    }
                    List<SopCell> confirmResults = stepContent.getConfirmResult();
                    if (CollUtil.isNotEmpty(confirmResults)){
                        SopCell confirmResult = confirmResults.get(0);
                        String description = confirmResult.getDescription();
                        CellUtil.setCellValue(CellUtil.getOrCreateCell(row,13),description,null);
                        List<CustomInfoPointsBean> customs = confirmResult.getCustoms();
                        if (CollUtil.isNotEmpty(customs)){
                            for (int n = 0; n < customs.size(); n++) {
                                CustomInfoPointsBean customInfoPointsBean = customs.get(n);
                                if ("异常情况".equals(customInfoPointsBean.getName())){
                                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,14),"是",null);
                                }else {
                                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,14),"否",null);
                                }
                            }
                        }
                        List<InfoPointsBean> infoPoints = confirmResult.getInfoPoints();
                        if (CollUtil.isNotEmpty(infoPoints)){
                            InfoPointsBean infoPointsBean = infoPoints.get(0);
                            CellUtil.setCellValue(CellUtil.getOrCreateCell(row,15),infoPointsBean.getName(),null);
                            List<WrongRangesBean> wrongRanges = infoPointsBean.getWrongRanges();
                            if (CollUtil.isNotEmpty(wrongRanges)){
                                for (WrongRangesBean wrongRange : wrongRanges) {
                                    if ("gt".equals(wrongRange.getType())){
                                        CellUtil.setCellValue(CellUtil.getOrCreateCell(row,17),wrongRange.getValues(),null);
                                    }
                                    if ("lt".equals(wrongRange.getType())){
                                        CellUtil.setCellValue(CellUtil.getOrCreateCell(row,16),wrongRange.getValues(),null);
                                    }
                                }
                            }
                        }
                    }
                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,0),sopBean.getSopName(),null);
                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,6),(null!=stepContent.getIsPhoto() && stepContent.getIsPhoto())?"是":"否",null);
                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,7),(null!=stepContent.getIsScan() && stepContent.getIsScan())?"是":"否",null);
                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,10),stepContent.getTimeLimit(),null);
                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,11),stepContent.getPreConform(),null);
                    CellUtil.setCellValue(CellUtil.getOrCreateCell(row,12),stepContent.getNotice(),null);
                    sheetRowIndex++;
                }
            }
        }
    }

    /***
     * Description: 生成需要保存的对象
     * @param uploadData : 上传的数据
     * @param codeMap : code映射
     * @param infoMap : 信息点映射
     * @param firstSheet : sheet对象
     * @param objTypeMap
     * @return : java.util.List<com.persagy.fm.sop.model.vo.SaveSopVo>
     * @author : lijie
     * @date :2021/4/30 17:04
     * Update By lijie 2021/4/30 17:04
     */
    private List<SaveSopVo> generateSaveSopVo(List<List<Object>> uploadData, Map<String, String> codeMap,
                                              Map<String, InfoPointsBean> infoMap, ExcelReader firstSheet, String projectId, Map<String, String> objTypeMap) {
        int size = uploadData.size();
        Map<String,SaveSopVo> resultMap = new HashMap<>();
        Map<String,Set<String>> objNameMap = new HashMap<>();
        Map<String,Set<String>> confirmResultMap = new HashMap<>();
        for (int i = 2; i < size; i++) {
            // 1.SOP的名称
            String sopName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(0,i))));
            SaveSopVo saveSopVo = resultMap.getOrDefault(sopName, new SaveSopVo());
            saveSopVo.setSopName(sopName);
            // 2.项目id
            saveSopVo.setProjectId(projectId);
            // 3.适用范围
            List<FitObjsBean> fitObjs = saveSopVo.getFitObjs();
            if (CollUtil.isEmpty(fitObjs)){
                fitObjs = new ArrayList<>();
                saveSopVo.setFitObjs(fitObjs);
            }
            // 4.步骤,批量上传的根据SOP的名称有且仅有一步
            List<SopStepsBean> steps = saveSopVo.getSteps();
            SopStepsBean sopStepsBean;
            if (CollUtil.isEmpty(steps)){
                steps = new ArrayList<>();
                sopStepsBean = SopStepsBean.builder().fromSop(false).build();
                steps.add(sopStepsBean);
                saveSopVo.setSteps(steps);
            }else {
                sopStepsBean=steps.get(0);
            }
            Set<String> objNameSet = objNameMap.getOrDefault(sopName, new HashSet<>());
            // 5.对象名称
            String objName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(2,i))));
            if (!objNameSet.contains(objName)){
                objNameSet.add(objName);
                objNameMap.putIfAbsent(sopName,objNameSet);
                // 新增适用范围
                fitObjs.add(FitObjsBean.builder()
                        .objId(codeMap.getOrDefault(objName,"").substring(1))
                        .objName(objName.substring(objName.lastIndexOf(SopCommonConst.UNDER_LINE)+1))
                        .objCode(codeMap.getOrDefault(objName,"").substring(1))
                        .objType(objTypeMap.getOrDefault(objName,""))
                        .objSource(ObjOwnerEnum.CONTENT_OBJ.getType()).build());
                List<StepContentBean> stepContents = sopStepsBean.getStepContent();
                StepContentBean stepContentBean;
                if (CollUtil.isEmpty(stepContents)){
                    stepContents = new ArrayList<>();
                    stepContentBean = StepContentBean.builder().fromSop(false).build();
                    stepContents.add(stepContentBean);
                }else {
                    stepContentBean = stepContents.get(0);
                }
                List<ContentObjsBean> contentObjs = stepContentBean.getContentObjs();
                if (CollUtil.isEmpty(contentObjs)){
                    contentObjs = new ArrayList<>();
                }
                contentObjs.add(ContentObjsBean.builder()
                        .objId(codeMap.getOrDefault(objName,"").substring(1))
                        .objName(objName.substring(objName.lastIndexOf(SopCommonConst.UNDER_LINE)+1))
                        .objCode(codeMap.getOrDefault(objName,"").substring(1))
                        .objType(objTypeMap.getOrDefault(objName,""))
                        .build());
                stepContentBean.setContentObjs(contentObjs);
                // 空间范围
                if (ObjTypeEnum.EQUIP_CLASS.getType().equals(objTypeMap.getOrDefault(objName,""))){
                    String spaceObjName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(4,i))));
                    if (StrUtil.isNotBlank(spaceObjName)){
                        List<SpaceScopeBean> spaceScopeBeans = CollUtil.defaultIfEmpty(stepContentBean.getSpaceScope(), new ArrayList<>());
                        spaceScopeBeans.add(SpaceScopeBean.builder()
                                .objId(codeMap.getOrDefault(spaceObjName,"").substring(1))
                                .objName(spaceObjName.substring(spaceObjName.lastIndexOf(SopCommonConst.UNDER_LINE)+1))
                                .objCode(codeMap.getOrDefault(spaceObjName,"").substring(1))
                                .objType(objTypeMap.getOrDefault(spaceObjName,""))
                                .build());
                        stepContentBean.setSpaceScope(spaceScopeBeans);
                    }
                }
                sopStepsBean.setStepContent(stepContents);
            }
            List<StepContentBean> stepContents = sopStepsBean.getStepContent();
            StepContentBean stepContentBean;
            if (CollUtil.isEmpty(stepContents)){
                stepContents = new ArrayList<>();
                stepContentBean = StepContentBean.builder().build();
                stepContents.add(stepContentBean);
                sopStepsBean.setStepContent(stepContents);
            }else {
                stepContentBean = stepContents.get(0);
            }
            String photo = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(6, i))));
            if (null == stepContentBean.getIsPhoto()){
                if (StrUtil.isBlank(photo) || "否".equals(photo)){
                    stepContentBean.setIsPhoto(false);
                }else {
                    stepContentBean.setIsPhoto(true);
                }
            }
            String scan = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(7, i))));
            if (null == stepContentBean.getIsScan()){
                if (StrUtil.isBlank(scan) || "否".equals(scan)){
                    stepContentBean.setIsScan(false);
                }else {
                    stepContentBean.setIsScan(true);
                }
            }
            // 处理时间
            if (null==stepContentBean.getTimeLimit()){
                Integer destMin = null;
                String day = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(8, i))));
                if (StrUtil.isNotBlank(day)){
                    destMin = Integer.parseInt(day)*24*60;
                }
                String hour = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(9, i))));
                if (StrUtil.isNotBlank(hour)){
                    destMin = null!=destMin ? (destMin + Integer.parseInt(hour)*60):Integer.parseInt(hour)*60;
                }
                String min = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(10, i))));
                if (StrUtil.isNotBlank(min)){
                    destMin = null!=destMin ? (destMin + Integer.parseInt(min)):Integer.parseInt(min);
                }
                if (null!=destMin){
                    stepContentBean.setTimeLimit(destMin);
                }
            }
            String preConform = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(11, i))));
            if (StrUtil.isBlank(stepContentBean.getPreConform()) && StrUtil.isNotBlank(preConform)){
                stepContentBean.setPreConform(preConform);
            }
            String notice = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(12, i))));
            if (StrUtil.isBlank(stepContentBean.getNotice()) && StrUtil.isNotBlank(notice)){
                stepContentBean.setNotice(notice);
            }
            String confirmResultContent = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(13, i))));
            Set<String> confirmResultContents = confirmResultMap.getOrDefault(sopName,new HashSet<>());
            if (!confirmResultContents.contains(confirmResultContent)){
                confirmResultContents.add(confirmResultContent);
                confirmResultMap.putIfAbsent(sopName,confirmResultContents);
                List<ConfirmResultBean> confirmResults = stepContentBean.getConfirmResult();
                if (CollUtil.isEmpty(confirmResults)){
                    confirmResults = new ArrayList<>();
                }
                List<CustomInfoPointsBean> customInfoPoints = new ArrayList<>();
                String custom = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(14, i))));
                if (StrUtil.isNotBlank(custom) && "是".equals(custom)){
                    CustomInfoPointsBean customInfoPoint = new CustomInfoPointsBean();
                    customInfoPoint.setItems(Arrays.asList("正常","异常"));
                    customInfoPoint.setWrongs(Collections.singletonList("异常"));
                    customInfoPoint.setType("2");
                    customInfoPoint.setName("异常情况");
                    customInfoPoints.add(customInfoPoint);
                }
                List<InfoPointsBean> infoPointsBeans = new ArrayList<>();
                String infoPointStr = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(15, i))));
                String downStr = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(16, i))));
                String upStr = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(17, i))));
                if (StrUtil.isNotBlank(infoPointStr)){
                    InfoPointsBean infoPointsBean = infoMap.get(objName+SopCommonConst.UNDER_LINE+infoPointStr);
                    if (null!=infoPointsBean){
                        List<WrongRangesBean> wrongRanges = infoPointsBean.getWrongRanges();
                        if (CollUtil.isEmpty(wrongRanges)){
                            wrongRanges = new ArrayList<>();
                        }
                        if (StrUtil.isNotBlank(downStr)){
                            wrongRanges.add(WrongRangesBean.builder().type("lt").values(downStr).build());
                        }
                        if (StrUtil.isNotBlank(upStr)){
                            wrongRanges.add(WrongRangesBean.builder().type("gt").values(upStr).build());
                        }
                        infoPointsBean.setWrongRanges(wrongRanges);
                    }
                    infoPointsBeans.add(infoPointsBean);
                }
                confirmResults.add(ConfirmResultBean.builder()
                        .description(confirmResultContent)
                        .infoPoints(infoPointsBeans)
                        .customs(customInfoPoints)
                        .build());
                stepContentBean.setConfirmResult(confirmResults);
            }
            resultMap.putIfAbsent(sopName,saveSopVo);
        }
        return CollUtil.newArrayList(resultMap.values());
    }

    /***
     * Description: 校验数据是否正确
     * @param uploadData : 上传的SOP数据
     * @param classNameSet : 分类名称集合
     * @param codeMap : 对象类映射
     * @param infoMap : 信息点映射
     * @param firstSheet
     * @return : java.util.List<java.lang.String>
     * @author : lijie
     * @date :2021/4/30 15:05
     * Update By lijie 2021/4/30 15:05
     */
    private List<String> checkDataIsValid(List<List<Object>> uploadData, Set<String> classNameSet,
                                          Map<String, String> codeMap, Map<String, InfoPointsBean> infoMap, ExcelReader firstSheet) {
        List<Sop> sops = sopService.list();
        Set<String> sopNameSet = sops.stream().map(Sop::getSopName).collect(Collectors.toSet());
        Set<String> codeNameSet = codeMap.keySet();
        Set<String> codeSet = CollUtil.newHashSet(codeMap.values());
        Set<String> infoNameSet = infoMap.keySet();
        List<String> errorList = new ArrayList<>();
        for (int i = 2; i < uploadData.size(); i++) {
            List<Object> objects = uploadData.get(i);
            String realCol="第"+(i+1)+"行，";
            // 1.SOP的名称
            String sopName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(0,i))));
            if (StrUtil.isBlank(sopName)){
                errorList.add(realCol+SopColumnEnum.SOP_NAME.getName()+SopErrorEnum.CONTENT_BLANK_ERROR);
            }
            if (StrUtil.isNotBlank(sopName) && sopNameSet.contains(sopName)){
                errorList.add(realCol+SopColumnEnum.SOP_NAME.getName()+SopErrorEnum.SOP_NAME_REPEAT_ERROR);
            }
            // 2.对象分类
            String className = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(1,i))));
            if (StrUtil.isBlank(className)){
                errorList.add(realCol+SopColumnEnum.OBJ_CLASSIFY.getName()+SopErrorEnum.CONTENT_BLANK_ERROR);
            }
            if (StrUtil.isNotBlank(className) && !classNameSet.contains(className)){
                errorList.add(realCol+SopColumnEnum.OBJ_CLASSIFY.getName()+SopErrorEnum.CONTENT_ERROR);
            }
            // 3.对象选择
            String objName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(2,i))));
            if (StrUtil.isBlank(objName)){
                errorList.add(realCol+SopColumnEnum.OBJ_CHOICE.getName()+SopErrorEnum.CONTENT_BLANK_ERROR);
            }
            if (StrUtil.isNotBlank(objName) && !codeNameSet.contains(objName)){
                errorList.add(realCol+SopColumnEnum.OBJ_CHOICE.getName()+SopErrorEnum.CONTENT_ERROR);
            }
            // 4.对象编码
            String objCode = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(3,i))));
            if (StrUtil.isBlank(objCode)){
                errorList.add(realCol+SopColumnEnum.OBJ_CODE.getName()+SopErrorEnum.CONTENT_BLANK_ERROR);
            }
            if (StrUtil.isNotBlank(objCode) && !codeSet.contains(objCode)){
                errorList.add(realCol+SopColumnEnum.OBJ_CODE.getName()+SopErrorEnum.CONTENT_ERROR);
            }
            // 5.工作时长(天)
            String dayStr = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(8,i))));
            if (StrUtil.isNotBlank(dayStr) && !NumberUtil.isNumber(dayStr)){
                errorList.add(realCol+SopColumnEnum.WORK_TIME_DAY+SopErrorEnum.NUMBER_ERROR);
            }
            // 6.工作时长(时)
            String hourStr = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(9,i))));
            if (StrUtil.isNotBlank(hourStr) && !NumberUtil.isNumber(hourStr)){
                errorList.add(realCol+SopColumnEnum.WORK_TIME_HOUR+SopErrorEnum.NUMBER_ERROR);
            }
            // 7.工作时长(分)
            String minStr = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(10,i))));
            if (StrUtil.isNotBlank(minStr) && !NumberUtil.isNumber(minStr)){
                errorList.add(realCol+SopColumnEnum.WORK_TIME_MIN+SopErrorEnum.NUMBER_ERROR);
            }
            // 8.标准作业操作内容
            String describe = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(13,i))));
            if (StrUtil.isBlank(describe)){
                errorList.add(realCol+SopColumnEnum.JOB_DESCRIPTION.getName()+SopErrorEnum.CONTENT_BLANK_ERROR);
            }
            // 9.添加信息点
            String infoCode = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(CellUtil.getCellValue(firstSheet.getOrCreateCell(15,i))));
            if (StrUtil.isNotBlank(infoCode) && !infoNameSet.contains(objName+SopCommonConst.UNDER_LINE+infoCode)){
                errorList.add(realCol+SopErrorEnum.CONTENT_ERROR);
            }
        }
        return errorList;
    }

    /***
     * Description: 校验文件的正确性
     * @param lists : 数据列表
     * @return : java.util.List<java.lang.String>
     * @author : lijie
     * @date :2021/3/15 11:51
     * Update By lijie 2021/3/15 11:51
     */
    private List<String> checkFileIsValid(List<List<Object>> lists) {
        List<String> errorList = new ArrayList<>();
        if (lists.size() < SopQueryConst.EXCEL_HEAD_INDEX){
            errorList.add(SopErrorEnum.FILE_ERROR.getDesc());
            return errorList;
        }
        List<Object> rows = lists.get(1);
        if (CollUtil.isEmpty(rows)){
            errorList.add(SopErrorEnum.FILE_ERROR.getDesc());
            return errorList;
        }
        List<SopColumnEnum> list = CollUtil.list(false, EnumUtil.getEnumMap(SopColumnEnum.class).values());
        Map<Integer, String> sopColumnMap = list.stream()
                .collect(Collectors.toMap(SopColumnEnum::getIndex, SopColumnEnum::getName, (k1, k2) -> k1));
        if (rows.size()<sopColumnMap.size()){
            errorList.add(SopErrorEnum.FILE_ERROR.getDesc());
            return errorList;
        }
        for (int i=0;i<sopColumnMap.size();i++){
            String columnName = sopColumnMap.get(i);
            if (StringUtils.isBlank(columnName)){
                errorList.add(SopErrorEnum.FILE_ERROR.getDesc());
                return errorList;
            }
            String realName = CharSequenceUtil.trimToEmpty(StrUtil.utf8Str(rows.get(i)));
            if (StrUtil.isBlank(realName) || !realName.contains(columnName)){
                errorList.add(SopErrorEnum.FILE_ERROR.getDesc());
                return errorList;
            }
        }
        return errorList;
    }

    /***
     * Description: 填充对象类数据
     * @param workbook : 工作簿
     * @return : void
     * @author : lijie
     * @date :2021/4/28 19:05
     * Update By lijie 2021/4/28 19:05
     */
    private void fillObjClassDataToWorkbook(Workbook workbook) {
        // 做列表的数据
        Sheet listSheet = workbook.getSheetAt(1);
        // 做VooLook列表的数据
        Sheet lookSheet = workbook.getSheetAt(2);
        // 1.填充对象类数据
        // 2.填充信息点数据
        // 做信息点的sheet
        Sheet infoSheet = workbook.getSheetAt(3);
        Map<String, List<InfoPointsBean>> infoCodeMap = queryAllInfoCodeData();
        writeObjClassDataToSheet(listSheet,lookSheet,workbook,infoSheet,infoCodeMap);

        // 3.设置第一个sheet的有效性:对象分类,第二列(1),第三行(2)开始
        Sheet firstSheet = workbook.getSheetAt(0);
        rewriteExcelTempAtEmp("=classData!$A$1:$A$6",firstSheet,2,65535,1,1);
        rewriteExcelTempAtEmp("=INDIRECT($B:$B)",firstSheet,2,65535,2,2);
        rewriteExcelTempAtEmp("=IF(VLOOKUP($D:$D,codeData!B:C,2,FALSE)=\"equip_class\",空间功能类型,\"\")",firstSheet,2,65535,4,4);
        rewriteExcelTempAtEmp("=INDIRECT($D:$D)",firstSheet,2,65535,15,15);
        rewriteExcelTempAtEmp("=classData!$A$7:$A$8",firstSheet,2,65535,6,6);
        rewriteExcelTempAtEmp("=classData!$A$7:$A$8",firstSheet,2,65535,7,7);
        rewriteExcelTempAtEmp("=classData!$A$7:$A$8",firstSheet,2,65535,14,14);
        //创建下拉框对象
        //CellUtil.setCellValue(CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(firstSheet,2),1),"通用系统类",null);
        //CellUtil.setCellValue(CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(firstSheet,2),2),"给排水专业-水景系统",null);
        for (int i=2;i<65535;i++){
            CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(firstSheet,i),3).setCellFormula("IFERROR(VLOOKUP($C:$C,codeData!A:B,2,FALSE),\"\")");
            CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(firstSheet,i),5).setCellFormula("IFERROR(VLOOKUP($E:$E,codeData!A:B,2,FALSE),\"\")");
        }
        // 4.隐藏三个sheet页
        workbook.setSheetHidden(1,true);
        workbook.setSheetHidden(2,true);
        workbook.setSheetHidden(3,true);
    }
    /***
     * Description: 写入信息点的数据
     * @return : void
     * @author : lijie
     * @date :2021/4/29 21:54
     * Update By lijie 2021/4/29 21:54
     */
    private Map<String,List<InfoPointsBean>> queryAllInfoCodeData() {
        JSONObject param = new JSONObject();
        param.put(CommonConstant.USER_ID,CommonConstant.SYSTEM_ID);
        String queryStr = saasClient.queryAllInfoPointForObject(param);
        JSONObject queryObj = JSONObject.parseObject(queryStr);
        if (ResultEnum.FAILURE.getType().equals(queryObj.getString(ResultEnum.RESULT.getType()))){
            return new HashMap<>();
        }
        JSONObject item = queryObj.getJSONObject(ResultEnum.ITEM.getType());
        if (null==item || 0==item.size()){
            return new HashMap<>();
        }
        Map<String,List<InfoPointsBean>> result = new HashMap<>();
        Set<String> keySet = item.keySet();
        for (String key : keySet) {
            JSONArray temp = item.getJSONArray(key);
            if (null==temp || temp.size()<=0){
                continue;
            }
            List<InfoPointsBean> infoPointsBeans = temp.toJavaList(InfoPointsBean.class);
            result.put(key,infoPointsBeans);
        }
        return result;
    }

    /**
     *<p>Description:将表中的某一区域设置成下拉框.</p>
     *<p>@param sheet
     *<p>@param dataSet
     *<p>@param firstRow
     *<p>@param lastRow
     *<p>@param firstCol
     *<p>@param lastCol void
     *@since 2018年5月10日: 上午11:28:02
     *@author lijie
     *@version 1.0
     */
    public static void rewriteExcelTempAtEmp(String formulaString, Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
        CellRangeAddressList addressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
        //创建下拉框对象
        XSSFDataValidationHelper helper = new XSSFDataValidationHelper((XSSFSheet) sheet);
        XSSFDataValidationConstraint dvConstraint =  (XSSFDataValidationConstraint) helper.createFormulaListConstraint(formulaString);
        DataValidation dataValidation = helper.createValidation(dvConstraint, addressList);
        //处理Excel兼容性问题
        if(dataValidation instanceof XSSFDataValidation) {
            dataValidation.setSuppressDropDownArrow(true);
            dataValidation.setShowErrorBox(true);
        }else {
            dataValidation.setSuppressDropDownArrow(false);
        }
        sheet.addValidationData(dataValidation);
    }

    /***
     * Description: 写数据到Sheet中
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @return : void
     * @author : lijie
     * @date :2021/4/28 21:27
     * Update By lijie 2021/4/28 21:27
     */
    private void writeObjClassDataToSheet(Sheet listSheet, Sheet lookSheet,Workbook workbook, Sheet infoSheet,
                                          Map<String, List<InfoPointsBean>> infoCodeMap) {
        // 1.填写行头
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,0),0), ClassTypeEnum.SYSTEM.getType(),null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,1),0), ClassTypeEnum.EQUIP.getType(),null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,2),0), ClassTypeEnum.SPACE.getType(),null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,3),0), ClassTypeEnum.BUILD.getType(),null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,4),0), ClassTypeEnum.FLOOR.getType(),null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,5),0), ClassTypeEnum.SHAFT.getType(),null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,6),0), "是",null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,7),0), "否",null);
        // 2.填充"通用系统类"数据
        fillSystemClassData(listSheet, lookSheet,workbook,infoSheet,infoCodeMap);
        // 3.填充"通用设备类"数据
        fillEquipClassData(listSheet,lookSheet,workbook,infoSheet,infoCodeMap);
        // 4.填充"空间功能类型"数据
        fillSpaceClassData(listSheet,lookSheet,workbook,infoSheet,infoCodeMap);
        // 5.填充"建筑功能类型"数据
        fillBuildClassData(listSheet,lookSheet,workbook,infoSheet,infoCodeMap);
        // 6.填充楼层类数据
        fillFloorClassData(listSheet,lookSheet,workbook,infoSheet,infoCodeMap);
        // 7.填充竖井类数据
        fillShaftClassData(listSheet,lookSheet,workbook,infoSheet,infoCodeMap);
    }
    /***
     * Description: 填充竖井类数据
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @param infoSheet
     * @param infoCodeMap
     * @return : java.lang.Integer
     * @author : lijie
     * @date :2021/4/28 21:12
     * Update By lijie 2021/4/28 21:12
     */
    private void fillShaftClassData(Sheet listSheet, Sheet lookSheet, Workbook workbook, Sheet infoSheet, Map<String, List<InfoPointsBean>> infoCodeMap) {
        List<ObjBean> objBeans = new ObjService() {
            @Override
            protected String queryDataFromSaas(JSONObject param) {
                return saasClient.queryShaftType(param);
            }
        }.queryObjsData();
        if (CollUtil.isEmpty(objBeans)){
            return;
        }
        int index = 1;
        for (ObjBean firstBean : objBeans) {
            if (StrUtil.isBlank(firstBean.getObjId()) || StrUtil.isBlank(firstBean.getObjName())){
                continue;
            }
            setCellValue(firstBean.getObjName(),firstBean.getObjId(),5,listSheet,lookSheet,index,ObjTypeEnum.SHAFT_TYPE.getType());
            setInfoValue(workbook,lookSheet,infoCodeMap,firstBean.getObjId(),firstBean.getObjName(),infoSheet,ObjTypeEnum.SHAFT.getType());
            index++;
        }
        setSheetRefersToFormula(workbook,1,6,index,SheetNameEnum.CLASS_DATA.getType(),ClassTypeEnum.SHAFT.getType());
    }

    /***
     * Description: 填充楼层类数据
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @param infoSheet
     * @param infoCodeMap
     * @return : java.lang.Integer
     * @author : lijie
     * @date :2021/4/28 21:12
     * Update By lijie 2021/4/28 21:12
     */
    private void fillFloorClassData(Sheet listSheet, Sheet lookSheet, Workbook workbook, Sheet infoSheet, Map<String, List<InfoPointsBean>> infoCodeMap) {
        List<ObjBean> objBeans = new ObjService() {
            @Override
            protected String queryDataFromSaas(JSONObject param) {
                return saasClient.queryFloorType(param);
            }
        }.queryObjsData();
        if (CollUtil.isEmpty(objBeans)){
            return;
        }
        int index = 1;
        for (ObjBean firstBean : objBeans) {
            if (StrUtil.isBlank(firstBean.getObjId()) || StrUtil.isBlank(firstBean.getObjName())){
                continue;
            }
            setCellValue(firstBean.getObjName(),firstBean.getObjId(),4,listSheet,lookSheet,index,ObjTypeEnum.FLOOR_TYPE.getType());
            setInfoValue(workbook,lookSheet,infoCodeMap,firstBean.getObjId(),firstBean.getObjName(),infoSheet,ObjTypeEnum.FLOOR.getType());
            index++;
        }
        setSheetRefersToFormula(workbook,1,5,index,SheetNameEnum.CLASS_DATA.getType(),ClassTypeEnum.FLOOR.getType());
    }

    /***
     * Description: 填充建筑功能类型
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @param infoSheet
     * @param infoCodeMap
     * @return : java.lang.Integer
     * @author : lijie
     * @date :2021/4/28 21:05
     * Update By lijie 2021/4/28 21:05
     */
    private void fillBuildClassData(Sheet listSheet, Sheet lookSheet,
                                    Workbook workbook, Sheet infoSheet, Map<String, List<InfoPointsBean>> infoCodeMap) {
        List<ObjBean> objBeans = new ObjService() {
            @Override
            protected String queryDataFromSaas(JSONObject param) {
                return saasClient.queryBuildFuncType(param);
            }
        }.queryObjsData();
        if (CollUtil.isEmpty(objBeans)){
            return;
        }
        int index = 1;
        for (ObjBean firstBean : objBeans) {
            if (StrUtil.isBlank(firstBean.getObjId()) || StrUtil.isBlank(firstBean.getObjName())){
                continue;
            }
            setCellValue(firstBean.getObjName(),firstBean.getObjId(),3,listSheet,lookSheet,index,ObjTypeEnum.BUILD_FUNC_TYPE.getType());
            setInfoValue(workbook,lookSheet,infoCodeMap,firstBean.getObjId(),firstBean.getObjName(),infoSheet,ObjTypeEnum.BUILDING.getType());
            index++;
            List<ObjBean> secondBeans = firstBean.getContent();
            if (CollUtil.isEmpty(secondBeans)){
                continue;
            }
            for (ObjBean secondBean : secondBeans) {
                if (StrUtil.isBlank(firstBean.getObjId()) || StrUtil.isBlank(firstBean.getObjName())){
                    continue;
                }
                setCellValue(firstBean.getObjName()+SopCommonConst.UNDER_LINE+secondBean.getObjName(),secondBean.getObjId(),3,listSheet,lookSheet,index,ObjTypeEnum.BUILD_FUNC_TYPE.getType());
                setInfoValue(workbook,lookSheet,infoCodeMap,secondBean.getObjId(),firstBean.getObjName()+SopCommonConst.UNDER_LINE+secondBean.getObjName(),infoSheet,ObjTypeEnum.BUILDING.getType());
                index++;
                List<ObjBean> thirdBeans = secondBean.getContent();
                if (CollUtil.isEmpty(thirdBeans)){
                    continue;
                }
                for (ObjBean thirdBean : thirdBeans) {
                    if (StrUtil.isBlank(firstBean.getObjId()) || StrUtil.isBlank(firstBean.getObjName())){
                        continue;
                    }
                    setCellValue(firstBean.getObjName()+SopCommonConst.UNDER_LINE+secondBean.getObjName()+SopCommonConst.UNDER_LINE+thirdBean.getObjName(),thirdBean.getObjId(),3,listSheet,lookSheet,index,ObjTypeEnum.BUILD_FUNC_TYPE.getType());
                    setInfoValue(workbook,lookSheet,infoCodeMap,thirdBean.getObjId(),firstBean.getObjName()+SopCommonConst.UNDER_LINE+secondBean.getObjName()+SopCommonConst.UNDER_LINE+thirdBean.getObjName(),infoSheet,ObjTypeEnum.BUILDING.getType());
                    index++;
                }
            }
        }
        setSheetRefersToFormula(workbook,1,4,index,SheetNameEnum.CLASS_DATA.getType(),ClassTypeEnum.BUILD.getType());
    }

    /***
     * Description: 填充空间功能类型数据
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @param infoSheet
     * @param infoCodeMap
     * @return : java.lang.Integer
     * @author : lijie
     * @date :2021/4/28 20:46
     * Update By lijie 2021/4/28 20:46
     */
    private void fillSpaceClassData(Sheet listSheet, Sheet lookSheet, Workbook workbook, Sheet infoSheet, Map<String, List<InfoPointsBean>> infoCodeMap) {
        List<ObjBean> objBeans = new ObjService() {
            @Override
            protected String queryDataFromSaas(JSONObject param) {
                return saasClient.querySpaceClass(param);
            }
        }.queryObjsData();
        if (CollUtil.isEmpty(objBeans)){
            return;
        }
        int index = 1;
        for (ObjBean objBean : objBeans) {
            if (StrUtil.isBlank(objBean.getObjId()) || StrUtil.isBlank(objBean.getObjName())){
                continue;
            }
            setCellValue(objBean.getObjName(),objBean.getObjId(),2,listSheet,lookSheet,index,ObjTypeEnum.SPACE_CLASS.getType());
            setInfoValue(workbook, lookSheet,infoCodeMap,objBean.getObjId(),objBean.getObjName(),infoSheet, ObjTypeEnum.SPACE.getType());
            index++;
        }
        setSheetRefersToFormula(workbook,1,3,index,SheetNameEnum.CLASS_DATA.getType(),ClassTypeEnum.SPACE.getType());
    }

    /***
     * Description: 填充通用设备类数据
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @return : void
     * @author : lijie
     * @date :2021/4/28 20:13
     * Update By lijie 2021/4/28 20:13
     */
    private void fillEquipClassData(Sheet listSheet, Sheet lookSheet, Workbook workbook,
                                    Sheet infoSheet, Map<String, List<InfoPointsBean>> infoCodeMap) {
        // 1.查询通用设备类数据
        List<ObjBean> objBeans = new ObjService() {
            @Override
            protected String queryDataFromSaas(JSONObject param) {
                return saasClient.queryEquipClass(param);
            }
        }.queryObjsData();
        int index = 1;
        // 专业层
        for (ObjBean domainBean : objBeans) {
            List<ObjBean> classBeans = domainBean.getContent();
            if (CollUtil.isEmpty(classBeans)){
                continue;
            }
            // 系统类层
            for (ObjBean classBean : classBeans) {
                if (CollUtil.isEmpty(classBean.getContent())){
                    continue;
                }
                List<ObjBean> equipBeans = classBean.getContent();
                // 设备类层
                for (ObjBean equipBean : equipBeans) {
                    if (StrUtil.isBlank(equipBean.getObjName()) || StrUtil.isBlank(equipBean.getObjId())){
                        continue;
                    }
                    String equipName = equipBean.getObjName();
                    if (StrUtil.isNotBlank(classBean.getObjName())){
                        equipName = classBean.getObjName()+SopCommonConst.UNDER_LINE+equipName;
                    }
                    if (StrUtil.isNotBlank(domainBean.getObjName())){
                        equipName = domainBean.getObjName()+SopCommonConst.UNDER_LINE+equipName;
                    }
                    setCellValue(equipName,equipBean.getObjId(),1,listSheet,lookSheet,index,ObjTypeEnum.EQUIP_CLASS.getType());
                    setInfoValue(workbook, lookSheet,infoCodeMap,equipBean.getObjId(),equipName,infoSheet,equipBean.getObjId());
                    index++;
                }
            }
        }
        setSheetRefersToFormula(workbook,1,2,index,SheetNameEnum.CLASS_DATA.getType(),ClassTypeEnum.EQUIP.getType());
    }
    /***
     * Description: 创建名称
     * @param workbook : 工作簿
     * @param offset : 便宜量
     * @param rowId : 行id
     * @param index : 索引数
     * @param sheetName : sheet名称
     * @param manageName : 名称管理器中的名称
     * @return : void
     * @author : lijie
     * @date :2021/4/29 10:23
     * Update By lijie 2021/4/29 10:23
     */
    private void setSheetRefersToFormula(Workbook workbook, int offset, int rowId, int index,String sheetName,String manageName) {
        String range = getRange('A', offset, rowId, index);
        Name name = workbook.createName();
        //key不可重复
        name.setNameName(manageName);
        String formula = sheetName+"!" + range;
        name.setRefersToFormula(formula);
    }

    /***
     * Description:填充Cell的值
     * @param objName : 对象名称
     * @param objId : 对象id
     * @param index : listSheet的行索引
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @param listCellIndex : listSheet的单元格索引
     * @return : void
     * @author : lijie
     * @date :2021/4/28 20:52
     * Update By lijie 2021/4/28 20:52
     */
    private void setCellValue(String objName, String objId, Integer index, Sheet listSheet,
                              Sheet lookSheet,Integer listCellIndex,String objType) {
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(listSheet,index),listCellIndex), objName,null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(lookSheet,LOOK_INDEX.get()),0), objName,null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(lookSheet,LOOK_INDEX.get()),1), "_"+objId,null);
        CellUtil.setCellValue(CellUtil
                .getOrCreateCell(RowUtil.getOrCreateRow(lookSheet,LOOK_INDEX.get()),2), objType,null);
        LOOK_INDEX.set(LOOK_INDEX.get()+1);
    }

    /***
     * Description: 填充通用系统类的数据
     * @param listSheet : 做下拉列表的sheet页
     * @param lookSheet : 做VOLOOKUP的sheet页
     * @return : void  
     * @author : lijie
     * @date :2021/4/28 19:32
     * Update By lijie 2021/4/28 19:32
     */
    private void fillSystemClassData(Sheet listSheet, Sheet lookSheet,
            Workbook workbook,Sheet infoSheet, Map<String, List<InfoPointsBean>> infoCodeMap) {
        // 1.查询通用系统类的数据
        List<ObjBean> objBeans = new ObjService() {
            @Override
            protected String queryDataFromSaas(JSONObject param) {
                return saasClient.querySystemClass(param);
            }
        }.queryObjsData();
        int index = 1;
        for (ObjBean objBean : objBeans) {
            List<ObjBean> subObjBeans = objBean.getContent();
            if (CollUtil.isEmpty(subObjBeans)){
                continue;
            }
            for (ObjBean subObjBean : subObjBeans) {
                if (StrUtil.isBlank(subObjBean.getObjName()) || StrUtil.isBlank(subObjBean.getObjId())){
                    continue;
                }
                String objName = StrUtil.isBlank(objBean.getObjName()) ?subObjBean.getObjName():objBean.getObjName()+ SopCommonConst.UNDER_LINE +subObjBean.getObjName();
                setCellValue(objName,subObjBean.getObjId(),0,listSheet,lookSheet,index,ObjTypeEnum.SYSTEM_CLASS.getType());
                setInfoValue(workbook,lookSheet,infoCodeMap,subObjBean.getObjId(),objName,infoSheet,subObjBean.getObjId());
                index++;
            }
        }
        setSheetRefersToFormula(workbook,1,1,index,SheetNameEnum.CLASS_DATA.getType(),ClassTypeEnum.SYSTEM.getType());
    }
    /***
     * Description: 设置信息点数据
     * @param workbook :工作簿
     * @param lookSheet :存储对象类code的数据
     * @param infoCodeMap :信息点映射
     * @param objId :对象id
     * @return : void
     * @author : lijie
     * @date :2021/4/30 9:39
     * Update By lijie 2021/4/30 9:39
     */
    private void setInfoValue(Workbook workbook, Sheet lookSheet, Map<String, List<InfoPointsBean>> infoCodeMap,
                              String objId, String objName,Sheet infoSheet,String objType) {
        List<InfoPointsBean> infoPointsBeans = infoCodeMap.getOrDefault(objType, new ArrayList<>());
        for (int i = 0; i < infoPointsBeans.size(); i++) {
            InfoPointsBean infoPointsBean = infoPointsBeans.get(i);
            CellUtil.setCellValue(CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(lookSheet, LOOK_INDEX.get()-1),i+3),infoPointsBean.getName(),null);
            CellUtil.setCellValue(CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(infoSheet, INFO_INDEX.get()),0),objName+SopCommonConst.UNDER_LINE+infoPointsBean.getName(),null);
            CellUtil.setCellValue(CellUtil.getOrCreateCell(RowUtil.getOrCreateRow(infoSheet, INFO_INDEX.get()),1),JSONObject.toJSONString(infoPointsBean),null);
            INFO_INDEX.set(INFO_INDEX.get()+1);
        }
        setSheetRefersToFormula(workbook,3,LOOK_INDEX.get(),infoPointsBeans.size()+1, SheetNameEnum.CODE_DATA.getType(),"_"+objId);
    }

    /**
     *  计算formula
     * @param offset 偏移量，如果给0，表示从A列开始，1，就是从B列
     * @param rowId 第几行
     * @param colCount 一共多少列
     * @return 如果给入参 1,1,10. 表示从B1-K1。最终返回 $B$1:$K$1
     *
     */
    public static String getRange(char colFlag, int offset, int rowId, int colCount) {
        char start = (char)(colFlag + offset);
        if (colCount <= 25) {
            char end;
            if ((colCount - offset)<0){
                end = (char)(start + colCount - 1);
            }else {
                end = (char)(start + colCount - offset);
            }
            return "$" + start + "$" + rowId + ":$" + end + "$" + rowId;
        } else {
            char endPrefix = colFlag;
            char endSuffix = colFlag;
            // 26-51之间，包括边界（仅两次字母表计算）
            if ((colCount - 25) / 26 == 0 || colCount == 51) {
                if ((colCount - 25) % 26 == 0) {// 边界值
                    endSuffix = (char)(colFlag + 25);
                } else {
                    endSuffix = (char)(colFlag + (colCount - 25) % 26 - 1);
                }
            } else {// 51以上
                if ((colCount - 25) % 26 == 0) {
                    endSuffix = (char)(colFlag + 25);
                    endPrefix = (char)(endPrefix + (colCount - 25) / 26 - 1);
                } else {
                    if (((colCount - 25) % 26 - offset)<0){
                        endSuffix = (char)(colFlag + (colCount - 25) % 26 - 1);
                    }else {
                        endSuffix = (char)(colFlag + (colCount - 25) % 26 - offset);
                    }
                    endPrefix = (char)(endPrefix + (colCount - 25) / 26);
                }
            }
            return "$" + start + "$" + rowId + ":$" + endPrefix + endSuffix + "$" + rowId;
        }
    }

}
