package com.persagy.fm.sop.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.fm.common.constant.CommonConstant;
import com.persagy.fm.common.old.utils.StringUtil;
import com.persagy.fm.sop.constant.SopCommonConst;
import com.persagy.fm.sop.service.IInstanceService;
import com.persagy.fm.sop.service.IObjectService;
import com.persagy.old.cache.GraphCache;
import com.persagy.fm.sop.service.BaseService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service("instanceService")
public class InstanceServiceImpl extends BaseService implements IInstanceService {
    @Autowired
    private GraphCache graphCache;
    @Autowired
    private IObjectService objectService;
	@Override
	public String convertClassToObject(JSONObject jsonObject) throws Exception {
		String projectId = jsonObject.getString("project_id");
		JSONArray clazzs = jsonObject.getJSONArray("convert_class");// 待实例化类
		JSONObject past = jsonObject.getJSONObject("past"); // 已实例化的对象
		past = past == null ? new JSONObject() : past;
		Integer frequency = past.getInteger("frequency"); // 已实例化次数
		frequency = frequency == null ? 0 : frequency;
		JSONObject rule = jsonObject.getJSONObject("rule"); // 实例化规则

		JSONObject reusltJson = new JSONObject();
		JSONObject convertObject = new JSONObject();
		JSONObject clazz;
		String objId, objType;
		JSONArray instancedObjs, pastTemp;
		boolean containScopeFlag;
		
		//--------------设备类放到最后实例化--------
		clazzs.sort((o1,o2)->{
			if ("equip_class".equals(((JSONObject)o1).getString("obj_type"))) {
				return 1;
			}
			return -1;
		});
		// ====================分别实例化每个类=============
		for (Object object : clazzs) {
			clazz = (JSONObject) object;
			objId = clazz.getString("obj_id");
			objType = clazz.getString("obj_type");
			containScopeFlag = false;
			if (CommonConstant.data_type_equip_class.equals(objType) && !StringUtil.isEmptyList(clazz, "space_scope")) {
				JSONObject spaceClazz = clazz.getJSONArray("space_scope").getJSONObject(0);
				containScopeFlag = true;
				objId = spaceClazz.getString("obj_id");
				objType = spaceClazz.getString("obj_type");
			}
			
			//----------避免重复实例化-----
			if (containScopeFlag && convertObject.containsKey(objId)) {
				instancedObjs = convertObject.getJSONArray(objId);
				instancedObjs = queryEquipByScope(projectId, clazz.getString("obj_id"), instancedObjs);
				convertObject.put(objId + "*" + clazz.getString("obj_id"), instancedObjs);
				continue;
			} else if (convertObject.containsKey(objId) || convertObject.containsKey(objId + "*" + clazz.getString("obj_id"))) {
				continue;
			}
			
			//----------实例化-------
			instancedObjs = querySpaceOrEquipByType(projectId, objId, objType);
			if (CollectionUtils.isEmpty(instancedObjs)) {
				continue;
			}
			//--------过滤--------
			pastTemp = past.getJSONArray(objId);
			pastTemp = pastTemp == null ? new JSONArray() : pastTemp;
			instancedObjs = filterObjByRule(projectId, instancedObjs, rule, pastTemp, frequency);
			past.put(objId, pastTemp);

			if (containScopeFlag) {
				instancedObjs = queryEquipByScope(projectId, clazz.getString("obj_id"), instancedObjs);
				convertObject.put(objId + "*" + clazz.getString("obj_id"), instancedObjs);
			} else {
				convertObject.put(objId, instancedObjs);
			}
		}
		past.put("frequency", frequency + 1);
		reusltJson.put("past", past);
		reusltJson.put("convert_object", convertObject);
		return reusltJson.toJSONString();
	}
	
	/**
	 * 
	 * Description: 获取空间内的设备对象
	 * @param projectId
	 * @param equipType
	 * @param spaceObjs
	 * @return
	 * @throws Exception JSONArray
	 * @author yuecaipu
	 * @since 2018年10月26日: 下午5:52:27
	 * Update By yuecaipu 2018年10月26日: 下午5:52:27
	 */
	private JSONArray queryEquipByScope(String projectId, String equipType, JSONArray spaceObjs) throws Exception {

		//=========获取所有该类型设备======
		JSONArray equips = objectService.queryEquipByType(projectId, equipType);
		Set<String> equipIdsFromScope = new HashSet<>();
		Map<String, JSONObject> id2equip = new HashMap<>();
		//---------定义变量-----
		JSONObject equip, space, relation;
		JSONArray relations ;
		String spaceId;
		for (Object object : equips) {
			equip = (JSONObject) object;
			id2equip.put(equip.getString("obj_id"), equip);
		}
		//======获取空间内的所有设备===
		String graphId = graphCache.getGraphIdByProjectIdAndGraph(projectId, CommonConstant.graph_type_equip_in_space);
		for (Object object : spaceObjs) {
			space = (JSONObject) object;
			spaceId = space.getString("obj_id");
			relations = objectService.queryRelations(projectId, null, spaceId, graphId, "1");
			if (relations!=null) {
				for (Object object2 : relations) {
					relation = (JSONObject) object2;
					equipIdsFromScope.add(relation.getString("from_id"));
				}
			}
		}

		//=========交集========
		equips.clear();
		for (String equipId : equipIdsFromScope) {
			if (id2equip.containsKey(equipId)) {
				equips.add(id2equip.get(equipId));
			}
		}
		return equips;
	}

	private JSONArray querySpaceOrEquipByType(String projectId, String id , String type) throws Exception {
		if (CommonConstant.data_type_equip_class.equals(type)) {
			return objectService.queryEquipByType(projectId, id);
		}else if (CommonConstant.data_type_space_class.equals(type)) {
			return objectService.querySpaceByType(projectId, id);
		}
		return new JSONArray();
	}

	/**
	 * 
	 * Description: 根据实例化规则过滤对象
	 * @param projectId
	 * @param objsArr   需要过滤的对象集合
	 * @param rule      实例化规则
	 * @param past      已经实例化过的对象
	 * @return
	 * @throws Exception JSONArray
	 * @author yuecaipu
	 * @param frequency 
	 * @since 2018年10月26日: 下午5:51:36 
	 * Update By yuecaipu 2018年10月26日: 下午5:51:36
	 */
	private JSONArray filterObjByRule(String projectId, JSONArray objsArr, JSONObject rule, JSONArray past, Integer frequency) throws Exception {
		if (objsArr.isEmpty() || rule == null || StringUtil.isNull(rule, "execute_all_obj_flag") || "1".equals(rule.getString("execute_all_obj_flag"))) {
			return objsArr;
		}

		//===========获取对象的位置信息===============
		String objType = objsArr.getJSONObject(0).getString("obj_type");
		JSONArray objIds = new JSONArray();
//		Map<String, JSONObject> id2obj = new HashMap<>();
		for (int i = 0; i < objsArr.size(); i++) {
			objIds.add(objsArr.getJSONObject(i).getString("obj_id"));
//			id2obj.put(objsArr.getJSONObject(i).getString("obj_id"), objsArr.getJSONObject(i));
		}
		JSONArray objRelationship = objectService.querySpaceOrEquipNoHierarchy(projectId, objType, objIds);

		//==========根据scope 楼层编码筛选===========
		if (!StringUtil.isEmptyList(rule, "scope")) {
			JSONArray scope = rule.getJSONArray("scope");
			Set<String> floorIdentity = new HashSet<>();
			for (int i = 0; i < scope.size(); i++) {
				floorIdentity.add(scope.getJSONObject(i).getString("obj_id"));
			}
			
			for (int i = 0; i < objRelationship.size(); i++) {
				if (!floorIdentity.contains(objRelationship.getJSONObject(i).getString("floor_identity"))) {
//					objIds.remove(objRelationship.getJSONObject(i).getString("obj_id"));
//					id2obj.remove(objRelationship.getJSONObject(i).getString("obj_id"));
					objRelationship.remove(i);
					i--;
				}
			}
		}
		// ----------楼层与竖井序号加上建筑id前缀------------------
		String floor_sequence_id, shaft_sequence_id, build_id;
		for (int i = 0; i < objRelationship.size(); i++) {
			floor_sequence_id = objRelationship.getJSONObject(i).getString(FLOOR_SEQUENCE_ID);
			shaft_sequence_id = objRelationship.getJSONObject(i).getString(SHAFT_SEQUENCE_ID);
			build_id = objRelationship.getJSONObject(i).getString("build_id");
			if (!StringUtil.isNull(floor_sequence_id)) {
				objRelationship.getJSONObject(i).put(FLOOR_SEQUENCE_ID, build_id + "_" + floor_sequence_id);
			}
			if (!StringUtil.isNull(shaft_sequence_id)) {
				objRelationship.getJSONObject(i).put(SHAFT_SEQUENCE_ID, build_id + "_" + shaft_sequence_id);
			}
		}
		
		//===========筛选======================
//		String sequenceCode = getSequenceCode(rule.getString("order"));
//		objRelationship = JSONUtil.sortByStringField(objRelationship, sequenceCode, 1);

		objRelationship = filterBySequence(objRelationship, rule, past, objType,frequency);
		
		return objRelationship;
	}
	
	private JSONArray filterBySequence(JSONArray objRelationship, JSONObject rule, JSONArray past, String objType, Integer frequency) {
		String order = rule.getString("order"); // 对象执行排序 1-楼层,2-竖井,3-空间/设备,4-随机序号
		boolean reset_random = rule.getBooleanValue("reset_random"); // 重新随机:当order=4时可选择
		String style = rule.getString("style"); // 实例化方式： 1-n次实例化完全部对象、2-每次实例化n(%)个楼层/竖井、3-每次实例化n(%)个对象
		Integer cycle = rule.getInteger("cycle"); // 实例化周期： n次实例化完全部对象,style=1时有效
		Integer count = rule.getInteger("count"); // 个数 style=2/3时有效
		String unit = rule.getString("unit"); // 单位：1-个，2-%(百分比)

		/*
		 * 如果设置(或修改为)一次实例化全部对象，为防止计划执行期间新创建对象(修改计划)后的第一次发单只实例化出新创对象(或未实例化过的对象)的bug
		 */
		if ("1".equals(style) && 1 == cycle) {
			past.clear();
		}
		
		JSONArray result = new JSONArray();
		String orderField = getSequenceCode(order);
		JSONObject node;
		if ("1".equals(order)) {
			//=======楼层排序========
			Map<String, JSONArray> sequence2obj = sortBySequence(objRelationship, orderField);
			if ("1".equals(style)) {
				//-----X次实例化完全部对象-----
				Integer cycleTemp = cycle - (frequency % cycle);
				/**
				 * 若周期数大于对象总个数，后面的几次实例化为空 yuecaipu update by 2019/3/4
				 */
				boolean clearFlag = true;
				if (cycleTemp != 0 && cycleTemp <= cycle - objRelationship.size()) {
					clearFlag =false;
				}
				JSONArray unpastObjRelationship = filterByPass(objRelationship, past, "obj_id",clearFlag);
				sortBySequence(unpastObjRelationship, orderField);
				
				cycle = cycleTemp == 0 ? cycle : cycleTemp;
				count = (int) Math.ceil(unpastObjRelationship.size() * 1.0 / cycle);
				for (int i = 0; i < count; i++) {
					result.add(unpastObjRelationship.getJSONObject(i));
					past.add(unpastObjRelationship.getJSONObject(i).getString("obj_id"));
				}
			}else if ("2".equals(style)) {
				//-----每次实例化X个楼层内的对象-----
				List<String> sequences = new ArrayList<>();
				sequences.addAll(sequence2obj.keySet());
//				sequences.sort((o1,o2)->o1.compareTo(o2));
				sortFloorOrShaftSequence(sequences);
				if ("2".equals(unit)) {
					count = (int) Math.ceil(1.0 * sequences.size() * count / 100);
				}
				count = count > sequences.size() ? sequences.size() : count;
				for (String sequence : sequences) {
					if (!past.contains(sequence) && count > 0) {
						past.add(sequence);
						result.addAll(sequence2obj.get(sequence));
						count--;
					}
				}
				if (count > 0) {
					past.clear();
					for (String sequence : sequences) {
						if (!past.contains(sequence) && count > 0) {
							past.add(sequence);
							result.addAll(sequence2obj.get(sequence));
							count--;
						}
					}
				}
				
			}else {
				//-----每次实例化X个对象-----
				int total = objRelationship.size();
				if ("2".equals(unit)) {
					count = (int) Math.ceil(1.0 * total * count / 100);
				}
				count = count > total ? total : count;
				for (int i = 0; i < total && count > 0; i++) {
					node = objRelationship.getJSONObject(i);
					if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
						past.add(node.getString("obj_id"));
						result.add(node);
						count--;
					}
				}
				if (count>0) {
					past.clear();
					for (int i = 0; i < total && count > 0; i++) {
						node = objRelationship.getJSONObject(i);
						if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
							past.add(node.getString("obj_id"));
							result.add(node);
							count--;
						}
					}
				}
			}
		}else if ("2".equals(order)) {
			//=======竖井排序========
			Map<String, JSONArray> sequence2obj = sortBySequence(objRelationship, orderField);
			if ("1".equals(style)) {
				//-----X次实例化完全部对象-----
				Integer cycleTemp = cycle - (frequency % cycle);
				/**
				 * 若周期数大于对象总个数，后面的几次实例化为空 yuecaipu update by 2019/3/4
				 */
				boolean clearFlag = true;
				if (cycleTemp != 0 && cycleTemp <= cycle - objRelationship.size()) {
					clearFlag =false;
				}
				JSONArray unpastObjRelationship = filterByPass(objRelationship, past, "obj_id",clearFlag);
				sortBySequence(unpastObjRelationship, orderField);
				cycle = cycleTemp == 0 ? cycle : cycleTemp;
				count = (int) Math.ceil(unpastObjRelationship.size() * 1.0 / cycle);
				for (int i = 0; i < count; i++) {
					result.add(unpastObjRelationship.getJSONObject(i));
					past.add(unpastObjRelationship.getJSONObject(i).getString("obj_id"));
				}
			}else if ("2".equals(style)) {
				//-----每次实例化X个竖井内的对象-----
				List<String> sequences = new ArrayList<>();
				sequences.addAll(sequence2obj.keySet());
//				sequences.sort((o1,o2)->o1.compareTo(o2));
				sortFloorOrShaftSequence(sequences);
				if ("2".equals(unit)) {
					count = (int) Math.ceil(1.0 * sequences.size() * count / 100);
				}
				count = count > sequences.size() ? sequences.size() : count;
				for (String sequence : sequences) {
					if (!past.contains(sequence) && count > 0) {
						past.add(sequence);
						result.addAll(sequence2obj.get(sequence));
						count--;
					}
				}
				if (count > 0) {
					past.clear();
					for (String sequence : sequences) {
						if (!past.contains(sequence) && count > 0) {
							past.add(sequence);
							result.addAll(sequence2obj.get(sequence));
							count--;
						}
					}
				}
				
			}else {
				//-----每次实例化X个对象-----
				int total = objRelationship.size();
				if ("2".equals(unit)) {
					count = (int) Math.ceil(1.0 * total * count / 100);
				}
				count = count > total ? total : count;
				for (int i = 0; i < total && count > 0 ; i++) {
					node = objRelationship.getJSONObject(i);
					if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
						past.add(node.getString("obj_id"));
						result.add(node);
						count--;
					}
				}
				if (count > 0) {
					past.clear();
					for (int i = 0; i < total && count > 0 ; i++) {
						node = objRelationship.getJSONObject(i);
						if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
							past.add(node.getString("obj_id"));
							result.add(node);
							count--;
						}
					}
				}
			}
		}else if ("3".equals(order)) {
			//=======对象实例排序========
			sortBySequence(objRelationship, orderField);
			if ("1".equals(style)) {
				//-----X次实例化完全部对象-----
				Integer cycleTemp = cycle - (frequency % cycle);
				/**
				 * 若周期数大于对象总个数，后面的几次实例化为空 yuecaipu update by 2019/3/4
				 */
				boolean clearFlag = true;
				if (cycleTemp != 0 && cycleTemp <= cycle - objRelationship.size()) {
					clearFlag =false;
				}
				JSONArray unpastObjRelationship = filterByPass(objRelationship, past, "obj_id",clearFlag);
				sortBySequence(unpastObjRelationship, orderField);
				cycle = cycleTemp == 0 ? cycle : cycleTemp;
				count = (int) Math.ceil(1.0*unpastObjRelationship.size()/cycle);
				for (int i = 0; i < count; i++) {
					result.add(unpastObjRelationship.getJSONObject(i));
					past.add(unpastObjRelationship.getJSONObject(i).getString("obj_id"));
				}
			}else {
				//-----每次实例化X个对象-----
				if ("2".equals(unit)) {
					count = (int) Math.ceil(1.0*objRelationship.size()*count/100);
				}
				count = count > objRelationship.size() ? objRelationship.size() : count;
				for (int i = 0; i < objRelationship.size() && count > 0; i++) {
					node = objRelationship.getJSONObject(i);
					if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
						past.add(node.getString("obj_id"));
						result.add(node);
						count--;
					}
				}
				if (count > 0) {
					past.clear();
					for (int i = 0; i < objRelationship.size() && count > 0 ; i++) {
						node = objRelationship.getJSONObject(i);
						if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
							past.add(node.getString("obj_id"));
							result.add(node);
							count--;
						}
					}
				}
			}
		}else {
			//=======对象实例随机排序========
			Collections.shuffle(objRelationship);
			if ("1".equals(style)) {
				//-----X次实例化完全部对象-----
				Integer cycleTemp = cycle - (frequency % cycle);
				/**
				 * 若周期数大于对象总个数，后面的几次实例化为空 yuecaipu update by 2019/3/4
				 */
				boolean clearFlag = true;
				if (cycleTemp != 0 && cycleTemp <= cycle - objRelationship.size()) {
					clearFlag = false;
				}
				JSONArray unpastObjRelationship = filterByPass(objRelationship, past, "obj_id",clearFlag);
				cycle = cycleTemp == 0 ? cycle : cycleTemp;
				count = (int) Math.ceil(1.0*unpastObjRelationship.size()/cycle);
				for (int i = 0; i < count; i++) {
					result.add(unpastObjRelationship.getJSONObject(i));
					past.add(unpastObjRelationship.getJSONObject(i).getString("obj_id"));
				}
			}else {
				//-----每次实例化X个对象-----
				if (reset_random) {//------每次重新随机---
					past.clear();
				}
				int total = objRelationship.size();
				if ("2".equals(unit)) {
					count = (int) Math.ceil(1.0*total*count/100);
				}
				count = count > total ? total : count;
				for (int i = 0; i < objRelationship.size() && count > 0; i++) {
					node = objRelationship.getJSONObject(i);
					if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
						past.add(node.getString("obj_id"));
						result.add(node);
						count--;
					}
				}
				if (count > 0) {
					past.clear();
					for (int i = 0; i < objRelationship.size() && count > 0; i++) {
						node = objRelationship.getJSONObject(i);
						if (!past.contains(node.getString("obj_id")) && !result.contains(node)) {
							past.add(node.getString("obj_id"));
							result.add(node);
							count--;
						}
					}
				}
			}
		}
		return result;
	}

	/**将楼层或竖井序号排序*/
	private void sortFloorOrShaftSequence(List<String> sequences) {
		//----split[0]:建筑ID  split[1]:楼层/竖井序号
		sequences.sort((o1,o2)->{
			String[] split1 = o1.split("_");
			String[] split2 = o2.split("_");
			if (split1[0].equals(split2[0])) {
				return split1[1].compareTo(split2[1]);
			}
			return split1[0].compareTo(split2[0]);
		});
	}

	private JSONArray filterByPass(JSONArray objRelationship, JSONArray past , String passField, boolean clearFlag) {
		JSONObject object ;
		JSONArray result = new JSONArray();
		if (past.isEmpty()) {
			result.addAll(objRelationship);
		}else {
			for (int i = 0; i < objRelationship.size(); i++) {
				object = objRelationship.getJSONObject(i);
				if (!past.contains(object.getString(passField))) {
					result.add(objRelationship.getJSONObject(i));
				}
			}
			if (clearFlag && result.isEmpty()) {
				past.clear();
				result.addAll(objRelationship);
			}
		}
		return result;
	}
	
	
	/**
	 * 排序规则： 
	 * 按楼层/竖井排序：建筑id->楼层/竖井序号->空间/设备序号 
	 * 按空间/设备排序：空间/设备序号
	 */
	private Map<String, JSONArray> sortBySequence(JSONArray objRelationship, String orderField) {

		Map<String, JSONArray> sequence2obj = new HashMap<>();
		JSONArray arr ;
		
		//========移除序号为空的对象======
		for (int i = 0; i < objRelationship.size(); i++) {
			if (StringUtil.isNull(objRelationship.getJSONObject(i), orderField)) {
				objRelationship.remove(i);
				i--;
			}
		}
		//==========排序============
		objRelationship.sort((o1, o2) -> {
			String o1Sequence = ((JSONObject) o1).getString(orderField);
			String o2Sequence = ((JSONObject) o2).getString(orderField);
			if (OBJ_SERIAL.equals(orderField)) {
				return o1Sequence.compareTo(o2Sequence);
			}
			
			String[] split1 = o1Sequence.split("_");
			String[] split2 = o2Sequence.split("_");
			
			// -------split[0]:建筑Id，split[1]:楼层/竖井序号----
			if (split1[0].equals(split2[0])) {
				if (split1[1].equals(split2[1])) {
					String o1SERIAL = ((JSONObject) o1).getString(OBJ_SERIAL);
                    String o2SERIAL = ((JSONObject) o2).getString(OBJ_SERIAL);
					o1SERIAL = o1SERIAL == null ? "0" : o1SERIAL;
					o2SERIAL = o2SERIAL == null ? "0" : o2SERIAL;
					return o1SERIAL.compareTo(o2SERIAL);
				}
				return split1[1].compareTo(split2[1]);
			} else {
				return split1[0].compareTo(split2[0]);
			}
			
		});
		
		//========根据序号分类======
		for (int j = 0; j < objRelationship.size(); j++) {

			String orderField_value = objRelationship.getJSONObject(j).getString(orderField);
			arr = sequence2obj.getOrDefault(orderField_value, new JSONArray());
			arr.add(objRelationship.getJSONObject(j));
			sequence2obj.put(orderField_value, arr);
		}
		return sequence2obj;
	}
	
	private static final String FLOOR_SEQUENCE_ID = "floor_sequence_id";
	private static final String SHAFT_SEQUENCE_ID = "shaft_sequence_id";
	private static final String OBJ_SERIAL = "obj_serial";
	
	private String getSequenceCode(String order) {
		String sequenceCode = null;
		if ("1".equals(order)) {
			sequenceCode = FLOOR_SEQUENCE_ID;
		} else if ("2".equals(order)) {
			sequenceCode = SHAFT_SEQUENCE_ID;
		} else {
			sequenceCode = OBJ_SERIAL;
		}
		return sequenceCode;
	}

}
