package com.persagy.fm.sop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.fm.sop.entity.SopCell;
import com.persagy.fm.sop.dao.SopCellMapper;
import com.persagy.fm.sop.service.ISopCellService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author lijie
 * @since 2021-04-02
 */
@Service
public class SopCellServiceImpl extends ServiceImpl<SopCellMapper, SopCell> implements ISopCellService {
    /***
     * Description: 查询所有引用sop 的sop（递归引用）
     * @param projectId : 项目id
     * @param sopId : SOP的id
     * @return : java.util.Set<java.lang.String>
     * @author : lijie
     * @date :2021/4/19 9:38
     * Update By lijie 2021/4/19 9:38
     */
    @Override
    public Set<String> querySopRel(String projectId, String sopId) {
        LambdaQueryWrapper<SopCell> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopCell::getProjectId,projectId);
        queryWrapper.eq(SopCell::getValid,true);
        List<SopCell> sopCells = this.list(queryWrapper);
        //key：被引用SOP的主键id  value:引用该SOP的SOP主键id
        Map<String, Set<String>> relSopId2sopId = new HashMap<>();
        for (SopCell sopCell : sopCells) {
            Set<String> sopIds = relSopId2sopId.getOrDefault(sopCell.getRelSopId(), new HashSet<>());
            sopIds.add(sopCell.getSopId());
            relSopId2sopId.put(sopCell.getRelSopId(),sopIds);
        }
        return getRelSopRecursively(relSopId2sopId,sopId);
    }
    /***
     * Description: 递归查询引用的SOP
     * @param relSopId2sopId : 引用SOP的映射`{被引用SOP的主键id:引用该SOP的SOP主键id}`
     * @param sopId : SOP主键id
     * @return : java.util.Set<java.lang.String>  
     * @author : lijie
     * @date :2021/4/19 9:38
     * Update By lijie 2021/4/19 9:38
     */
    private Set<String> getRelSopRecursively(Map<String, Set<String>> relSopId2sopId, String sopId) {
        Set<String> relSopIds = new HashSet<>();
        if (CollectionUtils.isEmpty(relSopId2sopId.get(sopId))) {
            return relSopIds;
        }
        Set<String> set = relSopId2sopId.get(sopId);
        relSopIds.addAll(set);
        for (String string : set) {
            relSopIds.addAll(getRelSopRecursively(relSopId2sopId, string));
        }
        return relSopIds;
    }
    
}
