package com.persagy.fm.sop.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.persagy.fm.sop.entity.SopDestroyMsg;
import com.persagy.fm.sop.dao.SopDestroyMsgMapper;
import com.persagy.fm.sop.enums.MsgStatusEnum;
import com.persagy.fm.sop.service.ISopDestroyMsgService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author lijie
 * @since 2021-04-02
 */
@Service
public class SopDestroyMsgServiceImpl extends ServiceImpl<SopDestroyMsgMapper, SopDestroyMsg> implements ISopDestroyMsgService {
    /***
     * Description: 根据SOP的id和状态查询作废消息列表
     * @param sopId : SOP的id
     * @param status : 消息状态,1-未处理,2-已处理
     * @return : java.util.List<com.persagy.fm.sop.entity.SopDestroyMsg>
     * @author : lijie
     * @date :2021/4/19 21:46
     * Update By lijie 2021/4/19 21:46
     */
    @Override
    public List<SopDestroyMsg> queryRecordByCondition(String sopId, String status) throws Exception {
        LambdaQueryWrapper<SopDestroyMsg> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopDestroyMsg::getValid,true);
        queryWrapper.eq(SopDestroyMsg::getStatus, MsgStatusEnum.UNTREATED.getType());
        if (StrUtil.isNotBlank(status)){
            queryWrapper.eq(SopDestroyMsg::getStatus,MsgStatusEnum.HANDLED.getType());
        }
        if (StrUtil.isNotBlank(sopId)){
            queryWrapper.eq(SopDestroyMsg::getSopId,sopId);
        }
        return this.list(queryWrapper);
    }
    /***
     * Description: 标记SOP作废消息已读
     * @param msgId : 消息id
     * @param sopId : SOP的id
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 21:58
     * Update By lijie 2021/4/19 21:58
     */
    @Override
    public void markReadSopDestroyMsg(String msgId, String sopId) {
        if (StrUtil.isBlank(msgId) && StrUtil.isBlank(sopId)){
            return;
        }
        LambdaUpdateWrapper<SopDestroyMsg> updateWrapper = new LambdaUpdateWrapper<>();
        if (StrUtil.isNotBlank(msgId)){
            updateWrapper.eq(SopDestroyMsg::getMsgId,msgId);
        }
        if (StrUtil.isNotBlank(sopId)){
            updateWrapper.eq(SopDestroyMsg::getSopId,sopId);
        }
        this.update(SopDestroyMsg.builder().status(MsgStatusEnum.HANDLED.getType()).build(),updateWrapper);
    }

}
