package com.persagy.fm.sop.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.fm.sop.entity.SopEdit;
import com.persagy.fm.sop.dao.SopEditMapper;
import com.persagy.fm.sop.service.ISopEditService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author lijie
 * @since 2021-04-02
 */
@Service
public class SopEditServiceImpl extends ServiceImpl<SopEditMapper, SopEdit> implements ISopEditService {
    /***
     * Description: 根据sopId删除草稿
     * @param sopId : sopId
     * @return : void
     * @author : lijie
     * @date :2021/4/8 16:10
     * Update By lijie 2021/4/8 16:10
     */
    @Override
    public void deleteDraftBySopId(String sopId) {
        LambdaQueryWrapper<SopEdit> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopEdit::getSopId,sopId);
        queryWrapper.eq(SopEdit::getValid,true);
        this.remove(queryWrapper);
    }
    /***
     * Description: 根据SOP的唯一标识查询SOP的草稿
     * @param sopId : sopId
     * @return : com.persagy.fm.sop.entity.SopEdit
     * @author : lijie
     * @date :2021/4/8 19:11
     * Update By lijie 2021/4/8 19:11
     */
    @Override
    public List<SopEdit> querySopEditBySopId(String sopId) {
        return queryRecordByKey(sopId,null);
    }
    /***
     * Description: 根据SOP的id和项目id查询SOP数据
     * @param sopId : sop的id
     * @param projectId : 项目id
     * @return : com.persagy.fm.sop.entity.SopEdit
     * @author : lijie
     * @date :2021/4/21 10:16
     * Update By lijie 2021/4/21 10:16
     */
    @Override
    public List<SopEdit> queryRecordByKey(String sopId, String projectId) {
        LambdaQueryWrapper<SopEdit> queryWrapper = new LambdaQueryWrapper<>();
        if (StrUtil.isNotBlank(sopId)){
            queryWrapper.eq(SopEdit::getSopId,sopId);
        }
        if (StrUtil.isNotBlank(projectId)){
            queryWrapper.eq(SopEdit::getProjectId,projectId);
        }
        queryWrapper.eq(SopEdit::getValid,true);
        return this.list(queryWrapper);
    }


}
