package com.persagy.fm.sop.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.persagy.fm.sop.entity.SopReviseMsg;
import com.persagy.fm.sop.dao.SopReviseMsgMapper;
import com.persagy.fm.sop.enums.MsgStatusEnum;
import com.persagy.fm.sop.service.ISopReviseMsgService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author lijie
 * @since 2021-04-02
 */
@Service
public class SopReviseMsgServiceImpl extends ServiceImpl<SopReviseMsgMapper, SopReviseMsg> implements ISopReviseMsgService {
    /***
     * Description: 根据条件查询数据
     * @param sopId : sopId
     * @param status : status
     * @param msgId : msgId
     * @return : java.util.List<com.persagy.fm.sop.entity.SopReviseMsg>
     * @author : lijie
     * @date :2021/4/8 18:45
     * Update By lijie 2021/4/8 18:45
     */
    @Override
    public List<SopReviseMsg> queryDatasByCondition(String sopId,String status,String msgId) {
        LambdaQueryWrapper<SopReviseMsg> queryWrapper = new LambdaQueryWrapper<>();
        if (StrUtil.isNotBlank(sopId)){
            queryWrapper.eq(SopReviseMsg::getSopId,sopId);
        }
        queryWrapper.eq(SopReviseMsg::getStatus,StrUtil.isNotBlank(status)?status:"1");
        if (StrUtil.isNotBlank(msgId)){
            queryWrapper.eq(SopReviseMsg::getMsgId,msgId);
        }
        return this.list(queryWrapper);
    }
    /***
     * Description: 忽略SOP的消息
     * @param sopReviseMsg : SOP的消息
     * @return : void
     * @author : lijie
     * @date :2021/4/8 19:04
     * Update By lijie 2021/4/8 19:04
     */
    @Override
    public void ignoreSopReviseMsg(SopReviseMsg sopReviseMsg){
        sopReviseMsg.setStatus("2");
        this.updateById(sopReviseMsg);
    }
    /***
     * Description: 根据obj_id,sop_id批量更新status=1
     * @param objIds : 关联的对象id
     * @param sopId : SOP的id
     * @return : void
     * @author : lijie
     * @date :2021/4/19 12:39
     * Update By lijie 2021/4/19 12:39
     */
    @Override
    public void updateBatchSopReviseMsg(List<String> objIds, String sopId) throws Exception {
        if (CollUtil.isEmpty(objIds) && StrUtil.isBlank(sopId)){
            return;
        }
        LambdaUpdateWrapper<SopReviseMsg> updateWrapper = new LambdaUpdateWrapper<>();
        if (CollUtil.isNotEmpty(objIds)){
            updateWrapper.in(SopReviseMsg::getObjId,objIds);
        }
        if (StrUtil.isNotBlank(sopId)){
            updateWrapper.eq(SopReviseMsg::getSopId,sopId);
        }
        this.update(updateWrapper);
    }
    /***
     * Description: 忽略SOP的消息
     * @param msgId : SOP的消息id
     * @param sopId : SOP的id
     * @return : void
     * @author : lijie
     * @date :2021/4/8 19:04
     * Update By lijie 2021/4/8 19:04
     */
    @Override
    public void ignoreSopReviseMsg(String msgId, String sopId) {
        if (StrUtil.isBlank(msgId) && StrUtil.isBlank(sopId)){
            return;
        }
        this.update(SopReviseMsg.builder().status(MsgStatusEnum.HANDLED.getType()).build(),createUpdateWrapper(msgId,sopId));
    }
    /***
     * Description: 更改SOP的待修订标记为已自动处理
     * @param msgId : 消息id
     * @param sopId : SOP的id
     * @return : java.lang.String  
     * @author : lijie
     * @date :2021/4/21 14:09
     * Update By lijie 2021/4/21 14:09
     */
    @Override
    public void autoSopReviseMsg(String msgId, String sopId){
        if (StrUtil.isBlank(msgId) && StrUtil.isBlank(sopId)){
            return;
        }
        this.update(SopReviseMsg.builder()
                .status(MsgStatusEnum.AUTO_HANDLED.getType())
                .build(),createUpdateWrapper(msgId,sopId));
    }
    /***
     * Description: 创建更新的Wrapper
     * @param msgId : 消息id
     * @param sopId : SOP的id
     * @return : com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper<com.persagy.fm.sop.entity.SopReviseMsg>
     * @author : lijie
     * @date :2021/4/21 14:34
     * Update By lijie 2021/4/21 14:34
     */
    private LambdaUpdateWrapper<SopReviseMsg> createUpdateWrapper(String msgId, String sopId){
        LambdaUpdateWrapper<SopReviseMsg> updateWrapper = new LambdaUpdateWrapper<>();
        if (StrUtil.isNotBlank(msgId)){
            updateWrapper.eq(SopReviseMsg::getMsgId,msgId);
        }
        if (StrUtil.isNotBlank(sopId)){
            updateWrapper.eq(SopReviseMsg::getSopId,sopId);
        }
        return updateWrapper;
    }
}
