SET NAMES utf8mb4;
SET FOREIGN_KEY_CHECKS = 0;
CREATE TABLE IF NOT EXISTS `custom_info_point`  (
  `custom_info_point_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `custom_info_point_name` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '信息点控件名',
  `details` json DEFAULT NULL COMMENT '控件明细',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`custom_info_point_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

CREATE TABLE IF NOT EXISTS `operate_log`  (
`log_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `operate_person_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作人id',
  `table_name` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作表名',
  `operate_type` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作类型',
  `request_method` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '请求方法',
  `request_content` json DEFAULT NULL COMMENT '请求内容',
  `operate_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作时间,格式为yyyyMMddHHmmss',
  `operate_result` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作结果',
  `result_content` json DEFAULT NULL COMMENT '操作结果内容',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`log_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

CREATE TABLE IF NOT EXISTS `sop`  (
  `sop_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT 'SOP的主键id',
  `project_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '项目id,集团SOP的值为group_sop_id',
  `extend_field` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '建筑功能类型',
  `operator_id` varchar(36) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作人id,为人员id或账号id',
  `operator_name` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '操作人姓名,为人员名称或账号名称',
  `sop_name` varchar(100) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的名称,目前长度定为100.交互支持50个字符',
  `sop_type` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP类型,1-通用,2-项目专用',
  `no_obj` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '是否有无对象的步骤,0-否 1-是.当为引用的SOP时,就算被引用的没有对象也为1',
  `related_data` json DEFAULT NULL COMMENT '相关资料,[{"name":"***","url"："***"}]',
  `sop_status` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '1-已发布、2-已作废',
  `publish_status` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '1-正常、2-待修订、3-修订中',
  `version` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '版本号',
  `version_count` int(10) DEFAULT NULL COMMENT '版本数量',
  `version_explain` json DEFAULT NULL COMMENT '版本说明,[“描述”,“描述”]',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_cell
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_cell`  (
  `sop_cell_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT 'SOP_CELL的主键id',
  `parent_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '父级id,SOP的id或者sop_cell_id',
  `project_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '项目id',
  `from_sop` int(1) DEFAULT NULL COMMENT 'true-引用的SOP.是否引用的SOP',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '所属的SOP的id',
  `rel_sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '引用的SOP',
  `pre_execute` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '执行前验证  默认为空,1-设备运行继续执行,2-设备停用继续执行,3-设备停机继续执行',
  `pre_conform` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '强制确认',
  `notice` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '注意事项',
  `obj_required_control` json DEFAULT NULL COMMENT '对象管控需求',
  `obj_show_item` json DEFAULT NULL COMMENT '对象展示项 working_time-对象运行时间',
  `time_limit` int(10) DEFAULT NULL COMMENT '工作时长(分钟数)',
  `info_points` json DEFAULT NULL COMMENT '标准信息点数组',
  `description` varchar(1024) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '内容描述',
  `customs` json DEFAULT NULL COMMENT '自定义信息点数组',
  `pic` json DEFAULT NULL COMMENT '图片',
  `att` json DEFAULT NULL COMMENT '附件',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  `content` varchar(255) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '步骤内容中的content,兼容老数据产生',
  `is_photo` int(1) DEFAULT NULL COMMENT '在到达每个对象位置时拍照',
  `edit_content` int(1) DEFAULT NULL COMMENT '前端传递的参数',
  `flag` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '前端传参,后端无用.兼容老数据产生',
  `temp_sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '临时的SOP的id,前端传的参,后端无用,兼容老数据产生',
  `is_scan` int(1) DEFAULT NULL COMMENT '在到达每个对象位置时扫码',
  `is_nfc` int(1) DEFAULT NULL COMMENT '在到达每个指定对象时进行NFC识别',
  `working_time` int(1) DEFAULT NULL COMMENT '对象运行时间',
  `pre_conform_focus` int(1) DEFAULT NULL COMMENT '前端传参,兼容老数据产生',
  `notice_focus` int(1) DEFAULT NULL COMMENT '前端传参,兼容老数据产生',
  `content_focus` int(1) DEFAULT NULL COMMENT '前端传参,兼容老数据产生',
  `verified` varchar(255) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '前端传参,兼容老数据产生',
  PRIMARY KEY (`sop_cell_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_destroy_msg
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_destroy_msg`  (
  `msg_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `project_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '项目id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的主键id',
  `ref_sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '被引用的sop主键id',
  `ref_sop_name` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '被引用的sop名称',
  `ref_sop_status` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '被引用的sop状态描述',
  `status` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '状态,1-未处理、2-已处理',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`msg_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_domain_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_domain_rel`  (
  `sop_domain_rel_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `parent_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '父级id,SOP的id或者sop_cell_id',
  `domain` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '专业code',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_domain_rel_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_edit
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_edit`  (
  `sop_edit_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的主键id',
  `project_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '项目id,集团SOP的值为group_sop_id',
  `extend_field` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '建筑功能类型',
  `operator` json DEFAULT NULL COMMENT '操作人信息',
  `sop_name` varchar(100) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的名称,目前长度定为100.交互支持50个字符',
  `sop_type` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP类型,1-通用,2-项目专用',
  `no_obj` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '是否有无对象的步骤,0-否 1-是.当为引用的SOP时,就算被引用的没有对象也为1',
  `domains` json DEFAULT NULL COMMENT '专业编码,[code,code]',
  `tools` json DEFAULT NULL COMMENT '工具数组,[{"tool"：“……”,"from_step":true}]',
  `order_type` json DEFAULT NULL COMMENT '关联的工单类型,数组[code,code]',
  `fit_objs` json DEFAULT NULL COMMENT '适用对象',
  `equip_model_ids` json DEFAULT NULL COMMENT '设备型号',
  `labels` json DEFAULT NULL COMMENT '自定义标签',
  `steps` json DEFAULT NULL COMMENT '步骤',
  `related_data` json DEFAULT NULL COMMENT '相关资料,[{"name":"***",“url”：“***”},{}]',
  `sop_status` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '1-已发布、2-已作废',
  `publish_status` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '1-正常、2-待修订、3-修订中',
  `ref_sop_ids` json DEFAULT NULL COMMENT '引用的sop',
  `version` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '版本号',
  `version_count` int(10) DEFAULT NULL COMMENT '版本数量',
  `version_explain` json DEFAULT NULL COMMENT '版本说明,[“描述”,“描述”]',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_edit_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_equip_model_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_equip_model_rel`  (
  `sop_equip_model_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的主键id',
  `equip_model_id` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '设备型号id',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_equip_model_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_his
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_his`  (
  `sop_his_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的主键id',
  `project_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '项目id,集团SOP的值为group_sop_id',
  `extend_field` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '建筑功能类型',
  `operator` json DEFAULT NULL COMMENT '操作人信息',
  `sop_name` varchar(100) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的名称,目前长度定为100.交互支持50个字符',
  `sop_type` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP类型,1-通用,2-项目专用',
  `no_obj` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '是否有无对象的步骤,0-否 1-是.当为引用的SOP时,就算被引用的没有对象也为1',
  `domains` json DEFAULT NULL COMMENT '专业编码,[code,code]',
  `tools` json DEFAULT NULL COMMENT '工具数组,[{"tool"：“……”,"from_step":true}]',
  `order_type` json DEFAULT NULL COMMENT '关联的工单类型,数组[code,code]',
  `fit_objs` json DEFAULT NULL COMMENT '适用对象',
  `equip_model_ids` json DEFAULT NULL COMMENT '设备型号',
  `labels` json DEFAULT NULL COMMENT '自定义标签',
  `steps` json DEFAULT NULL COMMENT '步骤',
  `related_data` json DEFAULT NULL COMMENT '相关资料,[{"name":"***",“url”：“***”},{}]',
  `sop_status` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '1-已发布、2-已作废',
  `publish_status` varchar(2) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '1-正常、2-待修订、3-修订中',
  `ref_sop_ids` json DEFAULT NULL COMMENT '引用的sop',
  `refed_sops` json DEFAULT NULL COMMENT '引用的sop数组',
  `version` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '版本号',
  `version_count` int(10) DEFAULT NULL COMMENT '版本数量',
  `version_explain` json DEFAULT NULL COMMENT '版本说明,[“描述”,“描述”]',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_his_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_label_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_label_rel`  (
  `sop_label_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'sop的主键id',
  `label` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '标签名称',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_label_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_media_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_media_rel`  (
  `sop_media_rel_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `parent_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '父级id,SOP的id或者sop_cell_id',
  `key` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '存储到图片服务器的key',
  `name` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '附件名称,图片名称等',
  `type` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'pic-图片,att-附件',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_media_rel_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_obj_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_obj_rel`  (
  `sop_obj_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `parent_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '父级id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的唯一标识',
  `obj_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '对象id',
  `obj_type` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '对象类型',
  `obj_name` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '对象名称',
  `obj_code` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '对象code',
  `type_code` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '对象所属类ID（用于空间系统设备对象，前端筛选同一类型对象使用）',
  `is_revise` int(1) DEFAULT NULL COMMENT 'true-需修订,false-不需修订',
  `parents` json DEFAULT NULL COMMENT '父类对象',
  `obj_source` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'content_obj-步骤中操作对象',
  `obj_owner` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'content_obj-步骤中的操作对象,space_scope-空间范围,fit_obj-适用范围',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_obj_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_order_type_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_order_type_rel`  (
  `sop_order_type_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的主键id',
  `project_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '项目id',
  `order_type` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '工单类型id',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_order_type_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_revise_msg
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_revise_msg`  (
  `msg_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `sop_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT 'SOP的主键id',
  `obj_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '作废对象id',
  `obj_name` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '作废对象名称',
  `obj_type` varchar(20) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '作废对象类型,1-对象模型、2-对象实例、3-信息点',
  `content` varchar(500) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '消息内容',
  `status` varchar(10) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '状态,1-未处理、2-已忽略、3-已自动处理',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`msg_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

-- ----------------------------
-- Table structure for sop_tool_rel
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sop_tool_rel`  (
  `sop_tool_rel_id` varchar(50) COLLATE utf8mb4_bin NOT NULL COMMENT '主键id',
  `parent_id` varchar(50) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '父级id,SOP的id或者sop_cell_id',
  `tool` varchar(200) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '工具名称',
  `from_step` int(1) DEFAULT NULL COMMENT '1-是步骤中的工具,0或空-不是步骤中的工具',
  `create_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '创建时间,格式为yyyyMMddHHmmss',
  `update_time` varchar(14) COLLATE utf8mb4_bin DEFAULT NULL COMMENT '更新时间,格式为yyyyMMddHHmmss',
  `valid` int(1) DEFAULT '1' COMMENT '是否有效',
  PRIMARY KEY (`sop_tool_rel_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin ROW_FORMAT=DYNAMIC;

SET FOREIGN_KEY_CHECKS = 1;
