package com.persagy.fm.workorder.cache;

import com.alibaba.fastjson.JSONObject;
import com.persagy.fm.workorder.constant.RedisConstant;
import com.persagy.fm.workorder.domain.entity.WoFlowPlan;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.log4j.Logger;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component
@Slf4j
@RequiredArgsConstructor
public class OrderTypeCache {
    private final RedisTemplate<String,String> redisTemplate;

    public  String getNameByTypeCode(String code){
        String orderTypeStr = (String) redisTemplate.opsForHash().get(RedisConstant.ALL_WO_FLOW_PLAN_PERSAGY_KEY, code);

        if (orderTypeStr!=null){
            JSONObject orderType = JSONObject.parseObject(orderTypeStr);
            return orderType.getString("plan_name");
        }
        log.error("OrderTypeCache:getNameByTypeCode" + "未获取到缓存:" + code);
        return "";
    }

    public  String getWorkTypeByCode(String code){
        String orderTypeStr = (String) redisTemplate.opsForHash().get(RedisConstant.ALL_WO_FLOW_PLAN_PERSAGY_KEY, code);

        if (orderTypeStr!=null){
            JSONObject orderType = JSONObject.parseObject(orderTypeStr);
            return orderType.getString("work_type");
        }
        log.error("OrderTypeCache:getWorkTypeByCode" + "未获取到缓存:" + code);
        return "";
    }
    /***
     * Description: 更新缓存
     * @param woFlowPlan : 工单类型
     * @return : boolean  
     * @author : lijie
     * @date :2021/5/8 14:58
     * Update By lijie 2021/5/8 14:58
     */
    public boolean updateCache(WoFlowPlan woFlowPlan){
        boolean resultFlag = true;
        String orderTypeId = woFlowPlan.getPlanId();
        String planName = woFlowPlan.getPlanName();
        String workType = woFlowPlan.getWorkType();
        String excuteType = woFlowPlan.getExecuteType();
        String functionType = woFlowPlan.getFunctionType();
        JSONObject orderTypeObj = new JSONObject();
        orderTypeObj.put("plan_name", planName);
        orderTypeObj.put("work_type", workType);
        orderTypeObj.put("execute_type", excuteType);
        orderTypeObj.put("function_type", functionType);
        try {
            redisTemplate.opsForHash().put(RedisConstant.ALL_WO_FLOW_PLAN_PERSAGY_KEY, orderTypeId, orderTypeObj.toJSONString());
        }catch (Exception e){
            resultFlag =  false;
            log.error("更新工单类型缓存失败:"+e);
        }
        return resultFlag;
    }
}
