/**
 * @包名称 com.sagacloud.resource
 * @文件名 RestFlowPlanController.java
 * @创建者 wanghailong
 * @邮箱 wanghailong@persagy.com  
 * @修改描述 
 */

package com.persagy.fm.workorder.controller;


import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.exception.BusinessException;
import com.persagy.fm.common.old.utils.StringUtil;
import com.persagy.fm.common.old.utils.ToolsUtil;
import com.persagy.fm.common.validator.Insert;
import com.persagy.fm.common.validator.Update;
import com.persagy.fm.workorder.domain.vo.WoFlowPlanVo;
import com.persagy.fm.workorder.domain.vo.BaseQueryFlowPlanVo;
import com.persagy.fm.workorder.domain.vo.QueryFlowPlanListVo;
import com.persagy.fm.workorder.domain.vo.QueryVerifyFlowPlanVo;
import com.persagy.fm.workorder.service.IWoFlowPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/***
 * Description: 工单流转方案接口Controller层
 * @author : lijie
 * @date :2021/4/26 17:11
 * Update By lijie 2021/4/26 17:11
 */
@RestController("/restFlowPlanService/")
@RequestMapping(produces="application/json;charset=UTF-8",value = "/restFlowPlanService")
public class WoFlowPlanController{
	@Autowired
	private IWoFlowPlanService flowPlanService;
	/***
	 * Description: 查询所有的工单类型
	 * @param queryFlowPlanListVo : 请求参数
	 * @return : java.lang.String
	 * @author : lijie
	 * @date :2021/4/27 17:54
	 * Update By lijie 2021/4/27 17:54
	 */
	@PostMapping("queryFlowPlan")
	public String queryFlowPlan(@RequestBody @Validated QueryFlowPlanListVo queryFlowPlanListVo) throws Exception {
	    return flowPlanService.queryFlowPlan(queryFlowPlanListVo);
	}
	
	/***
	 * Description:验证工单类型岗位职责信息
	 * @param queryVerifyFlowPlanVo : 请求参数
	 * @return : java.lang.String
	 * @author : lijie
	 * @date :2021/4/27 17:55
	 * Update By lijie 2021/4/27 17:55
	 */
	@PostMapping("/verifyFlowPlan")
	public String verifyFlowPlan(@RequestBody @Validated QueryVerifyFlowPlanVo queryVerifyFlowPlanVo) throws Exception {
		return flowPlanService.verifyFlowPlan(queryVerifyFlowPlanVo);
	}

	/***
	 * Description: 查询部门岗位树
	 * @param jsonObject : 请求参数
	 * @return : java.lang.String
	 * @author : lijie
	 * @date :2021/5/8 10:23
	 * Update By lijie 2021/5/8 10:23
	 */
	@PostMapping("/queryDeptPositionTree")
	public String queryDeptPositionTree(@RequestBody JSONObject jsonObject) throws Exception{
		if( ! StringUtil.isNull(jsonObject, "user_id")){
			return flowPlanService.queryDeptPositionTree(jsonObject.toJSONString());
		}
		return ToolsUtil.return_error_json;
	}

	/***
	 * Description: 根据Id删除工单类型
	 * @param jsonObject : 请求参数
	 * @return : java.lang.String  
	 * @author : lijie
	 * @date :2021/5/8 10:23
	 * Update By lijie 2021/5/8 10:23
	 */
	@PostMapping("/deleteFlowPlanById")
	public String deleteFlowPlanById(@RequestBody JSONObject jsonObject) throws Exception {
		if(!StringUtil.isNull(jsonObject, "user_id", "plan_id")){
			return flowPlanService.deleteFlowPlanById(jsonObject.toJSONString());
		}else{
			return ToolsUtil.return_error_json;
		}
	}

	/***
	 * Description: 查询项目下所有流转方案
	 * @param queryFlowPlanListVo : 请求参数
	 * @return : java.lang.String
	 * @author : lijie
	 * @date :2021/5/8 10:31
	 * Update By lijie 2021/5/8 10:31
	 */
	@PostMapping("/queryProjectFlowPlan")
	public String queryProjectFlowPlan(@RequestBody @Validated QueryFlowPlanListVo queryFlowPlanListVo) throws Exception {
		if (StrUtil.isBlank(queryFlowPlanListVo.getProjectId())){
			return ToolsUtil.return_error_json;
		}
		return flowPlanService.queryProjectFlowPlan(queryFlowPlanListVo);
	}

	/***
	 * Description: 添加工单流转方案信息
	 * @param woFlowPlanVo : 请求参数
	 * @return : java.lang.String
	 * @author : lijie
	 * @date :2021/5/8 11:21
	 * Update By lijie 2021/5/8 11:21
	 */
	@PostMapping("/addFlowPlan")
	@Transactional(rollbackFor = BusinessException.class)
	public String addFlowPlan(@RequestBody @Validated(Insert.class) WoFlowPlanVo woFlowPlanVo) throws Exception {
		return flowPlanService.addFlowPlan(woFlowPlanVo);
	}

	/***
	 * Description: 根据Id查询工单详细信息
	 * @param baseQueryFlowPlanVo : 请求参数
	 * @return : java.lang.String  
	 * @author : lijie
	 * @date :2021/5/8 15:01
	 * Update By lijie 2021/5/8 15:01
	 */
	@PostMapping("/queryFlowPlanById")
	public String queryFlowPlanById(@RequestBody @Validated BaseQueryFlowPlanVo baseQueryFlowPlanVo) throws Exception {
		if (StrUtil.isBlank(baseQueryFlowPlanVo.getPlanId())){
			return ToolsUtil.return_error_json;
		}
		return flowPlanService.queryFlowPlanById(baseQueryFlowPlanVo);
	}

	/***
	 * Description: 根据Id编辑流转方案信息
	 * @param woFlowPlanVo : 请求参数
	 * @return : java.lang.String  
	 * @author : lijie
	 * @date :2021/5/8 16:19
	 * Update By lijie 2021/5/8 16:19
	 */
	@PostMapping("/updateFlowPlanById")
	public String updateFlowPlanById(@RequestBody @Validated(Update.class) WoFlowPlanVo woFlowPlanVo) throws Exception {
		return flowPlanService.updateFlowPlanById(woFlowPlanVo);
	}

	/***
	 * Description: 根据工作类型查询所有有效工单流转方案信息
	 * @param jsonObject : 请求参数
	 * @return : java.lang.String  
	 * @author : lijie
	 * @date :2021/5/8 17:04
	 * Update By lijie 2021/5/8 17:04
	 */
	@PostMapping("/queryFlowPlanByWorkType")
	public String queryFlowPlanByWorkType(@RequestBody JSONObject jsonObject) throws Exception{
		if(!StringUtil.isNull(jsonObject, "user_id", "work_type")){
			return flowPlanService.queryFlowPlanByWorkType(jsonObject.getString("work_type"));
		} else {
			return ToolsUtil.return_error_json;
		}
	}
	/***
	 * Description: 根据时间类型查询所有有效流转方案信息
	 * @param jsonObject : 请求参数
	 * @return : java.lang.String  
	 * @author : lijie
	 * @date :2021/5/8 17:08
	 * Update By lijie 2021/5/8 17:08
	 */
	@PostMapping("/queryFlowPlanByExecuteType")
	public String queryFlowPlanByExecuteType(@RequestBody JSONObject jsonObject) throws Exception{
		if( ! StringUtil.isNull(jsonObject, "user_id", "execute_type")){
			return flowPlanService.queryFlowPlanByExecuteType(jsonObject.getString("execute_type"));
		} else {
			return ToolsUtil.return_error_json;
		}

	}
	/***
	 * Description: 通过工单类型名称查询所有有效工单
	 * @param jsonObject : 请求参数
	 * @return : java.lang.String  
	 * @author : lijie
	 * @date :2021/5/8 17:12
	 * Update By lijie 2021/5/8 17:12
	 */
	@PostMapping("/queryFlowPlanByPlanName")
	public String queryFlowPlanByPlanName(@RequestBody JSONObject jsonObject) throws Exception{
		if( ! StringUtil.isNull(jsonObject, "user_id", "plan_name")){
			return flowPlanService.queryFlowPlanByPlanName(jsonObject.getString("plan_name"));
		} else {
			return ToolsUtil.return_error_json;
		}
	}
	// TODO 维修项的需要等工单重构后才能做这两个接口
//
//
//
//	/**
//	 * @Author yuecaipu
//	 * @Description 维修管理-维修项内容配置:更新
//	 * @Date 11:08 2019/10/10
//	 * @Param
//	 * @return
//	 **/
//    @PostMapping("/updateMaintenanceConfiguration")
//    public String updateMaintenanceConfiguration(@RequestBody JSONObject jsonObject) throws Exception{
//        if( StringUtil.isNull(jsonObject, "user_id", "maintenance_configuration")){
//            return ToolsUtil.errorJsonMsg("入参不正确");
//        }
//
//        JSONObject maintenance_configuration = jsonObject.getJSONObject("maintenance_configuration");
//
//        boolean b = maintenanceConfigurationCache.updateMaintenanceConfiguration(maintenance_configuration);
//        return ToolsUtil.successJsonMsg(maintenance_configuration);
//
//    }
//
//
//    @Autowired
//    private MaintenanceItemServiceImpl maintenanceItemService;
//    /**
//     * @Author yuecaipu
//     * @Description 维修项内容配置:查询配置
//     * @Date 11:08 2019/10/10
//     * @Param
//     * @return
//     **/
//    @PostMapping("/queryMaintenanceConfiguration")
//    public String queryMaintenanceConfiguration(@RequestBody JSONObject jsonObject) throws Exception{
//        if( StringUtil.isNull(jsonObject, "user_id")){
//            return ToolsUtil.errorJsonMsg("入参不正确");
//        }
//        String order_id = jsonObject.getString("order_id");
//        String version = jsonObject.getString("version");
//        if (StringUtil.isNull(version) && StringUtil.isNotNull(order_id)){
//            //---查询当前工单使用的维修项配置版本
//            JSONArray maintenanceItems = maintenanceItemService.queryRecordByCondition(jsonObject);
//            if (CollectionUtils.isNotEmpty(maintenanceItems)){
//                version = maintenanceItems.getJSONObject(0).getString("maintenance_configuration_version");
//            }
//        }
//
//        jsonObject = maintenanceConfigurationCache.getMaintenanceConfiguration(version);
//        return ToolsUtil.successJsonMsg(jsonObject);
//    }


}
