package com.persagy.fm.workorder.domain.dto;

import com.fasterxml.jackson.annotation.JsonAlias;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.persagy.fm.workorder.constant.enums.WoApplyItemEnum;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class PostAndDutyNative {
    /**单精度窗口组件名称*/
    @ApiModelProperty("单精度窗口组件名称")
    @JsonAlias({"float_window_modular","floatWindowModular"})
    @JsonProperty("floatWindowModular")
    private String floatWindowModular;
    @ApiModelProperty("通知单")
    @JsonAlias({"messageReminder","message_reminder"})
    @JsonProperty("messageReminder")
    private MessageReminder messageReminder;
    @ApiModelProperty("时间限制")
    @JsonAlias({"timeLimit","time_limit"})
    @JsonProperty("timeLimit")
    private TimeLimitBean timeLimit;
    @ApiModelProperty("时间类型编码")
    @JsonAlias({"executeType","execute_type"})
    @JsonProperty("executeType")
    private String executeType;
    @ApiModelProperty("申请信息")
    private Apply apply;
    @ApiModelProperty("工作类型:1-维保,2-维修,3-巡检,4-运行,5-安全")
    @JsonAlias({"workType","work_type"})
    @JsonProperty("workType")
    private String workType;
    @ApiModelProperty("建筑功能类型")
    @JsonAlias({"functionType","function_type"})
    @JsonProperty("functionType")
    private String functionType;
    @ApiModelProperty("报修是否可以转此类工单,0-不可以,1-可以")
    @JsonAlias({"repairFlag","repair_flag"})
    @JsonProperty("repairFlag")
    private String repairFlag;
    @ApiModelProperty("执行信息")
    private BaseMode execute;
    @ApiModelProperty("审批信息")
    private List<Audit> audit;
    @ApiModelProperty("审核信息")
    private Audit approve;
    @ApiModelProperty("新建信息")
    private Create create;
    @ApiModelProperty("评价信息")
    private Audit evaluate;
    @ApiModelProperty("指派信息")
    private Assign assign;




    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class MessageReminder {
        @ApiModelProperty("下级路由节点通知")
        @JsonAlias({"nextRouteNobodyNotify","next_route_nobody_notify"})
        @JsonProperty("nextRouteNobodyNotify")
        private NextRouteNobodyNotify nextRouteNobodyNotify;
        @ApiModelProperty("工单时间通知")
        @JsonAlias({"woOrderTimeNotify","wo_order_time_notify"})
        @JsonProperty("woOrderTimeNotify")
        private List<WoOrderTimeNotify> woOrderTimeNotify;
        @ApiModelProperty("消息通知索引")
        @JsonAlias({"messageNotificationIndex","message_notification_index"})
        @JsonProperty("messageNotificationIndex")
        private Integer messageNotificationIndex;
    }
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class Apply  extends BaseMode{
        /**
         * 申请类型
         * {@link WoApplyItemEnum}
         * */
        @ApiModelProperty(value = "申请类型")
        private String code;
        @ApiModelProperty("申请类型名称")
        private String name;
        @ApiModelProperty("审批步骤")
        private List<Audit> audit;
        @ApiModelProperty("审核步骤")
        private Audit approve;
    }
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class BaseMode {
        @ApiModelProperty("检测")
        private Boolean checked;
        @ApiModelProperty("非位置")
        @JsonProperty("noPosition")
        @JsonAlias({"noPosition","no_position"})
        private Boolean noPosition;
        @ApiModelProperty("岗位信息")
        private List<Position> positions;
    }
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class Audit extends BaseMode{
        @ApiModelProperty(value = "审批类型")
        private String code;
        @ApiModelProperty(value = "审批类型")
        private String code_;
        @ApiModelProperty("是否展示")
        @JsonProperty("isShow")
        @JsonAlias({"isShow","is_show"})
        private Boolean isShow;
    }
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class Create extends BaseMode{
        @ApiModelProperty(value = "是否需要按照排班表过滤每次可转交的人员范围")
        @JsonAlias({"filter_scheduling","filterScheduling"})
        @JsonProperty("filter_scheduling")
        private Boolean filterScheduling;
        @ApiModelProperty(value = "到达工单要求开始时间时才允许执行")
        @JsonAlias({"arrival_time_allow_execute","arrivalTimeAllowExecute"})
        @JsonProperty("arrival_time_allow_execute")
        private Boolean arrivalTimeAllowExecute;
        @ApiModelProperty(value = "请选择是否启用专业控制")
        @JsonAlias({"limit_domain","limitDomain"})
        @JsonProperty("limit_domain")
        private Boolean limitDomain;
        @ApiModelProperty(value = "下级路由方式：execute-新建后执行，assign-新建后指派，robbing-新建后抢单")
        @JsonAlias({"executie_mode","executieMode"})
        @JsonProperty("executie_mode")
        private String executieMode;
    }
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class Assign extends Create{
        @ApiModelProperty(value = "期望该操作持续时间不超过")
        @JsonAlias({"expecte_operation","expecteOperation"})
        @JsonProperty("expecte_operation")
        private Boolean expecteOperation;
        @ApiModelProperty(value = "到达工单要求开始时间时才允许执行")
        private Integer minute;
        @ApiModelProperty(value = "可操作与否")
        private Boolean disabled;
    }



}
