package com.persagy.fm.workorder.domain.dto;

import com.fasterxml.jackson.annotation.JsonAlias;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class TimeLimitBean {
    @ApiModelProperty("是否需要执行人填写收费明细（如人工费、材料费等）,0-否,1-是")
    @JsonAlias({"importOutlayFlag","import_outlay_flag"})
    @JsonProperty("import_outlay_flag")
    private String importOutlayFlag;
    @ApiModelProperty("维修项 0-不展示 1-展示 2-展示且维修反馈必须执行后才可提交工单")
    @JsonAlias({"maintenanceConfigurationFlag","maintenance_configuration_flag"})
    @JsonProperty("maintenance_configuration_flag")
    private String maintenanceConfigurationFlag;
    @ApiModelProperty("否需要客户签字，0-否，1-是")
    @JsonAlias({"signFlag","sign_flag"})
    @JsonProperty("sign_flag")
    private String signFlag;
    @ApiModelProperty("审核结束后是否需要评价,0或空-否，1-是")
    @JsonAlias({"auditorEvaluationFlag","auditor_evaluation_flag"})
    @JsonProperty("auditor_evaluation_flag")
    private String auditorEvaluationFlag;
    @ApiModelProperty("自动数据获取,0或空-否，1-是")
    @JsonAlias({"automaticDataCaptureFlag","automatic_data_capture_flag"})
    @JsonProperty("automatic_data_capture_flag")
    private String automaticDataCaptureFlag;
    @ApiModelProperty("对接视频实时照片,0或空-否，1-是")
    @JsonAlias({"videoCaptureFlag","video_capture_flag"})
    @JsonProperty("video_capture_flag")
    private String videoCaptureFlag;
    @ApiModelProperty("超时后无人接单是否关闭")
    @JsonAlias({"timeoutOrderReceiving","timeout_order_receiving"})
    @JsonProperty("timeout_order_receiving")
    private TimeoutOrderReceiving timeoutOrderReceiving;
    @ApiModelProperty("签字说明")
    @JsonAlias({"signDesc","sign_desc"})
    @JsonProperty("sign_desc")
    private String signDesc;
    @ApiModelProperty("开始时间")
    @JsonAlias({"startTime","start_time"})
    @JsonProperty("startTime")
    private StartTime startTime;
    @ApiModelProperty("结束时间")
    @JsonAlias({"endTime","end_time"})
    @JsonProperty("endTime")
    private BaseTime endTime;
    @ApiModelProperty("消息提醒设置（工单配置提取）")
    @JsonAlias({"remindSet","remind_set"})
    @JsonProperty("remindSet")
    private RemindSetBean remindSet;
    @ApiModelProperty("扩展字段集")
    @JsonAlias({"customField","custom_field"})
    @JsonProperty("customField")
    private List<String> customField;
    /***
     * Description: 下级路由节点通知类
     * @author : lijie
     * @date :2021/4/26 19:35
     * Update By lijie 2021/4/26 19:35
     */
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class TimeoutOrderReceiving {
        @ApiModelProperty("9-异常中止，10-撤销")
        @JsonAlias({"closedState","closed_state"})
        @JsonProperty("closed_state")
        private String closedState;
        @ApiModelProperty("是否选中")
        private Boolean selected;
    }

    /***
     * Description: 消息提醒设置（工单配置提取）类
     * @author : lijie
     * @date :2021/4/26 19:35
     * Update By lijie 2021/4/26 19:35
     */
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class RemindSetBean {
        @ApiModelProperty("发单后下级岗位无人员时通知岗位")
        @JsonAlias({"nextRouteNobodyNotify","next_route_nobody_notify"})
        @JsonProperty("nextRouteNobodyNotify")
        private NextRouteNobodyNotify nextRouteNobodyNotify;
        @ApiModelProperty("到达工单开始、结束时间提醒")
        private List<WoOrderTimeNotify> woOrderTimeNotify;
    }

    /***
     * Description: 开始时间类
     * @author : lijie
     * @date :2021/4/26 19:35
     * Update By lijie 2021/4/26 19:35
     */
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class StartTime extends BaseTime{
        @ApiModelProperty("9-异常中止，10-撤销")
        @JsonAlias({"closedState","closed_state"})
        @JsonProperty("closedState")
        private String closedState;
        @ApiModelProperty("超时后是否发起超时事件     false -否，不发起  true-是，发起")
        @JsonAlias({"isGenerate","is_generate"})
        @JsonProperty("isGenerate")
        private Boolean isGenerate;
        @ApiModelProperty("处理部门")
        @JsonAlias({"dealDept","deal_dept"})
        @JsonProperty("dealDept")
        private Department dealDept;
        @ApiModelProperty("紧急程度，高、中、低")
        private String urgency;

        @Data
        @AllArgsConstructor
        @NoArgsConstructor
        public static class Department{
            @ApiModelProperty("部门id")
            @JsonProperty("deptId")
            @JsonAlias({"deptId","dept_id"})
            private String deptId;
            @ApiModelProperty("部门名称")
            @JsonProperty("deptName")
            @JsonAlias({"deptName","dept_name"})
            private String deptName;

        }

    }
    /***
     * Description: 开始时间类
     * @author : lijie
     * @date :2021/4/26 19:35
     * Update By lijie 2021/4/26 19:35
     */
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class BaseTime{
        @ApiModelProperty("是否选中")
        private Boolean selected;
        @ApiModelProperty("after-后  front-前")
        private String around;
        @ApiModelProperty("分钟")
        private Integer minute;
    }

}