package com.persagy.fm.workorder.domain.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;

import java.io.Serializable;
import java.util.List;

import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.persagy.fm.workorder.domain.dto.PostAndDuty;
import com.persagy.fm.workorder.domain.dto.PostAndDutyNative;
import com.persagy.fm.workorder.domain.dto.TimeLimitBean;
import com.persagy.fm.workorder.handler.JsonTypeHandlerFactory;
import lombok.*;
import lombok.experimental.Accessors;

/**
 * @author lijie
 * @since 2021-04-26
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName(value = "wo_flow_plan",autoResultMap = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class WoFlowPlan extends Model<WoFlowPlan> {

    private static final long serialVersionUID = 1L;

    /**
     * 工单类型id
     */
    @TableId(value = "plan_id", type = IdType.ASSIGN_ID)
    private String planId;

    /**
     * 工单类型名称
     */
    @TableField("plan_name")
    private String planName;

    /**
     * 项目id
     */
    @TableField("project_id")
    private String projectId;

    /**
     * 报修是否可以转此类工单,0-不可以,1-可以
     */
    @TableField("repair_flag")
    private String repairFlag;

    /**
     * 工作类型id,1-维保,2-维修,3-巡检,4-运行,5-安全
     */
    @TableField("work_type")
    private String workType;

    /**
     * 建筑功能类型
     */
    @TableField("function_type")
    private String functionType;

    /**
     * 部门id
     */
    @TableField("department_id")
    private String departmentId;

    /**
     * 设置的此类工单相关要求
     */
    @TableField(value = "time_limit",typeHandler = JacksonTypeHandler.class)
    private TimeLimitBean timeLimit;

    /**
     * 时间类型,temp-临时,plan-计划
     */
    @TableField("execute_type")
    private String executeType;

    /**
     * 岗位职责数据
     */
    @TableField(value = "post_and_duty",typeHandler = JsonTypeHandlerFactory.PostAndDutyHandler.class)
    private List<PostAndDuty> postAndDuty;

    /**
     * 原生岗位职责数据
     */
    @TableField(value = "post_and_duty_native",typeHandler = JacksonTypeHandler.class)
    private PostAndDutyNative postAndDutyNative;

    /**
     * 工单类型状态,0-需修订,1-正常
     */
    @TableField("plan_status")
    private String planStatus;

    /**
     * 版本号
     */
    @TableField("version")
    private Integer version;

    /**
     * 创建者id
     */
    @TableField("creator_id")
    private String creatorId;

    /**
     * 创建时间
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private String createTime;

    /**
     * 更新时间
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    private String updateTime;

    /**
     * 是否有效
     */
    @TableField("valid")
    private Boolean valid;


    @Override
    protected Serializable pkVal() {
        return this.planId;
    }

}
