package com.persagy.fm.workorder.domain.vo;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonAlias;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.persagy.fm.common.model.vo.BaseVo;
import com.persagy.fm.common.validator.Insert;
import com.persagy.fm.common.validator.Update;
import com.persagy.fm.workorder.domain.dto.PostAndDuty;
import com.persagy.fm.workorder.domain.dto.PostAndDutyNative;
import com.persagy.fm.workorder.domain.dto.TimeLimitBean;
import com.persagy.fm.workorder.handler.JsonTypeHandlerFactory;
import lombok.*;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

/**
 * @author lijie
 * @since 2021-04-26
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class WoFlowPlanVo extends BaseVo {


    /**
     * 工单类型id
     */
    @JsonAlias({"planId","plan_id"})
    @JsonProperty("plan_id")
    @NotBlank(message = "缺少必填项:plan_id",groups = {Update.class})
    private String planId;

    /**
     * 人员id
     */
    @JsonAlias({"personId","person_id"})
    @JsonProperty("person_id")
    private String personId;

    /**
     * 工单类型名称
     */
    @JsonAlias({"planName","plan_name"})
    @JsonProperty("plan_name")
    @NotBlank(message = "缺少必填项:plan_name",groups = {Insert.class})
    private String planName;

    /**
     * 项目id
     */
    @JsonAlias({"projectId","project_id"})
    @JsonProperty("project_id")
    private String projectId;

    /**
     * 报修是否可以转此类工单,0-不可以,1-可以
     */
    @JsonAlias({"repairFlag","repair_flag"})
    @JsonProperty("repair_flag")
    private String repairFlag;

    /**
     * 工作类型id,1-维保,2-维修,3-巡检,4-运行,5-安全
     */
    @JsonAlias({"workType","work_type"})
    @JsonProperty("work_type")
    @NotBlank(message = "缺少必填项:work_type")
    private String workType;

    /**
     * 建筑功能类型
     */
    @JsonAlias({"functionType","function_type"})
    @JsonProperty("function_type")
    private String functionType;

    /**
     * 设置的此类工单相关要求
     */
    @JsonAlias({"timeLimit","time_limit"})
    @JsonProperty("time_limit")
    private TimeLimitBean timeLimit;

    /**
     * 时间类型,temp-临时,plan-计划
     */
    @JsonAlias({"executeType","execute_type"})
    @JsonProperty("execute_type")
    @NotBlank(message = "缺少必填项:excute_type")
    private String executeType;

    /**
     * 岗位职责数据
     */
    @JsonAlias({"postAndDuty","post_and_duty"})
    @JsonProperty("post_and_duty")
    @NotEmpty(message = "缺少必填项:post_and_duty")
    private List<PostAndDuty> postAndDuty;

    /**
     * 原生岗位职责数据
     */
    @JsonAlias({"postAndDutyNative","post_and_duty_native"})
    @JsonProperty("post_and_duty_native")
    private PostAndDutyNative postAndDutyNative;

    /**
     * 工单类型状态,0-需修订,1-正常
     */
    @JsonAlias({"planStatus","plan_status"})
    @JsonProperty("plan_status")
    private String planStatus;

}
