package com.persagy.fm.workorder.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.persagy.fm.workorder.domain.entity.WoFlowPlan;
import com.persagy.fm.workorder.domain.vo.WoFlowPlanVo;
import com.persagy.fm.workorder.domain.vo.BaseQueryFlowPlanVo;
import com.persagy.fm.workorder.domain.vo.QueryFlowPlanListVo;
import com.persagy.fm.workorder.domain.vo.QueryVerifyFlowPlanVo;

/**
 * <p>
 *  服务类
 * </p>
 *
 * @author lijie
 * @since 2021-04-26
 */
public interface IWoFlowPlanService extends IService<WoFlowPlan> {
    /***
     * Description: 查询所有的工单类型
     * @param queryFlowPlanListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/26 19:13
     * Update By lijie 2021/4/26 19:13
     */
    String queryFlowPlan(QueryFlowPlanListVo queryFlowPlanListVo);
    /***
     * Description: 验证工单类型岗位职责信息
     * @param queryVerifyFlowPlanVo : 请求参数
     * @return : java.lang.String  
     * @author : lijie
     * @date :2021/4/27 17:54
     * Update By lijie 2021/4/27 17:54
     */
    String verifyFlowPlan(QueryVerifyFlowPlanVo queryVerifyFlowPlanVo);
    /***
     * Description: 查询部门岗位树
     * @param jsonStr : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 10:18
     * Update By lijie 2021/5/8 10:18
     */
    String queryDeptPositionTree(String jsonStr);
    /***
     * Description: 根据Id删除工单类型
     * @param jsonStr : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 10:23
     * Update By lijie 2021/5/8 10:23
     */
    String deleteFlowPlanById(String jsonStr);
    /***
     * Description: 查询项目下所有流转方案
     * @param queryFlowPlanListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 10:31
     * Update By lijie 2021/5/8 10:31
     */
    String queryProjectFlowPlan(QueryFlowPlanListVo queryFlowPlanListVo);
    /***
     * Description: 添加工单流转方案信息
     * @param woFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 11:21
     * Update By lijie 2021/5/8 11:21
     */
    String addFlowPlan(WoFlowPlanVo woFlowPlanVo);
    /***
     * Description: 根据Id查询工单详细信息
     * @param baseQueryFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 15:01
     * Update By lijie 2021/5/8 15:01
     */
    String queryFlowPlanById(BaseQueryFlowPlanVo baseQueryFlowPlanVo) throws Exception;
    /***
     * Description: 根据Id编辑流转方案信息
     * @param woFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 16:19
     * Update By lijie 2021/5/8 16:19
     */
    String updateFlowPlanById(WoFlowPlanVo woFlowPlanVo);
    /***
     * Description: 根据工作类型查询所有有效工单流转方案信息
     * @param workType : 工作类型
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:04
     * Update By lijie 2021/5/8 17:04
     */
    String queryFlowPlanByWorkType(String workType);
    /***
     * Description: 根据时间类型查询所有有效流转方案信息
     * @param executeType : 时间类型
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:08
     * Update By lijie 2021/5/8 17:08
     */
    String queryFlowPlanByExecuteType(String executeType);
    /***
     * Description: 通过工单类型名称查询所有有效工单
     * @param planName : 工单类型名称
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:12
     * Update By lijie 2021/5/8 17:12
     */
    String queryFlowPlanByPlanName(String planName);
}
