package com.persagy.fm.workorder.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.fm.common.constant.enums.ResultEnum;
import com.persagy.fm.workorder.service.BaseService;
import com.persagy.fm.workorder.service.IDictionaryService;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Service
public class DictionaryServiceImpl extends BaseService implements IDictionaryService {
	
	private final static  Logger log = Logger.getLogger(DictionaryServiceImpl.class);
	//get：数据平台-数据字典查询
	private final static String DATAPLAT_DICT_QUERY = "BASEPATH/dict/query/PARAM";
	//post：数据平台-抗震设防烈度
	public final static String DATAPLAT_SEISMIC_PRECAUTIONARY_INTENSITY_QUERY = "BASEPATH/dict/query/seismic_precautionary_intensity";
	
	/**
	 * 查询所有设备类型的数据字典
	 * @return
	 * @throws Exception
	 */
	@Override
	public String queryAllEquipmentCategory() throws Exception {
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"equipment_all");
		String resQuery = this.httpPostRequestDataPlatform(requestURL,"{}");
		return resQuery.replace("class", "name").replace("system", "name").replace("facility", "name");
	}
	
	/**
	 * 查询地理信息--省市信息
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllRegionCode() throws Exception {
		Map<String, String> regionMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"geography");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		JSONObject resultJson = JSONObject.parseObject(resultStr);
		if("success".equals(resultJson.getString("Result"))){
			JSONObject contentJson = resultJson.getJSONObject("Content");
			JSONObject provinceItem, cityItem, cityInfo, districtItem;
			JSONArray proviceArray, cityArray, cityInfoArray, districtArray;
			//处理普通省市地区
			proviceArray = contentJson.getJSONArray("normal");
			if(proviceArray != null){
				for(int i=0; i<proviceArray.size(); i++){
					provinceItem = proviceArray.getJSONObject(i);
					//处理市级数据
					cityArray = provinceItem.getJSONArray("cities");
					for(int j=0; j<cityArray.size(); j++){
						cityItem = cityArray.getJSONObject(j);
						cityInfoArray = cityItem.getJSONArray("cityInfo");
						//处理市区数据
						districtArray = cityItem.getJSONArray("district");
						for(int k=0; k<districtArray.size(); k++){
							districtItem = districtArray.getJSONObject(k);
							regionMap.put(districtItem.getString("code"), districtItem.getString("districtName"));
						}
						//获取市级信息
						for(int k=0; k<cityInfoArray.size(); k++){
							cityInfo = cityInfoArray.getJSONObject(k);
							if(!cityInfo.getString("name").contains("市本级")) {
								regionMap.put(cityInfo.getString("code"), cityInfo.getString("name"));
								break;
							}
						}
					}
					//处理省级数据
					regionMap.put(completLengthFromRight(provinceItem.getString("provinceCode"), 6, "0"), provinceItem.getString("provinceName"));
				}
			}
			//处理直辖市
			proviceArray = contentJson.getJSONArray("specialCities");
			if(proviceArray != null){
				for(int i=0; i<proviceArray.size(); i++){
					provinceItem = proviceArray.getJSONObject(i);
					//处理市区信息
					districtArray = provinceItem.getJSONArray("district");
					for(int j=0; j<districtArray.size(); j++){
						districtItem = districtArray.getJSONObject(j);
						regionMap.put(districtItem.getString("code"), districtItem.getString("districtName"));
					}
					//处理市级信息
					cityInfoArray = provinceItem.getJSONArray("cityInfo");
					for(int j=0; j<cityInfoArray.size(); j++){
						cityInfo = cityInfoArray.getJSONObject(j);
						if(!cityInfo.getString("name").contains("市本级")){
							regionMap.put(cityInfo.getString("code"), cityInfo.getString("name"));
							break;
						}
					}
				}
			}
		}
		return regionMap;
	}

	/**
	 * 查询气候区
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllClimateAreaCode() throws Exception {
		Map<String, String> climateMap = new HashMap<String, String>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"climate");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains(ResultEnum.SUCCESS.getType())){
			JSONObject climateJson = JSONObject.parseObject(resultStr);
			JSONArray climates = climateJson.getJSONArray("Content");
			getDictionaryCode(climateMap, climates);
		}
		return climateMap;
	}
	/**
	 * 查询天气状态
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllWeatherStatusCode() throws Exception{
		Map<String, String> weatherMap = new HashMap<>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"weather");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject weather;
			JSONObject weatherJson = JSONObject.parseObject(resultStr);
			JSONArray weathers = weatherJson.getJSONArray("Content");
			for(int i=0; i<weathers.size(); i++){
				weather = weathers.getJSONObject(i);
				weatherMap.put(weather.getString("code"), weather.getString("name"));
			}
		}
		return weatherMap;
	}
	

	/**
	 * 查询空气风力等级
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllWindLevelCode() throws Exception{
		Map<String, String> windMap = new HashMap<String, String>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"wind");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject wind;
			JSONObject windJson = JSONObject.parseObject(resultStr);
			JSONArray winds = windJson.getJSONArray("Content");
			for(int i=0; i<winds.size(); i++){
				wind = winds.getJSONObject(i);
				windMap.put(wind.getString("code"), wind.getString("name"));
			}
		}
		return windMap;
	}
	

	/**
	 * 查询方位信息
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllDirectionCode() throws Exception{
		Map<String, String> directionMap = new HashMap<String, String>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"direction");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				directionMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return directionMap;
	}
	
	
	/**
	 * 查询建筑功能类型
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllBuildingCode() throws Exception{
		Map<String, String> buildingMap = new HashMap<String, String>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"building");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			getDictionaryCode(buildingMap, directions);
		}
		return buildingMap;
	}
	public void getDictionaryCode(Map<String, String> dictionaryMap, JSONArray dictionarys){
		if(dictionarys == null || dictionarys.size() == 0){
			return;
		}
		JSONObject dictionary;
		for(int i=0; i<dictionarys.size(); i++){
			dictionary = dictionarys.getJSONObject(i);
			dictionaryMap.put(dictionary.getString("code"), dictionary.getString("name"));
			getDictionaryCode(dictionaryMap, dictionary.getJSONArray("content"));
		}
	}
	/**
	 * 查询空间功能类型
	 * @return
	 * @throws Exception
	 */
	public Map<String, String> queryAllSpaceCode() throws Exception{
		Map<String, String> spaceMap = new HashMap<String, String>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"space");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject spaceJson = JSONObject.parseObject(resultStr);
			JSONArray spaces = spaceJson.getJSONArray("Content");
			getDictionaryCode(spaceMap, spaces);
		}
		return spaceMap;
	}
	
	/**
	 * 查询租赁业态
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllRentalCode() throws Exception{
		Map<String, String> rentalMap = new HashMap<>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"rental");
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject rentalJson = JSONObject.parseObject(resultStr);
			JSONArray rentals = rentalJson.getJSONArray("Content");
			getDictionaryCode(rentalMap, rentals);
		}
		return rentalMap;
	}
	
	/**
	 * 查询抗震设防烈度
	 * @return
	 * @throws Exception
	 */
	@Override
	public Map<String, String> queryAllSeismicPrecautionaryIntensityCode() throws Exception{
		Map<String, String> rentalMap = new HashMap<>();
		String requestURL = this.getDataPlatDictPath(DATAPLAT_SEISMIC_PRECAUTIONARY_INTENSITY_QUERY ,null);
		String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				rentalMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return rentalMap;
	}
	
	  /**
     * 查询防尘等级
     * @return
     * @throws Exception
     */
	@Override
    public Map<String, String> queryAllDustproofCode() throws Exception {
		Map<String, String> dustproofMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"dustproof");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				dustproofMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return dustproofMap;
    }
	
	 /**
     * 查询防水等级
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllWaterproofCode() throws Exception {
    	Map<String, String> waterproofMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"waterproof");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				waterproofMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return waterproofMap;
    }
	
    /**
     * 查询防爆型式
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllExplosionproofEquipmentCode() throws Exception {
    	Map<String, String> proofEquipmentMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"explosionproof/equipment");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject rentalJson = JSONObject.parseObject(resultStr);
			JSONArray rentals = rentalJson.getJSONArray("Content");
			getDictionaryCode(proofEquipmentMap, rentals);
		}
		return proofEquipmentMap;
    }
    
    /**
     * 查询防爆设备类型
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllDangerCode() throws Exception {
    	Map<String, String> dangerTypeMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"danger");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				dangerTypeMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return dangerTypeMap;
    }
    /**
     * 查询危险区域等级
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllExplosionproofGasTemperatureCode() throws Exception {
    	Map<String, String> gasTemperatureMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"explosionproof/gas_temperature");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				gasTemperatureMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return gasTemperatureMap;
    }
    
    /**
     * 查询气体类型或温度类型
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllExplosionproofTypeCode() throws Exception {
    	Map<String, String> proofTypeMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"explosionproof/type");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject rentalJson = JSONObject.parseObject(resultStr);
			JSONArray rentals = rentalJson.getJSONArray("Content");
			getDictionaryCode(proofTypeMap, rentals);
		}
		return proofTypeMap;
    }
    
    
    /**
     * 查询污染等级
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllPollutionLevelCode() throws Exception {
    	Map<String, String> pollutionLevelMap = new HashMap<>();
        String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"pollution_level");
        String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
		if(resultStr.contains("success")){
			JSONObject direction;
			JSONObject directionJson = JSONObject.parseObject(resultStr);
			JSONArray directions = directionJson.getJSONArray("Content");
			for(int i=0; i<directions.size(); i++){
				direction = directions.getJSONObject(i);
				pollutionLevelMap.put(direction.getString("code"), direction.getString("name"));
			}
		}
		return pollutionLevelMap;
    }
    
    /**
     * 查询污秽等级
     * @return
     * @throws Exception
     */
    public Map<String, String> queryAllContaminationLevelCode() throws Exception {
    	Map<String, String> contaminationLevelMap = new HashMap<>();
    	String requestURL = this.getDataPlatDictPath(DATAPLAT_DICT_QUERY,"contamination_level");
    	String resultStr = this.httpPostRequestDataPlatform(requestURL,"{}");
    	if(resultStr.contains("success")){
    		JSONObject direction;
    		JSONObject directionJson = JSONObject.parseObject(resultStr);
    		JSONArray directions = directionJson.getJSONArray("Content");
    		for(int i=0; i<directions.size(); i++){
    			direction = directions.getJSONObject(i);
    			contaminationLevelMap.put(direction.getString("code"), direction.getString("name"));
    		}
    	}
    	return contaminationLevelMap;
    }
    
	/**
	 * 
	 * 功能描述：查询数据字典数据
	 * @param name
	 * @return 
	 * @创建者 wanghailong
	 * @邮箱 wanghailong@persagy.com 
	 * 修改描述
	 */
    @Override
	public Map<String, String> queryDictionaryDataByName(String name){
		Map<String, String> map = null;
		if(null == name || "".equals(name)) return map;
		try {
			if(name.contains("省市区代码")){
				map = queryAllRegionCode();
			}else if(name.contains("气候区代码")){
				map = queryAllClimateAreaCode();
			}else if(name.contains("天气状态")){
				map = queryAllWeatherStatusCode();
			}else if(name.contains("空气风力等级")){
				map = queryAllWindLevelCode();
			}else if(name.contains("方向代码")){
				map = queryAllDirectionCode();
			}else if(name.contains("建筑功能")){
				map = queryAllBuildingCode();
			}else if(name.contains("空间功能")){
				map = queryAllSpaceCode();
			}else if(name.contains("租赁业态")){
				map = queryAllRentalCode();
			}else if(name.contains("抗震设防烈度")){
				map = queryAllSeismicPrecautionaryIntensityCode();
			}else if(name.contains("防尘等级")){
				map = queryAllDustproofCode();
			}else if(name.contains("防水等级")){
				map = queryAllWaterproofCode();
			}else if(name.contains("防爆设备类型")){
				map = queryAllDangerCode();
			}else if(name.contains("防爆型式")){
				map = queryAllExplosionproofEquipmentCode();
			}else if(name.contains("危险区域等级")){
				map = queryAllExplosionproofGasTemperatureCode();
			}else if(name.contains("气体类型") || name.contains("温度类型") ){
				map = queryAllExplosionproofTypeCode();
			}else if(name.contains("防爆等级")){
//				map = queryAllPollutionLevelCode();
			}else if(name.contains("污秽等级")){
				map = queryAllContaminationLevelCode();
			}else if(name.contains("污染等级")){
				map = queryAllPollutionLevelCode();
			}else {
				log.warn("未查到数据字典["+name+"]的信息!");
			}
		} catch (Exception e) {
			log.warn("未查到数据字典["+name+"]的信息!");
		}
	
		return map;
	}
    
    /**
	 * 
	 * 功能描述：查询数据字典数据
	 * @param name
	 * @return 
     * @throws Exception 
     * @创建者 wanghailong
	 * @邮箱 wanghailong@persagy.com 
	 * 修改描述
	 */
    @Override
	public JSONArray queryDictionaryDataForJSONArrayByName(String name) throws Exception{
		JSONArray array = null;
    	Map<String, String> map = null;
		if(null == name || "".equals(name)) return array;
		if(name.contains("省市区代码")){
			map = queryAllRegionCode();
		}else if(name.contains("气候区代码")){
			map = queryAllClimateAreaCode();
		}else if(name.contains("天气状态")){
			map = queryAllWeatherStatusCode();
		}else if(name.contains("空气风力等级")){
			map = queryAllWindLevelCode();
		}else if(name.contains("方向代码")){
			map = queryAllDirectionCode();
		}else if(name.contains("建筑功能类型")){
			map = queryAllBuildingCode();
		}else if(name.contains("空间功能类型")){
			map = queryAllSpaceCode();
		}else if(name.contains("租赁业态类型")){
			map = queryAllRentalCode();
		}else if(name.contains("防尘等级")){
			map = queryAllDustproofCode();
		}else if(name.contains("防水等级")){
			map = queryAllWaterproofCode();
		}else if(name.contains("防爆设备类型")){
			map = queryAllDangerCode();
		}else if(name.contains("防爆型式")){
			map = queryAllExplosionproofEquipmentCode();
		}else if(name.contains("危险区域等级")){
			map = queryAllExplosionproofGasTemperatureCode();
		}else if(name.contains("气体类型") || name.contains("温度类型") ){
			map = queryAllExplosionproofTypeCode();
		}else if(name.contains("防爆等级")){
//				map = queryAllPollutionLevelCode();
		}else if(name.contains("污秽等级")){
			map = queryAllContaminationLevelCode();
		}else if(name.contains("污染等级")){
			map = queryAllPollutionLevelCode();
		}else {
			log.warn("未查到数据字典["+name+"]的信息!");
		}
		if(map != null){
			array = new JSONArray();
			JSONObject item;
			for(String key : map.keySet()){
				item = new JSONObject();
				item.put("code", key);
				item.put("name", map.get(key));
				array.add(item);
			}
		}
		return array;
	}
	
	
	
	

}
