package com.persagy.fm.workorder.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.common.exception.BusinessException;
import com.persagy.fm.common.constant.CommonConstant;
import com.persagy.fm.common.constant.enums.*;
import com.persagy.fm.common.model.vo.GeneralDictVo;
import com.persagy.fm.common.old.utils.StringUtil;
import com.persagy.fm.common.old.utils.ToolsUtil;
import com.persagy.fm.workorder.cache.OrderTypeCache;
import com.persagy.fm.workorder.client.SaasClient;
import com.persagy.fm.workorder.constant.FlowPlanConstant;
import com.persagy.fm.workorder.constant.enums.WoApplyItemEnum;
import com.persagy.fm.workorder.constant.enums.WoControlModelEnum;
import com.persagy.fm.workorder.constant.enums.WoFlowPlanStatusEnum;
import com.persagy.fm.workorder.dao.WoFlowPlanMapper;
import com.persagy.fm.workorder.domain.dto.*;
import com.persagy.fm.workorder.domain.entity.WoFlowPlan;
import com.persagy.fm.workorder.domain.vo.WoFlowPlanVo;
import com.persagy.fm.workorder.domain.vo.BaseQueryFlowPlanVo;
import com.persagy.fm.workorder.domain.vo.QueryFlowPlanListVo;
import com.persagy.fm.workorder.domain.vo.QueryVerifyFlowPlanVo;
import com.persagy.fm.workorder.service.IDictionaryService;
import com.persagy.fm.workorder.service.IGeneralDictService;
import com.persagy.fm.workorder.service.IWoFlowPlanService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.xml.transform.Result;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author lijie
 * @since 2021-04-26
 */
@Service
@RequiredArgsConstructor
public class WoFlowPlanServiceImpl extends ServiceImpl<WoFlowPlanMapper, WoFlowPlan> implements IWoFlowPlanService {

    private final IGeneralDictService generalDictService;
    private final SaasClient saasClient;
    private final OrderTypeCache orderTypeCache;
    private final IDictionaryService dictionaryService;


    private static final String promptMessageOne = "一套分配方案至少有一个岗位的职责要包含“新建工单”";
    private static final String promptMessageTwo = "一套分配方案至少有一个岗位的职责要包含“执行”";
    private static final String promptMessageThree = "一套分配方案至少有一个岗位的职责要包含“申请结束”";
    private static final String promptMessageFour = "配置一个岗位的职责中有“申请结束”时，必须要有“执行”";
    private static final String promptMessageFive = "“新建”职责设置新建后指派时，必须要有“指派”";
    private static final String promptMessageSix = "“申请”、“审核”职责必须选择下级路由岗位";

    /***
     * Description: 查询所有的工单类型
     * @param queryFlowPlanListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/26 19:13
     * Update By lijie 2021/4/26 19:13
     */
    @Override
    public String queryFlowPlan(QueryFlowPlanListVo queryFlowPlanListVo) {
        //依据条件进行筛选
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getValid,true);
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getPlanName())){
            queryWrapper.like(WoFlowPlan::getPlanName, queryFlowPlanListVo.getPlanName());
        }
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getWorkType())){
            queryWrapper.eq(WoFlowPlan::getWorkType, queryFlowPlanListVo.getWorkType());
        }
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getRepairFlag())){
            queryWrapper.eq(WoFlowPlan::getRepairFlag,queryFlowPlanListVo.getRepairFlag());
        }
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getExecuteType())){
            queryWrapper.eq(WoFlowPlan::getExecuteType,queryFlowPlanListVo.getExecuteType());
        }
        if (StrUtil.isBlank(queryFlowPlanListVo.getProjectId())){
            queryWrapper.eq(WoFlowPlan::getProjectId, FlowPlanConstant.GROUP_ID);
        }
        List<WoFlowPlan> woFlowPlans = this.list(queryWrapper);
        if (CollUtil.isEmpty(woFlowPlans)){
            return ToolsUtil.successJsonMsg(new ArrayList<>());
        }
        return ToolsUtil.successJsonMsg(fixQueryResult(woFlowPlans));
    }
    /***
     * Description: 验证工单类型岗位职责信息
     * @param queryVerifyFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/27 17:54
     * Update By lijie 2021/4/27 17:54
     */
    @Override
    public String verifyFlowPlan(QueryVerifyFlowPlanVo queryVerifyFlowPlanVo) {
        List<PostAndDuty> postAndDutys = queryVerifyFlowPlanVo.getPostAndDuty();
        JSONArray promptMessages = new JSONArray();
        Map<String, JSONArray> map = new HashMap<String, JSONArray>();
        map.put("promptMessageOne", JSONArray.parseArray("[\""+promptMessageOne+"\"]"));
        map.put("promptMessageTwo", JSONArray.parseArray("[\""+promptMessageTwo+"\"]"));
        map.put("promptMessageThree", JSONArray.parseArray("[\""+promptMessageThree+"\"]"));
        if (CollUtil.isNotEmpty(postAndDutys)){
            JSONObject applyItem;
            String controlCode,applyCode ;
            boolean applyCloseFlag, executeFlag, assignFlag = false, createEndAssignFlag = false, nextRouteFlag = false;
            for (PostAndDuty postAndDuty : postAndDutys) {
                applyCloseFlag = false;
                executeFlag = false;
                List<DutyBean> dutyBeans = postAndDuty.getDuty();
                if (CollUtil.isEmpty(dutyBeans)){
                    continue;
                }
                for (DutyBean dutyBean : dutyBeans) {
                    controlCode = dutyBean.getCode();
                    if (WoControlModelEnum.CREATE.getType().equals(controlCode)) {
                        //-------一个岗位的职责要包含“新建工单”-----
                        map.remove("promptMessageOne");
                        //-------判断有无新建后指派功能----------
                        if (WoControlModelEnum.ASSIGN.getType().equals(dutyBean.getExecutieMode())) {
                            createEndAssignFlag = true;
                        }
                    }
                    if (WoControlModelEnum.EXECUTE.getType().equals(controlCode)) {
                        //-------一个岗位的职责要包含“执行”-----
                        map.remove("promptMessageTwo");
                        executeFlag = true;
                    }
                    if (WoControlModelEnum.APPLY.getType().equals(controlCode)) {
                        List<ApplyItemBean> applyItems = dutyBean.getApplyItems();
                        if (CollUtil.isEmpty(applyItems)){
                            continue;
                        }
                        for (ApplyItemBean applyItemBean : applyItems) {
                            applyCode = applyItemBean.getCode();
                            if (WoApplyItemEnum.CLOSE.getType().equals(applyCode)) {
                                //-------一个岗位的职责要包含“申请结束”-----
                                map.remove("promptMessageThree");
                                applyCloseFlag = true;
                            }
                            if (CollUtil.isNotEmpty(applyItemBean.getNextRoute())){
                                nextRouteFlag = true;
                            }
                        }
                    }else if (WoControlModelEnum.ASSIGN.getType().equals(controlCode)) {
                        assignFlag = true;
                    }else if (WoControlModelEnum.AUDIT.getType().equals(controlCode)) {
                        List<ApplyItemBean> applyItems = dutyBean.getApplyItems();
                        if (CollUtil.isEmpty(applyItems)){
                            continue;
                        }
                        for (ApplyItemBean applyItemBean : applyItems) {
                            if (CollUtil.isNotEmpty(applyItemBean.getNextRoute())){
                                nextRouteFlag = true;
                            }
                        }
                    }
                }
                if (applyCloseFlag && !executeFlag) {
                    //----------配置一个岗位的职责中有“申请结束”时，必须要有“执行”
                    map.put("promptMessageFour", JSONArray.parseArray("[\""+promptMessageFour+"\"]"));
                }
            }
            if (createEndAssignFlag && !assignFlag) {
                map.put("promptMessageFive", JSONArray.parseArray("[\""+promptMessageFive+"\"]"));
            }
            if (nextRouteFlag) {
                map.put("promptMessageFive", JSONArray.parseArray("[\""+promptMessageSix+"\"]"));
            }
        }
        promptMessages.addAll(map.values());
        verifyFlowPlanName(queryVerifyFlowPlanVo.getPlanId(),queryVerifyFlowPlanVo.getPlanName(),null, null, promptMessages);
        return ToolsUtil.successJsonMsg(promptMessages);
    }
    /***
     * Description: 查询部门岗位树
     * @param jsonStr : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 10:18
     * Update By lijie 2021/5/8 10:18
     */
    @Override
    public String queryDeptPositionTree(String jsonStr){
        JSONObject jsonObj = JSONObject.parseObject(jsonStr);
        jsonObj.put("user_id", CommonConstant.SYSTEM_ID);
        jsonObj.put("user_type", "0");
        jsonObj.put("treeType", "dept_position_model");
        String respContent = saasClient.queryDeptPositionTree(jsonObj);
        //去掉“其他”部门
        JSONObject jsonObject = JSONObject.parseObject(respContent);
        JSONArray deptArr = jsonObject.getJSONArray(ResultEnum.CONTENT.getType());
        if(deptArr != null && deptArr.size() > 0){
            for (int i = 0; i < deptArr.size(); i++) {
                JSONObject obj = deptArr.getJSONObject(i);
                if("其他".equals(obj.getString("obj_name"))){
                    deptArr.remove(i);
                    i--;
                }
            }
        } else {
            deptArr = new JSONArray();
        }
        return ToolsUtil.successJsonMsg(deptArr);
    }
    /***
     * Description: 根据Id删除工单类型
     * @param jsonStr : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 10:23
     * Update By lijie 2021/5/8 10:23
     */
    @Override
    @Transactional(rollbackFor = BusinessException.class)
    public String deleteFlowPlanById(String jsonStr){
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        WoFlowPlan woFlowPlan = WoFlowPlan.builder()
                .planId(jsonObject.getString("plan_id"))
                .valid(false)
                .build();
        this.updateById(woFlowPlan);
        return ToolsUtil.successJsonMsg(new ArrayList<>());
    }
    /***
     * Description: 查询项目下所有流转方案
     * @param queryFlowPlanListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 10:31
     * Update By lijie 2021/5/8 10:31
     */
    @Override
    public String queryProjectFlowPlan(QueryFlowPlanListVo queryFlowPlanListVo) {
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getValid,true);
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getProjectId())){
            queryWrapper.eq(WoFlowPlan::getProjectId,queryFlowPlanListVo.getProjectId());
        }
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getExecuteType())){
            queryWrapper.eq(WoFlowPlan::getExecuteType,queryFlowPlanListVo.getExecuteType());
        }
        if (StrUtil.isNotBlank(queryFlowPlanListVo.getWorkType())){
            queryWrapper.eq(WoFlowPlan::getWorkType,queryFlowPlanListVo.getWorkType());
        }
        List<WoFlowPlan> woFlowPlans = this.list(queryWrapper);
        if (CollUtil.isEmpty(woFlowPlans)){
            return ToolsUtil.successJsonMsg(new ArrayList<>());
        }
        return ToolsUtil.successJsonMsg(fixQueryResult(woFlowPlans));
    }
    /***
     * Description: 添加工单流转方案信息
     * @param woFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 11:21
     * Update By lijie 2021/5/8 11:21
     */
    @Override
    public String addFlowPlan(WoFlowPlanVo woFlowPlanVo) {
        //添加工单类型信息
        // 1.过滤掉下级路由中重复的岗位
        List<WoFlowPlan> woFlowPlans = queryWoFlanByPlanName(woFlowPlanVo.getPlanName());
        if (CollUtil.isNotEmpty(woFlowPlans)){
            return ToolsUtil.errorJsonMsg("工单类型名称重复！");
        }
        woFlowPlanVo.setPostAndDuty(distinctNextRoute(CollUtil.defaultIfEmpty(woFlowPlanVo.getPostAndDuty(), new ArrayList<>())));
        woFlowPlanVo.setPostAndDuty(saveCreateAndAssignNextRoute(CollUtil.defaultIfEmpty(woFlowPlanVo.getPostAndDuty(), new ArrayList<>())));
        WoFlowPlan woFlowPlan = new WoFlowPlan();
        woFlowPlan.setPlanId(IdPrefixEnum.WO_FLOW_PLAN.getPrefix()+IdWorker.getIdStr());
        woFlowPlan.setPlanName(woFlowPlanVo.getPlanName());
        woFlowPlan.setProjectId(StrUtil.blankToDefault(woFlowPlanVo.getProjectId(),FlowPlanConstant.GROUP_ID));
        //工作类型编码
        woFlowPlan.setWorkType(woFlowPlanVo.getWorkType());
        //时间类型编码
        woFlowPlan.setExecuteType(woFlowPlanVo.getExecuteType());
        //项目功能类型
        woFlowPlan.setFunctionType(woFlowPlanVo.getFunctionType());
        //方案状态 0-需维护，1-正常
        woFlowPlan.setPlanStatus(WoFlowPlanStatusEnum.NORMAL.getType());
        //创建人
        woFlowPlan.setCreatorId(woFlowPlanVo.getPersonId());
        //报修是否可以转此工单，  0-否，1-是
        woFlowPlan.setRepairFlag(StrUtil.blankToDefault(woFlowPlanVo.getRepairFlag(), BoolEnum.FALSE.getType()));
        //岗位职责
        woFlowPlan.setPostAndDuty(woFlowPlanVo.getPostAndDuty());
        if (ObjectUtil.isNotNull(woFlowPlanVo.getPostAndDutyNative())){
            woFlowPlan.setPostAndDutyNative(woFlowPlanVo.getPostAndDutyNative());
        }
        //添加时间要求
        woFlowPlan.setTimeLimit(ObjectUtil.defaultIfNull(woFlowPlanVo.getTimeLimit(), new TimeLimitBean()));
        // 版本号
        woFlowPlan.setVersion(1);
        this.save(woFlowPlan);
        //====刷新缓存===
        orderTypeCache.updateCache(woFlowPlan);
        return ToolsUtil.successJsonMsg(new ArrayList<>());
    }
    /***
     * Description: 根据Id查询工单详细信息
     * @param baseQueryFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 15:01
     * Update By lijie 2021/5/8 15:01
     */
    @Override
    public String queryFlowPlanById(BaseQueryFlowPlanVo baseQueryFlowPlanVo) throws Exception {
        WoFlowPlan woFlowPlan = this.getById(baseQueryFlowPlanVo.getPlanId());
        if (null==woFlowPlan){
            return ToolsUtil.successJsonMsg(new JSONObject());
        }
        Map<String, String> buildingMap = dictionaryService.queryAllBuildingCode();
        Map<String, String> typeNameMap = queryWorkTypeAndExecuteTypeNameMap();
        WoFlowPlanBean woFlowPlanBean = BeanUtil.copyProperties(woFlowPlan, WoFlowPlanBean.class);
        //获取工单类型和时间类型
        woFlowPlanBean.setWorkTypeName(typeNameMap.getOrDefault(woFlowPlanBean.getWorkType(),""));
        woFlowPlanBean.setExecuteTypeName(typeNameMap.getOrDefault(woFlowPlanBean.getExecuteType(),""));
        JSONObject flowPlan = JSONObject.parseObject(JSONObject.toJSONString(woFlowPlanBean));
        //在这里无法将字符串解析为对象  ：是因为传入的参数的原因
        String post_and_duty = flowPlan.getString("post_and_duty");
        //-----旧数据兼容 中止->终止
        post_and_duty = post_and_duty.replace("中止","终止");
        flowPlan.put("post_and_duty", JSONArray.parseArray(post_and_duty));
        if (!StringUtil.isNull(flowPlan,"post_and_duty_native")){
            String post_and_duty_native = flowPlan.getString("post_and_duty_native");
            post_and_duty_native = post_and_duty_native.replace("中止","终止");
            flowPlan.put("post_and_duty_native", JSONObject.parseObject(post_and_duty_native));
        }

        String function_type =  flowPlan.getString("function_type");
        String function_type_name = "";
        if(!StringUtil.isNull(function_type)) {
            function_type_name = buildingMap.get(function_type);
        }
        flowPlan.put("function_type_name", function_type_name == null ? "" : function_type_name);
        //添加时间限制
        flowPlan.put("time_limit", JSONObject.parseObject(flowPlan.getString("time_limit")));
        return ToolsUtil.successJsonMsg(flowPlan);
    }
    /***
     * Description: 根据Id编辑流转方案信息
     * @param woFlowPlanVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 16:19
     * Update By lijie 2021/5/8 16:19
     */
    @Override
    public String updateFlowPlanById(WoFlowPlanVo woFlowPlanVo) {
        // 1.根据plan_id查询工单类型
        WoFlowPlan woFlowPlan = this.getById(woFlowPlanVo.getPlanId());
        if (null==woFlowPlan){
            return ToolsUtil.errorJsonMsg("未找到流转方案");
        }
        // 找到工单类型版本自动加一,自动更新不加版本
        if (StrUtil.isBlank(woFlowPlanVo.getPlanStatus())){
            woFlowPlan.setVersion(woFlowPlan.getVersion()+1);
        }
        woFlowPlanVo.setPostAndDuty(distinctNextRoute(CollUtil.defaultIfEmpty(woFlowPlanVo.getPostAndDuty(),new ArrayList<>())));
        // 正常更新不传状态
        if(StrUtil.isBlank(woFlowPlanVo.getPlanStatus())){
            // 添加新建和指派的下级路由
            woFlowPlanVo.setPostAndDuty(saveCreateAndAssignNextRoute(CollUtil.defaultIfEmpty(woFlowPlanVo.getPostAndDuty(),new ArrayList<>())));
            woFlowPlan.setPlanStatus(WoFlowPlanStatusEnum.NORMAL.getType());
        }
        // 工单类型名称
        if (StrUtil.isNotBlank(woFlowPlanVo.getPlanName())){
            woFlowPlan.setPlanName(woFlowPlanVo.getPlanName());
        }
        // 工作类型编码
        woFlowPlan.setWorkType(woFlowPlanVo.getWorkType());
        // 时间类型编码
        woFlowPlan.setExecuteType(woFlowPlanVo.getExecuteType());
        //项目功能类型
        if (StrUtil.isNotBlank(woFlowPlanVo.getFunctionType())){
            woFlowPlan.setFunctionType(woFlowPlanVo.getFunctionType());
        }
        //报修是否可以转此工单，  0-否，1-是
        woFlowPlan.setRepairFlag(StrUtil.blankToDefault(woFlowPlanVo.getRepairFlag(),BoolEnum.FALSE.getType()));
        // 岗位职责
        woFlowPlan.setPostAndDuty(CollUtil.defaultIfEmpty(woFlowPlanVo.getPostAndDuty(),new ArrayList<>()));
        if (null!=woFlowPlanVo.getPostAndDutyNative()){
            woFlowPlan.setPostAndDutyNative(woFlowPlanVo.getPostAndDutyNative());
        }
        woFlowPlan.setTimeLimit(ObjectUtil.defaultIfNull(woFlowPlanVo.getTimeLimit(),new TimeLimitBean()));
        woFlowPlan.setUpdateTime(null);
        this.updateById(woFlowPlan);
        orderTypeCache.updateCache(woFlowPlan);
        return ToolsUtil.successJsonMsg(new ArrayList<>());
    }
    /***
     * Description: 根据工作类型查询所有有效工单流转方案信息
     * @param workType : 工作类型
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:04
     * Update By lijie 2021/5/8 17:04
     */
    @Override
    @Deprecated
    public String queryFlowPlanByWorkType(String workType) {
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getValid,true);
        queryWrapper.eq(WoFlowPlan::getWorkType,workType);
        return queryWoFlowPlansByCondition(queryWrapper);
    }
    /***
     * Description: 根据时间类型查询所有有效流转方案信息
     * @param executeType : 时间类型
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:08
     * Update By lijie 2021/5/8 17:08
     */
    @Override
    public String queryFlowPlanByExecuteType(String executeType) {
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getValid,true);
        queryWrapper.eq(WoFlowPlan::getExecuteType,executeType);
        return queryWoFlowPlansByCondition(queryWrapper);
    }
    /***
     * Description: 通过工单类型名称查询所有有效工单
     * @param planName : 工单类型名称
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:12
     * Update By lijie 2021/5/8 17:12
     */
    @Override
    public String queryFlowPlanByPlanName(String planName) {
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getValid,true);
        queryWrapper.eq(WoFlowPlan::getPlanName, planName);
        return queryWoFlowPlansByCondition(queryWrapper);
    }

    /***
     * Description: 根据条件查询工单类型列表
     * @param queryWrapper : 查询Wrapper
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/5/8 17:11
     * Update By lijie 2021/5/8 17:11
     */
    private String queryWoFlowPlansByCondition(LambdaQueryWrapper<WoFlowPlan> queryWrapper) {
        List<WoFlowPlan> woFlowPlans = this.list(queryWrapper);
        if (CollUtil.isEmpty(woFlowPlans)){
            return ToolsUtil.successJsonMsg(new ArrayList<>());
        }
        return ToolsUtil.successJsonMsg(fixQueryResult(woFlowPlans));
    }

    /***
     * Description: 查询工作类型和执行类型的名称映射
     * @return : java.util.Map<java.lang.String,java.lang.String>
     * @author : lijie
     * @date :2021/5/8 16:05
     * Update By lijie 2021/5/8 16:05
     */
    private Map<String,String> queryWorkTypeAndExecuteTypeNameMap() {
        List<GeneralDictVo> workTypes = generalDictService.queryGeneralDictByKey(DictTypeEnum.WORK_TYPE.getType(), null);
        List<GeneralDictVo> woExecuteTypes = generalDictService.queryGeneralDictByKey(DictTypeEnum.WO_EXECUTE_TYPE.getType(), null);
        Map<String,String> dictTypeNameMap = new HashMap<>();
        dictTypeNameMap.putAll(workTypes.stream().collect(Collectors.toMap(GeneralDictVo::getCode,GeneralDictVo::getName,(k1,k2)->k1)));
        dictTypeNameMap.putAll(woExecuteTypes.stream().collect(Collectors.toMap(GeneralDictVo::getCode,GeneralDictVo::getName,(k1,k2)->k1)));
        return dictTypeNameMap;
    }

    /***
     * Description: 根据工单类型名称查询工单类型
     * @param planName : 工单类型名称
     * @return : void
     * @author : lijie
     * @date :2021/5/8 14:24
     * Update By lijie 2021/5/8 14:24
     */
    private List<WoFlowPlan> queryWoFlanByPlanName(String planName) {
        if (StrUtil.isBlank(planName)){
            return new ArrayList<>();
        }
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getValid,true);
        queryWrapper.eq(WoFlowPlan::getPlanName,planName);
        return this.list(queryWrapper);
    }

    /**
     * Description: 给新建指派控制模块添加下级路由
     * @param postAndDuties:岗位职责
     * @author ChenQuanWei
     * @since 2018年5月31日: 上午8:55:23
     * Update By ChenQuanWei 2018年5月31日: 上午8:55:23
     */
    private List<PostAndDuty> saveCreateAndAssignNextRoute(List<PostAndDuty> postAndDuties) {
        Map<String,List<NextRouteBean>> dutyForPostMap = new HashMap<>();
        // 1.遍历岗位职责,整理新建,指派和执行的岗位职责
        for (PostAndDuty postAndDuty : postAndDuties) {
            List<DutyBean> dutyBeans = CollUtil.defaultIfEmpty(postAndDuty.getDuty(), new ArrayList<>());
            for (DutyBean dutyBean : dutyBeans) {
                // 只整理新建、指派和执行
                if (WoControlModelEnum.CREATE.getType().equals(dutyBean.getCode())
                        || WoControlModelEnum.ASSIGN.getType().equals(dutyBean.getCode())
                        || WoControlModelEnum.EXECUTE.getType().equals(dutyBean.getCode())) {
                    List<NextRouteBean> nextRouteBeans = dutyForPostMap.getOrDefault(dutyBean.getCode(), new ArrayList<>());
                    nextRouteBeans.add(NextRouteBean.builder()
                            .type(postAndDuty.getType())
                            .name(postAndDuty.getName())
                            .positionId(postAndDuty.getPositionId())
                            .build());
                    dutyForPostMap.put(dutyBean.getCode(), nextRouteBeans);
                }
            }
        }
        List<NextRouteBean> excuteNextRouteBeans = dutyForPostMap.getOrDefault(WoControlModelEnum.EXECUTE.getType(), new ArrayList<>());
        List<NextRouteBean> assignNextRouteBeans = dutyForPostMap.getOrDefault(WoControlModelEnum.ASSIGN.getType(), new ArrayList<>());
        // 2.当指派路由大于1时去掉系统指派的路由
        if (assignNextRouteBeans.size()>1){
            for (int i = assignNextRouteBeans.size() - 1; i >= 0; i--) {
                NextRouteBean nextRouteBean = assignNextRouteBeans.get(i);
                if (FlowPlanConstant.SYSTEM_POSITION.equals(nextRouteBean.getPositionId())){
                    assignNextRouteBeans.remove(i);
                }
            }
        }
        // 3.处理新建和指派时的下级路由
        for (PostAndDuty postAndDuty : postAndDuties) {
            List<DutyBean> dutyBeans = CollUtil.defaultIfEmpty(postAndDuty.getDuty(), new ArrayList<>());
            for (DutyBean dutyBean : dutyBeans) {
                List<NextRouteBean> nextRouteBeanArr = new ArrayList<>();
                //只在新建中添加，设置默认
                if (WoControlModelEnum.CREATE.getType().equals(dutyBean.getCode())){
                    if (null==dutyBean.getArrivalTimeAllowExecute()){
                        //到达工单要求开始时间时才允许执行
                        dutyBean.setArrivalTimeAllowExecute(false);
                    }
                    if (null==dutyBean.getLimitDomain()){
                        //请选择是否启用专业控制
                        dutyBean.setLimitDomain(false);
                    }
                    // 保存的时候没有传来下级路由.  新建的下级路由可以是执行或指派
                    if (WoControlModelEnum.ASSIGN.getType().equals(dutyBean.getExecutieMode())){
                        for (NextRouteBean assignNextRouteBean : assignNextRouteBeans) {
                            nextRouteBeanArr.add(NextRouteBean.builder()
                                        .type("2")
                                        .positionId(assignNextRouteBean.getPositionId())
                                        .name(assignNextRouteBean.getName()).build());
                        }
                        dutyBean.setNextRoute(nextRouteBeanArr);
                        continue;
                    }
                    if (CollUtil.isEmpty(excuteNextRouteBeans)){
                        continue;
                    }
                    // 下级路由为执行和抢单
                    for (NextRouteBean excuteNextRouteBean : excuteNextRouteBeans) {
                        nextRouteBeanArr.add(NextRouteBean.builder()
                                .type("2")
                                .positionId(excuteNextRouteBean.getPositionId())
                                .name(excuteNextRouteBean.getName()).build());
                    }
                    dutyBean.setNextRoute(nextRouteBeanArr);
                    continue;
                }
                if (WoControlModelEnum.ASSIGN.getType().equals(dutyBean.getCode())){
                    if (null==dutyBean.getExpecteOperation()){
                        //期望该操作持续时间不超过
                        dutyBean.setExpecteOperation(false);
                    }
                    if (null==dutyBean.getLimitDomain()){
                        //请选择是否启用专业控制
                        dutyBean.setLimitDomain(false);
                    }
                    if (null==dutyBean.getFilterScheduling()){
                        //是否需要按照排班表过滤每次可转交的人员范围
                        dutyBean.setFilterScheduling(false);
                    }
                    if (CollUtil.isEmpty(excuteNextRouteBeans)){
                        continue;
                    }
                    // 下级路由为执行和抢单
                    for (NextRouteBean excuteNextRouteBean : excuteNextRouteBeans) {
                        nextRouteBeanArr.add(NextRouteBean.builder()
                                .type("2")
                                .positionId(excuteNextRouteBean.getPositionId())
                                .name(excuteNextRouteBean.getName()).build());
                    }
                    dutyBean.setNextRoute(nextRouteBeanArr);
                }
            }
            postAndDuty.setDuty(dutyBeans);
        }
        return postAndDuties;
    }
    /***
     * Description: 去重下级路由
     * @param postAndDuties : 工单类型
     * @return : void
     * @author : lijie
     * @date :2021/5/8 11:25
     * Update By lijie 2021/5/8 11:25
     */
    private List<PostAndDuty> distinctNextRoute(List<PostAndDuty> postAndDuties){
        for (int i = postAndDuties.size() - 1; i >= 0; i--) {
            PostAndDuty postAndDuty = postAndDuties.get(i);
            List<DutyBean> dutyBeans = CollUtil.defaultIfEmpty(postAndDuty.getDuty(),new ArrayList<>());
            for (int j = 0; j < dutyBeans.size(); j++) {
                DutyBean dutyBean = dutyBeans.get(j);
                List<NextRouteBean> nextRouteBeans = CollUtil.defaultIfEmpty(dutyBean.getNextRoute(), new ArrayList<>());
                Set<String> nextRoutePositionIds = new HashSet<>();
                for (int m = nextRouteBeans.size() - 1; m >= 0; m--) {
                    NextRouteBean nextRouteBean = nextRouteBeans.get(m);
                    if (nextRoutePositionIds.contains(nextRouteBean.getPositionId())){
                        nextRouteBeans.remove(m);
                        continue;
                    }
                    nextRoutePositionIds.add(nextRouteBean.getPositionId());
                }
                dutyBean.setNextRoute(nextRouteBeans);
            }
            postAndDuty.setDuty(dutyBeans);
        }
        return postAndDuties;
    }

    /**
     * Description: 验证工单类型名是否重复
     * @param planId: 工单类型id
     * @param planName: 工单类型名称
     * @param projectId: 项目id
     * @param reminds: 提醒语
     * @param remindArr: 提醒语数组
     * @author ChenQuanWei
     * @since 2018年7月27日: 下午5:16:08
     * Update By ChenQuanWei 2018年7月27日: 下午5:16:08
     */
    private void verifyFlowPlanName(String planId, String planName, String projectId, JSONArray reminds, JSONArray remindArr){
        List<WoFlowPlan> flowPlanArrays = queryFlowPlanByName(planName);
        if (CollUtil.isEmpty(flowPlanArrays)){
            return;
        }
        for (WoFlowPlan flowPlan : flowPlanArrays) {
            if (!planName.equals(flowPlan.getPlanName()) || flowPlan.getPlanId().equals(planId)){
                continue;
            }
            String destProjectId = flowPlan.getProjectId();
            if (StrUtil.isBlank(projectId)
                  || StrUtil.isBlank(destProjectId)
                  || FlowPlanConstant.GROUP_ID.equals(destProjectId)
                  || projectId.equals(destProjectId)){
                reminds = reminds == null ? new JSONArray() : reminds;
                reminds.add("工单类型名称重复！");
                remindArr.add(reminds);
                break;
            }
        }
    }

    /***
     * Description: 根据工单类型名称查询工单类型
     * @param planName : 工单类型名称
     * @return : com.alibaba.fastjson.JSONArray
     * @author : lijie
     * @date :2021/4/28 10:27
     * Update By lijie 2021/4/28 10:27
     */
    private List<WoFlowPlan> queryFlowPlanByName(String planName){
        if (StrUtil.isBlank(planName)){
            return new ArrayList<>();
        }
        LambdaQueryWrapper<WoFlowPlan> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WoFlowPlan::getPlanName,planName);
        queryWrapper.eq(WoFlowPlan::getValid, true);
        return this.list(queryWrapper);
    }

    /**
     * Description: 整理查询出的流转方案返回格式
     * @param woFlowPlans:结果集
     * @return
     * @throws Exception JSONArray
     * @author ChenQuanWei
     * @since 2018年7月5日: 上午9:37:54
     * Update By lijie 2021年4月27日17:52:11
     */
    private JSONArray fixQueryResult(List<WoFlowPlan> woFlowPlans){
        String workType, executeType;
        JSONArray flowPlanArrays = new JSONArray();
        Map<String,String> dictTypeNameMap = queryWorkTypeAndExecuteTypeNameMap();
        for (int i = 0; i < woFlowPlans.size(); i++) {
            JSONObject flowPlans = new JSONObject();
            //获取具体的一条信息，从中筛选前端所需信息
            WoFlowPlan flowPlan = woFlowPlans.get(i);
            workType = flowPlan.getWorkType();
            executeType = flowPlan.getExecuteType();
            flowPlans.put("plan_id", flowPlan.getPlanId());
            flowPlans.put("plan_name", flowPlan.getPlanName());
            flowPlans.put("project_id", flowPlan.getProjectId());
            flowPlans.put("work_type", workType);
            flowPlans.put("order_type", workType);
            flowPlans.put("execute_type", executeType);
            flowPlans.put("repair_flag", flowPlan.getRepairFlag());
            flowPlans.put("plan_status", flowPlan.getPlanStatus());
            flowPlans.put("create_time", flowPlan.getCreateTime());
            flowPlans.put("work_type_name", dictTypeNameMap.getOrDefault(workType,""));
            flowPlans.put("order_type_name", dictTypeNameMap.getOrDefault(workType,""));
            flowPlans.put("execute_type_name", dictTypeNameMap.getOrDefault(executeType,""));
            //岗位名称
            List<PostAndDuty> postAndDutys = flowPlan.getPostAndDuty();
            if (CollUtil.isNotEmpty(postAndDutys)){
                String positionStr = postAndDutys.stream()
                        .filter(postAndDuty -> StrUtil.isNotBlank(postAndDuty.getName()))
                        .map(PostAndDuty::getName)
                        .collect(Collectors.joining("、"));
                flowPlans.put("position", positionStr);
            }
            flowPlanArrays.add(flowPlans);
        }
        return flowPlanArrays;
    }
}
