package com.persagy.fm.sop.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.persagy.common.exception.BusinessException;
import com.persagy.fm.common.constant.CommonConstant;
import com.persagy.fm.common.constant.DataRequestPathUtil;
import com.persagy.fm.common.constant.enums.BoolEnum;
import com.persagy.fm.common.constant.enums.IdPrefixEnum;
import com.persagy.fm.common.constant.enums.ResultEnum;
import com.persagy.fm.common.constant.enums.ValidEnum;
import com.persagy.fm.common.model.vo.GeneralDictVo;
import com.persagy.fm.common.old.utils.StringUtil;
import com.persagy.fm.common.old.utils.ToolsUtil;
import com.persagy.fm.sop.constant.SopCommonConst;
import com.persagy.fm.sop.constant.SopQueryConst;
import com.persagy.fm.sop.entity.*;
import com.persagy.fm.sop.entity.Sop;
import com.persagy.fm.sop.entity.SopEdit;
import com.persagy.fm.sop.entity.SopLabelRel;
import com.persagy.fm.sop.entity.SopObjRel;
import com.persagy.fm.sop.entity.SopOrderTypeRel;
import com.persagy.fm.sop.enums.*;
import com.persagy.fm.sop.model.dto.*;
import com.persagy.fm.sop.model.vo.*;
import com.persagy.fm.sop.service.*;
import com.persagy.old.cache.GraphCache;
import com.persagy.old.cache.OrderTypeCache;
import com.persagy.old.cache.ProjectCache;
import com.persagy.old.common.*;
import com.persagy.old.common.enums.ObjectLevel;
import com.persagy.old.dao.DBConst;
import com.persagy.fm.sop.service.IEquipCompanyBookService;
import com.persagy.fm.sop.service.IObjectService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 服务接口实现
 *
 * @author lijie
 * @description 由 Mybatisplus Code Generator 创建
 * @since 2021-03-31 18:08:21
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class ProjectSopServiceImpl implements IProjectSopService {
    private final ISopService sopService;
    private final ISopCellService sopCellService;
    private final ISopDomainRelService sopDomainRelService;
    private final ISopEquipModelRelService sopEquipModelRelService;
    private final ISopLabelRelService sopLabelRelService;
    private final ISopMediaRelService sopMediaRelService;
    private final ISopObjRelService sopObjRelService;
    private final ISopOrderTypeRelService sopOrderTypeRelService;
    private final ISopToolRelService sopToolRelService;
    private final ISopEditService sopEditService;
    private final IObjectService objectService;
    private final ISopHisService sopHisService;
    private final ISopReviseMsgService sopReviseMsgService;
    private final IEquipModelService equipModelService;
    private final OrderTypeCache orderTypeCache;
    private final IWoFlowPlanService woFlowPlanService;
    private final IEquipCompanyBookService equipCompanyBookService;
    private final ISopDestroyMsgService sopDestroyMsgService;
    private final ProjectCache projectCache;
    private final IGeneralDictService generalDictService;
    private final IDictionaryService dictionaryService;
 	private final GraphCache graphCache;
 	private final BaseService baseService;
    @Value("${data.platform.basePath}")
    public String dataPlatBasePath;

    private static final Set<String> OBJ_TYPES = new HashSet<>(Arrays.asList(ObjTypeEnum.BUILD.getType(),
            ObjTypeEnum.EQUIP.getType(), ObjTypeEnum.FLOOR.getType(), ObjTypeEnum.SHAFT.getType(),
            ObjTypeEnum.SPACE.getType(), ObjTypeEnum.SYSTEM.getType()));

    /***
     * Description: 添加草稿状态sop
     * @param saveSopVo : 保存SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/2 14:30
     * Update By lijie 2021/4/2 14:30
     */
    @Override
    public QuerySopVo addDraftSop(SaveSopVo saveSopVo) {
        // 1.调用通用SOP名称判重逻辑校验名称是否重复
        if (!checkSopNameRepeated(saveSopVo.getSopName(),saveSopVo.getSopId(),saveSopVo.getProjectId())){
            throw new BusinessException("SOP名称重复,请修改SOP名称");
        }
        // 2.调用批量添加草稿逻辑
        List<String> sopIds = batchAddDraftSop(Collections.singletonList(saveSopVo));
        QuerySopVo querySopVo = new QuerySopVo();
        if (CollUtil.isNotEmpty(sopIds)) {
            querySopVo.setSopId(sopIds.get(0));
        }
        return querySopVo;
    }

    /***
     * Description: 添加发布状态sop
     * @param saveSopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/1 16:00
     * Update By lijie 2021/4/1 16:00
     */
    @Override
    @Transactional(rollbackFor = BusinessException.class)
    public void addPublishedSop(SaveSopVo saveSopVo) {
        //Set<String> validRequiredFields = validRequiredFields(saveSopVo);
        // 调用批量添加接口
        saveSopVo.setVersionExplain(Collections.singletonList(SopCommonConst.VERSION_EXPLAIN_CREATE));
        saveSopVo.setSopStatus(SopStatusEnum.PUBLISHED.getType());
        saveSopVo.setPublishStatus(SopPublishStatusEnum.NORMAL.getType());
        saveSopVo.setVersion(SopCommonConst.SOP_VERSION_INIT);
        saveSopVo.setVersionCount(ValidEnum.TRUE.getType());
        batchAddPublishedSop(Collections.singletonList(saveSopVo));
    }

    /***
     * Description: 填充SOP详情
     * @param sops : sop的集合
     * @return : void
     * @author : lijie
     * @date :2021/4/7 9:26
     * Update By lijie 2021/4/7 9:26
     */
    @Override
    public List<SopBean> fillSopDetail(List<Sop> sops) {
        if (CollUtil.isEmpty(sops)) {
            return new ArrayList<>();
        }
        Set<String> sopIds = sops.stream().map(Sop::getSopId).collect(Collectors.toSet());
        Map<String, List<SopEquipModelRel>> equipModelMap = queryEquipModelMapBySopIds(sopIds);
        Map<String, List<ToolsBean>> toolMap = queryToolMapBySopIds(sopIds);
        Map<String, List<SopDomainRel>> domainMap = querySopDomainRelsByParentIds(sopIds);
        Map<String, List<SopLabelRel>> labelMap = queryLabelMapBySopIds(sopIds);
        Map<String, List<String>> orderTypeMap = queryOrderTypeMapBySopIds(sopIds);
        Map<String, List<SopObjRel>> fitObjMap = queryObjMapBySopIds(sopIds, ObjOwnerEnum.FIT_OBJ.getType());
        Map<String, List<SopCell>> sopCellMap = queryStepsBySopIds(sopIds);
        List<SopBean> result = new ArrayList<>();
        // 填充Sop的内容
        sops.forEach(sop -> {
            SopBean sopBean = BeanUtil.copyProperties(sop, SopBean.class);
            sopBean.setOperator(OperatorBean.builder().id(sop.getOperatorId()).name(sop.getOperatorName()).build());
            sopBean.setDomains(domainMap.getOrDefault(sop.getSopId(), new ArrayList<>())
                    .stream().map(SopDomainRel::getDomain).collect(Collectors.toList()));
            sopBean.setOrderType(orderTypeMap.getOrDefault(sop.getSopId(), new ArrayList<>()));
            sopBean.setTools(toolMap.getOrDefault(sop.getSopId(), new ArrayList<>()));
            sopBean.setFitObjs(fitObjMap.getOrDefault(sop.getSopId(), new ArrayList<>()));
            sopBean.setEquipModelIds(equipModelMap.getOrDefault(sop.getSopId(), new ArrayList<>())
                    .stream().map(SopEquipModelRel::getEquipModelId).collect(Collectors.toList()));
            sopBean.setLabels(labelMap.getOrDefault(sop.getSopId(), new ArrayList<>())
                    .stream().map(SopLabelRel::getLabel).collect(Collectors.toList()));
            sopBean.setSteps(sopCellMap.getOrDefault(sop.getSopId(), new ArrayList<>()));
            result.add(sopBean);
        });
        return result;
    }

    /***
     * Description: 编辑页-发布sop
     * @param saveSopVo : 更新的SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/8 14:05
     * Update By lijie 2021/4/8 14:05
     */
    @Override
    public QuerySopVo publishSop(SaveSopVo saveSopVo) throws Exception {
        if (!checkSopNameRepeated(saveSopVo.getSopName(),saveSopVo.getSopId(),saveSopVo.getProjectId())){
            throw new BusinessException("SOP名称重复,请修改SOP名称");
        }
        // sop的id
        String sopId = saveSopVo.getSopId();
        List<Sop> sops = sopService.listByIds(Collections.singletonList(sopId));
        List<SopBean> sopBeans = fillSopDetail(sops);
        boolean isUpdate = false;
        Map<String, SopBean> sopRefProfile = proccessSopIdRefProfile(sopId);
        if (CollUtil.isNotEmpty(sopBeans)) {
            // 设置参数中的版本号为当前版本号
            Integer versionCount = sopBeans.get(0).getVersionCount();
            versionCount = versionCount + 1;
            saveSopVo.setVersionExplain(Collections.singletonList("修订标准操作"));
            saveSopVo.setSopStatus(SopStatusEnum.PUBLISHED.getType());
            saveSopVo.setPublishStatus(SopPublishStatusEnum.NORMAL.getType());
            saveSopVo.setVersion("V" + (double) versionCount / 10);
            saveSopVo.setVersionCount(versionCount);
            // 存在历史版本,进行历史版本的处理
            SopHis sopHis = BeanUtil.copyProperties(proccessSopItem(sopBeans.get(0), sopRefProfile), SopHis.class);
            sopHis.setRefedSops(getSopProfileOfRefCurrentSop(sopHis.getSopId()));
            sopHisService.save(sopHis);
            isUpdate = true;
        }
        //删除SOP关联的所有关系
        deleteAllRelsBySopId(saveSopVo.getSopId());
        //删除草稿sop
        sopEditService.deleteDraftBySopId(saveSopVo.getSopId());
        if (isUpdate){
            batchAddPublishedSop(Collections.singletonList(saveSopVo));
        }else {
            // 新增SOP
            addPublishedSop(saveSopVo);
        }
        return new QuerySopVo(saveSopVo.getSopId());
    }

    /***
     * Description: 根据SOP的id集合查询关联的设备模型
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopLabelRel>>
     * @author : lijie
     * @date :2021/4/6 18:05
     * Update By lijie 2021/4/6 18:05
     */
    @Override
    public Map<String, List<SopEquipModelRel>> queryEquipModelMapBySopIds(Set<String> sopIds) {
        if (CollUtil.isEmpty(sopIds)) {
            return new HashMap<>();
        }
        QueryWrapper<SopEquipModelRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(SopEquipModelRel.SOP_ID, sopIds);
        List<SopEquipModelRel> sopEquipModelRels = sopEquipModelRelService.list(queryWrapper);
        Map<String, List<SopEquipModelRel>> objRelMap = new HashMap<>();
        for (SopEquipModelRel sopEquipModelRel : sopEquipModelRels) {
            List<SopEquipModelRel> sopEquipModelRelSet = objRelMap.getOrDefault(sopEquipModelRel.getSopId(), new ArrayList<>());
            sopEquipModelRelSet.add(sopEquipModelRel);
            objRelMap.put(sopEquipModelRel.getSopId(), sopEquipModelRelSet);
        }
        return objRelMap;
    }

    /***
     * Description: 根据设备模型id查询SOP设备模型关系列表
     * @param equipModelIds : 设备模型id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopEquipModelRel>
     * @author : lijie
     * @date :2021/4/6 20:16
     * Update By lijie 2021/4/6 20:16
     */
    @Override
    public List<SopEquipModelRel> queryEquipModelRelsByEquipModelIds(List<String> equipModelIds) {
        if (CollUtil.isEmpty(equipModelIds)) {
            return new ArrayList<>();
        }
        QueryWrapper<SopEquipModelRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(SopEquipModelRel.EQUIP_MODEL_ID, equipModelIds);
        return sopEquipModelRelService.list(queryWrapper);
    }

    /***
     * Description: 根据适用范围对象id集合查询关联对象
     * @param fitObjIds : 适用范围对象id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopObjRel>
     * @author : lijie
     * @date :2021/4/6 20:38
     * Update By lijie 2021/4/6 20:38
     */
    @Override
    public List<SopObjRel> queryFitObjsByIds(List<String> fitObjIds) {
        if (CollUtil.isEmpty(fitObjIds)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SopObjRel> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(SopObjRel::getObjId, fitObjIds);
        lambdaQueryWrapper.eq(SopObjRel::getObjOwner, ObjOwnerEnum.FIT_OBJ.getType());
        return sopObjRelService.list(lambdaQueryWrapper);
    }

    /***
     * Description: 根据SOP的id集合查询关联的标签
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopLabelRel>>
     * @author : lijie
     * @date :2021/4/6 18:05
     * Update By lijie 2021/4/6 18:05
     */
    @Override
    public Map<String, List<SopLabelRel>> queryLabelMapBySopIds(Set<String> sopIds) {
        if (CollUtil.isEmpty(sopIds)) {
            return new HashMap<>();
        }
        QueryWrapper<SopLabelRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(SopLabelRel.SOP_ID, sopIds);
        List<SopLabelRel> sopLabelRels = sopLabelRelService.list(queryWrapper);
        Map<String, List<SopLabelRel>> objRelMap = new HashMap<>();
        for (SopLabelRel sopLabelRel : sopLabelRels) {
            List<SopLabelRel> sopLabelRelSet = objRelMap.getOrDefault(sopLabelRel.getSopId(), new ArrayList<>());
            sopLabelRelSet.add(sopLabelRel);
            objRelMap.put(sopLabelRel.getSopId(), sopLabelRelSet);
        }
        return objRelMap;
    }

    /***
     * Description: 根据标签名称查询SOP与标签关系数据
     * @param labels : 标签名称列表
     * @return : java.util.List<com.persagy.fm.sop.entity.SopLabelRel>
     * @author : lijie
     * @date :2021/4/6 20:24
     * Update By lijie 2021/4/6 20:24
     */
    @Override
    public List<SopLabelRel> queryLabelsByLabelName(List<String> labels) {
        if (CollUtil.isEmpty(labels)) {
            return new ArrayList<>();
        }
        QueryWrapper<SopLabelRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(SopLabelRel.LABEL, labels);
        return sopLabelRelService.list(queryWrapper);
    }

    /***
     * Description: 根据SOP的id集合查询关联的适用范围
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.model.dto.FitObjsBean>>
     * @author : lijie
     * @date :2021/4/6 17:58
     * Update By lijie 2021/4/6 17:58
     */
    //@Override
    private Map<String, List<SopObjRel>> queryObjMapBySopIds(Set<String> sopIds, String objOwner) {
        if (CollUtil.isEmpty(sopIds)) {
            return new HashMap<>();
        }
        LambdaQueryWrapper<SopObjRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(SopObjRel::getParentId, sopIds);
        queryWrapper.eq(SopObjRel::getObjOwner, objOwner);
        List<SopObjRel> objRels = sopObjRelService.list(queryWrapper);
        Map<String, List<SopObjRel>> objRelMap = new HashMap<>();
        for (SopObjRel objRel : objRels) {
            List<SopObjRel> fitObjsBeans = objRelMap.getOrDefault(objRel.getParentId(), new ArrayList<>());
            fitObjsBeans.add(objRel);
            objRelMap.put(objRel.getParentId(), fitObjsBeans);
        }
        return objRelMap;
    }

    /***
     * Description: 根据工单类型id查询所有的SOP
     * @param orderTypeIds : 工单类型id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopOrderTypeRel>
     * @author : lijie
     * @date :2021/4/6 20:28
     * Update By lijie 2021/4/6 20:28
     */
    @Override
    public List<SopOrderTypeRel> queryOrderTypeByOrderTypeIds(List<String> orderTypeIds) {
        if (CollUtil.isEmpty(orderTypeIds)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SopOrderTypeRel> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(SopOrderTypeRel::getOrderType, orderTypeIds);
        return sopOrderTypeRelService.list(lambdaQueryWrapper);
    }

    /***
     * Description: 根据SOP的id集合查询关联的工单类型id
     * @param sopIds : SOP的id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<java.lang.String>>
     * @author : lijie
     * @date :2021/4/6 17:44
     * Update By lijie 2021/4/6 17:44
     */
    @Override
    public Map<String, List<String>> queryOrderTypeMapBySopIds(Set<String> sopIds) {
        if (CollUtil.isEmpty(sopIds)) {
            return new HashMap<>();
        }
        QueryWrapper<SopOrderTypeRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(SopOrderTypeRel.SOP_ID, sopIds);
        List<SopOrderTypeRel> orderTypeRels = sopOrderTypeRelService.list(queryWrapper);
        Map<String, List<String>> orderTypeMap = new HashMap<>();
        for (SopOrderTypeRel orderTypeRel : orderTypeRels) {
            List<String> orderTypeIds = orderTypeMap.getOrDefault(orderTypeRel.getSopId(), new ArrayList<>());
            if (StrUtil.isBlank(orderTypeRel.getOrderType())) {
                continue;
            }
            orderTypeIds.add(orderTypeRel.getOrderType());
            orderTypeMap.put(orderTypeRel.getSopId(), orderTypeIds);
        }
        return orderTypeMap;
    }

    @Override
    public List<Sop> querySopByIds(Set<String> sopIds, Boolean fillDetailFlag) {
        if (CollUtil.isEmpty(sopIds)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<Sop> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(Sop::getSopId, sopIds);
        queryWrapper.eq(Sop::getValid, true);
        List<Sop> sops = sopService.list(queryWrapper);
        if (fillDetailFlag) {
            fillSopDetail(sops);
        }
        return sops;
    }

    /***
     * Description: 根据父级id查询关联的专业
     * @param parentIds : 父级id的集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopDomainRel>>
     * @author : lijie
     * @date :2021/4/7 14:11
     * Update By lijie 2021/4/7 14:11
     */
    @Override
    public Map<String, List<SopDomainRel>> querySopDomainRelsByParentIds(Set<String> parentIds) {
        if (CollUtil.isEmpty(parentIds)) {
            return new HashMap<>();
        }
        LambdaQueryWrapper<SopDomainRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(SopDomainRel::getParentId, parentIds);
        List<SopDomainRel> domainRels = sopDomainRelService.list(queryWrapper);
        Map<String, List<SopDomainRel>> domainRelMap = new HashMap<>();
        for (SopDomainRel sopDomainRel : domainRels) {
            List<SopDomainRel> sopDomainRels = domainRelMap.getOrDefault(sopDomainRel.getParentId(), new ArrayList<>());
            sopDomainRels.add(sopDomainRel);
            domainRelMap.put(sopDomainRel.getParentId(), sopDomainRels);
        }
        return domainRelMap;
    }

    /***
     * Description: 根据SOP的id查询SOP的步骤
     * @param sopIds : sop的Id
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopCell>>
     * @author : lijie
     * @date :2021/4/7 11:09
     * Update By lijie 2021/4/7 11:09
     */
    @Override
    public Map<String, List<SopCell>> queryStepsBySopIds(Set<String> sopIds) {
        // 1.根据sop的id集合查询所有的步骤元素
        Set<SopCell> stepCells = querySopCellByParentIds(sopIds);
        // 2.根据所有的步骤元素查询所有的工作内容元素
        Set<SopCell> stepContentCells = querySopCellByParentIds(stepCells.stream().map(SopCell::getSopCellId).collect(Collectors.toSet()));
        // 3.根据工作内容元素查询确认结果元素
        Set<SopCell> confirmResultCells = querySopCellByParentIds(stepContentCells.stream().map(SopCell::getSopCellId).collect(Collectors.toSet()));
        // 5.填充content_objs
        fillContentObjsForSopCell(stepContentCells);
        // 6.填充space_scope
        fillSpaceObjsForSopCell(stepContentCells);
        // 7.填充专业
        fillDomainForSopCell(stepContentCells);
        // 8.填充确认结果
        Map<String, List<SopCell>> confirmResultMap = getMapByParentId(confirmResultCells);
        stepContentCells.forEach(sopCell -> sopCell
                .setConfirmResult(confirmResultMap.getOrDefault(sopCell.getSopCellId(), new ArrayList<>())));
        // 9.填充工作内容
        Map<String, List<SopCell>> stepContentMap = getMapByParentId(stepContentCells);
        stepCells.forEach(sopCell -> sopCell
                .setStepContent(stepContentMap.getOrDefault(sopCell.getSopCellId(), new ArrayList<>())));
        return getMapByParentId(stepCells);
    }

    /***
     * Description: 根据SOP的id查询关联的工具对象
     * @param sopIds : sop的id集合
     * @return : java.util.Map<java.lang.String,java.util.List<com.persagy.fm.sop.entity.SopToolRel>>
     * @author : lijie
     * @date :2021/4/7 14:27
     * Update By lijie 2021/4/7 14:27
     */
    private Map<String, List<ToolsBean>> queryToolMapBySopIds(Set<String> sopIds) {
        List<SopToolRel> sopToolRels = querySopToolRelByParentIds(sopIds);
        if (CollUtil.isEmpty(sopToolRels)) {
            return new HashMap<>();
        }
        Map<String, List<ToolsBean>> result = new HashMap<>();
        sopToolRels.forEach(sopToolRel -> {
            List<ToolsBean> toolRels = result.getOrDefault(sopToolRel.getParentId(), new ArrayList<>());
            toolRels.add(ToolsBean.builder().fromStep(sopToolRel.getFromStep()).tool(sopToolRel.getTool()).build());
            result.put(sopToolRel.getParentId(), toolRels);
        });
        return result;
    }

    /***
     * Description: 编辑页-保存草稿状态sop
     * @param saveSopVo : 保存SOP的参数
     * @return : com.persagy.fm.sop.utils.OldResponseResult<com.persagy.fm.sop.model.vo.QuerySopVo>
     * @author : lijie
     * @date :2021/4/8 17:24
     * Update By lijie 2021/4/8 17:24
     */
    @Override
    public QuerySopVo saveDraftSop(SaveSopVo saveSopVo) {
        String destSopId;
        if (StrUtil.isBlank(saveSopVo.getSopId())) {
            List<String> sopIds = batchAddDraftSop(Collections.singletonList(saveSopVo));
            destSopId = CollUtil.isNotEmpty(sopIds) ? sopIds.get(0) : "";
        } else {
            Set<String> objIds = getObjIdsFromParam(saveSopVo);
            // 查询sop待修订消息
            ignoreSopReviseMsgBySopIdAndObjIds(objIds,saveSopVo.getSopId());
            // 根据SOP查询
            List<String> refSopIds = getSopRefSops(saveSopVo.getSteps());
            SopEdit sopEdit = BeanUtil.copyProperties(saveSopVo, SopEdit.class);
            if (SopStatusEnum.PUBLISHED.getType().equals(sopEdit.getSopStatus())) {
                sopEdit.setPublishStatus(SopPublishStatusEnum.REVISEING.getType());
            }
            sopEdit.setRefSopIds(refSopIds);
            List<SopEdit> sopEdits = sopEditService.querySopEditBySopId(saveSopVo.getSopId());
            if (CollUtil.isNotEmpty(sopEdits)) {
                SopEdit srcSopEdit = sopEdits.get(0);
                sopEdit.setSopEditId(srcSopEdit.getSopEditId());
                sopEditService.updateById(sopEdit);
                destSopId = sopEdit.getSopId();
            } else {
                List<String> sopIds = batchAddDraftSop(Collections.singletonList(saveSopVo));
                destSopId = CollUtil.isNotEmpty(sopIds) ? sopIds.get(0) : "";
            }
        }
        if (StrUtil.isNotBlank(destSopId)) {
            sopService.updateById(Sop.builder().sopId(destSopId).publishStatus(SopPublishStatusEnum.REVISEING.getType()).build());
        }
        return new QuerySopVo(destSopId);
    }
    /***
     * Description: 忽略SOP的待修订消息
     * @param objIds : 对象id
     * @param sopId : SOP的id
     * @return : void
     * @author : lijie
     * @date :2021/4/23 20:12
     * Update By lijie 2021/4/23 20:12
     */
    private void ignoreSopReviseMsgBySopIdAndObjIds(Set<String> objIds, String sopId) {
        if (StrUtil.isBlank(sopId) || CollUtil.isEmpty(objIds)){
            return;
        }
        List<SopReviseMsg> sopReviseMsgs = sopReviseMsgService.queryDatasByCondition(sopId, null, null);
        List<SopReviseMsg> ignoreSopReviseMsgs = new ArrayList<>();
        for (SopReviseMsg sopReviseMsg : sopReviseMsgs) {
            if (objIds.contains(sopReviseMsg.getObjId())) {
                sopReviseMsg.setStatus(MsgStatusEnum.HANDLED.getType());
                ignoreSopReviseMsgs.add(sopReviseMsg);
            }
        }
        if (CollUtil.isEmpty(ignoreSopReviseMsgs)){
            return;
        }
        sopReviseMsgService.updateBatchById(ignoreSopReviseMsgs,ignoreSopReviseMsgs.size());
    }

    /***
     * Description: 验证sop
     * @param saveSopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/2 15:28
     * Update By lijie 2021/4/2 15:28
     */
    @Override
    public List<String> verifySopForPublish(SaveSopVo saveSopVo) throws Exception {
        //1 验证必填项
        //2 验证对象是否存在
        //3 验证引用sop
        List<String> content = new ArrayList<>();
        Set<String> lackFields = validRequiredFields(saveSopVo);
        if (CollUtil.isNotEmpty(lackFields)) {
            content.add("尚有必填项未填写!" + JSONObject.toJSONString(lackFields));
        }
        // 获得关联对象id集合
        Set<String> objIds = getSopObjects(saveSopVo);
        if (CollUtil.isNotEmpty(objIds)) {
            String objectStr = objectService.queryBatchObj(JSONObject.toJSONString(objIds), saveSopVo.getProjectId(), false);
            JSONObject objectJson = JSONObject.parseObject(objectStr);
            if (DBConst.Result.FAILURE.equals(objectJson.getString(DBConst.Result.RESULT))) {
                throw new BusinessException("查询关联对象信息失败");
            }
            Map<String, String> objectIdName = objectService.getObjectIdNameMap(objectStr);
            for (String id : objectIdName.keySet()) {
                content.add("添加的" + objectIdName.get(id) + "对象已作废,请前往编辑!");
            }
        }
        List<String> refSops = getSopRefSops(saveSopVo.getSteps());
        if (refSops.size() > 0) {
            List<Sop> sopList = sopService.listByIds(refSops);
            for (Sop sop : sopList) {
                if (!sop.getValid()) {
                    content.add("引用的" + sop.getSopName() + "标准操作流程已被作废,请前往编辑!");
                }
            }
        }
        return content;
    }

    /***
     * Description: 新增sop时验证sop名称
     * @param queryVerifySopNameBaseVo : 请求参数
     * @return : com.persagy.fm.sop.model.vo.VerifySopNameResult
     * @author : lijie
     * @date :2021/4/2 12:19
     * Update By lijie 2021/4/2 12:19
     */
    @Override
    public VerifySopNameVo verifySopNameForAdd(QueryVerifySopNameBaseVo queryVerifySopNameBaseVo) {
        return verifySopNameForUpdate(queryVerifySopNameBaseVo);
    }

    /***
     * Description: 编辑sop时验证sop名称
     * @param queryVerifySopNameBaseVo : 请求参数
     * @return : com.persagy.fm.sop.model.vo.VerifySopNameResult
     * @author : lijie
     * @date :2021/4/2 12:19
     * Update By lijie 2021/4/2 12:19
     */
    @Override
    public VerifySopNameVo verifySopNameForUpdate(QueryVerifySopNameBaseVo queryVerifySopNameBaseVo) {
        return VerifySopNameVo.builder()
                .canUse(checkSopNameRepeated(queryVerifySopNameBaseVo.getSopName(),
                        queryVerifySopNameBaseVo.getSopId(),
                        queryVerifySopNameBaseVo.getProjectId()))
                .build();
    }

    /***
     * Description: 根据SOP的id集合查询SOP列表
     * @param sopIds : sop的id数组
     * @return : java.util.List<com.persagy.fm.sop.entity.Sop>
     * @author : lijie
     * @date :2021/4/10 14:07
     * Update By lijie 2021/4/10 14:07
     */
    @Override
    public List<SopBean> queryPublishedSopListByIds(List<String> sopIds) {
        if (CollUtil.isEmpty(sopIds)) {
            return new ArrayList<>();
        }
        // 1.根据SOP的id集合查询SOP内容
        List<Sop> sops = sopService.listByIds(sopIds);
        // 2.填充SOP的细节
        return fillSopDetail(sops);
    }

    /***
     * Description: 当对象名称变化或对象删除时更新sop
     * @param objectChangeVo : 请求参数
     * @author : lijie
     * @date :2021/4/10 14:34
     * Update By lijie 2021/4/10 14:34
     */
    @Override
    public void updateSopWhenObjectChange(ObjectChangeVo objectChangeVo, boolean isUpdateName) {
        // 1.根据对象id查询关联的SOP的id
        List<SopObjRel> sopObjRels = querySopObjRelByObjId(objectChangeVo.getMsObjId());
        if (CollUtil.isEmpty(sopObjRels)) {
            return;
        }
        Set<String> sopIds = sopObjRels.stream().map(SopObjRel::getSopId).collect(Collectors.toSet());
        if (CollUtil.isEmpty(sopIds)) {
            return;
        }
        if (!isUpdateName) {
            // 1.保存消息并修改SOP的修订状态
            if (!deleteObject(sopIds, objectChangeVo)) {
                throw new BusinessException("添加待修订消息或修改sop状态失败");
            }
            return;
        }
        // 更新对象名称
        for (SopObjRel sopObjRel : sopObjRels) {
            sopObjRel.setObjName(objectChangeVo.getMsObjName());
        }
        // PS:旧的逻辑里有一个更换step_content中的content内容里的@对象名称替换为@对象名称.但接口文档中没有content这个字段
        sopObjRelService.updateBatchById(sopObjRels);
    }

    /***
     * Description: 查询已发布的sop列表
     * @param queryPublishedListVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 15:56
     * Update By lijie 2021/4/10 15:56
     */
    @Override
    public PublishedListVo queryPublishedSopList(QueryPublishedListVo queryPublishedListVo) throws Exception {
        //========1.根据update和项目id查询======
        LambdaQueryWrapper<Sop> queryWrapper = new LambdaQueryWrapper<>();
        // 项目id
        Set<String> projectIds = new HashSet<>();
        projectIds.add(queryPublishedListVo.getProjectId());
        if (!SopCommonConst.GROUP_SOP_ID.equals(queryPublishedListVo.getProjectId())
             && (CollUtil.isEmpty(queryPublishedListVo.getSignCodes()) || queryPublishedListVo.getSignCodes().contains("public"))){
            projectIds.add(SopCommonConst.GROUP_SOP_ID);
        }
        queryWrapper.in(Sop::getProjectId,projectIds);
        queryWrapper.eq(Sop::getValid,true);
        queryWrapper.eq(Sop::getSopStatus, SopStatusEnum.PUBLISHED.getType());
        // 更新时间
        String updateTimeFrom = queryPublishedListVo.getUpdateTimeFrom();
        String updateTimeTo = queryPublishedListVo.getUpdateTimeTo();
        if (StrUtil.isNotBlank(updateTimeFrom)) {
            queryWrapper.ge(Sop::getUpdateTime,DateUtil.transferDateFormat(updateTimeFrom, CommonConstant.date_format_show, CommonConstant.date_format_save));
        }
        if (StrUtil.isNotBlank(updateTimeTo)) {
            queryWrapper.le(Sop::getUpdateTime,DateUtil.transferDateFormat(updateTimeTo, CommonConstant.date_format_show, CommonConstant.date_format_save));
        }
        // SOP名称
        if (StrUtil.isNotBlank(queryPublishedListVo.getSopName())) {
            queryWrapper.like(Sop::getSopName, queryPublishedListVo.getSopName());
        }
        // ========2.移除待修订状态sop=======
        if (!SopCommonConst.GROUP_SOP_ID.equals(queryPublishedListVo.getProjectId())) {
            queryWrapper.and(wrapper->{
                wrapper.or(subWrapper->subWrapper.ne(Sop::getProjectId,SopCommonConst.GROUP_SOP_ID));
                wrapper.or(subWrapper->{
                    subWrapper.ne(Sop::getPublishStatus,SopPublishStatusEnum.REVISE.getType());
                    subWrapper.eq(Sop::getProjectId, SopCommonConst.GROUP_SOP_ID);
                });
            });
            String rootProjFuncType = objectService.queryRootProjFuncTypeById(queryPublishedListVo.getProjectId());
            queryWrapper.and(wrapper->{
                wrapper.or(subWrapper->subWrapper.isNull(Sop::getExtendField));
                wrapper.or(subWrapper->{
                    subWrapper.isNotNull(Sop::getExtendField);
                    subWrapper.eq(Sop::getExtendField,rootProjFuncType);
                });
            });
        }

        List<Sop> sops = sopService.list(queryWrapper);

        //=========3.筛选SOP========
        sops = filterSopBySignCodes(queryPublishedListVo.getSignCodes(), sops, queryPublishedListVo.getProjectId());
        sops = filterSopByFitScope(queryPublishedListVo, sops);

        //=========4设置返回字段======
        List<SopBean> sopBeans = fillSopDetail(sops);
        sopBeans = additionalSopDetail(sopBeans, queryPublishedListVo.getProjectId(), "1");
        PublishedListVo item = new PublishedListVo();
        //------------设置过滤项-----
        if (queryPublishedListVo.getNeedReturnCriteria()) {
            getSopListCriteria(sopBeans, item);
        }
        item.setContent(sopBeans);
        item.setCount(sopBeans.size());
        return item;
    }

    /***
     * Description: 查询编辑中的SOP列表
     * @param queryPublishedListBaseVo : 请求参数
     * @return : java.util.List<com.persagy.fm.sop.model.dto.SopBean>
     * @author : lijie
     * @date :2021/4/10 18:12
     * Update By lijie 2021/4/10 18:12
     */
    @Override
    public List<SopBean> queryEditingSopList(QueryPublishedListBaseVo queryPublishedListBaseVo) throws Exception {
        QueryWrapper<SopEdit> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(SopEdit.VALID, ValidEnum.TRUE.getType());
        if (StrUtil.isNotBlank(queryPublishedListBaseVo.getSopName())) {
            queryWrapper.like(SopEdit.SOP_NAME, queryPublishedListBaseVo.getSopName());
        }
        //sop状态：0-草稿、1-已发布
        queryWrapper.eq(SopEdit.SOP_STATUS, SopStatusEnum.DRAFT.getType());
        queryWrapper.orderByDesc(SopEdit.UPDATE_TIME);
        List<SopEdit> editList = sopEditService.list(queryWrapper);
        return filterRemind(editList, false);
    }

    /***
     * Description: 查询已作废的sop列表
     * @param queryPublishedListBaseVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 18:27
     * Update By lijie 2021/4/10 18:27
     */
    @Override
    public List<SopBean> queryDestroyedSopList(QueryPublishedListBaseVo queryPublishedListBaseVo) throws Exception {
        String sopName = queryPublishedListBaseVo.getSopName();
        QueryWrapper<Sop> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Sop.SOP_STATUS, SopStatusEnum.INVALID.getType());
        queryWrapper.eq(Sop.VALID, ValidEnum.FALSE.getType());
        if (StrUtil.isNotBlank(sopName)) {
            queryWrapper.like(Sop.SOP_NAME, sopName);
        }
        queryWrapper.orderByDesc(SopEdit.UPDATE_TIME);
        List<Sop> list = sopService.list(queryWrapper);
        return filterRemindSopList(list, true);
    }

    /***
     * Description: 根据Id查询已发布的sop信息
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 18:47
     * Update By lijie 2021/4/10 18:47
     */
    @Override
    public SopBean queryPublishedSopById(QuerySopVo querySopVo) throws Exception {
        // 1 处理sop.steps查询所有引用sop_id、根据sop_id批量查询sops、根据引用sop_id更新sop.steps
        // 2 判断sop.publishStatus为待修订、修订中，查询修订消息，标记修订对象为is_revise:true
        List<Sop> sops = sopService.listByIds(Collections.singletonList(querySopVo.getSopId()));
        List<SopBean> sopBeans = fillSopDetail(sops);
        if (CollUtil.isEmpty(sopBeans)) {
            throw new BusinessException("SOP不存在,请刷新页面");
        }
        SopBean sopBean = sopBeans.get(0);
        List<EquipModelBean> equipModelBeans = queryEquipModelsByIds(sopBean.getEquipModelIds());
        //处理order_type
        List<String> orderTypeName = queryOrderTypeNameByIds(sopBean.getOrderType());
        Map<String, SopReviseMsg> changedObj=null;
        String publishStatus = sopBean.getPublishStatus();
        if ("2".equals(publishStatus) || "3".equals(publishStatus)) {
            //查询变化
            changedObj = queryChangedObjIdBySopId(sopBean.getSopId());
        }
        Map<String, SopBean> refSopMap = proccessSopIdRefProfile(sopBean.getSopId());
        sopBean = proccessSopItem(sopBean, refSopMap);
        processDomainName(sopBean);
        processSopReviseFlag(sopBean,changedObj);
        processDomainName(sopBean);
        sopBean.setEquipModels(equipModelBeans);
        sopBean.setOrderTypeNames(orderTypeName);
        String extendField = sopBean.getExtendField();
        if (!StringUtil.isNull(extendField)) {
            sopBean.setExtendFieldName(objectService.getProFuncTypeNameById(extendField));
        }
        return sopBean;
    }

    /***
     * Description: 根据Id查询可编辑的sop信息
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/10 19:27
     * Update By lijie 2021/4/10 19:27
     */
    @Override
    public SopBean querySopForEditById(QuerySopVo querySopVo) throws Exception {
        List<SopEdit> sopEdits = sopEditService.querySopEditBySopId(querySopVo.getSopId());
        if (CollUtil.isEmpty(sopEdits)){
            return queryPublishedSopById(querySopVo);
        }
        SopEdit sopEdit = sopEdits.get(0);
        SopBean sopBean = BeanUtil.copyProperties(sopEdit, SopBean.class);
        // 1 处理sop.steps查询所有引用sop_id、根据sop_id批量查询sops、根据引用sop_id更新sop.steps
        // 2 判断sop.publishStatus为待修订、修订中，查询修订消息，标记修订对象为is_revise:true
        List<EquipModelBean> equipModelBeans = queryEquipModelsByIds(sopBean.getEquipModelIds());
        List<String> orderTypeNames = queryOrderTypeNameByIds(sopBean.getOrderType());
        Map<String, SopReviseMsg> changedObj=null;
        String publishStatus = sopBean.getPublishStatus();
        if ("2".equals(publishStatus) || "3".equals(publishStatus)) {
            //查询变化
            changedObj = queryChangedObjIdBySopId(sopBean.getSopId());
        }
        Map<String, SopBean> refSops = proccessSopIdRefProfile(sopBean.getSopId());
        proccessSopItem(sopBean,refSops);
        processSopReviseFlag(sopBean,changedObj);
        processDomainName(sopBean);
        String extendField = sopBean.getExtendField();
        sopBean.setEquipModels(equipModelBeans);
        sopBean.setOrderTypeNames(orderTypeNames);
        if (!StringUtil.isNull(extendField)) {
            sopBean.setExtendFieldName(objectService.getProFuncTypeNameById(extendField));
        }
        return sopBean;
    }
    /***
     * Description: 处理SOP的专业名称
     * @param sopBean : SOP的对象
     * @return : void
     * @author : lijie
     * @date :2021/4/23 15:09
     * Update By lijie 2021/4/23 15:09
     */
    private void processDomainName(SopBean sopBean) {
        Map<String, String> domainNameMap = queryAllDomainNameMap();
        List<String> domainNames = new ArrayList<>();
        List<String> domainIds = CollUtil.defaultIfEmpty(sopBean.getDomains(), new ArrayList<>());
        for (String domainId : domainIds) {
            domainNames.add(domainNameMap.getOrDefault(domainId,""));
        }
        sopBean.setDomainNames(domainNames);
        List<SopCell> steps = CollUtil.defaultIfEmpty(sopBean.getSteps(), new ArrayList<>());
        for (SopCell step : steps) {
            List<SopCell> stepContents = CollUtil.defaultIfEmpty(step.getStepContent(), new ArrayList<>());
            for (SopCell stepContent : stepContents) {
                if (StrUtil.isBlank(stepContent.getDomain())){
                    continue;
                }
                stepContent.setDomainName(domainNameMap.getOrDefault(stepContent.getDomain(),""));
            }
        }
    }

    /***
     * Description: 处理SOP的未修订标记
     * @param sopBean : SOP对象
     * @param changedObj : 更改对象标记
     * @return : void
     * @author : lijie
     * @date :2021/4/23 14:46
     * Update By lijie 2021/4/23 14:46
     */
    private void processSopReviseFlag(SopBean sopBean, Map<String, SopReviseMsg> changedObj) {
        if (null==changedObj || changedObj.size()<=0){
            return;
        }
        Set<String> changedObjIds = changedObj.keySet();
        // 1.处理适用范围
        List<SopObjRel> fitObjs = CollUtil.defaultIfEmpty(sopBean.getFitObjs(), new ArrayList<>());
        for (SopObjRel fitObj : fitObjs) {
            if (changedObjIds.contains(fitObj.getObjId())){
                fitObj.setIsRevise(true);
            }
        }
        // 2.处理步骤
        List<SopCell> steps = CollUtil.defaultIfEmpty(sopBean.getSteps(),new ArrayList<>());
        // 时间复杂度: O(n^5),呵呵....
        for (SopCell step : steps) {
            List<SopCell> stepContents = CollUtil.defaultIfEmpty(step.getStepContent(), new ArrayList<>());
            for (SopCell stepContent : stepContents) {
                // 2.1 操作内容
                List<SopObjRel> contentObjs = CollUtil.defaultIfEmpty(stepContent.getContentObjs(), new ArrayList<>());
                for (SopObjRel contentObj : contentObjs) {
                    if (changedObjIds.contains(contentObj.getObjId())){
                        contentObj.setIsRevise(true);
                        continue;
                    }
                    List<ParentsBean> parentsBeans = CollUtil.defaultIfEmpty(contentObj.getParents(), new ArrayList<>());
                    for (ParentsBean parentsBean : parentsBeans) {
                        List<String> parentIds = CollUtil.defaultIfEmpty(parentsBean.getParentIds(), new ArrayList<>());
                        for (String parentId : parentIds) {
                            if (changedObjIds.contains(parentId)){
                                contentObj.setIsRevise(true);
                                break;
                            }
                        }
                    }

                }
                // 2.2 确认结果
                List<SopCell> confirmResults = CollUtil.defaultIfEmpty(stepContent.getConfirmResult(), new ArrayList<>());
                for (SopCell confirmResult : confirmResults) {
                    // 原逻辑里有作业操作内容中有对象的内容,从接口和页面上来判断,并没有这个内容.故这里就不加了
                    // 信息点
                    List<InfoPointsBean> infoPointsBeans = CollUtil.defaultIfEmpty(confirmResult.getInfoPoints(), new ArrayList<>());
                    for (InfoPointsBean infoPointsBean : infoPointsBeans) {
                        if (changedObjIds.contains(infoPointsBean.getId())){
                            infoPointsBean.setIsRevise(true);
                        }
                    }
                }
            }
        }



    }

    /***
     * Description: 作废sop信息
     * @param queryDestroySopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/13 9:10
     * Update By lijie 2021/4/13 9:10
     */
    @Override
    public void destroySopById(QueryDestroySopVo queryDestroySopVo) throws Exception {
        // 1.删除当前sop副本
        sopEditService.deleteDraftBySopId(queryDestroySopVo.getSopId());
        // 2.补全sop当前引用sop版本及状态，更新版本说明，不提交版本数据至版本表
        List<Sop> sops = sopService.listByIds(Collections.singletonList(queryDestroySopVo.getSopId()));
        if (CollUtil.isEmpty(sops)){
            return;
        }
        Sop sop = sops.get(0);
        // 2.1 作废SOP内容
        sop.setSopStatus(SopStatusEnum.INVALID.getType());
        sop.setOperatorId(queryDestroySopVo.getOperator().getId());
        sop.setOperatorName(queryDestroySopVo.getOperator().getName());
        sop.setVersionExplain(Collections.singletonList("作废标准操作"));
        sop.setValid(false);
        sopService.updateById(sop);
        // 2.2 删除SOP的各种关系
        destoryAllRelsBySopId(sop.getSopId());
        // 3.查询所有引用当前sop的sop，移除sop.steps中的引用，更新版本，提交版本至版本表，移除sop引用关系表数据
        revisionSopStepWhenRefSopDeleted(queryDestroySopVo.getUserId(),sop);
    }
    /***
     * Description: 恢复已作废的sop信息
     * @param queryDestroySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/13 16:12
     * Update By lijie 2021/4/13 16:12
     */
    @Override
    public List<String> recoverSopById(QueryDestroySopVo queryDestroySopVo) throws Exception {
        List<Sop> sops = sopService.listByIds(Collections.singletonList(queryDestroySopVo.getSopId()));
        if (CollUtil.isEmpty(sops)){
            return new ArrayList<>();
        }
        List<SopBean> sopBeans = fillSopDetail(sops);
        SopBean sopBean = sopBeans.get(0);
        Set<String> needRemoveCellIds = new HashSet<>();
        // 1.处理sop引用  -----移除已作废的sop引用
        LambdaQueryWrapper<SopCell> sopCellQueryWrapper = new LambdaQueryWrapper<>();
        sopCellQueryWrapper.eq(SopCell::getSopId,sopBean.getSopId());
        List<SopCell> relSopCells = CollUtil.defaultIfEmpty(sopCellService.list(sopCellQueryWrapper),new ArrayList<>());
        Set<String> relSopIds = relSopCells.stream()
                .filter(sopCell -> StrUtil.isNotBlank(sopCell.getRelSopId()))
                .map(SopCell::getRelSopId)
                .collect(Collectors.toSet());
        LambdaQueryWrapper<Sop> relSopWrapper = new LambdaQueryWrapper<>();
        relSopWrapper.in(Sop::getSopId,relSopIds);
        relSopWrapper.eq(Sop::getSopStatus,SopStatusEnum.PUBLISHED.getType());
        List<Sop> relSops = sopService.list(relSopWrapper);
        Set<String> publishedSopIds = relSops.stream().map(Sop::getSopId).collect(Collectors.toSet());
        relSopIds.removeAll(publishedSopIds);
        removeRefFromStep(sopBean,relSopIds,needRemoveCellIds);
        if (CollUtil.isEmpty(sopBean.getSteps())) {
            return Collections.singletonList(SopCommonConst.remind_recovery_failed);
        }
        //处理对象
        //查询所有对象ID
        JSONArray objType = JSONArray.parseArray("[\"build\",\"floor\",\"space\",\"system\",\"equip\"]");
        JSONArray dictType = JSONArray.parseArray("[\"system_class\",\"equip_class\",\"space_class\"]");
        JSONArray objIds = new JSONArray();
        JSONArray dictIds = new JSONArray();
        // 适用对象
        List<SopObjRel> fitObjs = CollUtil.defaultIfEmpty(sopBean.getFitObjs(),new ArrayList<>());
        for (SopObjRel sopObjRel : fitObjs) {
            if (objType.contains(sopObjRel.getObjType()) && !objIds.contains(sopObjRel.getObjId())) {
                objIds.add(sopObjRel.getObjId());
            }
            if (dictType.contains(sopObjRel.getObjType())) {
                dictIds.add(sopObjRel.getObjId());
            }
        }
        // 步骤
        List<SopCell> steps = CollUtil.defaultIfEmpty(sopBean.getSteps(),new ArrayList<>());
        for (SopCell sopStep : steps) {
            if (sopStep.getFromSop()){
                continue;
            }
            List<SopCell> stepContents = CollUtil.defaultIfEmpty(sopStep.getStepContent(), new ArrayList<>());
            for (SopCell stepContent : stepContents) {
                if (stepContent.getFromSop()){
                    continue;
                }
                List<SopObjRel> sopObjRels = CollUtil.defaultIfEmpty(stepContent.getContentObjs(), new ArrayList<>());
                for (SopObjRel sopObjRel : sopObjRels) {
                    if (objType.contains(sopObjRel.getObjType()) && !objIds.contains(sopObjRel.getObjId())) {
                        objIds.add(sopObjRel.getObjId());
                    }
                    if (dictType.contains(sopObjRel.getObjType())) {
                        dictIds.add(sopObjRel.getObjId());
                    }
                }
            }
        }
        //根据objIds、dictIds查询是否存在
        String publishStatus = SopPublishStatusEnum.NORMAL.getType();
        String objStr = objectService.queryBatchObj(objIds.toJSONString(), sopBean.getProjectId(), true);
        List<String> objIdList = proccessQueryResult(JSONObject.parseObject(objStr), "id");
        if(objIdList != null) {
            for (String objId : objIdList) {
                objIds.remove(objId);
            }
        }else{
            publishStatus = SopPublishStatusEnum.REVISE.getType();
        }
        dictIds = JSONUtil.distinctArray(dictIds);
        List<String> dictIdList = objectService.queryBatchAllDomainOrInfoPoint(dictIds.toJSONString());
        if(dictIdList != null) {
            for (String dictId : dictIdList) {
                dictIds.remove(dictId);
            }
        }else{
            publishStatus = SopPublishStatusEnum.REVISE.getType();
        }
        objIds.addAll(dictIds);
        if(objIds.size() > 0){
            removeObjFromStep(sopBean,objIds,needRemoveCellIds);
            if (CollUtil.isEmpty(sopBean.getSteps())){
                return Collections.singletonList("SOP操作对象已失效，恢复失败！");
            }
        }
        sopBean.setOperator(queryDestroySopVo.getOperator());
        sopBean.setSopStatus(SopStatusEnum.PUBLISHED.getType());
        sopBean.setVersionExplain(Collections.singletonList("恢复标准操作"));
        if (CollUtil.isEmpty(objIds) || CollUtil.isEmpty(relSopIds)){
            sopBean.setVersionExplain(Collections.singletonList("恢复标准操作，清除了废弃的对象和标准操作"));
        }
        sopBean.setPublishStatus(publishStatus);
        SaveSopVo saveSopVo = BeanUtil.copyProperties(sopBean, SaveSopVo.class);
        publishSop(saveSopVo);
        return new ArrayList<>();
    }

    /***
     * Description: 根据对象id移除SOP
     * @param sopBean : sop对象
     * @param objIds : 对象id数组
     * @return : void
     * @author : lijie
     * @date :2021/4/22 10:36
     * Update By lijie 2021/4/22 10:36
     */
    private void removeObjFromStep(SopBean sopBean, JSONArray objIds, Set<String> needRemoveCellIds) {
        if (null==sopBean || null==objIds || objIds.isEmpty()){
            return;
        }
        // 适用范围
        List<SopCell> sopSteps = CollUtil.defaultIfEmpty(sopBean.getSteps(), new ArrayList<>());
        for (int m = sopSteps.size() - 1; m >= 0; m--) {
            SopCell sopStep = sopSteps.get(m);
            if (sopStep.getFromSop()){
                continue;
            }
            List<SopCell> sopStepContents = CollUtil.defaultIfEmpty(sopStep.getStepContent(), new ArrayList<>());
            for (SopCell sopStepContent : sopStepContents) {
                if (sopStepContent.getFromSop()){
                    continue;
                }
                List<SopObjRel> contentObjs = CollUtil.defaultIfEmpty(sopStepContent.getContentObjs(),new ArrayList<>());
                for (int i = contentObjs.size() - 1; i >= 0; i--) {
                    SopObjRel contentObj = contentObjs.get(i);
                    if (objIds.contains(contentObj.getObjId())){
                        contentObjs.remove(i);
                    }
                }
                if (CollUtil.isEmpty(contentObjs)){
                    sopStepContent.setSpaceScope(null);
                }
                List<SopCell> confirmResults = CollUtil.defaultIfEmpty(sopStepContent.getConfirmResult(),new ArrayList<>());
                for (int i = confirmResults.size() - 1; i >= 0; i--) {
                    SopCell confirmResult = confirmResults.get(i);
                    List<InfoPointsBean> infoPointsBeans = CollUtil.defaultIfEmpty(confirmResult.getInfoPoints(), new ArrayList<>());
                    for (int j = infoPointsBeans.size() - 1; j >= 0; j--) {
                        InfoPointsBean infoPointsBean = infoPointsBeans.get(j);
                        if (objIds.contains(infoPointsBean.getId())){
                            infoPointsBeans.remove(j);
                        }
                    }
                }
            }
            if (CollUtil.isEmpty(sopStepContents)){
                sopSteps.remove(m);
            }
        }
    }

    /***
     * Description: 查询可供选择的sop，复制、引用时用
     * @param queryPublishedSelVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/14 12:23
     * Update By lijie 2021/4/14 12:23
     */
    @Override
    public PublishedListVo querySopListForSel(QueryPublishedSelVo queryPublishedSelVo) throws Exception {
        //========1.根据update和项目id查询======
        QueryWrapper<Sop> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Sop.PROJECT_ID, queryPublishedSelVo.getProjectId());
        String updateTimeFrom = queryPublishedSelVo.getUpdateTimeFrom();
        String updateTimeTo = queryPublishedSelVo.getUpdateTimeTo();
        if (!StringUtil.isNull(updateTimeFrom)) {
            queryWrapper.ge(Sop.UPDATE_TIME, DateUtil.transferDateFormat(updateTimeFrom, CommonConstant.date_format_show, CommonConstant.date_format_save));
        }
        if (!StringUtil.isNull(updateTimeTo)) {
            queryWrapper.le(Sop.UPDATE_TIME, DateUtil.transferDateFormat(updateTimeTo, CommonConstant.date_format_show, CommonConstant.date_format_save));
        }
        if (StrUtil.isNotBlank(queryPublishedSelVo.getSopName())) {
            queryWrapper.like(Sop.SOP_NAME, queryPublishedSelVo.getSopName());
        }
        List<Sop> sops = sopService.list(queryWrapper);
        //========2.移除待修订状态sop=======
        removeReviseSop(sops);
        if (!SopCommonConst.GROUP_SOP_ID.equals(queryPublishedSelVo.getProjectId())) {
            String rootProjFuncType = objectService.queryRootProjFuncTypeById(queryPublishedSelVo.getProjectId());
            removeWithoutProJFuncType(sops, rootProjFuncType);
        }
        removeRelSop(sops,queryPublishedSelVo.getProjectId(),queryPublishedSelVo.getSopId());
        List<SopBean> sopBeans = fillSopDetail(sops);
        if (null!=queryPublishedSelVo.getContainsClassObj()
                && !queryPublishedSelVo.getContainsClassObj()) {
            for (int i = sopBeans.size()-1; i >= 0; i--) {
                SopBean sopBean = sopBeans.get(i);
                //过滤掉操作内容有大类对象的sop
                if (isContainsClassObj(sopBean)) {
                    sops.remove(i);
                }
            }
        }

        //=========3.筛选SOP========
        sops = filterSopBySignCodes(queryPublishedSelVo.getSignCodes(), sops, queryPublishedSelVo.getProjectId());
        sops = filterSopByFitScope(queryPublishedSelVo, sops);

        //=========4设置返回字段======
        sopBeans = additionalSopDetail(sopBeans, queryPublishedSelVo.getProjectId(), "1");
        PublishedListVo item = new PublishedListVo();
        //------------设置过滤项-----
        if (null!=queryPublishedSelVo.getNeedReturnCriteria()
                && queryPublishedSelVo.getNeedReturnCriteria()) {
            getSopListCriteria(sopBeans, item);
        }
        for (Sop sop : sops) {
            sopBeans.add(BeanUtil.copyProperties(sop, SopBean.class));
        }
        item.setContent(sopBeans);
        item.setCount(sopBeans.size());
        return item;
    }
    /***
     * Description: 查询当前sop被引用的sop列表
     * @param queryReferencedSopVo : 请求参数
     * @return : java.util.List<com.persagy.fm.sop.model.vo.ReferencedSopListVo>
     * @author : lijie
     * @date :2021/4/19 10:53
     * Update By lijie 2021/4/19 10:53
     */
    @Override
    public List<RefedSopBean> queryReferencedSopListById(QueryReferencedSopVo queryReferencedSopVo) throws ParseException {
        Sop sop = sopService.getById(queryReferencedSopVo.getSopId());
        if (sop.getVersion().equals(queryReferencedSopVo.getVersion())){
            // 1.通过sop_sop_rel关系表查询到所有引用当前sop的sopID
            LambdaQueryWrapper<SopCell> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(SopCell::getRelSopId,queryReferencedSopVo.getSopId());
            List<SopCell> sopCells = sopCellService.list(queryWrapper);
            if (CollUtil.isEmpty(sopCells)){
                return new ArrayList<>();
            }
            // 2.根据引用的SOP的id查询所有SOP列表
            Set<String> refSopIds = sopCells.stream().map(SopCell::getSopId).collect(Collectors.toSet());
            List<Sop> refSops = sopService.listByIds(refSopIds);
            if (CollUtil.isEmpty(refSops)){
                return new ArrayList<>();
            }
            List<RefedSopBean> result = new ArrayList<>();
            for (Sop refSop : refSops) {
                RefedSopBean sopListVo = RefedSopBean.builder()
                        .refSopId(refSop.getSopId())
                        .refSopName(refSop.getSopName())
                        .version(refSop.getVersion())
                        .sopType(refSop.getSopType())
                        .statusExplain(getSopStatus(refSop.getSopStatus(), refSop.getPublishStatus()))
                        .updateTime(DateUtil.transferDateFormat(refSop.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show))
                        .build();
                if (!SopCommonConst.GROUP_SOP_ID.equals(refSop.getProjectId())){
                    sopListVo.setProjectId(refSop.getProjectId());
                    sopListVo.setProjectName(projectCache.getProjectNameById(refSop.getProjectId()));
                }
                if (StrUtil.isNotBlank(refSop.getOperatorId())
                        && StrUtil.isNotBlank(refSop.getOperatorName())){
                    sopListVo.setOperator(OperatorBean.builder().id(refSop.getOperatorId()).name(refSop.getOperatorName()).build());
                }
                result.add(sopListVo);
            }
            return result;
        }
        //sop为历史版本的sop
        SopHis sopHis = sopHisService.getById(queryReferencedSopVo.getSopId());
        if (null==sopHis || CollUtil.isEmpty(sopHis.getRefedSops())){
            return new ArrayList<>();
        }
        return sopHis.getRefedSops();
    }
    /***
     * Description:取消修订的sop信息
     * @param querySopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/19 12:30
     * Update By lijie 2021/4/19 12:30
     */
    @Override
    public void cancelReviseSopById(QuerySopVo querySopVo) throws Exception {
        //1 删除当前sop副本
        LambdaQueryWrapper<SopEdit> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopEdit::getSopId,querySopVo.getSopId());
        sopEditService.remove(queryWrapper);
        //2 判断当前sop是否存在待修订消息 有:publish_status:1 没有:publish_status:0
        LambdaQueryWrapper<SopObjRel> sopObjRelWrapper = new LambdaQueryWrapper<>();
        sopObjRelWrapper.eq(SopObjRel::getSopId,querySopVo.getSopId());
        List<SopObjRel> sopObjRels = sopObjRelService.list(sopObjRelWrapper);
        if (CollUtil.isNotEmpty(sopObjRels)){
            List<String> objIds = sopObjRels.stream().map(SopObjRel::getObjId).collect(Collectors.toList());
            // 修改sop_revise_msg表中与sop和对象相关的数据 设置状态为未处理  status：1-未处理、2-已忽略、3-已自动处理
            sopReviseMsgService.updateBatchSopReviseMsg(objIds,querySopVo.getSopId());
        }
        //查询待修订消息
        List<SopReviseMsg> sopReviseMsgs = sopReviseMsgService.queryDatasByCondition(querySopVo.getSopId(), null, null);
        LambdaUpdateWrapper<Sop> updateWrapper = new LambdaUpdateWrapper<>();
        if (CollUtil.isNotEmpty(sopReviseMsgs)){
            sopService.updateById(Sop.builder().sopId(querySopVo.getSopId()).publishStatus(SopPublishStatusEnum.REVISE.getType()).build());
            return;
        }
        sopService.updateById(Sop.builder().sopId(querySopVo.getSopId()).publishStatus(SopPublishStatusEnum.NORMAL.getType()).build());
    }
    /***
     * Description: 删除草稿的sop信息
     * @param querySopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/19 19:59
     * Update By lijie 2021/4/19 19:59
     */
    @Override
    public void deleteDraftSopById(QuerySopVo querySopVo) {
        sopEditService.deleteDraftBySopId(querySopVo.getSopId());
    }
    /***
     * Description: 编辑列表发布sop前验证
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 20:06
     * Update By lijie 2021/4/19 20:06
     */
    @Override
    public List<String> verifySopOnList(QuerySopVo querySopVo) throws Exception {
        //1 查询编辑表中当前sop的编辑数据
        List<SopEdit> sopEdits = sopEditService.querySopEditBySopId(querySopVo.getSopId());
        if (CollUtil.isEmpty(sopEdits)){
            throw new BusinessException("未找到sop");
        }
        SopEdit sopEdit = sopEdits.get(0);
        SaveSopVo saveSopVo = BeanUtil.copyProperties(sopEdit, SaveSopVo.class);
        saveSopVo.setUserId(querySopVo.getUserId());
        return verifySopForPublish(saveSopVo);
    }
    /***
     * Description: 发布sop信息
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 20:53
     * Update By lijie 2021/4/19 20:53
     */
    @Override
    public QuerySopVo publishSopOnList(QuerySopVo querySopVo) throws Exception {
        List<String> verifySopOnList = verifySopOnList(querySopVo);
        if (CollUtil.isNotEmpty(verifySopOnList)){
            throw new BusinessException("缺少非空项:"+JSONObject.toJSONString(verifySopOnList));
        }
        List<SopEdit> sopEdits = sopEditService.querySopEditBySopId(querySopVo.getSopId());
        if (CollUtil.isEmpty(sopEdits)){
            throw new BusinessException("未找到sop");
        }
        SopEdit sopEdit = sopEdits.get(0);
        SaveSopVo saveSopVo = BeanUtil.copyProperties(sopEdit, SaveSopVo.class);
        saveSopVo.setUserId(querySopVo.getUserId());
        return publishSop(saveSopVo);
    }
    /***
     * Description: 查询引用的sop作废的消息列表
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 21:03
     * Update By lijie 2021/4/19 21:03
     */
    @Override
    public List<DestroyMsgVo> querySopDestroyMsgList(QuerySopVo querySopVo) throws Exception {
        List<SopDestroyMsg> sopDestroyMsgs = sopDestroyMsgService.queryRecordByCondition(querySopVo.getSopId(), null);
        if (CollUtil.isEmpty(sopDestroyMsgs)){
            return new ArrayList<>();
        }
        List<DestroyMsgVo> result = new ArrayList<>();
        for (SopDestroyMsg sopDestroyMsg : sopDestroyMsgs) {
            result.add(DestroyMsgVo.builder()
                    .msgId(sopDestroyMsg.getMsgId())
                    .refSopId(sopDestroyMsg.getRefSopId())
                    .refSopName(sopDestroyMsg.getRefSopName())
                    .refSopStatus(sopDestroyMsg.getRefSopStatus())
                    .build());
        }
        return result;
    }
    /***
     * Description: 标记sop作废消息已读
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/19 21:55
     * Update By lijie 2021/4/19 21:55
     */
    @Override
    public void markReadSopDestroyMsg(QuerySopVo querySopVo) throws Exception {
        sopDestroyMsgService.markReadSopDestroyMsg(null,querySopVo.getSopId());
    }
    /***
     * Description: 查询待修订中的消息列表
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 11:22
     * Update By lijie 2021/4/20 11:22
     */
    @Override
    public List<ReviseMsgVo> querySopReviseMsgList(QuerySopVo querySopVo) throws ParseException {
        List<SopReviseMsg> sopReviseMsgs = sopReviseMsgService.queryDatasByCondition(querySopVo.getSopId(), null, null);
        if (CollUtil.isEmpty(sopReviseMsgs)){
            return new ArrayList<>();
        }
        List<ReviseMsgVo> result = new ArrayList<>();
        for (SopReviseMsg sopReviseMsg : sopReviseMsgs) {
            result.add(ReviseMsgVo.builder()
                    .msgId(sopReviseMsg.getSopId())
                    .content(sopReviseMsg.getContent())
                    .reviseObjId(sopReviseMsg.getObjId())
                    .createTime(DateUtil.transferDateFormat(sopReviseMsg.getCreateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show))
                    .build());
        }
        return result;
    }
    /***
     * Description: 忽略修订消息
     * @param queryReviseMsgVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 12:17
     * Update By lijie 2021/4/20 12:17
     */
    @Override
    public void ignoreSopReviseMsg(QueryReviseMsgVo queryReviseMsgVo) {
        sopReviseMsgService.ignoreSopReviseMsg(queryReviseMsgVo.getMsgId(),queryReviseMsgVo.getSopId());
        List<SopReviseMsg> sopReviseMsgs = sopReviseMsgService.queryDatasByCondition(queryReviseMsgVo.getSopId(), null, null);
        if (CollUtil.isNotEmpty(sopReviseMsgs)){
            return;
        }
        sopService.updateById(Sop.builder()
                .sopId(queryReviseMsgVo.getSopId())
                .publishStatus(SopPublishStatusEnum.NORMAL.getType())
                .build());
    }
    /***
     * Description: 查询sop的历史版本列表
     * @param querySopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 14:06
     * Update By lijie 2021/4/20 14:06
     */
    @Override
    public List<SopHisListVo> querySopHisList(QuerySopVo querySopVo) throws ParseException {
        // 1.查询当前版本
        List<Sop> sopList = sopService.listByIds(Collections.singletonList(querySopVo.getSopId()));
        if (CollUtil.isEmpty(sopList)){
            return new ArrayList<>();
        }
        List<SopHisListVo> result = new ArrayList<>();
        Sop destSop = sopList.get(0);
        SopHisListVo currentHisVo = SopHisListVo.builder()
                .sopId(destSop.getSopId())
                .version(destSop.getVersion())
                .versionExplain(destSop.getVersionExplain())
                .currentVersion(true)
                .updateTime(DateUtil.transferDateFormat(destSop.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show))
                .build();
        if (StrUtil.isNotBlank(destSop.getOperatorId())
                && StrUtil.isNotBlank(destSop.getOperatorName())){
            currentHisVo.setOperator(OperatorBean.builder().id(destSop.getOperatorId()).name(destSop.getOperatorName()).build());
        }
        result.add(currentHisVo);
        // 2.查询历史版本
        LambdaQueryWrapper<SopHis> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopHis::getSopId,querySopVo.getSopId());
        List<SopHis> sopHisList = sopHisService.list(queryWrapper);
        if (CollUtil.isEmpty(sopHisList)){
            return result;
        }
        for (SopHis sopHis : sopHisList) {
            result.add(SopHisListVo.builder()
                    .sopId(sopHis.getSopId())
                    .updateTime(DateUtil.transferDateFormat(sopHis.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show))
                    .currentVersion(false)
                    .versionExplain(sopHis.getVersionExplain())
                    .version(sopHis.getVersion())
                    .operator(sopHis.getOperator())
                    .build());
        }
        return result;
    }
    /***
     * Description: 根据ID查询历史sop或者引用的sop详细信息
     * @param querySopDetailVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 14:46
     * Update By lijie 2021/4/20 14:46
     */
    @Override
    public SopBean querySopDetailById(QuerySopDetailVo querySopDetailVo) throws Exception {
        // 1.当前版本
        LambdaQueryWrapper<Sop> sopQueryWrapper = new LambdaQueryWrapper<>();
        sopQueryWrapper.eq(Sop::getSopId,querySopDetailVo.getSopId());
        sopQueryWrapper.eq(Sop::getVersion,querySopDetailVo.getVersion());
        List<Sop> sopList = sopService.list(sopQueryWrapper);
        if (CollUtil.isNotEmpty(sopList)){
            List<SopBean> sopBeans = fillSopDetail(sopList);
            SopBean sopBean = sopBeans.get(0);
            Map<String, SopBean> sopBeanMap = proccessSopIdRefProfile(sopBean.getSopId());
            SopBean sopItem = proccessSopItem(sopBean, sopBeanMap);
            sopItem.setDomainNames(queryDomainNameByIds(sopItem.getDomains()));
            sopItem.setExtendFieldName(dictionaryService.queryAllBuildingCode().getOrDefault(sopItem.getExtendField(),""));
            sopItem.setOrderTypeNames(queryOrderTypeNameByIds(sopItem.getOrderType()));
            sopItem.setEquipModels(queryEquipModelsByIds(sopItem.getEquipModelIds()));
            return sopItem;
        }
        // 2.历史版本
        LambdaQueryWrapper<SopHis> hisQueryWrapper = new LambdaQueryWrapper<>();
        hisQueryWrapper.eq(SopHis::getSopId,querySopDetailVo.getSopId());
        hisQueryWrapper.eq(SopHis::getVersion,querySopDetailVo.getVersion());
        List<SopHis> sopHisList = sopHisService.list(hisQueryWrapper);
        if (CollUtil.isEmpty(sopHisList)){
            throw new BusinessException("当前版本与id的数据不存在");
        }
        SopHis sopHis = sopHisList.get(0);
        SopBean sopBean = BeanUtil.copyProperties(sopHis, SopBean.class);
        sopBean.setDomainNames(queryDomainNameByIds(sopBean.getDomains()));
        sopBean.setExtendFieldName(dictionaryService.queryAllBuildingCode().getOrDefault(sopBean.getExtendField(),""));
        sopBean.setOrderTypeNames(queryOrderTypeNameByIds(sopBean.getOrderType()));
        sopBean.setEquipModels(queryEquipModelsByIds(sopBean.getEquipModelIds()));
        return sopBean;
    }
    /***
     * Description: 查询自动修订方案
     * @param queryAutoReviseMsgVo :请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 18:52
     * Update By lijie 2021/4/20 18:52
     */
    @Override
    public List<AutoReviseMsgVo> queryAutoRevisePlan(QueryAutoReviseMsgVo queryAutoReviseMsgVo) {
        List<SopReviseMsg> sopReviseMsgs = sopReviseMsgService.queryDatasByCondition(queryAutoReviseMsgVo.getSopId(), null, queryAutoReviseMsgVo.getReviseObjId());
        if (CollUtil.isEmpty(sopReviseMsgs)){
            return new ArrayList<>();
        }
        List<Sop> sopList = sopService.listByIds(Collections.singletonList(queryAutoReviseMsgVo.getSopId()));
        List<SopBean> sopBeans = fillSopDetail(sopList);
        if (CollUtil.isEmpty(sopBeans)){
            return new ArrayList<>();
        }
        SopReviseMsg sopReviseMsg = sopReviseMsgs.get(0);
        String objId = sopReviseMsg.getObjId();
        String objName = sopReviseMsg.getObjName();
        String objType = sopReviseMsg.getObjType();
        SopBean sopBean = sopBeans.get(0);
        List<SopCell> sopBeanSteps = CollUtil.defaultIfEmpty(sopBean.getSteps(),new ArrayList<>());
        List<AutoReviseMsgVo> result = new ArrayList<>();
        for (int i = 0; i < sopBeanSteps.size(); i++) {
            SopCell sopBeanStep = sopBeanSteps.get(i);
            if (sopBeanStep.getFromSop()){
                continue;
            }
            List<SopCell> stepContents = CollUtil.defaultIfEmpty(sopBeanStep.getStepContent(),new ArrayList<>());
            for (SopCell stepContent : stepContents) {
                if (stepContent.getFromSop()){
                    continue;
                }
                result.addAll(generateRevisePlanFromStepContent(objId, objName, objType, stepContent, i+1));
            }
        }
        //适用对象生成自动修订方案
        List<SopObjRel> fitObjs = CollUtil.defaultIfEmpty(sopBean.getFitObjs(),new ArrayList<>());
        for (SopObjRel fitObj : fitObjs) {
            if (objId.equals(fitObj.getObjId())){
                result.add(AutoReviseMsgVo.builder()
                        .doStep("适用范围")
                        .effectPart("建筑体/楼层/房间/系统/设备:" + objName)
                        .doPlan("删除该适用范围")
                        .build());
            }
        }
        return result;
    }
    /***
     * Description: 自动修订
     * @param autoReviseSopVo : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/20 21:10
     * Update By lijie 2021/4/20 21:10
     */
    @Override
    public void autoReviseSop(AutoReviseSopVo autoReviseSopVo) {
        List<SopEdit> sopEdits = sopEditService.queryRecordByKey(autoReviseSopVo.getSopId(),autoReviseSopVo.getProjectId());
        SopEdit sopEdit = sopEdits.get(0);
        if (CollUtil.isEmpty(sopEdits)){
            sopEdit = BeanUtil.copyProperties(autoReviseSopVo,SopEdit.class);
        }else {
            BeanUtil.fillBeanWithMap(BeanUtil.beanToMap(autoReviseSopVo),sopEdit, CopyOptions.create(SopEdit.class,true));
        }
        String objName=null;
        List<FitObjsBean> fitObjsBeans = CollUtil.defaultIfEmpty(sopEdit.getFitObjs(), new ArrayList<>());
        for (int i = fitObjsBeans.size() - 1; i >= 0; i--) {
            if (autoReviseSopVo.getReviseObjId().equals(fitObjsBeans.get(i).getObjId())){
                objName = fitObjsBeans.get(i).getObjName();
                fitObjsBeans.remove(i);
            }
        }
        List<SopStepsBean> sopStepsBeans = CollUtil.defaultIfEmpty(sopEdit.getSteps(), new ArrayList<>());
        for (int i = sopStepsBeans.size() - 1; i >= 0; i--) {
            SopStepsBean sopStepsBean = sopStepsBeans.get(i);
            if (sopStepsBean.getFromSop()){
                continue;
            }
            List<StepContentBean> stepContentBeans = CollUtil.defaultIfEmpty(sopStepsBean.getStepContent(), new ArrayList<>());
            for (int j = stepContentBeans.size() - 1; j >= 0; j--) {
                StepContentBean stepContentBean = stepContentBeans.get(j);
                if (stepContentBean.getFromSop()){
                    continue;
                }
                //处理content_objs、confirm_result、confirm_result.info_points
                List<ContentObjsBean> contentObjsBeans = CollUtil.defaultIfEmpty(stepContentBean.getContentObjs(), new ArrayList<>());
                for (int m = contentObjsBeans.size() - 1; m >= 0; m--) {
                    ContentObjsBean contentObjsBean = contentObjsBeans.get(m);
                    if (autoReviseSopVo.getReviseObjId().equals(contentObjsBean.getObjId())){
                        objName = contentObjsBean.getObjName();
                        contentObjsBeans.remove(m);
                    }
                }
                if (CollUtil.isEmpty(contentObjsBeans)){
                    stepContentBean.setSpaceScope(new ArrayList<>());
                }
                List<ConfirmResultBean> confirmResultBeans = CollUtil.defaultIfEmpty(stepContentBean.getConfirmResult(), new ArrayList<>());
                for (int m = confirmResultBeans.size() - 1; m >= 0; m--) {
                    ConfirmResultBean confirmResultBean = confirmResultBeans.get(m);
                    if (autoReviseSopVo.getReviseObjId().equals(confirmResultBean.getObjId())){
                        confirmResultBeans.remove(m);
                        objName = confirmResultBean.getObjName();
                        continue;
                    }
                    List<InfoPointsBean> infoPointsBeans = CollUtil.defaultIfEmpty(confirmResultBean.getInfoPoints(), new ArrayList<>());
                    for (int n = infoPointsBeans.size() - 1; n >= 0; n--) {
                        InfoPointsBean infoPointsBean = infoPointsBeans.get(n);
                        if (autoReviseSopVo.getReviseObjId().equals(infoPointsBean.getId())){
                            infoPointsBeans.remove(n);
                            objName=infoPointsBean.getName();
                        }
                    }

                }
            }
        }
        List<String> versionExplains = CollUtil.defaultIfEmpty(sopEdit.getVersionExplain(), new ArrayList<>());
        if (StrUtil.isBlank(objName)){
            versionExplains.add("自动修订标准操作");
        }else {
            versionExplains.add("移除" + objName);
        }
        sopEdit.setVersionExplain(versionExplains);
        if (SopStatusEnum.PUBLISHED.getType().equals(sopEdit.getSopStatus())){
            sopEdit.setPublishStatus(SopPublishStatusEnum.REVISEING.getType());
        }
        sopEditService.saveOrUpdate(sopEdit);
        sopReviseMsgService.autoSopReviseMsg(autoReviseSopVo.getMsgId(),autoReviseSopVo.getSopId());
        sopService.updateById(Sop.builder()
                .sopId(autoReviseSopVo.getSopId())
                .publishStatus(SopPublishStatusEnum.REVISEING.getType())
                .build());
    }
    /***
     * Description: 根据工单类型id作废sop与工单类型的关系
     * @param jsonObject :
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 15:19
     * Update By lijie 2021/4/21 15:19
     */
    @Override
    public String deleteSopOrderTypeRelByOrderType(JSONObject jsonObject) throws Exception {
        String returnStr =null;
        String orderTypeId = jsonObject.getString("order_type_id");
        //1.删除sop中适用范围中的工单类型
        LambdaQueryWrapper<SopOrderTypeRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopOrderTypeRel::getOrderType,orderTypeId);
        List<SopOrderTypeRel> sopOrderTypeRels = sopOrderTypeRelService.list(queryWrapper);
        if (CollUtil.isEmpty(sopOrderTypeRels)){
            return ToolsUtil.successJsonMsg(new ArrayList<>());
        }
        Set<String> relIds = sopOrderTypeRels.stream().map(SopOrderTypeRel::getSopOrderTypeId).collect(Collectors.toSet());
        sopOrderTypeRelService.removeByIds(relIds);
        return ToolsUtil.successJsonMsg(new JSONArray());
    }
    /***
     * Description: 构建工单令牌
     * @param jsonObject : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:40
     * Update By lijie 2021/4/21 16:40
     */
    @Override
    public String buildWorkOrder(JSONObject jsonObject) throws Exception {
        String project_id = jsonObject.getString("project_id");
        if(StringUtil.isNull(project_id)) {
            return ToolsUtil.errorJsonMsg("project_id 不能为空!");
        }
        String user_id = jsonObject.getString("user_id");
        if(StringUtil.isNull(user_id)) {
            return ToolsUtil.errorJsonMsg("user_id 不能为空!");
        }
        JSONObject woBody = jsonObject.getJSONObject("wo_body");
        if(woBody == null || woBody.size() == 0) {
            return ToolsUtil.errorJsonMsg("wo_body 不能为空!");
        }
        String order = woBody.getString("order");
        JSONArray matterArray = woBody.getJSONArray("matters");
        JSONObject result = new JSONObject();
        JSONArray returnMatters = new JSONArray();
        JSONArray matterIdArray = new JSONArray();
        JSONArray objIds = new JSONArray();
        JSONArray objNames = new JSONArray();
        for(Object matter:matterArray){
            JSONObject handleMatter = buildMatterV2(project_id, (JSONObject) matter, order);
            returnMatters.add(handleMatter);
            matterIdArray.add(handleMatter.getString("matter_id"));
            for(Object matterStepObject:handleMatter.getJSONArray("matter_steps")){
                if(StrUtil.isNotBlank(((JSONObject)matterStepObject).getString("obj_id"))){
                    objIds.add(((JSONObject)matterStepObject).getString("obj_id"));
                    objNames.add(((JSONObject)matterStepObject).getString("obj_name"));
                }
            }
        }
        JSONObject itemObject = new JSONObject();
        JSONObject workOrderObject = new JSONObject();
        itemObject.put("work_order",workOrderObject);
        JSONObject woBodyReturn = new JSONObject();
        workOrderObject.put("wo_body",woBodyReturn);
        workOrderObject.put("@obj_ids",matterIdArray);
        woBodyReturn.put("obj_ids",objIds);
        woBodyReturn.put("obj_names",objNames);
        woBodyReturn.put("matters",returnMatters);
        result.put("Result","success");
        result.put("Item",itemObject);
        return result.toJSONString();
    }

    /***
     * Description: 获得工单事项预览
     * @param jsonObject : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:47
     * Update By lijie 2021/4/21 16:47
     */
    @Override
    public String getWoMattersPreview(JSONObject jsonObject) throws Exception {
        if(StringUtil.isNull(jsonObject, "user_id")
                || StringUtil.isEmptyList(jsonObject, "draft_matters")){
            return ToolsUtil.return_error_json;
        }
        JSONArray draftMatters = jsonObject.getJSONArray("draft_matters");
        JSONObject itemObject = new JSONObject();
        // 找出所有sop中的tools数据，将其tool的name值添加到required_tools中，去重；
        JSONArray requiredTools = new JSONArray();
        // summary的值：事项名称的串连;
        String summary = null;
        // 找出所有sop和desc_works中每个工作内容用到的专业编码domain的值，添加domain_list中，去重；
        JSONArray domainList = new JSONArray();
        JSONArray publishedMatters = new JSONArray();
        // 构建published_matters的值
        if (draftMatters != null && draftMatters.size() > 0) {
            JSONObject temp;
            for (int i = 0; i < draftMatters.size(); i++) {
                temp = transferMatter(draftMatters.getJSONObject(i));
                if (null!=temp) {
                    publishedMatters.add(temp);
                }
            }
        }
        for (int i = 0; i < publishedMatters.size(); i++) {
            JSONObject matter = publishedMatters.getJSONObject(i);
            JSONArray tools = matter.getJSONArray("tools");
            if (tools != null) {
                for (Object tool : tools) {
                    JSONObject toolTemp = (JSONObject) tool;
                    if (!requiredTools.contains(toolTemp.getString("tool"))) {
                        requiredTools.add(toolTemp.getString("tool"));
                    }
                }
            }
            matter.remove("tools");
            JSONArray domains = matter.getJSONArray("domains");
            if (domains != null) {
                for (Object domain : domains) {
                    if (!domainList.contains(domain.toString())) {
                        domainList.add(domain.toString());
                    }
                }
            }
            matter.remove("domains");
            String matterName = matter.getString("matter_name");
            if (summary == null) {
                summary = matterName;
            } else {
                summary = summary + "," + matterName;
            }
        }
        itemObject.put("required_tools", requiredTools);
        itemObject.put("summary", summary);
        itemObject.put("domain_list", domainList);
        itemObject.put("published_matters", publishedMatters);
        JSONObject result = new JSONObject();
        result.put("Result","success");
        result.put("Item",itemObject);
        return JSON.toJSONString(result, SerializerFeature.DisableCircularReferenceDetect);
    }
    /***
     * Description: 我的工单-新增页:查询推荐的sop
     * @param jsonObject : 请求参数
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:59
     * Update By lijie 2021/4/21 16:59
     */
    @Override
    public String queryRecommendSop(JSONObject jsonObject) throws Exception {
        if(StringUtil.isNull(jsonObject, "user_id")){
            return ToolsUtil.return_error_json;
        }
        //是否返回包含大类对象的sop  true：返回   false：不返回 (默认返回)
        Boolean flag = jsonObject.getBoolean("is_contains_classObj");
        flag = flag == null ? true : flag;
        //适用对象
        JSONArray fitObjIds = jsonObject.getJSONArray("fit_obj_ids");
        List<String> fitObjIdList = new ArrayList<>();
        JSONObject object ;
        for(int i = 0;i<fitObjIds.size();i++){
            object = fitObjIds.getJSONObject(i);
            if (StringUtil.isNull(object.getString("obj_id"))){
                fitObjIds.remove(i);
                i--;
            }
            fitObjIdList.add(object.getString("obj_id"));
        }
        //======若无@对象查询querySopListForSel接口返回===
        if(CollectionUtils.isEmpty(fitObjIds)) {
            jsonObject.remove("order_type");
            QueryPublishedSelVo queryPublishedSelVo = new QueryPublishedSelVo();
            queryPublishedSelVo.setProjectId(jsonObject.getString("project_id"));
            queryPublishedSelVo.setContainsClassObj(flag);
            queryPublishedSelVo.setFitObjIds(fitObjIdList);
            PublishedListVo publishedListVo = querySopListForSel(queryPublishedSelVo);
            return ToolsUtil.successJsonMsg(publishedListVo.getContent());
        }
        String project_id = jsonObject.getString("project_id");
        project_id = project_id == null ? SopCommonConst.GROUP_SOP_ID : project_id;
//    	String secret = projectCache.getProjectSecretById(project_id);
        List<String> lastSopIds = null;
        if (!CollectionUtils.isEmpty(fitObjIds)) {
            //查询图实例id
            String systemEquipGraphId = graphCache.getGraphIdByProjectIdAndGraph(project_id, "SystemEquip");
            String equipinSpaceGraphId = graphCache.getGraphIdByProjectIdAndGraph(project_id, "EquipinSpace");
            String shaftandSpaceGraphId = graphCache.getGraphIdByProjectIdAndGraph(project_id, "ShaftandSpace");
            lastSopIds = new ArrayList<>();
            for (int i = 0; i < fitObjIds.size(); i++) {
                JSONObject fit_obj_id = fitObjIds.getJSONObject(i);
                String obj_id = fit_obj_id.getString("obj_id");
                String obj_type = fit_obj_id.getString("obj_type");
                JSONArray obj_ids = new JSONArray();
                JSONArray relObjs = new JSONArray();
                relObjs.add(createObj(obj_id, obj_type));
                JSONArray equip_model_ids = new JSONArray();
                //获取该建筑下的所有楼层，空间，设备
                if(CommonConstant.data_type_build.equals(obj_type)) {
                    //				obj_ids.add(obj_id);
                    String obj = objectService.queryObjById(obj_id, project_id);
                    JSONObject objJson = JSONObject.parseObject(obj);
                    String buildFuncType = objJson.getJSONArray(ResultEnum.CONTENT.getType()).getJSONObject(0).getJSONObject("infos")
                            .getString("BuildFuncType");
                    relObjs.add(createObj(buildFuncType, "build_func_type"));
                    JSONObject params = new JSONObject();
                    params.put("valid", true);
                    params.put("projectId", project_id);
                    JSONObject criteria = new JSONObject();
                    criteria.put("id", obj_id);
                    JSONArray type = new JSONArray();
                    type.add("Fl");
                    type.add("GeneralZone");
                    type.add("Eq");
                    criteria.put("type", type);
                    params.put("criteria", criteria);
                    String queryResult = objectService.httpPostRequestDataPlatformPath(
                            DataRequestPathUtil.dataPlat_object_subset_query,
                            project_id,
                            null,
                            params.toString());
                    JSONObject queryJson = JSONObject.parseObject(queryResult);

                    if(ResultEnum.SUCCESS.getType().equals(queryJson.getString(ResultEnum.RESULT.getType()))) {
                        JSONArray queryConstents = queryJson.getJSONArray(ResultEnum.CONTENT.getType());
                        if(queryConstents != null && queryConstents.size() > 0) {
                            JSONObject queryConstent,idJson;
                            String id ;
                            JSONArray equipIds = new JSONArray();
                            for (int j = 0,length=queryConstents.size(); j < length; j++) {
                                queryConstent = queryConstents.getJSONObject(j);
                                id = queryConstent.getString("id");
                                if(id.startsWith("Eq")) {
                                    //设备实例
                                    idJson = new JSONObject();
                                    idJson.put("id", id);
                                    equipIds.add(idJson);
                                } else if(id.startsWith("Fl")){
                                    //								obj_ids.add(id);
                                    relObjs.add(createObj(id, "floor"));
                                }else if (id.startsWith("Sp")) {
                                    relObjs.add(createObj(id, "space"));
                                }
                            }
                            if (equipIds!=null && equipIds.size()!=0) {
                                JSONArray equipCodes = baseService.getEquipCodeFromEquipIds(project_id, equipIds,dataPlatBasePath);
                                //							obj_ids.addAll(equipCodes);
                                for (int j = 0; j < equipCodes.size(); j++) {
                                    relObjs.add(createObj(equipCodes.getString(j), "equip_class"));
                                }
                            }
                        }
                    }
                } else if(CommonConstant.data_type_floor.equals(obj_type)) {

                    //				obj_ids.add(obj_id);
                    String obj = objectService.queryObjById(obj_id, project_id);
                    JSONObject objJson = JSONObject.parseObject(obj);
                    String floorType = objJson.getJSONArray(ResultEnum.CONTENT.getType()).getJSONObject(0).getJSONObject("infos")
                            .getString("FloorType");
                    relObjs.add(createObj(floorType, "floor_type"));
                    JSONObject params = new JSONObject();
                    params.put("valid", true);
                    params.put("projectId", project_id);
                    JSONObject criteria = new JSONObject();
                    criteria.put("id", obj_id);
                    JSONArray type = new JSONArray();
                    type.add("GeneralZone");
                    //				type.add("Eq");
                    criteria.put("type", type);
                    params.put("criteria", criteria);
                    //				String queryResult = httpPostRequest(querySubsetQueryUrl, params.toString());
                    String queryResult = objectService.httpPostRequestDataPlatformPath(
                            DataRequestPathUtil.dataPlat_object_subset_query,
                            project_id,
                            null,
                            params.toString());
                    JSONObject queryJson = JSONObject.parseObject(queryResult);

                    if(ResultEnum.SUCCESS.getType().equals(queryJson.getString(ResultEnum.RESULT.getType()))) {
                        JSONArray queryConstents = queryJson.getJSONArray(ResultEnum.CONTENT.getType());
                        if(queryConstents != null && queryConstents.size() > 0) {
                            JSONArray criterias = new JSONArray();
                            for (int j = 0; j < queryConstents.size(); j++) {
                                JSONObject queryConstent = queryConstents.getJSONObject(j);
                                String id = queryConstent.getString("id");
                                //							obj_ids.add(id);
                                relObjs.add(createObj(id, "space"));
                                criteria = new JSONObject();
                                //							criteria.put("from_id", id);
                                criteria.put("to_id", id);
                                criteria.put("graph_id", equipinSpaceGraphId);
                                criteria.put("rel_type", "1");
                                criterias.add(criteria);
                            }
                            JSONArray relations = objectService.queryRelation(project_id, null, criterias);
                            if(relations != null && relations.size() > 0) {
                                JSONArray equipIds = new JSONArray();
                                JSONObject idJson;
                                for (int j = 0; j < relations.size(); j++) {
                                    JSONObject relation = relations.getJSONObject(j);
                                    String equip_id = relation.getString("from_id");
                                    //								obj_ids.add(equip_id);
                                    relObjs.add(createObj(equip_id, "equip"));
                                    //设备类型
                                    idJson = new JSONObject();
                                    idJson.put("id", equip_id);
                                    equipIds.add(idJson);
                                }
                                if (equipIds!=null && equipIds.size()!=0) {
                                    JSONArray EquipCodes = baseService.getEquipCodeFromEquipIds(project_id, equipIds, dataPlatBasePath);
                                    //								obj_ids.addAll(EquipCodes);
                                    for (int j = 0; j < EquipCodes.size(); j++) {
                                        relObjs.add(createObj(EquipCodes.getString(j), "equip_class"));
                                    }
                                }
                            }
                        }
                    }
                }  else if(CommonConstant.data_type_shaft.equals(obj_type)) {
                    //				obj_ids.add(obj_id);
                    String obj = objectService.queryObjById(obj_id, project_id);
                    JSONObject objJson = JSONObject.parseObject(obj);
                    String shaftType = objJson.getJSONArray(ResultEnum.CONTENT.getType()).getJSONObject(0).getJSONObject("infos")
                            .getString("ShaftFuncType");
                    relObjs.add(createObj(shaftType, "shaft_type"));
                    JSONObject criteria = new JSONObject();
                    JSONArray criterias = new JSONArray();
                    criteria.put("from_id", obj_id);
                    criteria.put("graph_id", shaftandSpaceGraphId);
                    criteria.put("rel_type", "1");
                    criterias.add(criteria);
                    JSONArray shaftandSpaceGraphs = objectService.queryRelation(project_id, null, criterias);
                    if (shaftandSpaceGraphs != null && shaftandSpaceGraphs.size() > 0) {
                        criterias.clear();
                        for (int j = 0; j < shaftandSpaceGraphs.size(); j++) {
                            JSONObject shaftandSpaceGraph = shaftandSpaceGraphs.getJSONObject(j);
                            String spaceId = shaftandSpaceGraph.getString("to_id");
                            //						obj_ids.add(spaceId);
                            relObjs.add(createObj(spaceId, "space"));
                            criteria = new JSONObject();
                            criteria.put("to_id", spaceId);
                            criteria.put("graph_id", equipinSpaceGraphId);
                            criteria.put("rel_type", "1");
                            criterias.add(criteria);
                        }
                        JSONArray equipinSpaceGraphs = objectService.queryRelation(project_id, null, criterias);
                        if (equipinSpaceGraphs != null && equipinSpaceGraphs.size() > 0) {
                            JSONArray equipIds = new JSONArray();
                            JSONObject idJson;
                            for (int j = 0; j < equipinSpaceGraphs.size(); j++) {
                                JSONObject equipinSpaceGraph = equipinSpaceGraphs.getJSONObject(j);
                                String equipId = equipinSpaceGraph.getString("from_id");
                                //							obj_ids.add(equipId);
                                relObjs.add(createObj(equipId, "equip"));
                                // 设备类型
                                idJson = new JSONObject();
                                idJson.put("id", equipId);
                                equipIds.add(idJson);
                            }
                            if (equipIds != null && equipIds.size() != 0) {
                                JSONArray EquipCodes = baseService.getEquipCodeFromEquipIds(project_id, equipIds, dataPlatBasePath);
                                //							obj_ids.addAll(EquipCodes);
                                for (int j = 0; j < EquipCodes.size(); j++) {
                                    relObjs.add(createObj(EquipCodes.getString(j), "equip_class"));
                                }
                            }
                        }
                    }
                }else if(CommonConstant.data_type_space.equals(obj_type)) {
                    JSONArray criterias = new JSONArray();
                    JSONObject criteria = new JSONObject();
                    criteria.put("to_id", obj_id);
                    criteria.put("graph_id", equipinSpaceGraphId);
                    criteria.put("rel_type", "1");
                    criterias.add(criteria);
                    JSONArray relations = objectService.queryRelation(project_id, null, criterias);
                    if(relations != null && relations.size() > 0) {
                        JSONArray equipIds = new JSONArray();
                        JSONObject idJson;
                        for (int j = 0; j < relations.size(); j++) {
                            JSONObject relation = relations.getJSONObject(j);
                            String equip_id = relation.getString("from_id");
                            //						obj_ids.add(equip_id);
                            relObjs.add(createObj(equip_id, "equip"));
                            //设备类型
                            idJson = new JSONObject();
                            idJson.put("id", equip_id);
                            equipIds.add(idJson);
                        }
                        if (equipIds!=null && equipIds.size()!=0) {
                            JSONArray EquipCodes = baseService.getEquipCodeFromEquipIds(project_id, equipIds, dataPlatBasePath);
                            //						obj_ids.addAll(EquipCodes);
                            for (int j = 0; j < EquipCodes.size(); j++) {
                                relObjs.add(createObj(EquipCodes.getString(j), "equip_class"));
                            }
                        }
                    }
                    //查询空间所属空间类
                    String queryResult = objectService.queryObjById(obj_id,project_id);
                    if (queryResult.contains(ResultEnum.SUCCESS.getType()) && queryResult.contains(ResultEnum.CONTENT.getType())) {
                        JSONObject queryJson = JSONObject.parseObject(queryResult);
                        JSONArray queryContents = queryJson.getJSONArray(ResultEnum.CONTENT.getType());
                        if (queryContents != null && queryContents.size() > 0) {
                            JSONObject queryItem = queryContents.getJSONObject(0);
                            queryItem = queryItem.getJSONObject("infos");
                            if (queryItem != null) {
                                String RoomFuncType = queryItem.getString("ProjRoomFuncType");
                                if (RoomFuncType != null && !"".equals(RoomFuncType)) {
                                    //								obj_ids.add(RoomFuncType);
                                    relObjs.add(createObj(RoomFuncType, "space_class"));
                                }
                            }
                        }
                    }
                } else if(CommonConstant.data_type_system.equals(obj_type)) {
                    //系统id
                    //系统类型id
                    String systemClass = baseService.getEquipOrSystemCodeFromId(obj_id, CommonConstant.tag_dict_sytstem, dataPlatBasePath);
                    relObjs.add(createObj(systemClass, "system_class"));
                    //系统
                    JSONObject params = new JSONObject();
                    JSONObject criteria = new JSONObject();
                    criteria.put("system_id", obj_id);
                    criteria.put("graph_id", systemEquipGraphId);
                    criteria.put("rel_type", "1");
                    params.put("criteria", criteria);
                    params.put("valid", true);
                    params.put("projectId", project_id);
                    String queryResult = objectService.httpPostRequestDataPlatformPath(
                            DataRequestPathUtil.dataPlat_equipment_complex_query,
                            project_id,
                            null,
                            params.toString());
                    JSONObject queryJson = JSONObject.parseObject(queryResult);
                    if(ResultEnum.SUCCESS.getType().equals(queryJson.getString(ResultEnum.RESULT.getType()))) {
                        JSONArray queryConstents = queryJson.getJSONArray(ResultEnum.CONTENT.getType());
                        if(queryConstents != null && queryConstents.size() > 0) {
                            for (int j = 0; j < queryConstents.size(); j++) {
                                JSONObject queryConstent = queryConstents.getJSONObject(j);
                                String id = queryConstent.getString("id");
                                String equipClass = baseService.getEquipOrSystemCodeFromId(id, CommonConstant.tag_dict_equip, dataPlatBasePath);
                                //设备类型id
                                if(!obj_ids.contains(equipClass)) {
                                    obj_ids.add(equipClass);
                                    relObjs.add(createObj(equipClass, "equip_class"));
                                }
                                //设备id
                                //							obj_ids.add(id);
                                relObjs.add(createObj(id, "equip"));
                            }
                        }
                    }
                } else if(CommonConstant.data_type_equip.equals(obj_type)) {
                    //设备id
                    String equipClass = baseService.getEquipOrSystemCodeFromId(obj_id, CommonConstant.tag_dict_equip, dataPlatBasePath);
                    //设备类型id
                    if(!obj_ids.contains(equipClass)) {
                        obj_ids.add(equipClass);
                        relObjs.add(createObj(equipClass, "equip_class"));
                    }
                    //系统id
                    JSONObject relation = objectService.queryRelation(project_id, null, null, obj_id, systemEquipGraphId, "1");
                    if(relation != null) {
                        String system_id = relation.getString("from_id");
                        obj_ids.add(system_id);
                        relObjs.add(createObj(system_id, "system"));
                    }
                    //设备型号
                    String queryResult = objectService.queryObjById(obj_id,project_id);
                    if (queryResult.contains(ResultEnum.SUCCESS.getType()) && queryResult.contains(ResultEnum.CONTENT.getType())) {
                        JSONObject queryJson = JSONObject.parseObject(queryResult);
                        JSONArray queryContents = queryJson.getJSONArray(ResultEnum.CONTENT.getType());
                        if (queryContents != null && queryContents.size() > 0) {
                            JSONObject queryItem = queryContents.getJSONObject(0);
                            queryItem = queryItem.getJSONObject("infos");
                            if (queryItem != null) {
                                String specification = queryItem.getString("Specification");
                                //String manufacturerID = queryItem.getString("ManufacturerID");
                                JSONObject param = new JSONObject();
                                param.put("equip_model_name", specification);
                                String equipModels = equipModelService.queryRecordByEquipModelName(param.toJSONString());
                                JSONObject equipModelsJson = JSONObject.parseObject(equipModels);
                                if (ResultEnum.SUCCESS.getType().equals(equipModelsJson.getString(ResultEnum.RESULT.getType()))) {
                                    JSONArray content = equipModelsJson.getJSONArray(ResultEnum.CONTENT.getType());
                                    if (content!=null) {
                                        for (int j = 0; j < content.size(); j++) {
                                            String equip_model_id = content.getJSONObject(j).getString("equip_model_id");
                                            equip_model_ids.add(equip_model_id);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                JSONObject params = new JSONObject();
                params.put("project_id", project_id);
                params.put("equip_model_ids", equip_model_ids);
                params.put("fit_obj_ids", obj_ids);
                params.put("relObjs", relObjs);
                lastSopIds = querySopsByFitObjs(lastSopIds, i, params);
            }
        }
        JSONObject params = new JSONObject();
        //根据工单类型查询
        JSONArray contents = new JSONArray();
        if(lastSopIds.size() > 0) {
            params.clear();
            LambdaQueryWrapper<Sop> sopLambdaQueryWrapper = new LambdaQueryWrapper<>();
            List<String> querySopIds = lastSopIds;
            String queryProjectId = project_id;
            sopLambdaQueryWrapper.or(wrapper->{
               wrapper.in(Sop::getSopId,querySopIds);
               wrapper.eq(Sop::getProjectId,queryProjectId);
            });
            if (!SopCommonConst.GROUP_SOP_ID.equals(project_id)){
                sopLambdaQueryWrapper.or(wrapper->{
                    wrapper.in(Sop::getSopId,querySopIds);
                    wrapper.eq(Sop::getProjectId,SopCommonConst.GROUP_SOP_ID);
                });
            }
            sopLambdaQueryWrapper.in(Sop::getSopId,JSONArray.parseArray(JSON.toJSONString(lastSopIds)).toJavaList(String.class));
            sopLambdaQueryWrapper.eq(Sop::getProjectId,project_id);
            List<Sop> sopList = sopService.list(sopLambdaQueryWrapper);
            List<SopBean> sopBeans = fillSopDetail(sopList);
            List<SopBean> sopBeanList = CollUtil.sort(sopBeans, Comparator.comparing(SopBean::getCreateTime));
            String rootProjFuncType = null;
            if (!SopCommonConst.GROUP_SOP_ID.equals(project_id)) {
                rootProjFuncType = objectService.queryRootProjFuncTypeById(project_id);
            }
            for (SopBean sopBean : sopBeanList) {
                if(contents.size() >= 50) {
                    break;
                }
                //发布状态为待修订的sop不推荐
                if ("2".equals(sopBean.getPublishStatus())) {
                    continue;
                }
                //如果前端过滤包含大类对象的sop 且 该sop操作内容中包含则不推荐
                if (!flag && isContainsClassObj(sopBean)) {
                    continue;
                }
                // 如果集团sop与本项目类型所对应的一级项目类型不一致 则不推荐
                if (rootProjFuncType != null && !StringUtil.isNull(sopBean.getExtendField())
                        && !sopBean.getExtendField().equals(rootProjFuncType)) {
                    continue;
                }
                JSONObject sop = new JSONObject();
                sop.put("sop_id", sopBean.getSopId());
                sop.put("sop_name", sopBean.getSopName());
                sop.put("version", sopBean.getVersion());
                sop.put("project_id", sopBean.getProjectId());
                sop.put("create_time", sopBean.getCreateTime());
                sop.put("update_time", sopBean.getUpdateTime());
                contents.add(sop);
            }
        }
        contents = JSONUtil.sortByField(contents, "update_time", -1);
        return ToolsUtil.successJsonMsg(contents);
    }

    /***
     * Description: 校验SOP名称是否重复
     * @param sopName : SOP的名称
     * @param sopId : SOP的id
     * @param projectId : 项目id
     * @return : boolean:true-可以使用,没有重复;false-不能使用,有重复
     * @author : lijie
     * @date :2021/4/23 10:12
     * Update By lijie 2021/4/23 10:12
     */
    private boolean checkSopNameRepeated(String sopName, String sopId, String projectId) {
        LambdaQueryWrapper<Sop> sopQueryWrapper = new LambdaQueryWrapper<>();
        sopQueryWrapper.eq(Sop::getSopName, sopName);
        List<Sop> list = sopService.list(sopQueryWrapper);
        for (Sop sop : list) {
            if (!SopCommonConst.GROUP_SOP_ID.equals(sop.getProjectId())
                    && !sop.getProjectId().equals(projectId)) {
                continue;
            }
            if (!sop.getSopId().equals(sopId)) {
                return false;
            }
        }
        // sop编辑表中存在同名sop 判断是否为同一sop
        LambdaQueryWrapper<SopEdit> sopEditQueryWrapper = new LambdaQueryWrapper<>();
        sopEditQueryWrapper.eq(SopEdit::getSopName, sopName);
        List<SopEdit> sopEdits = sopEditService.list(sopEditQueryWrapper);
        for (SopEdit sopEdit : sopEdits) {
            if (!SopCommonConst.GROUP_SOP_ID.equals(sopEdit.getProjectId())
                    && !sopEdit.getProjectId().equals(projectId)) {
                continue;
            }
            if (!sopEdit.getSopId().equals(sopId)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Description:
     * @param lastSopIds
     * @param i
     * @param params
     * @return
     * @throws Exception List<String>
     * @author yuecaipu
     * @since 2018年5月21日: 下午4:18:21
     * Update By yuecaipu 2018年5月21日: 下午4:18:21
     */
    private List<String> querySopsByFitObjs(List<String> lastSopIds, int i, JSONObject params) throws Exception {
        JSONArray contents = new JSONArray();
        Set<String> sopIds = new HashSet<>();
        //根据设备型号查询
        if (!StringUtil.isEmptyList(params, "equip_model_ids")) {
            LambdaQueryWrapper<SopEquipModelRel> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(SopEquipModelRel::getEquipModelId, params.getJSONArray("equip_model_ids").toJavaList(String.class));
            List<SopEquipModelRel> sopEquipModelRels = sopEquipModelRelService.list(queryWrapper);
            if (CollUtil.isNotEmpty(sopEquipModelRels)){
                sopIds.addAll(sopEquipModelRels.stream().map(SopEquipModelRel::getSopId).collect(Collectors.toSet()));
            }
        }
        //根据sop关联对象查询
        if(!StringUtil.isEmptyList(params, "relObjs")){
            JSONArray relObjs = params.getJSONArray("relObjs");
            LambdaQueryWrapper<SopObjRel> queryWrapper = new LambdaQueryWrapper<>();
            for(int j=0;j<relObjs.size();j++){
                JSONObject temp = relObjs.getJSONObject(j);
                queryWrapper.or(wrapper->{
                    wrapper.eq(SopObjRel::getValid,temp.getBoolean("valid"));
                    wrapper.eq(SopObjRel::getObjId,temp.getString("obj_id"));
                    wrapper.eq(SopObjRel::getObjType,temp.getString("obj_type"));
                });
            }
            List<SopObjRel> sopObjRels = sopObjRelService.list(queryWrapper);
            if (CollUtil.isNotEmpty(sopObjRels)){
                sopIds.addAll(sopObjRels.stream().map(SopObjRel::getSopId).collect(Collectors.toSet()));
            }
        }

        //根据no_obj查询
        LambdaQueryWrapper<Sop> sopLambdaQueryWrapper = new LambdaQueryWrapper<>();
        sopLambdaQueryWrapper.eq(Sop::getNoObj,"1");
        List<Sop> sopList = sopService.list(sopLambdaQueryWrapper);
        if (CollUtil.isNotEmpty(sopList)){
            sopIds.addAll(sopList.stream().map(Sop::getSopId).collect(Collectors.toSet()));
        }

        if(i == 0) {
            return proccessQueryResult(contents, "sop_id");
        }
        List<String> tempSopIds = proccessQueryResult(contents, "sop_id");
        return new ArrayList<>(CollectionUtils.retainAll(lastSopIds,tempSopIds));
    }

    /**
     * 处理查询结果 获取sop主键ID
     * @param jsonObject
     * @param infoName
     * @return
     */
    public List<String> proccessQueryResult(JSONObject jsonObject, String infoName){
        List<String> set = null;
        if(jsonObject != null){
            JSONArray array = jsonObject.getJSONArray("Content");
            if(array != null){
                set = new ArrayList<String>();
                for(int i=0; i<array.size(); i++){
                    set.add(array.getJSONObject(i).getString(infoName));
                }
            }
        }
        return set;
    }

    /**
     * 处理查询结果 获取sop主键ID
     * @param infoName
     * @return
     */
    public List<String> proccessQueryResult(JSONArray jsonArray, String infoName){
        List<String> set = null;
        if(jsonArray != null){
            set = new ArrayList<String>();
            for(int i=0; i<jsonArray.size(); i++){
                String infoNameValue = jsonArray.getJSONObject(i).getString(infoName);
                if(!set.contains(infoNameValue)) {
                    set.add(infoNameValue);
                }
            }
        }
        return set;
    }

    /***
     * Description: 是否包含物理世界对象类
     * @param sop : sop对象
     * @return : boolean
     * @author : lijie
     * @date :2021/4/21 17:08
     * Update By lijie 2021/4/21 17:08
     */
    private boolean isContainsClassObj(JSONObject sop) throws Exception {
        boolean flag = false;
        JSONArray steps = JSONArray.parseArray(sop.getString("steps"));
        if (steps!=null) {
            OUT:
            for (int i = 0; i < steps.size(); i++) {
                JSONObject step = steps.getJSONObject(i);
                Boolean from_sop = step.getBoolean("from_sop");
                if (!from_sop) {//非引用sop
                    JSONArray step_contents = step.getJSONArray("step_content");
                    if (step_contents!=null) {
                        for (int j = 0; j < step_contents.size(); j++) {
                            JSONObject step_content = step_contents.getJSONObject(j);
                            if (!step_content.getBoolean("from_sop")) {
                                JSONArray objs = step_content.getJSONArray("content_objs");
                                if (objs!=null) {
                                    for (int k = 0; k < objs.size(); k++) {
                                        JSONObject obj = objs.getJSONObject(k);
                                        String obj_type = obj.getString("obj_type");
                                        //对象类型以"_class"结尾的对象为大类对象
//										if (obj_type.endsWith("_class")) {
                                        if (obj_type.contains("_")) {
                                            flag =true;
                                            break OUT;
                                        }
                                    }
                                }
                            }else {
                                String sopId = step.getString("sop_id");
                                JSONObject relSop = querySopById(sopId);
                                if (relSop != null) {
                                    flag = isContainsClassObj(relSop);
                                    if (flag) {
                                        break OUT;
                                    }
                                }
                            }
                        }
                    }
                }else {//引用sop
                    String sopId = step.getString("sop_id");
                    JSONObject relSop = querySopById(sopId);
                    if (relSop != null) {
                        flag = isContainsClassObj(relSop);
                        if (flag) {
                            break OUT;
                        }
                    }
                }
            }
        }
        return flag;
    }
    /***
     * Description: 根据id查询SOP
     * @param sopId : SOP的id
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/21 17:04
     * Update By lijie 2021/4/21 17:04
     */
    private JSONObject querySopById(String sopId){
        List<Sop> sops = sopService.listByIds(Collections.singletonList(sopId));
        if (CollUtil.isEmpty(sops)){
            return null;
        }
        List<SopBean> sopBeans = fillSopDetail(sops);
        SopBean sopBean = sopBeans.get(0);
        return JSONObject.parseObject(JSONObject.toJSONString(sopBean));
    }

    /***
     * Description: 创建物理世界对象
     * @param objId : 对象id
     * @param objType : 对象类型
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/21 17:02
     * Update By lijie 2021/4/21 17:02
     */
    private JSONObject createObj(String objId, String objType) {
		JSONObject object = new JSONObject();
		object.put("valid", true);
		object.put("obj_id", objId);
		object.put("obj_type", objType);
		return object;
	}

    /***
     * Description: 事项处理 构建published_matters的值
     * @param matter : 事项
     * @return : com.alibaba.fastjson.JSONObject  
     * @author : lijie
     * @date :2021/4/21 16:51
     * Update By lijie 2021/4/21 16:51
     */
    private JSONObject transferMatter(JSONObject matter) throws Exception {
        JSONObject matterNew = new JSONObject();
        matterNew.putAll(matter);
        matterNew.put("matter_id", ToolsUtil.getUuid());
        matterNew.remove("desc_sops");
        matterNew.remove("desc_works");
        String desc_forepart = matter.getString("desc_forepart");
        String desc_aftpart = matter.getString("desc_aftpart");
        String desc_works_desc = matter.getString("desc_works_desc");
        desc_forepart = desc_forepart == null ? "" : desc_forepart;
        desc_aftpart = desc_aftpart == null ? "" : desc_aftpart;
        desc_works_desc = desc_works_desc == null ? "" : desc_works_desc;

        String description = desc_forepart + desc_aftpart + desc_works_desc;
        matterNew.put("description", description);
        //      matterNew.put("matter_name", matter.getString("matter_name"));
        //       matterNew.put("description", matter.getString("description"));

        JSONArray matterSteps = new JSONArray();
        JSONArray domains = new JSONArray();
        JSONArray tools = new JSONArray();
        JSONArray descSops = matter.getJSONArray("desc_sops");
        JSONArray descObjs = matter.getJSONArray("desc_objs");
        // 提取所有sop中的step_content，按照顺序添加到matter_steps中，step_type的值为"5";
        JSONArray	steps = new JSONArray();
        if (descSops != null && descSops.size() > 0) {
            for (int i = 0; i < descSops.size(); i++) {
                String sopId = descSops.getJSONObject(i).getString("sop_id");
                steps = transformSopSteps(steps, sopId, domains, tools,descObjs);
            }

        }
        // 如果draft_matters中的desc_works不为空，则将desc_works中的所有工作内容作为一个step_content，添加到matter_steps中，step_type的值为"5"；
        JSONArray descWorks = matter.getJSONArray("desc_works");
        if (descWorks != null) {
            for (int i = 0; i < descWorks.size(); i++) {
                steps.add(descWorks.getJSONObject(i));
                if(StringUtils.isNotBlank(descWorks.getJSONObject(i).getString("domain"))){
                    domains.add(descWorks.getJSONObject(i).getString("domain"));
                }

            }

        }
        JSONObject matter_step = new JSONObject();
        matter_step.put("step_type", "5");
        matter_step.put("step_content", steps);
        matterSteps.add(matter_step);
        // 如果工作事项下勾选了 "结束该事项时签字",则将"结束该事项时签字"，做为content的值，为matter_steps生成最后一个step_content，step_type的值为"6";
        JSONArray requiredControl = matter.getJSONArray("required_control");
        if (requiredControl != null) {
            if (requiredControl.contains("matter_end_scan")) {
                JSONArray contentArray = new JSONArray();
                JSONObject content = new JSONObject();
                content.put("content","结束该事项时签字");
                contentArray.add(content);
                JSONObject endScanStep = new JSONObject();
                endScanStep.put("step_type", "6");
                endScanStep.put("step_content", contentArray);
                matterSteps.add(endScanStep);
            }
            if (requiredControl.contains("matter_end_photo")) {
                JSONArray contentArray = new JSONArray();
                JSONObject content = new JSONObject();
                content.put("content","结束该事项时拍照");
                contentArray.add(content);
                JSONObject endScanStep = new JSONObject();
                endScanStep.put("step_type", "7");
                endScanStep.put("step_content", contentArray);
                matterSteps.add(endScanStep);
            }
            if (requiredControl.contains("matter_start_photo")) {
                JSONArray contentArray = new JSONArray();
                JSONObject content = new JSONObject();
                content.put("content","该事项执行前拍照");
                contentArray.add(content);
                JSONObject endScanStep = new JSONObject();
                endScanStep.put("step_type", "8");
                endScanStep.put("step_content", contentArray);
                matterSteps.add(endScanStep);
            }
            if (requiredControl.contains("matter_start_arrival_confirmation")) {
                JSONArray contentArray = new JSONArray();
                JSONObject content = new JSONObject();
                content.put("content","开始该事项时到场确认");
                contentArray.add(content);
                JSONObject endScanStep = new JSONObject();
                endScanStep.put("step_type", "9");
                endScanStep.put("step_content", contentArray);
                matterSteps.add(endScanStep);
            }
        }
        matterNew.put("matter_steps", matterSteps);
        matterNew.put("domains", domains);
        matterNew.put("tools", tools);
        return matterNew;
    }
    /***
     * Description: 转换SOP步骤
     * @param steps : 步骤
     * @param sopId : SOP的id
     * @param domains : 专业数组
     * @param tools : 工具数组
     * @param descObjs : 描述对象
     * @return : com.alibaba.fastjson.JSONArray
     * @author : lijie
     * @date :2021/4/21 16:52
     * Update By lijie 2021/4/21 16:52
     */
    private JSONArray transformSopSteps(JSONArray steps, String sopId,JSONArray domains,JSONArray tools,JSONArray descObjs) throws Exception {
        // 1.根据sopid 获取sop
        JSONObject sop = querySopById(sopId);
        /**SOP已作废或无效  则该SOP不组装*/
        if (null == sop || "2".equals(sop.getString("sop_status")) || !sop.getBooleanValue("valid")) {
            return steps;
        }
        domains.addAll(sop.getJSONArray("domains"));
        tools.addAll(sop.getJSONArray("tools"));
        JSONArray currentSteps = sop.getJSONArray("steps");
        Map<String, String> domainNameMap = queryAllDomainNameMap();
        //2.sop步骤中的stepContent添加到steps集合中
        for (Object object : currentSteps) {// 步骤
            JSONObject step = (JSONObject) object;
            if (step.getBooleanValue("from_sop")) {
                transformSopSteps(steps, step.getString("sop_id"), domains, tools,descObjs);
            } else {
                JSONArray stepContents = step.getJSONArray("step_content");
                for (Object object2 : stepContents) {// 步骤内容
                    JSONObject stepContent = (JSONObject) object2;
                    if (stepContent.getBooleanValue("from_sop")) {
                        transformSopSteps(steps, stepContent.getString("sop_id"), domains, tools,descObjs);
                    } else {
                        String domainCode = stepContent.getString("domain");
                        if (StringUtils.isNotBlank(domainCode)) {
                            stepContent.put("domain_name", domainNameMap.getOrDefault(domainCode,""));
                        }
                        if (StringUtil.isNull(stepContent,"time_limit")){
                            stepContent.remove("time_limit");
                        }
                        // 若sop步骤中无对象，将@的对象填充到步骤中 2019-10-28
                        JSONArray content_objs = (JSONArray)stepContent.getOrDefault("content_objs",new JSONArray());
                        if (CollectionUtils.isEmpty(content_objs) && CollectionUtils.isNotEmpty(descObjs)){
                            content_objs.addAll(descObjs);
                            stepContent.put("content_objs",content_objs);
                        }
                        if (CollectionUtils.isNotEmpty(content_objs)){
                            steps.add(stepContent);
                        }
                    }
                }
            }
        }
        return steps;
    }

    /***
     * Description: 组装工单令牌
     * @param project_id : 项目id
     * @param matter : 工单的单个matter
     * @param order : 排序
     * @return : com.alibaba.fastjson.JSONObject  
     * @author : lijie
     * @date :2021/4/21 16:20
     * Update By lijie 2021/4/21 16:20
     */ 
    private JSONObject buildMatterV2(String project_id, JSONObject matter, String order) throws Exception {
        JSONArray matterStepArray = matter.getJSONArray("matter_steps");
        if (CollUtil.isEmpty(matterStepArray)) {
            return matter;
        }
        // {"对象ID":[[第一步stepConten数组],[第二步骤stepContent数组]]}
        Map<String, JSONArray> matterStepMap = new HashMap<>();
        Map<String, JSONObject> id2obj = new HashMap<>();
        JSONArray signatureStepArray = new JSONArray();
        JSONArray photoStepArray = new JSONArray();
        JSONArray prePhotoStepArray = new JSONArray();
        JSONArray preArrivalStepArray = new JSONArray();
        for (Object matterStepObject : matterStepArray) {
            JSONArray stepContentArray = ((JSONObject) matterStepObject).getJSONArray("step_content");
            if(CollUtil.isEmpty(stepContentArray)){
                continue;
            }
            if ("1".equals(((JSONObject) matterStepObject).getString("step_type"))) {
                continue;
            }
            if ("6".equals(((JSONObject) matterStepObject).getString("step_type"))) {
                signatureStepArray.add(stepContentArray);
            }
            if ("7".equals(((JSONObject) matterStepObject).getString("step_type"))) {
                photoStepArray.add(stepContentArray);
            }
            if ("8".equals(((JSONObject) matterStepObject).getString("step_type"))) {
                prePhotoStepArray.add(stepContentArray);
            }
            if ("9".equals(((JSONObject) matterStepObject).getString("step_type"))) {
                preArrivalStepArray.add(stepContentArray);
            }
            // 拆分content_objs中的对象，按照对象出现的先后顺序，拆分出每个对象对应的stepContent：
            if ("5".equals(((JSONObject) matterStepObject).getString("step_type"))) {
                JSONObject newStepContent;
                JSONArray confirmResultArray, contentObjs, newStepContentArray;
                String objId;
                Map<String, JSONArray> stepContentMap = new HashMap<>();
                for (Object stepContentObject : stepContentArray) {
                    if (StringUtil.isNull((JSONObject)stepContentObject,"time_limit")){
                        ((JSONObject)stepContentObject).remove("time_limit");
                    }
                    confirmResultArray = ((JSONObject) stepContentObject).getJSONArray("confirm_result");
                    contentObjs = ((JSONObject) stepContentObject).getJSONArray("content_objs");
                    if(CollUtil.isEmpty(confirmResultArray)||CollUtil.isEmpty(contentObjs)){
                        continue;
                    }
                    for (Object obj : contentObjs) {
                        objId = ((JSONObject) obj).getString("obj_id");
                        if (!id2obj.containsKey(objId)) {
                            id2obj.put(objId, (JSONObject) obj);
                        }
                        newStepContentArray = stepContentMap.getOrDefault(objId, new JSONArray());
                        newStepContent = new JSONObject();
                        newStepContent.put("step_type", "5");
                        newStepContent.put("pre_conform",((JSONObject) stepContentObject).getString("pre_conform"));
                        newStepContent.put("pre_execute",((JSONObject) stepContentObject).getString("pre_execute"));
                        newStepContent.put("notice",((JSONObject) stepContentObject).getString("notice"));
                        newStepContent.put("domain",((JSONObject) stepContentObject).getString("domain"));
                        newStepContent.put("domain_name",((JSONObject) stepContentObject).getString("domain_name"));
                        newStepContent.put("obj_required_control",((JSONObject) stepContentObject).getJSONArray("obj_required_control"));
                        newStepContent.put("obj_show_item",((JSONObject) stepContentObject).getJSONArray("obj_show_item"));
                        newStepContent.put("time_limit",((JSONObject) stepContentObject).get("time_limit"));
                        JSONArray confirm_result = ((JSONObject) stepContentObject).getJSONArray("confirm_result");
                        if(confirm_result == null) {
                            confirm_result = new JSONArray();
                        }
                        newStepContent.put("confirm_result", JSONArray.parseArray(confirm_result.toJSONString()));
                        newStepContentArray.add(newStepContent);
                        stepContentMap.put(objId, newStepContentArray);
                    }
                }
                JSONArray tempArray ;
                for (Map.Entry<String, JSONArray> entity : stepContentMap.entrySet()) {
                    tempArray = matterStepMap.getOrDefault(entity.getKey(), new JSONArray());
                    tempArray.add(entity.getValue());
                    matterStepMap.put(entity.getKey(), tempArray);
                }
            }
        }

        JSONObject newMatter = JSONObject.parseObject(matter.toJSONString());
        newMatter.put("matter_steps", new JSONArray());

        // stepType=9 到场确认
        JSONObject preArrivalStep = initPreArrivalStepArray(preArrivalStepArray, matter.getString("description"));
        if (preArrivalStep != null) {
            newMatter.getJSONArray("matter_steps").add(preArrivalStep);
        }
        // stepType=8 执行前拍照
        JSONObject prePhotoStep = initPrePhotoStep(prePhotoStepArray, matter.getString("description"));
        if (prePhotoStep != null) {
            newMatter.getJSONArray("matter_steps").add(prePhotoStep);
        }
        // stepType=5 对象
        JSONArray workMatterSteps = initWorkMatterSteps(project_id, matterStepMap, id2obj, matter.getString("description"), order);
        if (CollUtil.isNotEmpty(workMatterSteps)) {
            workMatterSteps = sortMatterStep(workMatterSteps);
            newMatter.getJSONArray("matter_steps").addAll(workMatterSteps);
        }
        // stepType=6 结束时签名
        JSONObject signatureStep = initSignatureStep(signatureStepArray, matter.getString("description"));
        if (signatureStep != null) {
            newMatter.getJSONArray("matter_steps").add(signatureStep);
        }
        // stepType=7 结束时拍照
        JSONObject photoStep = initPhotoStep(photoStepArray, matter.getString("description"));
        if (photoStep != null) {
            newMatter.getJSONArray("matter_steps").add(photoStep);
        }
        return newMatter;
    }

    /***
     * Description: 初始化手机步骤
     * @param signatureStepArray : 量化步骤
     * @param description : 描述
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/21 16:38
     * Update By lijie 2021/4/21 16:38
     */
    private JSONObject initPhotoStep(JSONArray signatureStepArray ,String description) throws Exception{
        if(CollectionUtils.isNotEmpty(signatureStepArray)){
            String objStepId = ToolsUtil.getUuid();
            String sequenceId =  "0-1";
            JSONObject signatureStepContent = new JSONObject();

            signatureStepContent.put("$ID", objStepId+"_"+sequenceId);
            signatureStepContent.put("step_id", objStepId+"_"+sequenceId);
            signatureStepContent.put("step_sequence", sequenceId);
            signatureStepContent.put("step_type", "7");
            signatureStepContent.put("content","结束该事项时拍照");

            JSONObject signatureMatterStepObject = new JSONObject();
            signatureMatterStepObject.put("$ID", objStepId);
            signatureMatterStepObject.put("obj_step_id", objStepId);
            signatureMatterStepObject.put("obj_id", "");
            signatureMatterStepObject.put("obj_name", "");
            signatureMatterStepObject.put("description", description);
            JSONArray steps = new JSONArray();
            steps.add(signatureStepContent);
            signatureMatterStepObject.put("steps", steps);
            return signatureMatterStepObject;
        }
        return null;
    }

    /***
     * Description: 初始化步骤
     * @param signatureStepArray :量化步骤
     * @param description : 描述
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/21 16:37
     * Update By lijie 2021/4/21 16:37
     */
    private JSONObject initSignatureStep(JSONArray signatureStepArray ,String description) throws Exception{
        if(CollectionUtils.isNotEmpty(signatureStepArray)){
            String objStepId = ToolsUtil.getUuid();
            String sequenceId =  "0-1";
            JSONObject signatureStepContent = new JSONObject();

            signatureStepContent.put("$ID", objStepId+"_"+sequenceId);
            signatureStepContent.put("step_id", objStepId+"_"+sequenceId);
            signatureStepContent.put("step_sequence", sequenceId);
            signatureStepContent.put("step_type", "6");
            signatureStepContent.put("content","结束该事项时签字");

            JSONObject signatureMatterStepObject = new JSONObject();
            signatureMatterStepObject.put("$ID", objStepId);
            signatureMatterStepObject.put("obj_step_id", objStepId);
            signatureMatterStepObject.put("obj_id", "");
            signatureMatterStepObject.put("obj_name", "");
            signatureMatterStepObject.put("description", description);
            JSONArray steps = new JSONArray();
            steps.add(signatureStepContent);
            signatureMatterStepObject.put("steps", steps);
            return signatureMatterStepObject;
        }
        return null;
    }

    /***
     * Description: 排序Matter
     * @param matterStepArray : 源Matter
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/21 16:32
     * Update By lijie 2021/4/21 16:32
     */
    private JSONArray sortMatterStep(JSONArray matterStepArray) throws Exception {
        if (CollectionUtils.isEmpty(matterStepArray)|| matterStepArray.size()==1) {
            return matterStepArray;
        }
        Map<String, JSONObject> objId2matterStep = new HashMap<>();
        JSONObject matterStepObject, parents;
        List<TreeNode> nodes = new ArrayList<>();
        TreeNode node ,pnode;
        for (int i = 0; i < matterStepArray.size(); i++) {
            matterStepObject = matterStepArray.getJSONObject(i);
            if (StringUtil.isNull(matterStepObject, "obj_id")) {
                continue;
            }
            objId2matterStep.put(matterStepObject.getString("obj_id"), matterStepObject);
            parents = matterStepObject.getJSONObject("parents");
            node = new TreeNode();
            node.setKey(matterStepObject.getString("obj_id"));
            node.setName(matterStepObject.getString("obj_name"));
            node.setSerial(matterStepObject.getString("obj_serial"));
            if (parents != null && !StringUtil.isNull(parents, "parent_id")) {
                node.setParentId(parents.getString("parent_id"));

                pnode = new TreeNode();
                pnode.setKey(parents.getString("parent_id"));
                pnode.setName(parents.getString("parent_name"));
                if (!nodes.contains(pnode)) {
                    nodes.add(pnode);
                }
            }
            int indexOf = nodes.indexOf(node);
            if (indexOf != -1) {
                nodes.set(indexOf, node);
            } else {
                nodes.add(node);
            }
        }

        nodes = new Tree(nodes).buildTree();
        matterStepArray.clear();
        sortMatterStep(nodes,matterStepArray,objId2matterStep);

        return matterStepArray;
    }

    /***
     * Description: 排序
     * @param nodes : 节点数组
     * @param matterStepArray : 步骤数组
     * @param objId2matterStep : 对象id映射步骤
     * @return : void
     * @author : lijie
     * @date :2021/4/21 16:34
     * Update By lijie 2021/4/21 16:34
     */
    private void sortMatterStep(List<TreeNode> nodes,JSONArray matterStepArray, Map<String, JSONObject> objId2matterStep) {
        if (CollectionUtils.isEmpty(nodes)) {
            return;
        }
        //------按对象分类为建筑--系统--竖井--楼层--空间--设备 的顺序整理所有对象，依次排序给出。
        nodes.sort((o1, o2) -> {
            int compareTo = ObjectLevel.getLevel(o1.getKey()).compareTo(ObjectLevel.getLevel(o2.getKey()));
            if (0 == compareTo) {
                String serial1 = o1.getSerial();
                String serial2 = o2.getSerial();
                if (StringUtil.isNull(serial1) || StringUtil.isNull(serial2)) {
                    serial1 = o1.getName();
                    serial2 = o2.getName();
                }
                try {
                    return Integer.parseInt(serial1) - (Integer.parseInt(serial2));
                } catch (NumberFormatException e) {
                    return serial1.compareTo(serial2);
                }
            }
            return compareTo;
        });
        TreeNode node ;
        List<TreeNode> children;
        for (int i = 0; i < nodes.size(); i++) {
            node = nodes.get(i);
            if (null != objId2matterStep.get(node.getKey())) {
                matterStepArray.add(objId2matterStep.get(node.getKey()));
            }
            children = node.getChildren();
            sortMatterStep(children, matterStepArray, objId2matterStep);
        }
    }

    /***
     * Description: step_type = 5的情况
     * @param project_id : 项目id
     * @param matterStepMap : 步骤映射
     * @param id2obj : 对象id与对象的映射
     * @param description : 描述
     * @param order : 排序
     * @return : com.alibaba.fastjson.JSONArray
     * @author : lijie
     * @date :2021/4/21 16:26
     * Update By lijie 2021/4/21 16:26
     */
    private JSONArray initWorkMatterSteps(String project_id, Map<String, JSONArray> matterStepMap, Map<String, JSONObject> id2obj, String description, String order) throws Exception {
        JSONArray workMatterSteps = new JSONArray();
        Map<String, JSONObject> id2parent = getParent(project_id, matterStepMap.keySet(),order);
        for (Map.Entry<String, JSONArray> entity : matterStepMap.entrySet()) {
            JSONArray mergeMatterStepArray = new JSONArray();
            JSONArray stepContentArray = entity.getValue();
            Set<Object> objRequiredControlSet = new HashSet<>();
            Set<Object> objShowItemSet = new HashSet<>();
            for (Object stepContent : stepContentArray) {
                for (Object stepContentObject : (JSONArray) stepContent) {
                    JSONArray objRequiredControl = ((JSONObject) stepContentObject).getJSONArray("obj_required_control");
                    JSONArray objShowItem = ((JSONObject) stepContentObject).getJSONArray("obj_show_item");
                    if(CollUtil.isNotEmpty(objRequiredControl)){
                        objRequiredControlSet.addAll(objRequiredControl);
                    }
                    if(CollUtil.isNotEmpty(objShowItem)){
                        objShowItemSet.addAll(objShowItem);
                    }
                }
            }
            String objStepId = ToolsUtil.getUuid();
            int startIndex = 1;
            if (objRequiredControlSet.contains("obj_first_photo")) {
                JSONObject stepObject = new JSONObject();
                String stepSequence = "0-1";
                stepObject.put("$ID", objStepId+"_"+stepSequence);
                stepObject.put("step_id", objStepId+"_"+stepSequence);
                stepObject.put("step_sequence", stepSequence);
                stepObject.put("step_type", "3");
                stepObject.put("content", "到达指定位置拍照");
                mergeMatterStepArray.add(stepObject);
            }
            if (objRequiredControlSet.contains("obj_first_sign")) {
                JSONObject stepObject = new JSONObject();
                String stepSequence = "0-2";
                stepObject.put("$ID", objStepId+"_"+stepSequence);
                stepObject.put("step_id", objStepId+"_"+stepSequence);
                stepObject.put("step_sequence", stepSequence);
                stepObject.put("step_type", "4");
                stepObject.put("content", "到达指定位置扫码");
                mergeMatterStepArray.add(stepObject);
            }
			if (objRequiredControlSet.contains("obj_first_nfc")) {
				JSONObject stepObject = new JSONObject();
				String stepSequence = "0-3";
				stepObject.put("$ID", objStepId+"_"+stepSequence);
				stepObject.put("step_id", objStepId+"_"+stepSequence);
				stepObject.put("step_sequence", stepSequence);
				stepObject.put("step_type", "10");
				stepObject.put("content", "到达对象位置时进行NFC识别");
				mergeMatterStepArray.add(stepObject);
			}

            for (Object stepContent : stepContentArray) {
                JSONArray stepContentTemp = (JSONArray) stepContent;
                for (int i = 0; i < stepContentTemp.size(); i++) {
                    String stepSequence = startIndex + "-" + (i + 1);
                    stepContentTemp.getJSONObject(i).put("$ID", objStepId + "_" + stepSequence);
                    stepContentTemp.getJSONObject(i).put("step_id", objStepId + "_" + stepSequence);
                    stepContentTemp.getJSONObject(i).put("step_sequence", stepSequence);
                    stepContentTemp.getJSONObject(i).put("step_type", "5");
                    mergeMatterStepArray.add(stepContentTemp.getJSONObject(i));
                }
                startIndex = startIndex + 1;
            }
            JSONObject newMatterStep = new JSONObject();
            newMatterStep.put("$ID", objStepId);
            newMatterStep.put("obj_step_id", objStepId);

            newMatterStep.put("obj_id", entity.getKey());
            newMatterStep.put("obj_name", id2obj.get(entity.getKey()).getString("obj_name"));
            newMatterStep.put("obj_code", id2obj.get(entity.getKey()).getString("obj_code"));
            newMatterStep.put("obj_serial", id2obj.get(entity.getKey()).getString("obj_serial"));
            newMatterStep.put("parents", id2parent.get(entity.getKey()));
			newMatterStep.put("description", description);
            newMatterStep.put("steps", mergeMatterStepArray);
            newMatterStep.put("obj_show_item", objShowItemSet);
            workMatterSteps.add(newMatterStep);
        }
        return workMatterSteps;
    }

    /***
     * Description: 查询设备或空间的parent
     * @param projectId : 项目id
     * @param ids : id数组
     * @param order : 排序
     * @return : java.util.Map<java.lang.String,com.alibaba.fastjson.JSONObject>  
     * @author : lijie
     * @date :2021/4/21 16:28
     * Update By lijie 2021/4/21 16:28
     */
    private Map<String, JSONObject> getParent(String projectId, Set<String> ids, String order) throws Exception {
    	if (CollUtil.isEmpty(ids)) {
			return new HashMap<>();
		}
    	JSONArray equipIds = new JSONArray();
    	JSONArray spaceIds = new JSONArray();
    	JSONArray shaftIds = new JSONArray();
    	JSONArray bdSyFlIds = new JSONArray();
    	//======id分类=======
    	for (String id : ids) {
			if (id.startsWith("Eq")) {
				equipIds.add(id);
			}else if (id.startsWith("Sp")) {
				spaceIds.add(id);
			}else if (id.startsWith("Sh")) {
				shaftIds.add(id);
			}else {
				bdSyFlIds.add(id);
			}
		}

		JSONArray equips = null, spaces = null;
		Map<String, JSONObject> id2parent = new HashMap<>();
		JSONObject parent ,node;
		JSONObject parentPrototype = new JSONObject();
		String parentId , parentName , spaceName, floorName, buildName;
		//======设备查询===========
    	if (!equipIds.isEmpty()) {
    		equips = objectService.querySpaceOrEquipNoHierarchy(projectId, "equip", equipIds);
    		//-------------组装parent-------------
    		for (int i = 0; i < equips.size(); i++) {
    			node = equips.getJSONObject(i);
				parent = (JSONObject) parentPrototype.clone();
				parentId = node.getString("space_id");
				parentName = node.getString("space_name");
				floorName = node.getString("floor_name");
				buildName = node.getString("build_name");
				if (!StringUtil.isNull(floorName)){
					parentName = floorName + "-" + parentName;
				}
				if (!StringUtil.isNull(buildName)){
					parentName = buildName + "-" + parentName;
				}

				if (parentId == null) {
					parentId = node.getString("build_id");
					parentName = node.getString("build_name");
				}
				parent.put("parent_id", parentId);
				parent.put("parent_name", parentName);
				id2parent.put(node.getString("obj_id"), parent);
			}
		}
		//======空间查询===========
    	if (!spaceIds.isEmpty()) {
    		spaces = objectService.querySpaceOrEquipNoHierarchy(projectId, "space", spaceIds);
    		//-------------组装parent----------------
    		for (int i = 0; i < spaces.size(); i++) {
    			node = spaces.getJSONObject(i);
    			parent = (JSONObject) parentPrototype.clone();
                buildName = node.getString("build_name");
				if (order != null && "2".equals(order)) {
                    parentId = node.getString("shaft_id");
                    parentName = buildName + "-" + node.getString("shaft_name");
				} else {
					parentId = node.getString("floor_id");
                    parentName = buildName + "-" + node.getString("floor_name");
				}

				if (parentId == null) {
					parentId = node.getString("build_id");
					parentName = node.getString("build_name");
				}

				parent.put("parent_id", parentId);
				parent.put("parent_name", parentName);
				id2parent.put(node.getString("obj_id"), parent);
			}
    	}
    	//----------建筑id name 映射--------
    	Map<String, String> buildId2name = null;
		if (!bdSyFlIds.isEmpty() || !shaftIds.isEmpty()) {
			buildId2name = new HashMap<>();
			JSONArray build = objectService.subsetQuery(projectId, projectId, "Bd");
			for (Object object : build) {
				node = (JSONObject) object;
				buildId2name.put(node.getString("id"), getLocalName(node));
			}
		}
    	//=====建筑系统楼层对象查询====
    	if (!bdSyFlIds.isEmpty()){
			String resultStr = objectService.queryBatchObj(bdSyFlIds.toJSONString(), projectId, true);
			JSONObject parseObject = JSONObject.parseObject(resultStr);
			if (ResultEnum.SUCCESS.getType().equals(parseObject.getString(ResultEnum.RESULT.getType()))) {
				JSONArray content = parseObject.getJSONArray(ResultEnum.CONTENT.getType());
				for (Object object : content) {
					node = (JSONObject)object;
					parent = (JSONObject) parentPrototype.clone();
					JSONObject location  = node.getJSONObject("location");
					if (null != location && !StringUtil.isNull(location, "building")) {
						parentId = location.getString("building");
						parent.put("parent_id", parentId);
						parent.put("parent_name", buildId2name.get(parentId));
					}
					id2parent.put(node.getString("id"), parent);
				}
			}
		}
    	//=====竖井查询====
    	if (!shaftIds.isEmpty()) {
    		String graphId = graphCache.getGraphIdByProjectIdAndGraph(projectId, CommonConstant.graph_type_shaft_and_building);
    		JSONObject relation ;
    		for (Object object : shaftIds) {
    			relation = objectService.queryRelation(projectId, null, (String) object, null, graphId, "1");
    			parent = (JSONObject) parentPrototype.clone();
    			if (null!=relation) {
    				parentId = relation.getString("to_id");
    				parent.put("parent_id", parentId);
    				parent.put("parent_name", buildId2name.get(parentId));
				}
				id2parent.put((String)object, parent);
			}
		}
		return id2parent;
	}

    /***
     * Description: 根据id获得信息点key值
     * @param obj : 对象
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/21 16:30
     * Update By lijie 2021/4/21 16:30
     */
    private String getLocalName(JSONObject obj){
    	String id = obj.getString("id");
    	String LocalNameInfo =null;
    	if (id.startsWith("Sy")) {
    		LocalNameInfo = "SysLocalName";
		}else if (id.startsWith("Bd")) {
			LocalNameInfo = "BuildLocalName";
		}else if (id.startsWith("Fl")) {
			LocalNameInfo = "FloorLocalName";
		}else if (id.startsWith("Sh")) {
			LocalNameInfo = "ShaftLocalName";
		}else if (id.startsWith("Sp")) {
			LocalNameInfo = "SpaceLocalName";
		}
		return obj.getJSONObject("infos").getString(LocalNameInfo);
    }

    /***
     * Description: 初始化前置手机步骤 
     * @param signatureStepArray : 签名步骤数组
     * @param description : 描述
     * @return : com.alibaba.fastjson.JSONObject  
     * @author : lijie
     * @date :2021/4/21 16:25
     * Update By lijie 2021/4/21 16:25
     */
    private JSONObject initPrePhotoStep(JSONArray signatureStepArray , String description) throws Exception {
    	if(CollUtil.isEmpty(signatureStepArray)){
    	    return null;
        }
        String objStepId = ToolsUtil.getUuid();
        String sequenceId = "0-1";
        JSONObject signatureStepContent = new JSONObject();

        signatureStepContent.put("$ID", objStepId+"_"+sequenceId);
        signatureStepContent.put("step_id", objStepId+"_"+sequenceId);
        signatureStepContent.put("step_sequence", sequenceId);
        signatureStepContent.put("step_type", "8");
        signatureStepContent.put("content","执行该事项前拍照");

        JSONObject signatureMatterStepObject = new JSONObject();
        signatureMatterStepObject.put("$ID", objStepId);
        signatureMatterStepObject.put("obj_step_id", objStepId);
        signatureMatterStepObject.put("obj_id", "");
        signatureMatterStepObject.put("obj_name", "");
        signatureMatterStepObject.put("description", description);
        JSONArray steps = new JSONArray();
        steps.add(signatureStepContent);
        signatureMatterStepObject.put("steps", steps);
        return signatureMatterStepObject;
    }

    /***
     * Description: 初始化Step
     * @param preArrivalStepArray :预步骤
     * @param description : 描述
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/21 16:23
     * Update By lijie 2021/4/21 16:23
     */
    private JSONObject initPreArrivalStepArray(JSONArray preArrivalStepArray , String description) throws Exception {
        if(CollUtil.isEmpty(preArrivalStepArray)){
            return null;
        }
        String objStepId = ToolsUtil.getUuid();
        String sequenceId = "0-1";
        JSONObject signatureStepContent = new JSONObject();

        signatureStepContent.put("$ID", objStepId+"_"+sequenceId);
        signatureStepContent.put("step_id", objStepId+"_"+sequenceId);
        signatureStepContent.put("step_sequence", sequenceId);
        signatureStepContent.put("step_type", "9");
        signatureStepContent.put("content","执行该事项前到场确认");

        JSONObject signatureMatterStepObject = new JSONObject();
        signatureMatterStepObject.put("$ID", objStepId);
        signatureMatterStepObject.put("obj_step_id", objStepId);
        signatureMatterStepObject.put("obj_id", "");
        signatureMatterStepObject.put("obj_name", "");
        signatureMatterStepObject.put("description", description);
        JSONArray steps = new JSONArray();
        steps.add(signatureStepContent);
        signatureMatterStepObject.put("steps", steps);
        return signatureMatterStepObject;
    }

    /**
     * 根据sop阶段内容、变化对象生成自动修订方案
     * @param objId:对象id
     * @param objName:对象名称
     * @param objType:对象类型
     * @param stepContent:步骤内容
     * @param stepNo:步骤编号
     * @return
     */
    private List<AutoReviseMsgVo> generateRevisePlanFromStepContent(String objId, String objName, String objType, SopCell stepContent, int stepNo){
        List<AutoReviseMsgVo> result = new ArrayList<>();
        List<SopObjRel> contentObjs = CollUtil.defaultIfEmpty(stepContent.getContentObjs(),new ArrayList<>());
        for (SopObjRel contentObj : contentObjs) {
            if ("1".equals(objType)){
                result.addAll(generateRevisePlanForTypeOne(objId,objName,contentObj,stepNo,"对象选择"));
            }
            if ("2".equals(objType)){
                result.addAll(generateRevisePlanForTypeTwo(objId,objName,contentObj,stepNo,"对象选择"));
            }
        }
        List<SopCell> sopCells = CollUtil.defaultIfEmpty(stepContent.getConfirmResult(), new ArrayList<>());
        for (SopCell sopCell : sopCells) {
            result.addAll(generateRevisePlanForTypeThree(objId,objName,sopCell,stepNo,"需确认的操作结果"));
        }
        return result;
    }
    /***
     * Description: objType为1时生成的响应参数
     * @param objId : 对象id
     * @param objName : 对象名称
     * @param contentObj : SOP与对象关系对象
     * @param stepNo : 步骤编号
     * @param objFrom : 对象来源
     * @return : java.util.List<com.persagy.fm.sop.model.vo.AutoReviseMsgVo>
     * @author : lijie
     * @date :2021/4/20 19:46
     * Update By lijie 2021/4/20 19:46
     */
    private List<AutoReviseMsgVo> generateRevisePlanForTypeOne(String objId, String objName, SopObjRel contentObj, int stepNo, String objFrom){
        List<AutoReviseMsgVo> result = new ArrayList<>();
        //对象模型
        List<ParentsBean> parentsBeans = CollUtil.defaultIfEmpty(contentObj.getParents(), new ArrayList<>());
        if (CollUtil.isEmpty(parentsBeans)){
            return new ArrayList<>();
        }
        for (ParentsBean parentsBean : parentsBeans) {
            List<String> parentIds = CollUtil.defaultIfEmpty(parentsBean.getParentIds(), new ArrayList<>());
            if (parentIds.contains(objId)){
                result.add(AutoReviseMsgVo.builder()
                        .doStep("步骤" + stepNo)
                        .effectPart(objFrom + "：数据采集->" + objName)
                        .doPlan("删除该对象")
                        .build());
                break;
            }
        }
        return result;
    }
    /***
     * Description: objType为1时生成的响应参数
     * @param objId : 对象id
     * @param objName : 对象名称
     * @param contentObj : SOP与对象关系对象
     * @param stepNo : 步骤编号
     * @param objFrom : 对象来源
     * @return : java.util.List<com.persagy.fm.sop.model.vo.AutoReviseMsgVo>
     * @author : lijie
     * @date :2021/4/20 19:46
     * Update By lijie 2021/4/20 19:46
     */
    private List<AutoReviseMsgVo> generateRevisePlanForTypeTwo(String objId, String objName, SopObjRel contentObj, int stepNo, String objFrom){
        List<AutoReviseMsgVo> result = new ArrayList<>();
        //对象实例
        if(objId.equals(contentObj.getObjId())){
            result.add(AutoReviseMsgVo.builder()
                    .doStep("步骤" + stepNo)
                    .effectPart(objFrom + "：数据采集->" + objName)
                    .doPlan("删除该对象")
                    .build());
        }
        return result;
    }
    /***
     * Description: objType为1时生成的响应参数
     * @param objId : 对象id
     * @param objName : 对象名称
     * @param sopCell : SOP与对象关系对象
     * @param stepNo : 步骤编号
     * @param objFrom : 对象来源
     * @return : java.util.List<com.persagy.fm.sop.model.vo.AutoReviseMsgVo>
     * @author : lijie
     * @date :2021/4/20 19:46
     * Update By lijie 2021/4/20 19:46
     */
    private List<AutoReviseMsgVo> generateRevisePlanForTypeThree(String objId, String objName, SopCell sopCell, int stepNo, String objFrom){
        List<AutoReviseMsgVo> result = new ArrayList<>();
        //信息点
        List<InfoPointsBean> infoPoints = sopCell.getInfoPoints();
        if (CollUtil.isEmpty(infoPoints)){
            return new ArrayList<>();
        }
        for (InfoPointsBean infoPoint : infoPoints) {
            if (objId.equals(infoPoint.getId())){
                result.add(AutoReviseMsgVo.builder()
                        .doStep("步骤" + stepNo)
                        .effectPart(objFrom + "：数据采集->" + objName)
                        .doPlan("删除该对象")
                        .build());
            }
        }
        return result;
    }

    /***
     * Description: 填充SOP的专业名称
     * @param domainIds : sop的专业id集合
     * @return : void
     * @author : lijie
     * @date :2021/4/20 16:17
     * Update By lijie 2021/4/20 16:17
     */
    private List<String> queryDomainNameByIds(List<String> domainIds) {
        if (CollUtil.isEmpty(domainIds)){
            return new ArrayList<>();
        }
        Map<String, String> domainNameMap = queryAllDomainNameMap();
        List<String> result = new ArrayList<>();
        for (String domainId : domainIds) {
            result.add(domainNameMap.getOrDefault(domainId,""));
        }
        return result;
    }
    /***
     * Description: 查询所有专业名称映射
     * @return : java.util.Map<java.lang.String,java.lang.String>
     * @author : lijie
     * @date :2021/4/21 16:54
     * Update By lijie 2021/4/21 16:54
     */
    private Map<String,String> queryAllDomainNameMap(){
        List<GeneralDictVo> generalDictVos = generalDictService.queryGeneralDictByKey(DBConst.DICT_DOMAINS, null);
        if (CollUtil.isEmpty(generalDictVos)){
            return new HashMap<>();
        }
        return generalDictVos.stream()
                .collect(Collectors.toMap(GeneralDictVo::getCode, GeneralDictVo::getName, (k1, k2) -> k1));
    }

    /***
     * Description: 是否包含大类对象
     * @param sopBean : sop的对象
     * @return : boolean
     * @author : lijie
     * @date :2021/4/19 9:53
     * Update By lijie 2021/4/19 9:53
     */
    private boolean isContainsClassObj(SopBean sopBean) throws Exception {
        List<SopCell> beanSteps = sopBean.getSteps();
        if (CollUtil.isEmpty(beanSteps)){
            return false;
        }
        for (int i=0;i<beanSteps.size();i++){
            SopCell sopCell = beanSteps.get(i);
            Boolean fromSop = sopCell.getFromSop();
            if (!fromSop){
                // 非引用SOP
                List<SopCell> stepContents = sopCell.getStepContent();
                if (CollUtil.isEmpty(stepContents)){
                    continue;
                }
                for (SopCell stepContent : stepContents) {
                    List<SopObjRel> contentObjs = stepContent.getContentObjs();
                    if (!stepContent.getFromSop()){
                        if (CollUtil.isEmpty(contentObjs)){
                            continue;
                        }
                        for (SopObjRel contentObj : contentObjs) {
                            if (StrUtil.isNotBlank(contentObj.getObjType())
                                    && contentObj.getObjType().contains("_")){
                                return true;
                            }
                        }
                        continue;
                    }
                    List<Sop> relSops = sopService.listByIds(Collections.singletonList(stepContent.getRelSopId()));
                    List<SopBean> relSopBeans = fillSopDetail(relSops);
                    if (CollUtil.isEmpty(relSopBeans)){
                        return false;
                    }
                    return isContainsClassObj(relSopBeans.get(0));
                }
                continue;
            }
            List<Sop> relSops = sopService.listByIds(Collections.singletonList(sopCell.getRelSopId()));
            List<SopBean> relSopBeans = fillSopDetail(relSops);
            if (CollUtil.isEmpty(relSopBeans)){
                return false;
            }
            return isContainsClassObj(relSopBeans.get(0));
        }
        return false;
    }

    /***
     * Description: 移除已引用的SOP
     * @param sops : sop列表
     * @param projectId : 项目id
     * @param sopId : SOP的id
     * @return : void
     * @author : lijie
     * @date :2021/4/19 9:33
     * Update By lijie 2021/4/19 9:33
     */
    private void removeRelSop(List<Sop> sops, String projectId, String sopId) {
        Set<String> relSopIds = sopCellService.querySopRel(projectId, sopId);
        relSopIds.add(sopId);
        for (int i = sops.size()-1; i >= 0; i--) {
            String nodeId = sops.get(i).getSopId();
            if (relSopIds.contains(nodeId)) {
                sops.remove(i);
            }
        }
    }

    /***
     * Description: 当引用sop作废时更新引用作废sop的sop的步骤
     * @param userId : 用户id
     * @param sop : Sop内容
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/13 12:23
     * Update By lijie 2021/4/13 12:23
     */
    public void revisionSopStepWhenRefSopDeleted(String userId, Sop sop) throws Exception{
        String sopId = sop.getSopId();
        // 1.根据引用的SOP的唯一标识查询引用其的SOP的id
        LambdaQueryWrapper<SopCell> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopCell::getRelSopId,sop.getSopId());
        List<SopCell> sopCells = sopCellService.list(queryWrapper);
        if (CollUtil.isEmpty(sopCells)){
            return;
        }
        Set<String> relSopIds = sopCells.stream().map(SopCell::getSopId).collect(Collectors.toSet());
        List<Sop> sops = sopService.listByIds(relSopIds);
        if (CollUtil.isEmpty(sops)){
            return;
        }
        List<SopBean> sopBeans = fillSopDetail(sops);
        for (SopBean relSop : sopBeans) {
            SopBean refSop = removeRefFromStep(relSop, new HashSet<>(Collections.singletonList(sopId)),new HashSet<>());
            if (CollUtil.isEmpty(refSop.getSteps())){
                // 当Sop的步骤为空时则删除该SOP
                String refSopId = refSop.getSopId();
                log.error("移除作废的引用sop后步骤为空，作废该sop："+refSopId);
                QueryDestroySopVo destroySopVo = QueryDestroySopVo.builder()
                        .operator(OperatorBean.builder().id(CommonConstant.SYSTEM_ID).name("系统").build())
                        .build();
                destroySopVo.setSopId(refSopId);
                destroySopVo.setUserId(userId);
                destroySopById(destroySopVo);
                continue;
            }
            List<RefedSopBean> refedSopBeans = getSopProfileOfRefCurrentSop(refSop.getSopId());
            SopHis sopHis = BeanUtil.copyProperties(refSop, SopHis.class);
            sopHis.setRefedSops(refedSopBeans);
            sopHis.setSopHisId(IdPrefixEnum.SOP_HIS.getPrefix()+IdWorker.getIdStr());
            sopHis.setVersionExplain(CollUtil.isEmpty(sopHis.getVersionExplain())?new ArrayList<>():sopHis.getVersionExplain());
            sopHisService.save(sopHis);
            refSop.setVersionExplain(Collections.singletonList("清除废弃标准操作"));
            refSop.setVersionCount(refSop.getVersionCount()+1);
            refSop.setVersion("V" + (double) refSop.getVersionCount() / 10);
            refSop.setSopStatus(SopStatusEnum.PUBLISHED.getType());
            refSop.setPublishStatus(refSop.getSopStatus());
            sopService.updateById(BeanUtil.copyProperties(refSop,Sop.class));
            SopDestroyMsg sopDestroyMsg = SopDestroyMsg.builder()
                    .refSopId(sopId)
                    .refSopName(sop.getSopName())
                    .refSopStatus(getSopStatus(sop.getSopStatus(),sop.getPublishStatus()))
                    .projectId(refSop.getProjectId())
                    .sopId(refSop.getSopId())
                    .msgId(IdPrefixEnum.SOP_DESTROY_MSG.getPrefix()).build();
            sopDestroyMsgService.save(sopDestroyMsg);
        }
    }


    /***
     * Description: sop引用的sop作废或删除时更新sop步骤(移除删除sop的步骤)
     * @param sopBean : 原来的SOP
     * @param removedSopIds : 需要移除的SOP的id集合
     * @return : com.alibaba.fastjson.JSONObject  
     * @author : lijie
     * @date :2021/4/13 14:25
     * Update By lijie 2021/4/13 14:25
     */
    public SopBean removeRefFromStep(SopBean sopBean, Set<String> removedSopIds, Set<String> needRemoveCellIds){
        if (null==sopBean || CollUtil.isEmpty(removedSopIds)){
            return sopBean;
        }
        String sopId = sopBean.getSopId();
        List<SopCell> sopCells = sopBean.getSteps();
        if (CollUtil.isEmpty(sopCells)){
            return sopBean;
        }
        for (int i=sopCells.size()-1;i>=0;i--){
            SopCell sopCell = sopCells.get(i);
            if (sopCell.getFromSop() && removedSopIds.contains(sopCell.getRelSopId())){
                needRemoveCellIds.add(sopCell.getSopCellId());
                sopCells.remove(i);
                continue;
            }
            List<SopCell> stepContents = sopCell.getStepContent();
            if (CollUtil.isEmpty(stepContents)){
                continue;
            }
            for (int j=stepContents.size()-1;j>=0;j--){
                SopCell stepContent = stepContents.get(j);
                if (stepContent.getFromSop() && removedSopIds.contains(stepContent.getRelSopId())){
                    log.error("sop:"+sopId + "步骤中引用sop:"+stepContent.getRelSopId()+"已作废,删除引用！");
                    needRemoveCellIds.add(stepContent.getSopCellId());
                    stepContents.remove(j);
                }
            }
            if (CollUtil.isEmpty(stepContents)){
                needRemoveCellIds.add(sopCell.getSopCellId());
                sopCells.remove(i);
            }
        }
        return sopBean;
    }


    /***
     * Description: 查询引用的SOP内容
     * @param sopId : sop的唯一标识
     * @return : void
     * @author : lijie
     * @date :2021/4/13 9:38
     * Update By lijie 2021/4/13 9:38
     */
    private void queryQuotedSopBySopId(String sopId) {





    }

    /***
     * Description: 根据工单类型id数组批量查询工单名称
     * @param orderType : 工单类型id数组
     * @return : java.util.List<java.lang.String>
     * @author : lijie
     * @date :2021/4/10 19:40
     * Update By lijie 2021/4/10 19:40
     */
    private List<String> queryOrderTypeNameByIds(List<String> orderType) {
        if (CollUtil.isEmpty(orderType)) {
            return new ArrayList<>();
        }
        List<String> orderTypeName = new ArrayList<>();
        String woFlowPlaname;
        for (int i = 0; i < orderType.size(); i++) {
            woFlowPlaname = woFlowPlanService.getNameById(orderType.get(i));
            orderTypeName.add(woFlowPlaname);
        }
        return orderTypeName;
    }
    /***
     * Description: 根据设备模型id查询设备模型列表
     * @param equipModelIds : 设备模型id数组
     * @return : java.util.List<com.persagy.fm.sop.model.dto.EquipModelBean>
     * @author : lijie
     * @date :2021/4/10 19:46
     * Update By lijie 2021/4/10 19:46
     */
    private List<EquipModelBean> queryEquipModelsByIds(List<String> equipModelIds) throws Exception {
        if (CollUtil.isEmpty(equipModelIds)) {
            return new ArrayList<>();
        }
        JSONObject param = new JSONObject();
        param.put(SopQueryConst.EQUIP_MODEL_IDS, equipModelIds);
        String resultStr = equipModelService.queryBathchMergeRecordByequipModelIds(param.toJSONString(), SopQueryConst.EQUIP_MODEL_IDS, SopQueryConst.EQUIP_MODEL_ID);
        JSONObject resultObj = JSONObject.parseObject(resultStr);
        JSONArray resultContent = resultObj.getJSONArray(ResultEnum.CONTENT.getType());
        List<EquipModelBean> equipModelBeans = new ArrayList<>();
        if (null != resultContent && resultContent.size() > 0) {
            JSONObject equipModel, resultCompanyObj;
            String resultCompanyStr;
            for (int i = 0, length = resultContent.size(); i < length; i++) {
                equipModel = resultContent.getJSONObject(i);
                resultCompanyStr = equipCompanyBookService.queryRecordByCompanyId(equipModel.toJSONString());
                resultCompanyObj = JSONObject.parseObject(resultCompanyStr);
                if (ResultEnum.SUCCESS.getType().equals(resultCompanyObj.getString(ResultEnum.RESULT.getType()))
                        && resultCompanyObj.getJSONArray(ResultEnum.CONTENT.getType()).size() > 0) {
                    equipModel.put("company_name", resultCompanyObj.getJSONArray(ResultEnum.CONTENT.getType()).getJSONObject(0).getString("company_name"));
                }
            }
            equipModelBeans = resultContent.toJavaList(EquipModelBean.class);
        }
        return equipModelBeans;
    }

    /***
     * Description: 根据sopID查询改sop待修订消息中
     * @param sopId : sopId
     * @return : java.util.Map<java.lang.String,com.alibaba.fastjson.JSONObject>  
     * @author : lijie
     * @date :2021/4/10 19:11
     * Update By lijie 2021/4/10 19:11
     */
    public Map<String, SopReviseMsg> queryChangedObjIdBySopId(String sopId) throws Exception {
        LambdaQueryWrapper<SopReviseMsg> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopReviseMsg::getSopId, sopId);
        List<SopReviseMsg> sopReviseMsgs = sopReviseMsgService.list(queryWrapper);
        if (CollUtil.isEmpty(sopReviseMsgs)) {
            return new HashMap<>();
        }
        return sopReviseMsgs.stream().collect(Collectors.toMap(SopReviseMsg::getObjId, sopReviseMsg -> sopReviseMsg, (k1, k2) -> k1));
    }

    /***
     * Description: 封装SOP列表
     * @param sops : SOP列表
     * @param destoryFlag : 标志
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/6 18:45
     * Update By lijie 2021/4/6 18:45
     */
    private List<SopBean> filterRemindSopList(List<Sop> sops, boolean destoryFlag) throws Exception {
        List<SopBean> editSopArray = new ArrayList<>();
        Set<String> sopIds = sops.stream().map(Sop::getSopId).collect(Collectors.toSet());
        Map<String, List<String>> orderTypeIdMap = queryOrderTypeMapBySopIds(sopIds);
        Map<String, List<SopObjRel>> fitObjMap = queryObjMapBySopIds(sopIds, ObjOwnerEnum.FIT_OBJ.getType());
        Map<String, List<SopLabelRel>> labelMap = queryLabelMapBySopIds(sopIds);
        Map<String, List<SopEquipModelRel>> equipModelMap = queryEquipModelMapBySopIds(sopIds);
        Map<String, String> allPlanNameMap = woFlowPlanService.queryAllPlanNameMap();
        for (Sop sop : sops) {
            //组合适用范围 工单类型order_type、适用对象fit_objs、品牌brands、自定义标签labels
            List<String> scopes = new ArrayList<>();
            //工单类型
            List<String> orderTypeSet = orderTypeIdMap.getOrDefault(sop.getSopId(), new ArrayList<>());
            for (String orderTypeId : orderTypeSet) {
                if (StrUtil.isNotBlank(allPlanNameMap.get(orderTypeId))) {
                    scopes.add(allPlanNameMap.get(orderTypeId));
                }
            }
            //适用对象
            List<SopObjRel> fitObjSet = fitObjMap.getOrDefault(sop.getSopId(), new ArrayList<>());
            for (SopObjRel sopObjRel : fitObjSet) {
                if (StrUtil.isNotBlank(sopObjRel.getObjName())) {
                    scopes.add(sopObjRel.getObjName());
                }
            }

            // 设备型号
            List<SopEquipModelRel> equipModelRels = equipModelMap.get(sop.getSopId());
            if (CollUtil.isNotEmpty(equipModelRels)) {
                Set<String> equipModelIds = equipModelRels.stream()
                        .map(SopEquipModelRel::getEquipModelId).collect(Collectors.toSet());
                JSONObject equipModelObj = new JSONObject();
                equipModelObj.put("equip_model_ids", equipModelIds);
                String resultStr = equipModelService.queryBathchMergeRecordByequipModelIds(equipModelObj.toJSONString(), "equip_model_ids", "equip_model_id");
                JSONObject resultObj = JSONObject.parseObject(resultStr);
                JSONArray resultContent = resultObj.getJSONArray(ResultEnum.CONTENT.getType());
                if (ResultEnum.SUCCESS.getType().equals(resultObj.getString(ResultEnum.RESULT.getType()))
                        && resultContent.size() > 0) {
                    for (int j = 0; j < resultContent.size(); j++) {
                        JSONObject content = resultContent.getJSONObject(j);
                        scopes.add(content.getString("equip_model_name"));
                    }
                }
            }

            //自定义标签
            List<SopLabelRel> sopLabelRels = labelMap.getOrDefault(sop.getSopId(), new ArrayList<>());
            scopes.addAll(sopLabelRels.stream().map(SopLabelRel::getLabel).collect(Collectors.toSet()));
            //组合数据
            SopBean sopBean = SopBean.builder()
                    .sopId(sop.getSopId())
                    .sopName(sop.getSopName())
                    .version(sop.getVersion())
                    .scopes(scopes)
                    .updateTime(DateUtil.transferDateFormat(sop.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show))
                    .build();
            if (!destoryFlag) {
                sopBean.setSopStatus(sop.getSopStatus());
            }
            if (StrUtil.isNotBlank(sop.getOperatorId()) && StrUtil.isNotBlank(sop.getOperatorName())) {
                sopBean.setOperator(OperatorBean.builder().id(sop.getOperatorId()).name(sop.getOperatorName()).build());
            }
            editSopArray.add(sopBean);
        }
        return editSopArray;
    }

    /***
     * Description: 条件过滤
     * @param sopEdits : SOP的编辑列表数据
     * @param destoryFlag : 是否作废标记
     * @return : com.alibaba.fastjson.JSONObject
     * @author : lijie
     * @date :2021/4/10 18:16
     * Update By lijie 2021/4/10 18:16
     */
    public List<SopBean> filterRemind(List<SopEdit> sopEdits, boolean destoryFlag) throws Exception {
        String sopName;
        List<SopBean> editSopArray = new ArrayList<>();
        for (SopEdit sopEdit : sopEdits) {
            sopName = sopEdit.getSopName();
            //组合适用范围 工单类型order_type、适用对象fit_objs、品牌brands、自定义标签labels
            List<String> scopes = new ArrayList<>();
            //工单类型
            List<String> orderTypeIds = sopEdit.getOrderType();
            if (CollUtil.isNotEmpty(orderTypeIds)) {
                for (int j = 0; j < orderTypeIds.size(); j++) {
                    String dictName = woFlowPlanService.getNameById(orderTypeIds.get(j));
                    if (dictName != null) {
                        scopes.add(dictName);
                    }
                }
            }
            //适用对象
            List<FitObjsBean> fitObjs = sopEdit.getFitObjs();
            if (CollUtil.isNotEmpty(fitObjs)) {
                for (int j = 0; j < fitObjs.size(); j++) {
                    FitObjsBean fitObjsBean = fitObjs.get(j);
                    if (StrUtil.isNotBlank(fitObjsBean.getObjName())) {
                        scopes.add(fitObjsBean.getObjName());
                    }
                }
            }
            //设备型号
            List<String> equipModelIds = sopEdit.getEquipModelIds();
            if (CollUtil.isNotEmpty(equipModelIds)) {
                JSONObject equipModelObj = new JSONObject();
                equipModelObj.put("equip_model_ids", equipModelIds);
                String resultStr = equipModelService.queryBathchMergeRecordByequipModelIds(equipModelObj.toJSONString(), "equip_model_ids", "equip_model_id");
                JSONObject resultObj = JSONObject.parseObject(resultStr);
                JSONArray resultContent = resultObj.getJSONArray(ResultEnum.CONTENT.getType());
                if (ResultEnum.SUCCESS.getType().equals(resultObj.getString(ResultEnum.RESULT.getType()))
                        && resultContent.size() > 0) {
                    for (int j = 0; j < resultContent.size(); j++) {
                        JSONObject content = resultContent.getJSONObject(j);
                        scopes.add(content.getString("equip_model_name"));
                    }
                }
            }

            //自定义标签
            List<String> labels = sopEdit.getLabels();
            if (CollUtil.isNotEmpty(labels)) {
                scopes.addAll(labels);
            }
            //组合数据
            SopBean sopBean = SopBean.builder()
                    .sopId(sopEdit.getSopId())
                    .sopName(sopName)
                    .version(sopEdit.getVersion())
                    .scopes(scopes)
                    .updateTime(DateUtil.transferDateFormat(sopEdit.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show))
                    .build();
            if (!destoryFlag) {
                sopBean.setSopStatus(sopEdit.getSopStatus());
            }
            if (null != sopEdit.getOperator()) {
                sopBean.setOperator(sopEdit.getOperator());
            }
            editSopArray.add(sopBean);
        }
        return editSopArray;
    }

    /***
     * Description: 获得查询SOP列表的返回值
     * @param content : sop列表
     * @param publishedListVo : 返回的结果对象
     * @return : com.jayway.jsonpath.Criteria
     * @author : lijie
     * @date :2021/4/10 17:29
     * Update By lijie 2021/4/10 17:29
     */
    private void getSopListCriteria(List<SopBean> content, PublishedListVo publishedListVo) throws Exception {
        // 遍历结果过滤字段、组合返回查询参数
        if (content != null) {
            HashMap<String, String> equipModelMap = collectEquipModeFromSops(content);
            List<EquipModelBean> equipModels = new ArrayList<>();
            for (String equipModelId : equipModelMap.keySet()) {
                equipModels.add(EquipModelBean.builder()
                        .equipModelId(equipModelId)
                        .equipModelName(equipModelMap.getOrDefault(equipModelId, ""))
                        .build());
            }

            HashMap<String, FitObjsBean> fitObjMap = collectFitObjsFromSops(content);
            List<FitObjsBean> fitObjs = new ArrayList<>();
            for (String fitObjId : fitObjMap.keySet()) {
                fitObjs.add(fitObjMap.get(fitObjId));
            }

            HashMap<String, OrderTypeBean> OrderTypeMap = collectOrderTypeFromSops(content);
            List<OrderTypeBean> orderTypes = new ArrayList<>();
            for (String OrderTypeId : OrderTypeMap.keySet()) {
                orderTypes.add(OrderTypeMap.get(OrderTypeId));
            }

            Set<String> labels = new HashSet<>();
            for (int i = 0; i < content.size(); i++) {
                SopBean sop = content.get(i);
                List<String> labelsItem = sop.getLabels();
                if (labelsItem == null) {
                    continue;
                }
                labels.addAll(labelsItem);
            }

            HashMap<String, SopSignBean> signMap = getBathchMergeRecordBySign(content);
            List<SopSignBean> signs = new ArrayList<>();
            for (String signCode : signMap.keySet()) {
                signs.add(signMap.get(signCode));
            }
            publishedListVo.setCriteria(PublishedListVo.Criteria.builder()
                    .equipModels(equipModels)
                    .labels(new ArrayList<>(labels))
                    .fitObjs(fitObjs)
                    .orderType(orderTypes)
                    .signs(signs).build());
        }
    }

    /***
     * Description: 获得SOP的标记对象
     * @param sops : sop列表
     * @return : java.util.HashMap<java.lang.String,java.lang.Object>
     * @author : lijie
     * @date :2021/4/10 17:44
     * Update By lijie 2021/4/10 17:44
     */
    private HashMap<String, SopSignBean> getBathchMergeRecordBySign(List<SopBean> sops) {
        HashMap<String, SopSignBean> signMap = new HashMap<>();
        if (sops.isEmpty()) {
            return signMap;
        }
        for (int i = 0; i < sops.size(); i++) {
            SopBean sop = sops.get(i);
            if (null != sop.getSign()) {
                SopSignBean sign = sop.getSign();
                if (!signMap.containsKey(sign.getCode())) {
                    signMap.put(sign.getCode(), sign);
                }
            }
        }
        return signMap;
    }

    /***
     * Description: 收集工单类型数据
     * @param sops : sops列表数据
     * @return : java.util.HashMap<java.lang.String,java.lang.Object>
     * @author : lijie
     * @date :2021/4/10 17:41
     * Update By lijie 2021/4/10 17:41
     */
    private HashMap<String, OrderTypeBean> collectOrderTypeFromSops(List<SopBean> sops) throws Exception {
        HashMap<String, OrderTypeBean> map = new HashMap<>();
        if (sops.isEmpty()) {
            return map;
        }
        for (int i = 0; i < sops.size(); i++) {
            SopBean sop = sops.get(i);
            List<String> orderTypes = sop.getOrderType();
            for (int j = 0; j < orderTypes.size(); j++) {
                String orderTypeId = orderTypes.get(j);
                if (!map.containsKey(orderTypeId)) {
                    map.put(orderTypeId, OrderTypeBean.builder()
                            .code(orderTypeId)
                            .name(orderTypeCache.getOrderTypeNameById(orderTypeId)).build());
                }
            }
        }
        return map;
    }

    /***
     * Description: 收集SOP的对象信息
     * @param sops : sop列表
     * @return : java.util.HashMap<java.lang.String,com.persagy.fm.sop.entity.SopObjRel>
     * @author : lijie
     * @date :2021/4/10 17:40
     * Update By lijie 2021/4/10 17:40
     */
    private HashMap<String, FitObjsBean> collectFitObjsFromSops(List<SopBean> sops) {
        HashMap<String, FitObjsBean> map = new HashMap<>();
        if (!sops.isEmpty()) {
            for (int i = 0; i < sops.size(); i++) {
                SopBean sop = sops.get(i);
                List<SopObjRel> fitObjs = sop.getFitObjs();
                for (int j = 0; j < fitObjs.size(); j++) {
                    SopObjRel objRel = fitObjs.get(j);
                    map.put(objRel.getObjId(), BeanUtil.copyProperties(objRel, FitObjsBean.class));
                }
            }
        }
        return map;
    }

    /**
     * Description: 获取到sop中所有设备型号
     *
     * @param sopBeans 集合中的对象中包含equip_model_ids
     * @return HashMap<String, String>    key:equip_model_id ; value:equip_model_name
     * @throws Exception
     * @author yuecaipu
     * @since 2018年7月12日: 上午11:11:17
     * Update By yuecaipu 2018年7月12日: 上午11:11:17
     */
    private HashMap<String, String> collectEquipModeFromSops(List<SopBean> sopBeans) throws Exception {
        HashMap<String, String> equipModelMap = new HashMap<>();
        if (!sopBeans.isEmpty()) {
            Map<String, List<SopEquipModelRel>> setMap = queryEquipModelMapBySopIds(sopBeans.stream().map(SopBean::getSopId).collect(Collectors.toSet()));
            Set<String> sopModelIds = new HashSet<>();
            setMap.values().forEach(set -> sopModelIds.addAll(set.stream().map(SopEquipModelRel::getEquipModelId).collect(Collectors.toSet())));
            if (!sopModelIds.isEmpty()) {
                JSONObject param = new JSONObject();
                param.put(SopQueryConst.EQUIP_MODEL_IDS, sopModelIds);
                String equipModelStrs = equipModelService.queryBathchMergeRecordByequipModelIds(param.toJSONString(),
                        SopQueryConst.EQUIP_MODEL_IDS, SopQueryConst.EQUIP_MODEL_ID);
                JSONObject equipModelObjs = JSONObject.parseObject(equipModelStrs);
                JSONArray equipModels = equipModelObjs.getJSONArray(ResultEnum.CONTENT.getType());
                if (null!=equipModels && equipModels.size()>=0){
                    for (int j = 0, length = equipModels.size(); j < length; j++) {
                        JSONObject equipModel = equipModels.getJSONObject(j);
                        equipModelMap.put(equipModel.getString(SopQueryConst.EQUIP_MODEL_ID), equipModel.getString(SopQueryConst.EQUIP_MODEL_NAME));
                    }
                }
            }
        }
        return equipModelMap;
    }

    /**
     * Description: 补全sop详情信息
     *
     * @param sopBeans:sop对象数组
     * @param projectId:项目id
     * @param queryType:1-查询发布  2-查询可复制引用
     * @throws Exception
     * @author yuecaipu
     * @since 2018年7月12日: 下午12:23:55
     * Update By yuecaipu 2018年7月12日: 下午12:23:55
     */
    private List<SopBean> additionalSopDetail(List<SopBean> sopBeans, String projectId, String queryType) throws Exception {
        List<SopBean> content = new ArrayList<>();
        if (!sopBeans.isEmpty()) {
            HashMap<String, String> equipModelMap = collectEquipModeFromSops(sopBeans);
            SopBean sop;
            sopBeans = sopBeans.stream()
                    .sorted((sop1, sop2) -> sop2.getUpdateTime().compareTo(sop1.getUpdateTime()))
                    .collect(Collectors.toList());
            for (int i = 0, length = sopBeans.size(); i < length; i++) {
                sop = sopBeans.get(i);
                sop.setUpdateTime(DateUtil.transferDateFormat(sop.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show));
                if ("2".equals(queryType)) {
                    sop.setSopType(sop.getSopType());
                    sop.setStepCount(sop.getSteps().size());
                }
                if ("1".equals(queryType)) {
                    sop.setSign(getSopSign(sop, projectId));
                    sop.setStatus(projectId.equals(sop.getProjectId()) ? sop.getPublishStatus() : BoolEnum.TRUE.getType());
                    List<String> scopes = new ArrayList<>();
                    List<SopObjRel> fitObjs = sop.getFitObjs();
                    for (int j = 0, len = fitObjs.size(); j < len; j++) {
                        scopes.add(fitObjs.get(j).getObjName());
                    }

                    List<String> equipModelIds = sop.getEquipModelIds();
                    if (equipModelIds != null && equipModelIds.size() > 0) {
                        for (String equipModelId : equipModelIds) {
                            if (StrUtil.isNotBlank(equipModelMap.get(equipModelId))) {
                                scopes.add(equipModelMap.get(equipModelId));
                            }
                        }
                    }
                    List<String> orderTypes = sop.getOrderType();
                    String orderTypeName;
                    for (String orderTypeId : orderTypes) {
                        orderTypeName = orderTypeCache.getOrderTypeNameById(orderTypeId);
                        if (StrUtil.isNotBlank(orderTypeName)) {
                            scopes.add(orderTypeName);
                        }
                    }
                    sop.setScopes(scopes);
                }
                content.add(sop);
            }
        }
        return content;
    }

    /***
     * Description: 获得SOP标签对象
     * @param sopBean : sop对象
     * @param projectId : 项目id
     * @return : com.persagy.fm.sop.model.vo.SignVo
     * @author : lijie
     * @date :2021/4/10 17:36
     * Update By lijie 2021/4/10 17:36
     */
    private SopSignBean getSopSign(SopBean sopBean, String projectId) {
        SopSignBean sign = null;
        if (SopCommonConst.GROUP_SOP_ID.equals(projectId)) {
            String publishStatus = sopBean.getPublishStatus();
            if (SopConst.PUBLISH_STATUS_REVISE.equals(publishStatus)) {
                sign = SopSignBean.builder().code("revise").name("待修订").build();
            } else if (SopConst.PUBLISH_STATUS_REVISEING.equals(publishStatus)) {
                sign = SopSignBean.builder().code("revising").name("编辑中").build();
            }
        } else {
            if (SopConst.SOP_TYPE_GROUP.equals(sopBean.getSopType())) {//sop_type:1 集团sop  2项目专用sop
                sign = SopSignBean.builder().code("public").name("集团").build();
            } else {
                String publishStatus = sopBean.getPublishStatus();
                if (SopConst.PUBLISH_STATUS_REVISE.equals(publishStatus)) {
                    sign = SopSignBean.builder().code("revise").name("待修订").build();
                } else if (SopConst.PUBLISH_STATUS_REVISEING.equals(publishStatus)) {
                    sign = SopSignBean.builder().code("revising").name("编辑中").build();
                }
            }
        }
        return sign;
    }

    /***
     * Description: 根据适用范围筛选sop
     * @param publishedListVo : 请求参数
     * @param sops : sop列表
     * @return : java.util.List<com.persagy.fm.sop.entity.Sop>
     * @author : lijie
     * @date :2021/4/10 17:18
     * Update By lijie 2021/4/10 17:18
     */
    private List<Sop> filterSopByFitScope(QueryPublishedListVo publishedListVo, List<Sop> sops) throws Exception {
        List<Sop> resultSops = new ArrayList<>();
        if (CollUtil.isEmpty(publishedListVo.getEquipModelIds())
                && CollUtil.isEmpty(publishedListVo.getLabels())
                && CollUtil.isEmpty(publishedListVo.getOrderTypeIds())
                && CollUtil.isEmpty(publishedListVo.getFitObjIds())) {
            return sops;
        }
        HashMap<String, Sop> sopMap = getSopMap(sops);
        Set<String> filtSopIds = new HashSet<>();
        //1.根据适用返回查询sopid
        //1.1根据设备型号查询
        if (CollUtil.isNotEmpty(publishedListVo.getEquipModelIds())) {
            List<SopEquipModelRel> sopEquipModelRels = queryEquipModelRelsByEquipModelIds(publishedListVo.getEquipModelIds());
            filtSopIds.addAll(sopEquipModelRels.stream().map(SopEquipModelRel::getSopId).collect(Collectors.toSet()));
        }
        //1.2根据自定义标签查询
        if (CollUtil.isNotEmpty(publishedListVo.getLabels())) {
            List<SopLabelRel> labelsResJson = queryLabelsByLabelName(publishedListVo.getLabels());
            filtSopIds.addAll(labelsResJson.stream().map(SopLabelRel::getSopId).collect(Collectors.toSet()));
        }
        //1.3根据工单类型查询
        if (CollUtil.isNotEmpty(publishedListVo.getOrderTypeIds())) {
            List<SopOrderTypeRel> orderTypeResJson = queryOrderTypeByOrderTypeIds(publishedListVo.getOrderTypeIds());
            filtSopIds.addAll(orderTypeResJson.stream().map(SopOrderTypeRel::getSopId).collect(Collectors.toSet()));
        }
        //1.4根据适用对象查询
        if (CollUtil.isNotEmpty(publishedListVo.getFitObjIds())) {
            List<SopObjRel> fitObjResJson = queryFitObjsByIds(publishedListVo.getFitObjIds());
            filtSopIds.addAll(fitObjResJson.stream().map(SopObjRel::getParentId).collect(Collectors.toSet()));
        }
        for (String sopId : filtSopIds) {
            if (sopMap.containsKey(sopId)) {
                resultSops.add(sopMap.get(sopId));
            }
        }
        return resultSops;
    }

    /***
     * Description: 获得SOP与SOP的唯一标识的映射
     * @param sops : sop列表
     * @return : java.util.HashMap<java.lang.String,com.persagy.fm.sop.entity.Sop>
     * @author : lijie
     * @date :2021/4/10 17:21
     * Update By lijie 2021/4/10 17:21
     */
    private HashMap<String, Sop> getSopMap(List<Sop> sops) {
        HashMap<String, Sop> resultMap = new HashMap<>();
        for (int i = 0, length = sops.size(); i < length; i++) {
            Sop sop = sops.get(i);
            resultMap.put(sop.getSopId(), sop);
        }
        return resultMap;
    }

    /**
     * Description: 根据signCodes筛选sop（交集）
     *
     * @param signCodes
     * @param sops
     * @return JSONArray
     * @author yuecaipu
     * @since 2018年7月10日: 下午12:09:41
     * Update By yuecaipu 2018年7月10日: 下午12:09:41
     */
    private List<Sop> filterSopBySignCodes(List<String> signCodes, List<Sop> sops, String projectId) {
        List<Sop> resultSops = new ArrayList<>();
        if (CollUtil.isEmpty(signCodes) || CollUtil.isEmpty(sops)) {
            return sops;
        }
        HashSet<String> signCodeSet = new HashSet<>(signCodes);
        for (Sop sop : sops) {
            // 1.公共 && 集团SOP
            if (signCodeSet.contains(SopSignCodeEnum.PUBLIC.getType())
                    && SopTypeEnum.GROUP.getType().equals(sop.getSopType())) {
                resultSops.add(sop);
            }
            // 2.待修订 && 项目SOP
            if (signCodeSet.contains(SopSignCodeEnum.REVISE.getType())
                    && sop.getProjectId().equals(projectId)
                    && SopPublishStatusEnum.REVISE.getType().equals(sop.getPublishStatus())) {
                resultSops.add(sop);
            }
            // 3.修订中 && 项目SOP
            if (signCodeSet.contains(SopSignCodeEnum.REVISING.getType())
                    && sop.getProjectId().equals(projectId)
                    && SopPublishStatusEnum.REVISEING.getType().equals(sop.getPublishStatus())) {
                resultSops.add(sop);
            }
        }
        return resultSops;
    }

    /***
     * Description: 根据项目类型移除SOP列表
     * @param sops : sop列表
     * @param rootProjFuncType : 项目功能类型
     * @return : void
     * @author : lijie
     * @date :2021/4/10 16:56
     * Update By lijie 2021/4/10 16:56
     */
    private void removeWithoutProJFuncType(List<Sop> sops, String rootProjFuncType) {
        for (int i = 0, len = sops.size(); i < len; i++) {
            Sop sop = sops.get(i);
            if (!StringUtil.isNull(sop.getExtendField())
                    && !sop.getExtendField().equals(rootProjFuncType)) {
                sops.remove(i);
                i--;
                len--;
            }
        }
    }

    /***
     * Description:移除待修订集团sop
     * @param sops : sop列表
     * @return : void
     * @author : lijie
     * @date :2021/4/10 16:55
     * Update By lijie 2021/4/10 16:55
     */
    private void removeGroupReviseSop(List<Sop> sops) {
        if (CollUtil.isEmpty(sops)) {
            return;
        }
        for (int i = 0; i < sops.size(); i++) {
            Sop sop = sops.get(i);
            if (SopConst.PUBLISH_STATUS_REVISE.equals(sop.getPublishStatus())
                    && SopCommonConst.GROUP_SOP_ID.equals(sop.getProjectId())) {
                sops.remove(i);
                i--;
            }
        }
    }

    /***
     * Description: 移除待修订sop
     * @param sops : sop列表
     * @return : void
     * @author : lijie
     * @date :2021/4/10 16:54
     * Update By lijie 2021/4/10 16:54
     */
    private void removeReviseSop(List<Sop> sops) {
        if (!sops.isEmpty()) {
            for (int i = 0; i < sops.size(); i++) {
                Sop sop = sops.get(i);
                if (SopConst.PUBLISH_STATUS_REVISE.equals(sop.getPublishStatus())) {
                    sops.remove(i);
                    i--;
                }
            }
        }
    }

    /***
     * Description: 当sop涉及到的对象删除时的操作
     * @param sopIds : SOP的id集合
     * @param objectChangeVo : 请求参数
     * @return : boolean  
     * @author : lijie
     * @date :2021/4/10 15:13
     * Update By lijie 2021/4/10 15:13
     */
    private boolean deleteObject(Set<String> sopIds, ObjectChangeVo objectChangeVo) {
        if (CollUtil.isEmpty(sopIds)) {
            return true;
        }
        // 1.保存消息
        List<SopReviseMsg> sopReviseMsgs = new ArrayList<>();
        for (String sopId : sopIds) {
            sopReviseMsgs.add(SopReviseMsg.builder()
                    .objId(objectChangeVo.getMsObjId())
                    .objName(objectChangeVo.getMsObjName())
                    .objType(MsgTypeEnum.TWO.getType())
                    .content("因" + objectChangeVo.getMsObjName() + "作废，此标准操作知识部分信息点受影响，待修订")
                    .status("1")
                    .sopId(sopId).build());
        }
        sopReviseMsgService.saveBatch(sopReviseMsgs);
        // 2.修改sop状态为待修订
        LambdaUpdateWrapper<Sop> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.in(Sop::getSopId, sopIds);
        return sopService.update(Sop.builder().publishStatus(SopPublishStatusEnum.REVISE.getType()).build(), updateWrapper);
    }


    /***
     * Description: 根据对象id查询关联的关系
     * @param objId : 对象id
     * @return : java.util.List<com.persagy.fm.sop.entity.SopObjRel>
     * @author : lijie
     * @date :2021/4/10 14:59
     * Update By lijie 2021/4/10 14:59
     */
    private List<SopObjRel> querySopObjRelByObjId(String objId) {
        if (StrUtil.isBlank(objId)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SopObjRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SopObjRel::getObjId, objId);
        return sopObjRelService.list(queryWrapper);
    }

    /***
     * Description: 批量添加草稿状态的SOP
     * @param saveSopVos : 需要保存的SOP列表
     * @return : java.util.List<java.lang.String>
     * @author : lijie
     * @date :2021/4/2 14:35
     * Update By lijie 2021/4/2 14:35
     */
    private List<String> batchAddDraftSop(List<SaveSopVo> saveSopVos) {
        if (CollUtil.isEmpty(saveSopVos)) {
            return new ArrayList<>();
        }
        // 根据SOP的id获取编辑表中的草稿数据
        Set<String> querySopIds = saveSopVos.stream().map(SaveSopVo::getSopId).collect(Collectors.toSet());
        List<SopEdit> hisSopEdits = querySopEditsBySopIds(querySopIds);
        Map<String, SopEdit> hisSopEditMap = hisSopEdits.stream()
                .collect(Collectors.toMap(SopEdit::getSopId, sopEdit -> sopEdit, (k1, k2) -> k1));
        List<String> sopIds = new ArrayList<>();
        List<SopEdit> sopEdits = new ArrayList<>();
        for (SaveSopVo saveSopVo : saveSopVos) {
            // 1.获取关联的SOP的关联对象id
            List<String> refSopIds = getSopRefSops(saveSopVo.getSteps());
            // 2.创建草稿状态的SOP编辑对象
            SopEdit sopEdit = BeanUtil.copyProperties(saveSopVo, SopEdit.class);
            if (StrUtil.isBlank(sopEdit.getSopEditId())){
                SopEdit hisSopEdit = hisSopEditMap.get(sopEdit.getSopId());
                if (null!=hisSopEdit && StrUtil.isNotBlank(hisSopEdit.getSopEditId())){
                    sopEdit.setSopEditId(hisSopEdit.getSopEditId());
                }else {
                    sopEdit.setSopEditId(IdPrefixEnum.SOP_EDIT.getPrefix() + IdWorker.getIdStr());
                }
            }
            // 3.设置SOP的唯一标识id
            if (StrUtil.isBlank(sopEdit.getSopId())) {
                sopEdit.setSopId(IdPrefixEnum.SOP.getPrefix() + IdWorker.getIdStr());
            }
            // 4.设置SOP的类型 1 通用,2 项目专用
            if (StrUtil.isBlank(saveSopVo.getSopType())) {
                if (!SopCommonConst.GROUP_SOP_ID.equals(saveSopVo.getProjectId())) {
                    sopEdit.setSopType(SopCommonConst.SOP_TYPE_PROJECT);
                } else {
                    sopEdit.setSopType(SopCommonConst.SOP_TYPE_GROUP);
                }
            }
            // 5.sop状态 0 草稿,1 已发布,2 已作废
            if (StrUtil.isBlank(saveSopVo.getSopStatus())) {
                sopEdit.setSopStatus(SopStatusEnum.DRAFT.getType());
            } else {
                sopEdit.setSopStatus(SopStatusEnum.PUBLISHED.getType());
            }
            sopIds.add(sopEdit.getSopId());
            sopEdit.setRefSopIds(refSopIds);
            sopEdits.add(sopEdit);
        }
        if (CollUtil.isNotEmpty(sopEdits)) {
            sopEditService.saveOrUpdateBatch(sopEdits);
        }
        return sopIds;
    }

    /***
     * Description: 根据SOP的id集合查询所有编辑中的SOP草稿数据
     * @param querySopIds : sop的id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopEdit>
     * @author : lijie
     * @date :2021/4/23 11:30
     * Update By lijie 2021/4/23 11:30
     */
    private List<SopEdit> querySopEditsBySopIds(Set<String> querySopIds) {
        if (CollUtil.isEmpty(querySopIds)){
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SopEdit> sopEditLambdaQueryWrapper = new LambdaQueryWrapper<>();
        sopEditLambdaQueryWrapper.in(SopEdit::getSopId,querySopIds);
        return sopEditService.list(sopEditLambdaQueryWrapper);
    }

    /***
     * Description: 批量添加SOP方法
     * @param saveSopVos : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/1 16:30
     * Update By lijie 2021/4/1 16:30
     */
    @Override
    public void batchAddPublishedSop(List<SaveSopVo> saveSopVos) {
        if (CollUtil.isEmpty(saveSopVos)) {
            return;
        }
        List<Sop> sops = new ArrayList<>();
        List<SopCell> sopCells = new ArrayList<>();
        List<SopDomainRel> sopDomainRels = new ArrayList<>();
        List<SopToolRel> sopToolRels = new ArrayList<>();
        List<SopObjRel> sopObjRels = new ArrayList<>();
        List<SopEquipModelRel> sopEquipModelRels = new ArrayList<>();
        List<SopOrderTypeRel> sopOrderTypeRels = new ArrayList<>();
        List<SopLabelRel> sopLabelRels = new ArrayList<>();
        for (SaveSopVo saveSopVo : saveSopVos) {
            // 1.处理SOP对象
            Sop sop = replenishSopProperty(saveSopVo);
            sops.add(sop);
            // 2.创建SOP的cell数据
            createSopStepCells(saveSopVo.getSteps(), sop.getSopId(), sopCells, sopDomainRels, sopObjRels, sop);
            // 3.创建SOP的专业code的数据
            createSopDomainRel(saveSopVo.getDomains(), sop.getSopId(), sopDomainRels);
            // 4.创建SOP与工具的关系数据
            createSopToolRel(saveSopVo.getTools(), sop.getSopId(), sopToolRels);
            // 5.创建SOP与对象的关系数据
            createSopFitObjRel(saveSopVo.getFitObjs(), sop.getSopId(), sopObjRels);
            // 6.创建SOP与设备型号的关系数据
            createSopEquipModelRel(saveSopVo.getEquipModelIds(), sop.getSopId(), sopEquipModelRels);
            // 7.创建SOP与工单类型的关系数据
            createSopOrderTypeRel(saveSopVo.getOrderType(), sop.getSopId(), sop.getProjectId(), sopOrderTypeRels);
            // 8.创建SOP与标签的关系数据
            createSopLabelRel(saveSopVo.getLabels(), sop.getSopId(), sopLabelRels);
        }
        // 2.新增对象
        if (CollUtil.isNotEmpty(sops)) {
            sopService.saveOrUpdateBatch(sops);
        }
        if (CollUtil.isNotEmpty(sopCells)) {
            sopCellService.saveBatch(sopCells);
        }
        if (CollUtil.isNotEmpty(sopDomainRels)) {
            sopDomainRelService.saveBatch(sopDomainRels);
        }
        if (CollUtil.isNotEmpty(sopToolRels)) {
            sopToolRelService.saveBatch(sopToolRels);
        }
        if (CollUtil.isNotEmpty(sopObjRels)) {
            sopObjRelService.saveBatch(sopObjRels);
        }
        if (CollUtil.isNotEmpty(sopEquipModelRels)) {
            sopEquipModelRelService.saveBatch(sopEquipModelRels);
        }
        if (CollUtil.isNotEmpty(sopLabelRels)) {
            sopLabelRelService.saveBatch(sopLabelRels);
        }
        if (CollUtil.isNotEmpty(sopOrderTypeRels)) {
            sopOrderTypeRelService.saveBatch(sopOrderTypeRels);
        }
    }
    /***
     * Description: 补充SOP的参数 
     * @param saveSopVo : 保存参数
     * @return : void  
     * @author : lijie
     * @date :2021/4/13 19:12
     * Update By lijie 2021/4/13 19:12
     */
    private Sop replenishSopProperty(SaveSopVo saveSopVo) {
        Sop sop = BeanUtil.copyProperties(saveSopVo, Sop.class);
        // 主键id
        if (StrUtil.isBlank(sop.getSopId())) {
            sop.setSopId(IdPrefixEnum.SOP.getPrefix() + IdWorker.getIdStr());
        }
        // 操作人信息
        if (null != saveSopVo.getOperator()) {
            sop.setOperatorId(saveSopVo.getOperator().getId());
            sop.setOperatorName(saveSopVo.getOperator().getName());
        }
        // SOP类型
        if (StrUtil.isBlank(saveSopVo.getSopType())) {
            if (!SopCommonConst.GROUP_SOP_ID.equals(saveSopVo.getProjectId())) {
                sop.setSopType(SopTypeEnum.PROJECT.getType());
            } else {
                sop.setSopType(SopTypeEnum.GROUP.getType());
            }
        }
        // 发布状态
        if (StrUtil.isBlank(saveSopVo.getSopStatus())) {
            sop.setSopStatus(SopStatusEnum.PUBLISHED.getType());
        }
        // 发布状态
        if (StrUtil.isBlank(saveSopVo.getPublishStatus())) {
            sop.setPublishStatus(SopPublishStatusEnum.NORMAL.getType());
        }
        // 初始化版本
        if (StrUtil.isBlank(saveSopVo.getVersion())) {
            sop.setVersion(SopCommonConst.SOP_VERSION_INIT);
        }
        // 版本号
        if (null == sop.getVersionCount()) {
            sop.setVersionCount(ValidEnum.TRUE.getType());
        }
        // 版本表达式
        if (CollUtil.isEmpty(saveSopVo.getVersionExplain())) {
            sop.setVersionExplain(Collections.singletonList(SopCommonConst.VERSION_EXPLAIN_CREATE));
        }
        return sop;
    }

    /***
     * Description: 创建SOP确认结果
     * @param confirmResult : 确认结果数组
     * @param sopContent : SOP内容
     * @param sopCells : SOP的cell集合
     * @return : void
     * @author : lijie
     * @date :2021/4/1 20:43
     * Update By lijie 2021/4/1 20:43
     */
    private void createSopConfirmResult(List<ConfirmResultBean> confirmResult,
                                        SopCell sopContent,
                                        List<SopCell> sopCells,
                                        Sop sop) {
        if (CollUtil.isEmpty(confirmResult)) {
            return;
        }
        for (ConfirmResultBean confirmResultBean : confirmResult) {
            // 第三层的cell数据
            // 1.创建确认结果对象
            SopCell sopConfirmResult = BeanUtil.copyProperties(confirmResultBean, SopCell.class);
            sopConfirmResult.setParentId(sopContent.getSopCellId());
            sopConfirmResult.setProjectId(sop.getProjectId());
            sopConfirmResult.setSopId(sop.getSopId());
            if (StrUtil.isBlank(sopConfirmResult.getSopCellId())) {
                sopConfirmResult.setSopCellId(IdPrefixEnum.SOP_CELL.getPrefix() + IdWorker.getIdStr());
            }
            sopCells.add(sopConfirmResult);
        }
    }

    /***
     * Description: 创建工作内容关联的对象关系
     * @param contentObjs : 工作内容关联的对象数组
     * @param parentId : 父级id
     * @param sopObjRels : 关联对象集合
     * @param sopId : sopId
     * @return : void
     * @author : lijie
     * @date :2021/4/2 9:42
     * Update By lijie 2021/4/2 9:42
     */
    private void createSopContentObjsRel(List<ContentObjsBean> contentObjs,
                                         String parentId,
                                         List<SopObjRel> sopObjRels,
                                         String sopId) {
        if (CollUtil.isEmpty(contentObjs)) {
            return;
        }
        for (ContentObjsBean contentObj : contentObjs) {
            // 创建关联对象
            sopObjRels.add(SopObjRel.builder()
                    .isRevise(contentObj.getIsRevise())
                    .objId(contentObj.getObjId())
                    .parents(contentObj.getParents())
                    .objType(contentObj.getObjType())
                    .objName(contentObj.getObjName())
                    .objCode(contentObj.getObjCode())
                    .typeCode(contentObj.getTypeCode())
                    .parentId(parentId)
                    .sopId(sopId)
                    .sopObjId(IdPrefixEnum.SOP_OBJ_REL.getPrefix() + IdWorker.getIdStr())
                    .objSource(contentObj.getObjSource())
                    .objOwner(ObjOwnerEnum.CONTENT_OBJ.getType())
                    .build());
        }
    }

    /***
     * Description: 创建SOP与专业code的关系
     * @param domains : 专业code的集合
     * @param sopId : SOP的唯一标识id
     * @param sopDomainRels : SOP专业关系集合
     * @return : void
     * @author : lijie
     * @date :2021/4/1 20:49
     * Update By lijie 2021/4/1 20:49
     */
    private void createSopDomainRel(List<String> domains, String sopId, List<SopDomainRel> sopDomainRels) {
        if (CollUtil.isEmpty(domains)) {
            return;
        }
        for (String domain : domains) {
            if (StrUtil.isBlank(domain)) {
                continue;
            }
            // 1.封装专业与SOP的关系
            sopDomainRels.add(SopDomainRel.builder()
                    .domain(domain)
                    .parentId(sopId)
                    .sopDomainRelId(IdPrefixEnum.SOP_DOMAIN_REL.getPrefix() + IdWorker.getIdStr())
                    .build());
        }
    }

    /***
     * Description: 创建SOP于设备类型的关系
     * @param equipModelIds : 设备类型id数组
     * @param parentId : 父级id
     * @param sopEquipModelRels : 设备类型模型列表
     * @return : void
     * @author : lijie
     * @date :2021/4/2 10:39
     * Update By lijie 2021/4/2 10:39
     */
    private void createSopEquipModelRel(List<String> equipModelIds, String parentId, List<SopEquipModelRel> sopEquipModelRels) {
        if (CollUtil.isEmpty(equipModelIds)) {
            return;
        }
        for (String equipModelId : equipModelIds) {
            sopEquipModelRels.add(SopEquipModelRel.builder()
                    .equipModelId(equipModelId)
                    .sopEquipModelId(IdPrefixEnum.SOP_EQUIP_MODEL_REL.getPrefix() + IdWorker.getIdStr())
                    .sopId(parentId)
                    .build());
        }
    }

    /***
     * Description: 创建Sop与适用范围的关系
     * @param fitObjs : 适用范围对象
     * @param parentId : 父级id
     * @param sopObjRels : 对象列表
     * @return : void
     * @author : lijie
     * @date :2021/4/2 10:34
     * Update By lijie 2021/4/2 10:34
     */
    private void createSopFitObjRel(List<FitObjsBean> fitObjs, String parentId, List<SopObjRel> sopObjRels) {
        if (CollUtil.isEmpty(fitObjs)) {
            return;
        }
        for (FitObjsBean fitObj : fitObjs) {
            SopObjRel sopObjRel = BeanUtil.copyProperties(fitObj, SopObjRel.class);
            sopObjRel.setObjOwner(ObjOwnerEnum.FIT_OBJ.getType());
            sopObjRel.setParentId(parentId);
            sopObjRel.setSopId(parentId);
            sopObjRel.setSopObjId(IdPrefixEnum.SOP_OBJ_REL + IdWorker.getIdStr());
        }
    }

    /***
     * Description:创建SOP与标签的对象关系
     * @param labels : 标签数组
     * @param sopId : SOP的唯一标识id
     * @param sopLabelRels : SOP标签关系列表
     * @return : void
     * @author : lijie
     * @date :2021/4/2 11:04
     * Update By lijie 2021/4/2 11:04
     */
    private void createSopLabelRel(List<String> labels, String sopId, List<SopLabelRel> sopLabelRels) {
        if (CollUtil.isEmpty(labels)) {
            return;
        }
        for (String label : labels) {
            sopLabelRels.add(SopLabelRel.builder()
                    .label(label)
                    .sopId(sopId)
                    .sopLabelId(IdPrefixEnum.SOP_LABEL_REL.getPrefix()+IdWorker.getIdStr())
                    .build());
        }
    }

    /***
     * Description: 创建SOP与工单类型的关系
     * @param orderType : 工单类型id列表
     * @param sopId : SOP的唯一标识id
     * @param sopOrderTypeRels : SOP与工单类型关系列表
     * @return : void
     * @author : lijie
     * @date :2021/4/2 10:57
     * Update By lijie 2021/4/2 10:57
     */
    private void createSopOrderTypeRel(List<String> orderType, String sopId, String projectId, List<SopOrderTypeRel> sopOrderTypeRels) {
        if (CollUtil.isEmpty(orderType)) {
            return;
        }
        for (String orderTypeId : orderType) {
            sopOrderTypeRels.add(SopOrderTypeRel.builder()
                    .orderType(orderTypeId)
                    .projectId(projectId)
                    .sopId(sopId)
                    .sopOrderTypeId(IdPrefixEnum.SOP_ORDER_TYPE_REL.getPrefix() + IdWorker.getIdStr())
                    .build());
        }
    }

    /***
     * Description: 创建SOP与图片的关系对象
     * @param pics : 图片数组
     * @param parentId : 父级id
     * @param sopMediaRels : SOP与附件的关系对象数组
     * @return : void
     * @author : lijie
     * @date :2021/4/1 21:55
     * Update By lijie 2021/4/1 21:55
     */
    private void createSopPicRel(List<String> pics, String parentId, List<SopMediaRel> sopMediaRels) {
        if (CollUtil.isEmpty(pics)) {
            return;
        }
        for (String pic : pics) {
            sopMediaRels.add(SopMediaRel.builder()
                    .key(pic)
                    .type(AttTypeEnum.PIC.getType())
                    .parentId(parentId)
                    .sopMediaRelId(IdPrefixEnum.SOP_MEDIA_REL.getPrefix() + IdWorker.getIdStr())
                    .build());
        }
    }

    /***
     * Description: 创建工作内容关联的对象关系
     * @param spaceScopeBeans : 工作内容关联的对象数组
     * @param parentId : 父级id
     * @param sopObjRels : 对象关系集合
     * @param sopId : sopId
     * @return : void
     * @author : lijie
     * @date :2021/4/2 9:42
     * Update By lijie 2021/4/2 9:42
     */
    private void createSopSpaceObjsRel(List<SpaceScopeBean> spaceScopeBeans,
                                       String parentId,
                                       List<SopObjRel> sopObjRels,
                                       String sopId) {
        if (CollUtil.isEmpty(spaceScopeBeans)) {
            return;
        }
        for (SpaceScopeBean spaceScopeBean : spaceScopeBeans) {
            // 创建关联对象
            sopObjRels.add(SopObjRel.builder()
                    .objId(spaceScopeBean.getObjId())
                    .objType(spaceScopeBean.getObjType())
                    .parentId(parentId)
                    .sopId(sopId)
                    .objName(spaceScopeBean.getObjName())
                    .objCode(spaceScopeBean.getObjCode())
                    .sopObjId(IdPrefixEnum.SOP_OBJ_REL.getPrefix() + IdWorker.getIdStr())
                    .objOwner(ObjOwnerEnum.SPACE_SCOPE.getType())
                    .build());
        }
    }

    /***
     * Description: 创建SOP的cell数据
     * @param steps : 步骤数据
     * @param sopId : SOP的唯一标识id
     * @param sopCells : 需要存储的SOP的cell对象
     * @param sopDomainRels : 需要存储的SOP与专业的关系对象
     * @return : void
     * @author : lijie
     * @date :2021/4/1 18:34
     * Update By lijie 2021/4/1 18:34
     */
    private void createSopStepCells(List<SopStepsBean> steps, String sopId, List<SopCell> sopCells,
                                    List<SopDomainRel> sopDomainRels, List<SopObjRel> sopObjRels, Sop sop) {
        if (CollUtil.isEmpty(steps)) {
            return;
        }
        for (SopStepsBean step : steps) {
            // 1.第一层的Cell数据
            SopCell sopCell = BeanUtil.copyProperties(step, SopCell.class);
            sopCell.setParentId(sopId);
            sopCell.setRelSopId(step.getSopId());
            if (StrUtil.isBlank(sopCell.getSopCellId())) {
                sopCell.setSopCellId(IdPrefixEnum.SOP_CELL.getPrefix() + IdWorker.getIdStr());
            }
            sopCell.setSopId(sop.getSopId());
            sopCell.setProjectId(sop.getProjectId());
            sopCells.add(sopCell);
            // 2.创建步骤内容对象
            createSopStepContent(step.getStepContent(), sopCell, sopCells, sopDomainRels, sopObjRels, sop);
        }
    }

    /***
     * Description: 解析step_content
     * @param stepContent : 步骤内容
     * @param sopStep : 步骤
     * @param sopCells : 存储的Cell对象
     * @param sopDomainRels : 存储的SOP与专业关系对象
     * @return : void
     * @author : lijie
     * @date :2021/4/1 20:16
     * Update By lijie 2021/4/1 20:16
     */
    private void createSopStepContent(List<StepContentBean> stepContent,
                                      SopCell sopStep,
                                      List<SopCell> sopCells,
                                      List<SopDomainRel> sopDomainRels,
                                      List<SopObjRel> sopObjRels,
                                      Sop sop) {
        if (CollUtil.isEmpty(stepContent)) {
            return;
        }
        for (StepContentBean stepContentBean : stepContent) {
            // 第二层的cell数据
            // 1.创建SOP内容对象
            SopCell sopContent = BeanUtil.copyProperties(stepContentBean, SopCell.class);
            sopContent.setRelSopId(stepContentBean.getSopId());
            sopContent.setParentId(sopStep.getSopCellId());
            sopContent.setSopId(sop.getSopId());
            sopContent.setProjectId(sop.getProjectId());
            if (StrUtil.isBlank(sopContent.getSopCellId())) {
                sopContent.setSopCellId(IdPrefixEnum.SOP_CELL.getPrefix() + IdWorker.getIdStr());
            }
            sopCells.add(sopContent);
            // 2.创建确认结果对象
            createSopConfirmResult(stepContentBean.getConfirmResult(), sopContent, sopCells, sop);
            // 3.创建专业与SOP内容对象
            createSopDomainRel(Collections.singletonList(stepContentBean.getDomain()), sopContent.getSopCellId(),
                    sopDomainRels);
            // 4.创建工作内容涉及的对象与SOP的关系
            createSopContentObjsRel(stepContentBean.getContentObjs(), sopContent.getSopCellId(), sopObjRels, sop.getSopId());
            // 5.创建工作内容涉及的空间对象与SOP的关系
            createSopSpaceObjsRel(stepContentBean.getSpaceScope(), sopContent.getSopCellId(), sopObjRels, sop.getSopId());
        }
    }

    /***
     * Description: 创建SOP与工具的关联关系
     * @param tools : 工具数组
     * @param sopId : SOP的唯一标识id
     * @param sopToolRels : SOP与工具的关系数组
     * @return : void
     * @author : lijie
     * @date :2021/4/1 21:27
     * Update By lijie 2021/4/1 21:27
     */
    private void createSopToolRel(List<ToolsBean> tools, String sopId, List<SopToolRel> sopToolRels) {
        if (CollUtil.isEmpty(tools)) {
            return;
        }
        // 1.创建SOP于工具的关系对象
        for (ToolsBean tool : tools) {
            sopToolRels.add(SopToolRel.builder()
                    .fromStep(tool.getFromStep())
                    .parentId(sopId)
                    .sopToolRelId(IdPrefixEnum.SOP_TOOL_REL.getPrefix() + IdWorker.getIdStr())
                    .tool(tool.getTool())
                    .build());
        }
    }

    /***
     * Description: 删除所有的Sop的关系
     * @param sopId : sopId
     * @return : void
     * @author : lijie
     * @date :2021/4/8 16:28
     * Update By lijie 2021/4/8 16:28
     */
    private void deleteAllRelsBySopId(String sopId) {
        // 查询其关联的所有SopCell
        List<SopCell> sopCells = querySopCellBySopId(sopId);
        List<String> cellIds = sopCells.stream().map(SopCell::getSopCellId).collect(Collectors.toList());
        // 删除所有的Cell
        sopCellService.removeByIds(cellIds);
        // 删除所有的工具关系
        cellIds.add(sopId);
        LambdaQueryWrapper<SopToolRel> toolQueryWrapper = new LambdaQueryWrapper<>();
        toolQueryWrapper.in(SopToolRel::getParentId, cellIds);
        sopToolRelService.remove(toolQueryWrapper);
        // 删除所有专业关系
        LambdaQueryWrapper<SopDomainRel> domainQueryWrapper = new LambdaQueryWrapper<>();
        domainQueryWrapper.in(SopDomainRel::getParentId, cellIds);
        sopDomainRelService.remove(domainQueryWrapper);
        // 删除所有标签关系
        LambdaQueryWrapper<SopLabelRel> labelQueryWrapper = new LambdaQueryWrapper<>();
        labelQueryWrapper.in(SopLabelRel::getSopId, cellIds);
        sopLabelRelService.remove(labelQueryWrapper);
        // 删除所有对象关系
        LambdaQueryWrapper<SopObjRel> objRelQueryWrapper = new LambdaQueryWrapper<>();
        objRelQueryWrapper.in(SopObjRel::getParentId, cellIds);
        sopObjRelService.remove(objRelQueryWrapper);
        // 删除所有工单类型关系
        LambdaQueryWrapper<SopOrderTypeRel> orderTypeQueryWrapper = new LambdaQueryWrapper<>();
        orderTypeQueryWrapper.in(SopOrderTypeRel::getSopId, cellIds);
        sopOrderTypeRelService.remove(orderTypeQueryWrapper);
        // 删除所有设备模型关系
        LambdaQueryWrapper<SopEquipModelRel> modelQueryWrapper = new LambdaQueryWrapper<>();
        modelQueryWrapper.in(SopEquipModelRel::getSopId, cellIds);
        sopEquipModelRelService.remove(modelQueryWrapper);
        // 删掉SOP
        sopService.removeById(sopId);
    }

    /***
     * Description: 删除所有的Sop的关系
     * @param sopId : sopId
     * @return : void
     * @author : lijie
     * @date :2021/4/8 16:28
     * Update By lijie 2021/4/8 16:28
     */
    private void destoryAllRelsBySopId(String sopId) {
        // 查询其关联的所有SopCell
        List<SopCell> sopCells = querySopCellBySopId(sopId);
        List<String> cellIds = sopCells.stream().map(SopCell::getSopCellId).collect(Collectors.toList());
        // 删除所有的Cell
        SopCell sopCell = new SopCell();
        sopCell.setValid(false);
        LambdaUpdateWrapper<SopCell> sopCellLambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        sopCellLambdaUpdateWrapper.in(SopCell::getSopCellId,cellIds);
        sopCellService.update(sopCell,sopCellLambdaUpdateWrapper);
        // 删除所有的工具关系
        cellIds.add(sopId);
        LambdaUpdateWrapper<SopToolRel> toolQueryWrapper = new LambdaUpdateWrapper<>();
        toolQueryWrapper.in(SopToolRel::getParentId, cellIds);
        sopToolRelService.update(SopToolRel.builder().valid(false).build(),toolQueryWrapper);
        // 删除所有专业关系
        LambdaUpdateWrapper<SopDomainRel> domainQueryWrapper = new LambdaUpdateWrapper<>();
        domainQueryWrapper.in(SopDomainRel::getParentId, cellIds);
        sopDomainRelService.update(SopDomainRel.builder().valid(false).build(),domainQueryWrapper);
        // 删除所有标签关系
        LambdaUpdateWrapper<SopLabelRel> labelQueryWrapper = new LambdaUpdateWrapper<>();
        labelQueryWrapper.in(SopLabelRel::getSopId, cellIds);
        sopLabelRelService.update(SopLabelRel.builder().valid(false).build(),labelQueryWrapper);
        // 删除所有对象关系
        LambdaUpdateWrapper<SopObjRel> objRelQueryWrapper = new LambdaUpdateWrapper<>();
        objRelQueryWrapper.in(SopObjRel::getParentId, cellIds);
        sopObjRelService.update(SopObjRel.builder().valid(false).build(),objRelQueryWrapper);
        // 删除所有工单类型关系
        LambdaUpdateWrapper<SopOrderTypeRel> orderTypeQueryWrapper = new LambdaUpdateWrapper<>();
        orderTypeQueryWrapper.in(SopOrderTypeRel::getSopId, cellIds);
        sopOrderTypeRelService.update(SopOrderTypeRel.builder().valid(false).build(),orderTypeQueryWrapper);
        // 删除所有设备模型关系
        LambdaUpdateWrapper<SopEquipModelRel> modelQueryWrapper = new LambdaUpdateWrapper<>();
        modelQueryWrapper.in(SopEquipModelRel::getSopId, cellIds);
        sopEquipModelRelService.update(SopEquipModelRel.builder().valid(false).build(),modelQueryWrapper);
        //删除引用关系作废消息
        sopDestroyMsgService.markReadSopDestroyMsg(null,sopId);
        //删除待修订消息
        sopReviseMsgService.ignoreSopReviseMsg(null,sopId);
    }

    /***
     * Description: 填充工作内容中涉及的对象
     * @param sopCells :Sop的cell对象
     * @return : void
     * @author : lijie
     * @date :2021/4/7 12:43
     * Update By lijie 2021/4/7 12:43
     */
    private void fillContentObjsForSopCell(Set<SopCell> sopCells) {
        if (CollUtil.isEmpty(sopCells)) {
            return;
        }
        // 1.根据SOP的Cell id查询关联的对象列表
        Map<String, List<SopObjRel>> objMap = queryObjMapBySopIds(sopCells.stream()
                .map(SopCell::getSopCellId).collect(Collectors.toSet()), ObjOwnerEnum.CONTENT_OBJ.getType());
        // 2.填充对象
        sopCells.forEach(sopCell -> {
            sopCell.setContentObjs(objMap.getOrDefault(sopCell.getSopCellId(), new ArrayList<>()));
        });
    }

    /***
     * Description: 填充专业
     * @param sopCells : SOP的cell对象
     * @return : void
     * @author : lijie
     * @date :2021/4/7 14:01
     * Update By lijie 2021/4/7 14:01
     */
    private void fillDomainForSopCell(Set<SopCell> sopCells) {
        if (CollUtil.isEmpty(sopCells)) {
            return;
        }
        // 1.根据SOP的Cell id查询关联的对象列表
        Map<String, List<SopDomainRel>> domainMap = querySopDomainRelsByParentIds(sopCells.stream()
                .map(SopCell::getSopCellId).collect(Collectors.toSet()));
        // 2.填充对象
        sopCells.forEach(sopCell -> {
            List<SopDomainRel> sopDomainRels = domainMap.get(sopCell.getSopCellId());
            if (CollUtil.isEmpty(sopDomainRels)) {
                return;
            }
            sopCell.setDomain(sopDomainRels.get(0).getDomain());
        });
    }

    /***
     * Description: 填充空间对象
     * @param sopCells : SOP的cell对象
     * @return : void
     * @author : lijie
     * @date :2021/4/7 14:01
     * Update By lijie 2021/4/7 14:01
     */
    private void fillSpaceObjsForSopCell(Set<SopCell> sopCells) {
        if (CollUtil.isEmpty(sopCells)) {
            return;
        }
        // 1.根据SOP的Cell id查询关联的对象列表
        Map<String, List<SopObjRel>> objMap = queryObjMapBySopIds(sopCells.stream()
                .map(SopCell::getSopCellId).collect(Collectors.toSet()), ObjOwnerEnum.SPACE_SCOPE.getType());
        // 2.填充对象
        sopCells.forEach(sopCell -> {
            sopCell.setSpaceScope(objMap.getOrDefault(sopCell.getSopCellId(), new ArrayList<>()));
        });
    }

    /***
     * Description: 根据父级id获得SopCell映射
     * @param sopCells : SopCell数组
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopCell>>
     * @author : lijie
     * @date :2021/4/7 11:32
     * Update By lijie 2021/4/7 11:32
     */
    private Map<String, List<SopCell>> getMapByParentId(Set<SopCell> sopCells) {
        if (CollUtil.isEmpty(sopCells)) {
            return new HashMap<>();
        }
        Map<String, List<SopCell>> result = new HashMap<>();
        for (SopCell sopCell : sopCells) {
            List<SopCell> sons = result.getOrDefault(sopCell.getParentId(), new ArrayList<>());
            sons.add(sopCell);
            result.put(sopCell.getParentId(), sons);
        }
        return result;
    }

    /***
     * Description: 获得对象id集合
     * @param saveSopVo : 请求参数
     * @return : java.util.Set<java.lang.String>
     * @author : lijie
     * @date :2021/4/8 18:53
     * Update By lijie 2021/4/8 18:53
     */
    private Set<String> getObjIdsFromParam(SaveSopVo saveSopVo) {
        Set<String> result = new HashSet<>();
        // 处理适用范围
        List<FitObjsBean> fitObjs = saveSopVo.getFitObjs();
        if (CollUtil.isNotEmpty(fitObjs)) {
            for (FitObjsBean fitObj : fitObjs) {
                if (StrUtil.isNotBlank(fitObj.getObjId())) {
                    result.add(fitObj.getObjId());
                }
            }
        }
        List<SopStepsBean> steps = saveSopVo.getSteps();
        if (CollUtil.isEmpty(steps)) {
            return result;
        }
        for (SopStepsBean step : steps) {
            List<StepContentBean> stepContent = step.getStepContent();
            if (CollUtil.isEmpty(stepContent)) {
                continue;
            }
            for (StepContentBean stepContentBean : stepContent) {
                List<ContentObjsBean> contentObjs = stepContentBean.getContentObjs();
                if (CollUtil.isNotEmpty(contentObjs)) {
                    for (ContentObjsBean contentObj : contentObjs) {
                        if (StrUtil.isNotBlank(contentObj.getObjId())) {
                            result.add(contentObj.getObjId());
                        }
                    }
                }
                List<SpaceScopeBean> spaceScope = stepContentBean.getSpaceScope();
                if (CollUtil.isNotEmpty(spaceScope)) {
                    for (SpaceScopeBean spaceScopeBean : spaceScope) {
                        if (StrUtil.isNotBlank(spaceScopeBean.getObjId())) {
                            result.add(spaceScopeBean.getObjId());
                        }
                    }
                }
                List<ConfirmResultBean> confirmResultBeans = CollUtil.defaultIfEmpty(stepContentBean.getConfirmResult(), new ArrayList<>());
                for (ConfirmResultBean confirmResultBean : confirmResultBeans) {
                    List<InfoPointsBean> infoPointsBeans = CollUtil.defaultIfEmpty(confirmResultBean.getInfoPoints(),new ArrayList<>());
                    for (InfoPointsBean infoPointsBean : infoPointsBeans) {
                        if (StrUtil.isNotBlank(infoPointsBean.getId())){
                            result.add(infoPointsBean.getId());
                        }
                    }
                }
            }
        }
        return result;
    }

    /***
     * Description: 获取对象id数组
     * @param saveSopVo : 请求参数
     * @return : void
     * @author : lijie
     * @date :2021/4/2 16:57
     * Update By lijie 2021/4/2 16:57
     */
    private Set<String> getSopObjects(SaveSopVo saveSopVo) {
        Set<String> objIds = new HashSet<>();
        // 1.适用范围
        List<FitObjsBean> fitObjs = CollUtil.defaultIfEmpty(saveSopVo.getFitObjs(), new ArrayList<>());
        for (FitObjsBean fitObj : fitObjs) {
            if (StrUtil.isNotBlank(fitObj.getObjId())
                    && StrUtil.isNotBlank(fitObj.getObjType())
                    && OBJ_TYPES.contains(fitObj.getObjType())) {
                objIds.add(fitObj.getObjId());
            }
        }
        // 3.步骤
        List<SopStepsBean> stepsBeans = CollUtil.defaultIfEmpty(saveSopVo.getSteps(), new ArrayList<>());
        for (SopStepsBean stepsBean : stepsBeans) {
            // 1.步骤校验
            List<StepContentBean> stepContentBeans = CollUtil.defaultIfEmpty(stepsBean.getStepContent(), new ArrayList<>());
            for (StepContentBean stepContentBean : stepContentBeans) {
                // 1.关联对象
                List<ContentObjsBean> contentObjsBeans = CollUtil.defaultIfEmpty(stepContentBean.getContentObjs(), new ArrayList<>());
                for (ContentObjsBean contentObjsBean : contentObjsBeans) {
                    if (StrUtil.isNotBlank(contentObjsBean.getObjId())
                            && StrUtil.isNotBlank(contentObjsBean.getObjType())
                            && OBJ_TYPES.contains(contentObjsBean.getObjType())) {
                        objIds.add(contentObjsBean.getObjId());
                    }
                }
            }
        }
        return objIds;
    }

    /**
     * 处理引用当前sop的所有sop简介
     *
     * @return
     * @throws Exception
     */
    public List<RefedSopBean> getSopProfileOfRefCurrentSop(String srcSopId) throws Exception {
        // 1.根据原始SOP的唯一标识查询其关联的SOP内容
        List<SopCell> sopCells = querySopCellBySopId(srcSopId);
        if (CollUtil.isEmpty(sopCells)) {
            return new ArrayList<>();
        }
        // 2.查询所有的引用SOP的详情
        List<String> relSopIds = sopCells.stream().map(SopCell::getRelSopId).collect(Collectors.toList());
        List<Sop> sops = sopService.listByIds(relSopIds);
        List<SopBean> sopBeans = fillSopDetail(sops);
        List<RefedSopBean> result = new ArrayList<>();
        for (SopBean sopBean : sopBeans) {
            RefedSopBean refedSopBean = RefedSopBean.builder()
                    .refSopId(sopBean.getSopId())
                    .refSopName(sopBean.getSopName())
                    .statusExplain(sopBean.getStautsExplain())
                    .stepCount(sopBean.getStepCount())
                    .version(sopBean.getVersion())
                    .updateTime(sopBean.getUpdateTime())
                    .sopType(sopBean.getSopType())
                    .operatorId(sopBean.getOperatorId())
                    .operatorName(sopBean.getOperatorName())
                    .sopStatus(sopBean.getSopStatus())
                    .publishStatus(sopBean.getPublishStatus())
                    .build();
            if (!SopCommonConst.GROUP_SOP_ID.equals(sopBean.getProjectId())){
                refedSopBean.setProjectId(sopBean.getProjectId());
                refedSopBean.setProjectName(projectCache.getProjectNameById(sopBean.getProjectId()));
            }
            result.add(refedSopBean);
        }
        return result;
    }

    /***
     * Description: 获取当前传入的sop引用的sopId集合
     * @param sopStepsBeans : sop的步骤数组
     * @return : com.alibaba.fastjson.JSONArray
     * @author : lijie
     * @date :2021/4/2 14:38
     * Update By lijie 2021/4/2 14:38
     */
    public List<String> getSopRefSops(List<SopStepsBean> sopStepsBeans) {
        Set<String> refSopIds = new HashSet<>();
        if (CollUtil.isEmpty(sopStepsBeans)) {
            return new ArrayList<>();
        }
        for (SopStepsBean sopStepsBean : sopStepsBeans) {
            if (sopStepsBean.getFromSop()) {
                refSopIds.add(sopStepsBean.getSopId());
                continue;
            }
            List<StepContentBean> stepContents = sopStepsBean.getStepContent();
            if (CollUtil.isEmpty(stepContents)) {
                continue;
            }
            for (StepContentBean stepContent : stepContents) {
                if (stepContent.getFromSop()) {
                    refSopIds.add(stepContent.getSopId());
                }
            }
        }
        return new ArrayList<>(refSopIds);
    }

    /***
     * Description: 转换sop状态为汉字
     * @param status : SOP状态
     * @param publishStatus : 发布状态
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/14 9:18
     * Update By lijie 2021/4/14 9:18
     */
    public String getSopStatus(String status,String publishStatus) {
        if (SopStatusEnum.PUBLISHED.getType().equals(status)) {
            status = SopStatusEnum.PUBLISHED.getDescribe();
            if (SopPublishStatusEnum.REVISE.getType().equals(publishStatus)) {
                status = status + "、" + SopPublishStatusEnum.REVISE.getDescribe();
            } else if (SopPublishStatusEnum.REVISEING.getType().equals(publishStatus)) {
                status = status + "、" + SopPublishStatusEnum.REVISEING.getDescribe();
            }
        } else {
            status = SopStatusEnum.INVALID.getDescribe();
        }
        return status;
    }

    /**
     * 处理引用的sop id与简介(sop_name,version,step_count,status_explain,update_time)
     *
     * @param srcSopId 原来的SOP的唯一标识
     * @return map      key:sopId  value:sop对象
     * @throws Exception
     */
    public Map<String, SopBean> proccessSopIdRefProfile(String srcSopId) throws Exception {
        // 1.根据原始SOP的唯一标识查询其关联的SOP内容
        List<SopCell> sopCells = querySopCellBySopId(srcSopId);
        if (CollUtil.isEmpty(sopCells)) {
            return new HashMap<>();
        }
        // 2.查询所有的引用SOP的详情
        List<String> relSopIds = sopCells.stream().map(SopCell::getRelSopId).collect(Collectors.toList());
        List<Sop> sops = sopService.listByIds(relSopIds);
        List<SopBean> sopBeans = fillSopDetail(sops);
        Map<String, SopBean> map = new HashMap<>();
        for (SopBean sopBean : sopBeans) {
            sopBean.setStepCount(sopBean.getSteps().size());
            sopBean.setStautsExplain(getSopStatus(sopBean.getSopStatus(),sopBean.getPublishStatus()));
            sopBean.setUpdateTime(DateUtil.transferDateFormat(sopBean.getUpdateTime(), CommonConstant.date_format_save, CommonConstant.date_format_show));
            map.put(sopBean.getSopId(), sopBean);
        }
        return map;
    }

    /***
     * Description: 根据父级id集合批量查询SOP的cell
     * @param parentIds : 父级id集合
     * @return : java.util.Map<java.lang.String,java.util.Set<com.persagy.fm.sop.entity.SopCell>>
     * @author : lijie
     * @date :2021/4/7 11:26
     * Update By lijie 2021/4/7 11:26
     */
    private Set<SopCell> querySopCellByParentIds(Set<String> parentIds) {
        if (CollUtil.isEmpty(parentIds)) {
            return new HashSet<>();
        }
        LambdaQueryWrapper<SopCell> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(SopCell::getParentId, parentIds);
        return new HashSet<>(sopCellService.list(queryWrapper));
    }

    /**
     * 处理sop详情 补充引用sop简介，添加内容对象变化标识
     *
     * @param sopBean
     * @param refSopProfile
     * @return
     * @Param changedObjIds     objId //对象变化数据key:objId value:{"objId":objId, "objName":objName, "objType":objType}
     */
    public SopBean proccessSopItem(SopBean sopBean, Map<String, SopBean> refSopProfile) {
        List<SopCell> steps = sopBean.getSteps();
        if (CollUtil.isEmpty(steps)) {
            return sopBean;
        }
        for (SopCell step : steps) {
            if (step.getFromSop()) {
                setSopHisCell(step, refSopProfile.get(step.getRelSopId()));
                continue;
            }
            List<SopCell> stepContents = step.getStepContent();
            if (CollUtil.isEmpty(stepContents)) {
                continue;
            }
            for (SopCell stepContent : stepContents) {
                if (stepContent.getFromSop()) {
                    setSopHisCell(stepContent, refSopProfile.get(step.getRelSopId()));
                }
            }
        }
        return sopBean;
    }

    /***
     * Description: 根据Sop的唯一标识判断
     * @param srcSopId : 原始SOP的唯一标识
     * @return : java.util.List<com.persagy.fm.sop.entity.SopCell>
     * @author : lijie
     * @date :2021/4/8 16:03
     * Update By lijie 2021/4/8 16:03
     */
    private List<SopCell> querySopCellBySopId(String srcSopId) {
        LambdaQueryWrapper<SopCell> sopCellLambdaQueryWrapper = new LambdaQueryWrapper<>();
        sopCellLambdaQueryWrapper.eq(SopCell::getValid, true);
        if (StrUtil.isNotBlank(srcSopId)) {
            sopCellLambdaQueryWrapper.eq(SopCell::getSopId, srcSopId);
        }
        return sopCellService.list(sopCellLambdaQueryWrapper);
    }

    /***
     * Description: 根据父级id集合查询所有的附件
     * @param parentIds :
     * @return : java.util.Set<com.persagy.fm.sop.entity.SopMediaRel>
     * @author : lijie
     * @date :2021/4/7 11:38
     * Update By lijie 2021/4/7 11:38
     */
    private Set<SopMediaRel> querySopMediaRelByParentIds(Set<String> parentIds) {
        if (CollUtil.isEmpty(parentIds)) {
            return new HashSet<>();
        }
        LambdaQueryWrapper<SopMediaRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(SopMediaRel::getParentId, parentIds);
        List<SopMediaRel> sopMediaRels = sopMediaRelService.list(queryWrapper);
        return new HashSet<>(sopMediaRels);
    }

    /***
     * Description: 根据父级id查询关联的工具
     * @param parentIds : 父级id集合
     * @return : java.util.List<com.persagy.fm.sop.entity.SopToolRel>
     * @author : lijie
     * @date :2021/4/7 14:29
     * Update By lijie 2021/4/7 14:29
     */
    private List<SopToolRel> querySopToolRelByParentIds(Set<String> parentIds) {
        if (CollUtil.isEmpty(parentIds)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SopToolRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(SopToolRel::getParentId, parentIds);
        return sopToolRelService.list(queryWrapper);
    }

    /***
     * Description: 填充历史SOP的内容
     * @param sopCell : SOP的Cell
     * @param refSop : 引用的Sop
     * @return : void
     * @author : lijie
     * @date :2021/4/8 15:59
     * Update By lijie 2021/4/8 15:59
     */
    private void setSopHisCell(SopCell sopCell, SopBean refSop) {
        if (null != refSop) {
            sopCell.setSopId(refSop.getSopId());
            sopCell.setStautsExplain(refSop.getStautsExplain());
            sopCell.setStepCount(refSop.getStepCount());
            sopCell.setUpdateTime(refSop.getUpdateTime());
            sopCell.setSopName(refSop.getSopName());
            sopCell.setVersion(refSop.getVersion());
        }
    }

    /***
     * Description: 校验SOP参数列表
     * @param saveSopVo : 请求参数
     * @return : java.util.Set<java.lang.String>
     * @author : lijie
     * @date :2021/4/2 15:45
     * Update By lijie 2021/4/2 15:45
     */
    private Set<String> validRequiredFields(SaveSopVo saveSopVo) {
        Set<String> lackFields = new HashSet<>();
        // 1.sop第一层的普通字段校验
        if (StrUtil.isBlank(saveSopVo.getUserId())) {
            lackFields.add(SaveSopVo.USER_ID);
        }
        if (StrUtil.isBlank(saveSopVo.getProjectId())) {
            lackFields.add(SaveSopVo.PROJECT_ID);
        }
        if (StrUtil.isBlank(saveSopVo.getSopName())) {
            lackFields.add(SaveSopVo.SOP_NAME);
        }
        if (CollUtil.isEmpty(saveSopVo.getSteps())) {
            lackFields.add(SaveSopVo.STEPS);
        }
        // 2.适用范围
        List<FitObjsBean> fitObjs = CollUtil.defaultIfEmpty(saveSopVo.getFitObjs(), new ArrayList<>());
        for (FitObjsBean fitObj : fitObjs) {
            if (StrUtil.isBlank(fitObj.getObjId())) {
                lackFields.add(SaveSopVo.FIT_OBJS + "." + SaveSopVo.OBJ_ID);
            }
            if (StrUtil.isBlank(fitObj.getObjName())) {
                lackFields.add(SaveSopVo.FIT_OBJS + "." + SaveSopVo.OBJ_NAME);
            }
            if (StrUtil.isBlank(fitObj.getObjType())) {
                lackFields.add(SaveSopVo.FIT_OBJS + "." + SaveSopVo.OBJ_TYPE);
            }
        }
        // 3.步骤
        List<SopStepsBean> stepsBeans = CollUtil.defaultIfEmpty(saveSopVo.getSteps(), new ArrayList<>());
        for (SopStepsBean stepsBean : stepsBeans) {
            // 1.步骤校验
            List<StepContentBean> stepContentBeans = CollUtil.defaultIfEmpty(stepsBean.getStepContent(), new ArrayList<>());
            for (StepContentBean stepContentBean : stepContentBeans) {
                // 1.关联对象
                List<ContentObjsBean> contentObjsBeans = CollUtil.defaultIfEmpty(stepContentBean.getContentObjs(), new ArrayList<>());
                for (ContentObjsBean contentObjsBean : contentObjsBeans) {
                    if (StrUtil.isBlank(contentObjsBean.getObjId())) {
                        lackFields.add(SaveSopVo.CONTENT_OBJS + "." + SaveSopVo.OBJ_ID);
                    }
                    if (StrUtil.isBlank(contentObjsBean.getObjType())) {
                        lackFields.add(SaveSopVo.CONTENT_OBJS + "." + SaveSopVo.OBJ_TYPE);
                    }
                    if (StrUtil.isBlank(contentObjsBean.getObjName())) {
                        lackFields.add(SaveSopVo.CONTENT_OBJS + "." + SaveSopVo.OBJ_NAME);
                    }
                }
                // 2.工作内容
                List<ConfirmResultBean> confirmResultBeans = CollUtil.defaultIfEmpty(stepContentBean.getConfirmResult(), new ArrayList<>());
                for (ConfirmResultBean confirmResultBean : confirmResultBeans) {
                    // 1.描述
                    if (StrUtil.isBlank(confirmResultBean.getDescription())) {
                        lackFields.add(SaveSopVo.CONFIRM_RESULT + "." + SaveSopVo.DESCRIPTION);
                    }
                    // 2.信息点内容
                    List<InfoPointsBean> infoPointsBeans = CollUtil.defaultIfEmpty(confirmResultBean.getInfoPoints(), new ArrayList<>());
                    for (InfoPointsBean infoPointsBean : infoPointsBeans) {
                        if (StrUtil.isBlank(infoPointsBean.getId())) {
                            lackFields.add(SaveSopVo.INFO_POINTS + "." + SaveSopVo.ID);
                        }
                        if (StrUtil.isBlank(infoPointsBean.getCode())) {
                            lackFields.add(SaveSopVo.INFO_POINTS + "." + SaveSopVo.CODE);
                        }
                        if (StrUtil.isBlank(infoPointsBean.getName())) {
                            lackFields.add(SaveSopVo.INFO_POINTS + "." + SaveSopVo.NAME);
                        }
                    }
                    // 3.自定义信息点内容
                    List<CustomInfoPointsBean> customInfoPointsBeans = CollUtil.defaultIfEmpty(confirmResultBean.getCustoms(), new ArrayList<>());
                    for (CustomInfoPointsBean customInfoPointsBean : customInfoPointsBeans) {
                        if (StrUtil.isBlank(customInfoPointsBean.getName())) {
                            lackFields.add(SaveSopVo.CUSTOMS + "." + SaveSopVo.NAME);
                        }
                        if (StrUtil.isBlank(customInfoPointsBean.getType())) {
                            lackFields.add(SaveSopVo.CUSTOMS + "." + SaveSopVo.TYPE);
                        }
                    }
                }
                // 3.引用SOP标识
                if (null == stepsBean.getFromSop()) {
                    lackFields.add(SaveSopVo.STEP_CONTENT + "." + SaveSopVo.FROM_SOP);
                }
            }
            // 2.引用SOP标识
            if (null == stepsBean.getFromSop()) {
                lackFields.add(SaveSopVo.STEPS + "." + SaveSopVo.FROM_SOP);
            }
        }
        return lackFields;
    }

}